﻿using System;
using Syncfusion.Styles;
using System.ComponentModel;

namespace Syncfusion.ComponentModel
{

    /// <summary>
    /// A framework independent utility class for the new Nullable type in .NET Framework 2.0
    /// </summary>
    public class NullableHelper
    {
        /// <summary>
        /// Use this method instead of Convert.ChangeType. Makes Convert.ChangeType work with Nullable types.
        /// </summary>
        /// <param name="value"></param>
        /// <param name="type"></param>
        /// <returns></returns>
        public static object ChangeType(object value, Type type)
        {
#if !( SyncfusionFramework1_0 || SyncfusionFramework1_1 )
            Type nullableUnderlyingType = Nullable.GetUnderlyingType(type);
            if (nullableUnderlyingType != null)
            {
                if (value is string && nullableUnderlyingType != typeof(string))
                {
                    if (ValueConvert.IsEmpty((string)value))
                        return null;
                }

                value = ChangeType(value, nullableUnderlyingType);
                if (value is DBNull)
                    return null;
                return value;
            }
#endif

            return TypeConverterHelper.ChangeType(value, type);
        }

        /// <summary>
        /// Use this method instead of Convert.ChangeType. Makes Convert.ChangeType work with Nullable types.
        /// </summary>
        /// <param name="value"></param>
        /// <param name="type"></param>
        /// <param name="provider"></param>
        /// <returns></returns>
        public static object ChangeType(object value, Type type, IFormatProvider provider)
        {
#if !( SyncfusionFramework1_0 || SyncfusionFramework1_1 )
            Type nullableUnderlyingType = Nullable.GetUnderlyingType(type);
            if (nullableUnderlyingType != null)
            {
                if (value is string && nullableUnderlyingType != typeof(string))
                {
                    if (ValueConvert.IsEmpty((string)value))
                        return null;
                }

                value = ChangeType(value, nullableUnderlyingType, provider);
                if (value is DBNull)
                    return null;
                return value;
            }
#endif
            return TypeConverterHelper.ChangeType(value, type, provider);
        }

        /// <summary>
        /// Returns null if value is DBNull and specified type is a Nullable type. Otherwise the value is returned unchanged.
        /// </summary>
        /// <param name="value"></param>
        /// <param name="type"></param>
        /// <returns></returns>
        public static object FixDbNUllasNull(object value, Type type)
        {
            if (type == null)
                return value;

#if !( SyncfusionFramework1_0 || SyncfusionFramework1_1 )
            Type nullableUnderlyingType = Nullable.GetUnderlyingType(type);
            if (nullableUnderlyingType != null)
            {
                if (value is DBNull)
                    return null;
            }
#endif

            /*
             * Do not return DBNull for strong typed properties of an object. For example, if Parsing a string failed 
             * (e.g. if an empty string was passed in as argument) we need to check if it as object and in that
             * case return null. Only if it is a ValueType type (that is not nullable) then we should return DBNull
             * so that it also works with DataRowView.
             * */
            if (!type.IsValueType)
            {
                if (value is DBNull)
                    return null;
            }
            return value;
        }

        /// <summary>
        /// Returns the underlying type of a Nullable type. For .NET 1.0 and 1.1 this method will always return null.
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static Type GetUnderlyingType(Type type)
        {
#if !( SyncfusionFramework1_0 || SyncfusionFramework1_1 )
            return type == null ? null : Nullable.GetUnderlyingType(type);
#else
            return null;
#endif

        }

        /// <exclude/>
        class TypeConverterHelper
        {
            public static object ChangeType(object value, Type type)
            {
                return ChangeType(value, type, null);
            }

            public static object ChangeType(object value, Type type, IFormatProvider provider)
            {
			//Fix for defects: 13036, 13024, 12601  & 12716
			if (value == null)
                    return null;

                TypeConverter typeConverter = TypeDescriptor.GetConverter(value.GetType());
                if (typeConverter != null && typeConverter.CanConvertTo(type))
                    return typeConverter.ConvertTo(value, type);

                if (value is DBNull)
                    return DBNull.Value;

                return Convert.ChangeType(value, type, provider);
            }
        }

    }
}
