//-------------------------------------------------------------------------------------------------
// <copyright file="GridVolatileData.cs" company="syncfusion">
//  Copyright (c) Syncfusion Inc. 2001 - 2006. All rights reserved.
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
// </copyright>
//-------------------------------------------------------------------------------------------------

using System;
using System.Collections;
using System.Diagnostics;
using System.Threading;
using Syncfusion.Styles;
using Syncfusion.Diagnostics;

namespace Syncfusion.Windows.Forms.Grid
{
    [Syncfusion.Documentation.DocumentationExclude()]
    internal interface IGridVolatileDataContainer
    {
        GridBaseStylesMap BaseStylesMap { get; }
        int HeaderRowCount { get; }
        int HeaderColCount { get; }
        void ChangeCell(int rowIndex, int colIndex, GridStyleInfo style);
        void GetCellInfo(int rowIndex, int colIndex, GridStyleInfo style);
        GridCellModelBase LookupCellModel(string id);
    }

    /// <summary>
    /// Defines the interface for a volatile data cache to be used with the <see cref="GridModel.VolatileData"/>
    /// property.
    /// </summary>
    public interface IGridVolatileData : IGridData
    {
        /// <summary>
        /// Empty the cache.
        /// </summary>
        void Clear();

        /// <summary>
        /// Gets or sets a cached value for row count.
        /// </summary>
        int RowCount 
        { 
            get; set; 
        }

        /// <summary>
        /// Gets a value indicating whether a value for row count has been cached.
        /// </summary>
        bool HasRowCount 
        { 
            get; 
        }

        /// <summary>
        /// Clears cached value for row count.
        /// </summary>
        void ResetRowCount();

        /// <summary>
        /// Gets or sets a cached value for column count.
        /// </summary>
        int ColCount 
        { 
            get; set;
        }

        /// <summary>
        /// Gets a value indicating whether a value for column count has been cached.
        /// </summary>
        bool HasColCount 
        { 
            get; 
        }

        /// <summary>
        /// Clears cached value for column count.
        /// </summary>
        void ResetColCount();

        /// <summary>
        /// Resets cache for a specific cell.
        /// </summary>
        /// <param name="cell">The cell coordinates.</param>
        void ResetItem(GridCellPos cell);
    }

    /// <summary>
    /// GridVolatileData provides GridStyleInfo objects on demand that are returned by the 
    /// grid model's indexer. GridVolatileData caches the objects and their identity information 
    /// holding a weak reference to them.
    /// </summary>
    /// <remarks>
    /// GridModel holds a GridData object. This object contains the plain data with style settings 
    /// specific to a cell. When GridVolatileData creates a style object, it asks the grid to 
    /// return the cell settings from GridData and then associate these settings with identity
    /// information. This allows access to style properties using inheritance from base styles. 
    /// Identity information also ensures that changes to the style object will be written back 
    /// correctly to GridData.
    /// </remarks>
    public class GridVolatileData : IGridVolatileData, IDisposable
    {
        private Hashtable data;
        private IGridVolatileDataContainer grid;
        private int rowCount;
        private int colCount;
        private bool hasRowCount;
        private bool hasColCount;
        private System.Threading.Thread guiThread;
        //private bool inResetItem;
        private ArrayList resetCellsList = new ArrayList();

        /// <summary>
        /// Attaches the object to the GridModel.
        /// </summary>
        /// <param name="grid">The parent grid model that will access style information through this object.</param>
        public GridVolatileData(GridModel grid)
        {
            this.grid = grid;
            this.data = new Hashtable();
            guiThread = System.Threading.Thread.CurrentThread;
        }

        internal GridVolatileData(IGridVolatileDataContainer grid)
        {
            this.grid = grid;
            this.data = new Hashtable();
            guiThread = System.Threading.Thread.CurrentThread;
        }

        #region IDisposable Members

        /// <summary>
        /// Disposes the object and collection items.
        /// </summary>
        public void Dispose()
        {
            grid = null;

            //// Data contains WeakReferences as values. So, none
            //// of the values can be disposable.
            ////foreach (object obj in data.Values)
            ////{
            ////    IDisposable dispose = obj as IDisposable;
            ////    if (dispose != null)
            ////        dispose.Dispose();
            ////}
            GC.SuppressFinalize(this);
        }

        #endregion

        /// <summary>
        /// Gets the base styles map.
        /// </summary>
        public GridBaseStylesMap BaseStylesMap
        {
            get { return grid.BaseStylesMap; }
        }

        /// <summary>
        /// Looks up a <see cref="GridCellModelBase"/> for a given cell type as specified with <see cref="GridStyleInfo.CellType"/>.
        /// </summary>
        /// <param name="id">Cell type name.</param>
        /// <returns>Cell model.</returns>
        public GridCellModelBase LookupCellModel(string id)
        {
            return grid.LookupCellModel(id);
        }

        /// <summary>
        /// Gets a reference to the parent grid model.
        /// </summary>
        public GridModel Grid
        {
            get
            {
                return (GridModel)grid;
            }
        }

        // RemoveRows, MoveRows, DeleteRows etc. events will simply empty the cache.

        // Clear will remove all weak references. Although the referenced style will 
        // not necessarily be finalized immediately, they won't be accessed through GridVolatileData 
        // anymore. GridVolatileData will return a new style object instead.

        /// <summary>
        /// Empty the cache.
        /// </summary>
        public virtual void Clear()
        {
            data.Clear();
            hasRowCount = false;
            hasColCount = false;
        }

        /// <summary>
        /// Gets or sets a cached value for row count.
        /// </summary>
        public int RowCount
        {
            get
            {
                return rowCount;
            }

            set
            {
                rowCount = value;
                hasRowCount = true;
            }
        }

        /// <summary>
        /// Gets a value indicating whether a value for row count has been cached.
        /// </summary>
        public bool HasRowCount
        {
            get
            {
                return hasRowCount;
            }
        }

        /// <summary>
        /// Clears cached value for row count.
        /// </summary>
        public void ResetRowCount()
        {
            hasRowCount = false;
        }

        /// <summary>
        /// Gets or sets a cached value for column count.
        /// </summary>
        public int ColCount
        {
            get
            {
                return colCount;
            }

            set
            {
                colCount = value;
                hasColCount = true;
            }
        }

        /// <summary>
        /// Gets a value indicating whether a value for column count has been cached.
        /// </summary>
        public bool HasColCount
        {
            get
            {
                return hasColCount;
            }
        }

        /// <summary>
        /// Clears cached value for column count.
        /// </summary>
        public void ResetColCount()
        {
            hasColCount = false;
        }

        /// <summary>
        /// Gets the cached style information for a specific cell.
        /// </summary>
        /// <param name="cell">The cell coordinate.</param>
        /// <returns>The cached GridStyleInfo; NULL if no cache entry was found.</returns>
        public GridStyleInfo GetItem(GridCellPos cell)
        {
            // Returns NULL if cell is not found.
            object obj = data[cell];
            if (obj is WeakReference)
            {
                return ((WeakReference)obj).Target as GridStyleInfo;
            }

            return null;
        }

        internal int forceGcCollect = int.MaxValue;

#if DEB
        int maxDataCount = 100;
#endif

        /// <summary>
        /// Saves style information to be cached.
        /// </summary>
        /// <param name="cell">The cell coordinate.</param>
        /// <param name="style">The GridStyleInfo to be cached.</param>
        public void SetItem(GridCellPos cell, GridStyleInfo style)
        {
            try
            {
                if (data.Count >= forceGcCollect)
                {
                    GC.Collect();
                }

                ProcessResetCellsList();

#if DEB
                if (data.Count > maxDataCount)
                {
                    Console.WriteLine("VolatileData HashTable: " + data.Count.ToString());
                    maxDataCount = data.Count + data.Count/10; // let's just show a Trace statement for larger increases ...
                }
#endif
                data[cell] = new WeakReference(style);
            }
            catch (InvalidOperationException ex)
            {
                // should be InvalidOperation_HashInsertFailed=Hashtable insert failed.  Load factor too high.
                TraceUtil.TraceExceptionCatched(ex);
                Trace.WriteLine("Detected too many items in volatile date hashtable. Reducing GridVolatileData.forceGcCollect to a value below " + data.Count.ToString());
                TraceUtil.TraceCurrentMethodInfo("Hashtable count: ", data.Count);
                forceGcCollect = Math.Max(1000, data.Count - 1000);
                GC.Collect();
                TraceUtil.TraceCurrentMethodInfo("Hashtable count after GC.Collect: ", data.Count);
                data[cell] = new WeakReference(style);
            }
        }

        /// <summary>
        /// Resets cache for a specific cell.
        /// </summary>
        /// <param name="cell">The cell coordinates.</param>
        public void ResetItem(GridCellPos cell)
        {
#if DEBUG
            if (Switches.VolatileData.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(cell);
            }
#else
            ;
#endif

            if (Thread.CurrentThread != guiThread)
            {
                // In GC Thread - prevent ProcessResetCellsList 
#if SyncfusionFramework4_0
                bool lockTaken = false;
                Monitor.Enter(this, ref lockTaken);
#else
                Monitor.Enter(this);
#endif
                try
                {
                    resetCellsList.Add(cell);
                }
                finally
                {
                    Monitor.Exit(this);
                }
            }
            else
            {
                if (data.Contains(cell))
                {
                    data.Remove(cell);
                }
            }
        }

        void ProcessResetCellsList()
        {
            // Wait until list has filled up a bit.
            if (resetCellsList.Count < 10)
            {
                return;
            }

            // Skip processing list at this time if GC thread is accessing resetCellsList.
            // It is fine to process the list at a later SetItem() call. No need to lock the GC thread.

#if SyncfusionFramework4_0
            bool lockTaken = false;
            Monitor.TryEnter(this, ref lockTaken);
            if (!lockTaken)
            {
                return;
            }
#else
            if (!Monitor.TryEnter(this))
            {
                return;
            }
#endif

            ArrayList list = resetCellsList;

            // Immediately swap out resetCellsList in case GC thread finalizes a
            // style identity and try to modify the list while we process this current list below.
            resetCellsList = new ArrayList();

            Monitor.Exit(this);

            // We now work on the old resetCellsList. No other object has access to this list
            // anymore at this point.
            for (int n = 0; n < list.Count; n++)
            {
                object cell = list[n];
                WeakReference wr = (WeakReference)data[cell]; // Returns null if cell is not found.
                // double check that cell has not been recreated in the meantime.
                if (wr == null || wr.Target == null)
                {
                    data.Remove(cell);
                }
            }
        }

        /// <summary>
        /// Gets an array that consists of table, row, and column base styles for the specified row and column index.
        /// </summary>
        /// <param name="name">The name of the base style.</param>
        /// <param name="level">The maximum number of levels to look at when walking reference base styles.</param>
        /// <returns>An array that consists of table, row, and column base styles.</returns>
        public GridStyleInfo[] GetBaseStylesMapStyles(string name, out int level)
        {
            GridBaseStylesMap styleInfoMap = this.BaseStylesMap;
            return styleInfoMap.GetBaseStylesMapStyles(name, out level);
        }

        bool inGetBaseStyles = false;

        /// <summary>
        /// Gets an array that consists of table, row, and column base styles for the specified row and column index.
        /// </summary>
        /// <param name="thisStyleInfo">Cell style information.</param>
        /// <param name="rowIndex">Row index.</param>
        /// <param name="colIndex">Column index.</param>
        /// <returns>An array that consists of table, row, and column base styles.</returns>
        public GridStyleInfo[] GetBaseStyles(GridStyleInfo thisStyleInfo, int rowIndex, int colIndex)
        {
            GridStyleInfo[] cellStyles;
            int cellStyleCount = 0;

            // Row, Column, and Table style
            int headerRowCount = grid.HeaderRowCount;
            int headerColCount = grid.HeaderColCount;
            if (colIndex > headerColCount && rowIndex > headerRowCount)
            {
                if (rowIndex == 0)
                {
                    cellStyles = new GridStyleInfo[] 
                        {
                            RowStyles[rowIndex],
                            TableStyle
                        };
                }
                else if (colIndex == 0)
                {
                    cellStyles = new GridStyleInfo[] 
                        {
                            ColStyles[colIndex],
                            TableStyle
                        };
                }
                else
                {
                    cellStyles = new GridStyleInfo[] 
                        {
                            RowStyles[rowIndex],
                            ColStyles[colIndex],
                            TableStyle
                        };
                }

                cellStyleCount = cellStyles.Length;
            }
            else if ((colIndex >= 0 && rowIndex < 0) || (rowIndex >= 0 && colIndex < 0))
            {
                cellStyles = new GridStyleInfo[] 
                    {
                        TableStyle
                    };
                cellStyleCount = cellStyles.Length;
            }
            else
            {
                cellStyles = null;
            }

            // Base style.
            GridStyleInfo item = thisStyleInfo;
            if (thisStyleInfo == null)
            {
                item = this[rowIndex, colIndex];
            }

            string baseStyleName = item.HasBaseStyle ? item.BaseStyle : string.Empty;

            if (baseStyleName.Length == 0 && cellStyles != null)
            {
                foreach (GridStyleInfo si in cellStyles)
                {
                    if (si.HasBaseStyle)
                    {
                        baseStyleName = si.BaseStyle;
                        if (baseStyleName.Length > 0)
                        {
                            break;
                        }
                    }
                }
            }

            // Row or column header.
            if (baseStyleName.Length == 0)
            {
                if (rowIndex >= 0 && colIndex >= 0)
                {
                    if (rowIndex <= headerRowCount && colIndex <= headerColCount)
                    {
                        baseStyleName = "Header";
                    }
                    else if (rowIndex <= headerRowCount)
                    {
                        baseStyleName = "Column Header";
                    }
                    else if (colIndex <= headerColCount)
                    {
                        baseStyleName = "Row Header";
                    }
                }
            }

            // Load all parent base styles (including standard style).
            int level;
            GridStyleInfo[] infoMapStyles = GetBaseStylesMapStyles(baseStyleName, out level);

            // Combine the two arrays.
            GridStyleInfo[] baseStyles = new GridStyleInfo[cellStyleCount + level];
            if (cellStyles != null)
            {
                Array.Copy(cellStyles, 0, baseStyles, 0, cellStyleCount);
            }

            if (infoMapStyles != null)
            {
                Array.Copy(infoMapStyles, 0, baseStyles, cellStyleCount, level);
            }

            // Each GridStyleInfoIdentity will cache the baseStyles.
            return baseStyles;
        }

        // GridControlBaseCore.this[row, col] will forward its requests to GridVolatileData.this[].

        // GridVolatileData allocates its own style objects and maintains a weak reference to them.
        // Obsolete -> Once the style object is finalized, the weak reference is removed from the hashtable. <-

        // The user is !!!always!!! operating on "smart" copy of a style object. This style object 
        // knows about its base styles and other information (see GridStyleInfoIdentity). Changes in this object will result in 
        // SetStyleRange/StoreStyleRowCol to update changes to GridData. GridData can be dumb. It does 
        // not need to know about rowindex, colindex, base style etc. 

        /// <summary>
        /// Gives access to a <see cref="GridStyleInfo"/> at a given row and column index. 
        /// </summary>
        public GridStyleInfo /*IGridData*/ this[int rowIndex, int colIndex]
        {
            get
            {
                GridCellPos cell = new GridCellPos(rowIndex, colIndex);

                GridStyleInfo style;
                if (inGetBaseStyles)
                {
                    style = new GridStyleInfo(new GridStyleInfoIdentity(this, rowIndex, colIndex, true));
                    grid.GetCellInfo(rowIndex, colIndex, style);
                }
                else
                {
                    style = GetItem(cell);
                    ////                
                    if (style == null)
                    {
#if DEBUG
                        Trace.WriteLineIf(Switches.VolatileData.TraceVerbose, String.Format("VolatileData[{0},{1} allocates style object", rowIndex, colIndex));
#endif
                        //// Allocate style object and initialize its identity.
                        style = new GridStyleInfo(new GridStyleInfoIdentity(this, rowIndex, colIndex));
                        //// Avoid OnStyleChanged methods.
                        style.BeginInit();
                        //// Load data.
                        grid.GetCellInfo(rowIndex, colIndex, style);
                        style.EndInit();
                        //// Save weak reference to style object.
                        SetItem(cell, style);
                    }
                }

                return style;
            }

            set
            {
                grid.ChangeCell(rowIndex, colIndex, value);
                if (value != null)
                {
                    value.Store.ResetChangedBits();
                }
            }
        }

        //// GridVolatileData allocates its own style objects and maintains a weak reference to them.
        //// Once the style object is finalized, the weak reference is removed from the hashtable.

        /// <summary>
        /// Cached row styles within GridVolatileData.
        /// </summary>
        public class RowStylesIndexer
        {
            GridVolatileData dataParent;

            internal RowStylesIndexer(GridVolatileData data)
            {
                dataParent = data;
            }

            /// <summary>
            /// Gives access to a <see cref="GridStyleInfo"/> at a given row index. 
            /// </summary>
            public GridStyleInfo this[int rowIndex]
            {
                get
                {
                    return dataParent[rowIndex, -1];
                }

                set
                {
                    dataParent[rowIndex, -1] = value;
                }
            }
        }

        private RowStylesIndexer rowStyles = null;

        /// <summary>
        /// Gets the cached row styles within GridVolatileData.
        /// </summary>
        public RowStylesIndexer RowStyles
        {
            get
            {
                if (rowStyles == null)
                {
                    rowStyles = new RowStylesIndexer(this);
                }

                return rowStyles;
            }
        }

        /// <summary>
        /// Cached column styles within GridVolatileData.
        /// </summary>
        public class ColStylesIndexer
        {
            GridVolatileData dataParent;

            internal ColStylesIndexer(GridVolatileData data)
            {
                dataParent = data;
            }

            /// <summary>
            /// Gives access to a <see cref="GridStyleInfo"/> at a given column index. 
            /// </summary>
            public GridStyleInfo this[int colIndex]
            {
                get
                {
                    return dataParent[-1, colIndex];
                }

                set
                {
                    dataParent[-1, colIndex] = value;
                }
            }
        }

        private ColStylesIndexer colStyles = null;

        /// <summary>
        /// Gets the cached column styles within GridVolatileData
        /// </summary>
        public ColStylesIndexer ColStyles
        {
            get
            {
                if (colStyles == null)
                {
                    colStyles = new ColStylesIndexer(this);
                }

                return colStyles;
            }
        }

        /// <summary>
        /// Gets or sets access to a <see cref="GridStyleInfo"/> for the table style. 
        /// </summary>
        public GridStyleInfo TableStyle
        {
            get
            {
                return this[-1, -1];
            }

            set
            {
                this[-1, -1] = value;
            }
        }
    }
}
