//-------------------------------------------------------------------------------------------------
// <copyright file="GridFormulaCell.cs" company="syncfusion">
//  Copyright (c) Syncfusion Inc. 2001 - 2006. All rights reserved.
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
// </copyright>
//-------------------------------------------------------------------------------------------------

using System;
using System.Windows.Forms;
using System.Collections;
using System.Reflection;
using System.Globalization;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.Security;
using System.Security.Permissions;
using System.Data;

using Syncfusion.Windows.Forms;
using Syncfusion.Styles;
using Syncfusion.Diagnostics;
using Syncfusion.Drawing;
using System.Text;
using System.Text.RegularExpressions;

namespace Syncfusion.Windows.Forms.Grid
{
    #region  CellModel Class

    /// <summary>
    /// This GridCellModel supports entering formulas in a grid cell. This support is provided
    /// through the <see cref="GridFormulaCellModel.Engine"/> member of this class. Engine is
    /// a <see cref="GridFormulaEngine"/>.
    /// </summary>
    /// <remarks>
    /// The default behavior is that any cell whose CellType is GridFormulaCellModel will
    /// be interpreted as a formula cell provided the text in the cell starts with '='. 
    /// So, for such cells you would enter formulas such as
    /// =A1+A2+A3 or =Sum(A1:A3), an an attempt would be made to parse and compute the entry.
    /// <para/>
    /// Alternatively, you can use the <see cref="GridFormulaCellModel.formulaChar"/> character
    /// to specify which cells of CellType GridFormulaCellModel are to be used as formulas.
    /// You set this formulaChar private property through the class constructor which passes
    /// this parameter. If you set this value to '\0', every cell of CellType GridFormulaCellModel
    /// will be treated as a formula.
    /// <para/>
    /// The <see cref="RefreshCells"/> method redraws all the formula cells that depend upon
    /// a particular cell. This method is called when the user changes a value that affects
    /// other formula cells.
    /// <para/>
    /// The <see cref="GetFormattedText"/> method is where the actual calculations are done,
    /// if necessary, through calls to the Engine methods. This method also conditionally
    /// parses the formula only if it has been modified since the previous parse.
    /// </remarks>
    [Serializable]
    public class GridFormulaCellModel : GridTextBoxCellModel
    {
        /// <summary>
        /// Gets <see cref="GridFormulaEngine"/> encapsulates all formula features. Use this object
        /// to add / remove library functions.
        /// </summary>
        public GridFormulaEngine Engine
        {
            get
            {
                return engine;
            }
        }

        private GridFormulaEngine engine;
        private GridModel gridModel;

        /// <summary>
        /// Set this field to a particular character such as '=' to conditionally flag a cell
        /// whose CellType is GridFormulaCellModel as holding a formula. If you do not 
        /// set this field, then all cells of CellType GridFormulaCellModel that begin with '='
        /// will be treated as a formula. You set this private field as the call constructor. 
        /// If you set this value to '\0', every cell of CellType GridFormulaCellModel
        /// will be treated as a formula.
        /// </summary>
        private char formulaChar;

        /// <overload>
        /// Initializes a new GridFormulaCellModel object.
        /// </overload>
        /// <summary>
        /// Use this constructor to force all cells of CellType GridFormulaCellModel whose
        /// text begin with '=' to be regarded as holding formulas.
        /// </summary>
        /// <param name="grid">The GridModel using this GridFormulaCellModel.</param>
        public GridFormulaCellModel(GridModel grid)
            : this(grid, '=')
        {
            this.gridModel = grid;
        }

        /// <summary>
        /// Only cells whose contents begin with formulaChar will be treated as a formula.
        /// Other cells of CellType GridFormulaCellModel will be treated as GridTextBoxCellModel.
        /// The character formulaChar is ignored for parsing and computing purposes.
        /// </summary>
        /// <param name="grid">The GridModel using this GridFormulaCellModel.</param>
        /// <param name="formulaChar">Any GridFormulaCellModel beginning with this
        /// character will be treated as a formula. If you set this value to '\0', 
        /// every cell of CellType GridFormulaCellModel
        /// will be treated as a formula.</param>
        public GridFormulaCellModel(GridModel grid, char formulaChar)
            : base(grid)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(formulaChar);
            }
#else
            ;
#endif
            this.gridModel = grid;
            this.formulaChar = formulaChar;
            engine = new GridFormulaEngine(grid);
            Engine.formulaChar = formulaChar;

            ////Used to compute dependent cells.
            Grid.CellsChanged += new GridCellsChangedEventHandler(GridCellsChanged);

            ////Used to adjust formula references.
            Grid.ClipboardPasted += new GridCutPasteEventHandler(GridClipboardPasted);
            Grid.ClipboardPaste += new GridCutPasteEventHandler(GridClipboardPaste);
            Grid.ColsInserted += new GridRangeInsertedEventHandler(GridColsInserted);
            Grid.ColsRemoved += new GridRangeRemovedEventHandler(GridColsRemoved);
            Grid.RowsInserted += new GridRangeInsertedEventHandler(GridRowsInserted);
            Grid.RowsRemoved += new GridRangeRemovedEventHandler(GridRowsRemoved);

            ////Handle copy to copy formulas / values according to GridFormulaCopyFlags.
            Grid.ClipboardCopy += new GridCutPasteEventHandler(GridClipboardCopy);

            ////handle d&d reference updates
            if (Grid.ActiveGridView != null)
            {
                Grid.ActiveGridView.DragDrop += new DragEventHandler(GridDragDrop);
            }
        }

        /// <summary>
        /// Initializes a new <see cref="GridFormulaCellModel"/> from a serialization stream.
        /// </summary>
        /// <param name="info">An object that holds all the data needed to serialize or deserialize this instance.</param>
        /// <param name="context">Describes the source and destination of the serialized stream specified by info. </param>
        protected GridFormulaCellModel(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
#if DEBUG
            if (Switches.Serialization.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(info.FullTypeName, info.MemberCount);
            }
#else
            ;
#endif
            this.formulaChar = (char)info.GetValue("FormulaChar", typeof(char));
        }

        /// <summary>
        /// Constructor for use with an existing GridFormulaEngine.
        /// </summary>
        /// <param name="grid">The GridModel using this GridFormulaCellModel.</param>
        /// <param name="engine">An existing GridFormulaEngine.</param>
        /// <param name="formulaChar">Any GridFormulaCellModel beginning with this
        /// character will be treated as a formula. If you set this value to '\0', 
        /// every cell of CellType GridFormulaCellModel
        /// will be treated as a formula.</param>
        /// <remarks>Use this constructor if you want to use derived GridFormulaCellModel
        ///  and GridFormulaCellRenderer classes that share a single GridFormulaEngine.
        /// </remarks>
        public GridFormulaCellModel(GridModel grid, GridFormulaEngine engine, char formulaChar)
            : this(grid, formulaChar)
        {
            ////unsubscribe to the baseclass events...
            Grid.CellsChanged -= new GridCellsChangedEventHandler(GridCellsChanged);
            Grid.ClipboardPasted -= new GridCutPasteEventHandler(GridClipboardPasted);
            Grid.ClipboardPaste -= new GridCutPasteEventHandler(GridClipboardPaste);
            Grid.ColsInserted -= new GridRangeInsertedEventHandler(GridColsInserted);
            Grid.ColsRemoved -= new GridRangeRemovedEventHandler(GridColsRemoved);
            Grid.RowsInserted -= new GridRangeInsertedEventHandler(GridRowsInserted);
            Grid.RowsRemoved -= new GridRangeRemovedEventHandler(GridRowsRemoved);
            Grid.ClipboardCopy -= new GridCutPasteEventHandler(GridClipboardCopy);
            if (Grid.ActiveGridView != null)
            {
                Grid.ActiveGridView.DragDrop -= new DragEventHandler(GridDragDrop);
            }

            this.engine = null;
            this.engine = engine;
        }

        /// <summary>
        /// Called from GridModel implementation of IDeserializationCallback.OnDeserialization.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="model">The <see cref="GridModel"/> that was deserialized.</param>
        protected override void OnModelDeserialization(object sender, GridModel model)
        {
            base.OnModelDeserialization(sender, model);

            ////Now we can instantiate the engine since we have a reference to the gridModel.
            engine = new GridFormulaEngine(model);
            Engine.formulaChar = this.formulaChar;
            model.CellsChanged += new GridCellsChangedEventHandler(GridCellsChanged);
            model.ClipboardPasted += new GridCutPasteEventHandler(GridClipboardPasted);
            model.ClipboardPaste += new GridCutPasteEventHandler(GridClipboardPaste);
            model.ColsInserted += new GridRangeInsertedEventHandler(GridColsInserted);
            model.ColsRemoved += new GridRangeRemovedEventHandler(GridColsRemoved);
            model.RowsInserted += new GridRangeInsertedEventHandler(GridRowsInserted);
            model.RowsRemoved += new GridRangeRemovedEventHandler(GridRowsRemoved);
            model.ClipboardCopy += new GridCutPasteEventHandler(GridClipboardCopy);
            if (model.ActiveGridView != null)
            {
                model.ActiveGridView.DragDrop += new DragEventHandler(GridDragDrop);
            }
        }

        /// <summary>
        /// Implements the ISerializable interface and returns the data needed to serialize the cell model.
        /// </summary>
        /// <param name="info">A SerializationInfo object containing the information required to serialize the cell model.</param>
        /// <param name="context">A StreamingContext object containing the source and destination of the serialized stream.</param>
        [SecurityPermissionAttribute(SecurityAction.Demand, SerializationFormatter = true)]
        [SecurityPermissionAttribute(SecurityAction.LinkDemand, SerializationFormatter = true)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
#if DEBUG
            if (Switches.Serialization.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(info.FullTypeName, info.MemberCount);
            }
#else
            ;
#endif
            info.AddValue("FormulaChar", this.formulaChar); // FormulaChar
            base.GetObjectData(info, context);
        }

        /// <summary>
        /// This method forces any formula cell dependent upon the passed-in cell in the row
        /// and column to be recomputed. This method is called by the model whenever
        /// the contents of the passed-in cell is changed. For example, the user changing a
        /// single non-formula cell may require the recomputing of several additional formula cells. 
        /// This method triggers the redrawing of these dependent cells.
        /// </summary>
        /// <param name="row">Row index of the cell that was modified by the user.</param>
        /// <param name="col">Column index of the cell that was modified by the user.</param>
        public void RefreshCells(int row, int col)
        {
            if (CalculatingSuspended)
            {
                return;
            }

            GridSheetFamilyItem family = GridFormulaEngine.GetSheetFamilyItem(this.gridModel);
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(row, col, this);
            }
#else
            ;
#endif
            string sheet = family.GridModelToToken != null
                        ? family.GridModelToToken[this.Grid] as string : string.Empty;
            string s = sheet + GridRangeInfo.GetAlphaLabel(col) + row.ToString();

            Engine.Refresh(s);
        }

        /// <summary>
        /// Recalculates every cell that depends upon any cell in the passed-in range.
        /// </summary>
        /// <remarks>Calls the Engine.RefreshRange.</remarks>
        /// <param name="range">GridRangeInfo object to be refreshed.</param>
        public void RefreshRange(GridRangeInfo range)
        {
            Engine.RefreshRange(range);
        }

        /// <summary>
        /// Gets or sets a value indicating whether formulas are immediately calculated as dependent cells are changed.
        /// </summary>
        /// <remarks>Use this property to suspend calculations while a series of changes 
        /// are made to dependent cells either by the user or programmatically. When the changes are 
        /// complete, set this property to false, and then call RecalculateRange to recalculate
        /// the affected range. 
        /// <para/> 
        /// The implementation delegates this property to the Engine.CalculatingSuspended property.
        /// </remarks>
        /// <example> Use this code to suspend, and then later resume, calculations.
        /// <code lang="C#">
        ///        //suspend calculations...
        ///        GridFormulaCellModel formulaModel = this.gridControl1.CellModels["FormulaCell"] as GridFormulaCellModel;
        ///        formulaModel.CalculatingSuspended = true;
        ///  <para/>       
        ///        ....
        ///        //make changes, say to, cells in the GridRangeInfo.Cells(1,1,10,20).
        ///        ....
        /// <para/>        
        ///        //later resume calculations and refresh any cells that depend on GridRangeInfo.Cells(1,1,10,20)
        ///        GridFormulaCellModel formulaModel = this.gridControl1.CellModels["FormulaCell"] as GridFormulaCellModel;
        ///        formulaModel.CalculatingSuspended = false;
        ///        formulaModel.RefreshRange(GridRangeInfo.Cells(1,1,10,20));
        ///  <para/>        
        /// </code>
        /// <code lang="VB">
        ///        'suspend calculations...
        ///        Dim formulaModel As GridFormulaCellModel = Me.gridControl1.CellModel("FormulaCell") 
        ///        formulaModel.CalculatingSuspended = True
        /// <para/>                                        
        ///        ....
        ///        'make changes, say to, cells in the GridRangeInfo.Cells(1,1,10,20).
        ///        ....
        /// <para/>        
        ///        'later resume calculations and refresh any cells that depend on GridRangeInfo.Cells(1,1,10,20)
        ///        Dim formulaModel As GridFormulaCellModel = Me.gridControl1.CellModel("FormulaCell") 
        ///        formulaModel.CalculatingSuspended = False
        ///        formulaModel.RefreshRange(GridRangeInfo.Cells(1, 1, 10, 2))
        /// <para/>        
        /// </code>
        /// </example>
        public bool CalculatingSuspended
        {
            get { return Engine.CalculatingSuspended; }
            set { Engine.CalculatingSuspended = value; }
        }

        /// <summary>
        /// Recalculates any formula cells in the specified range.
        /// </summary>
        /// <remarks>The implementation is delegated to Engine.RecalculateRange.</remarks>
        /// <param name="range">GridRangeInfo object that specifies the cells to be recalculated.</param>
        public void RecalculateRange(GridRangeInfo range)
        {
            this.Engine.RecalculateRange(range, false);
        }

        /// <summary>
        /// Recalculates any formula cells in the specified range.
        /// </summary>
        /// <remarks>The implementation is delegated to Engine.RecalculateRange.</remarks>
        /// <param name="range">GridRangeInfo object that specifies the cells to be recalculated.</param>
        /// <param name="forceCalculations">Determines whether the calculation is performed 
        /// immediately or delayed until the next time the cell is drawn.</param>
        public void RecalculateRange(GridRangeInfo range, bool forceCalculations)
        {
            this.Engine.RecalculateRange(range, forceCalculations);
        }

        ////Used to tell GetFormattedText to return formula.
        private bool copyFormulasToClipboard = false;

        ////used to track sheet where a copy took place
        private string lastCopiedSheetName = string.Empty;

        /// <summary>
        /// Specifies if the clipboard buffer came from cut operation or not.
        /// </summary>
        private bool fromClipboardCut = false;

        ////Used to copy formulas / values according to GridFormulaCopyFlags.
        private void GridClipboardCopy(object sender, GridCutPasteEventArgs e)
        {
            if (Grid.Model.CommandStack.CurrentTransactionCommand != null && Grid.Model.CommandStack.CurrentTransactionCommand.Description.Equals(SR.GetString("GRID_IDM_CUTDATA")))
            {
                fromClipboardCut = true;
                Engine.fromClipboardCut = true;
            }
            else
            {
                fromClipboardCut = false;
                Engine.fromClipboardCut = false;
            }
            ////Need to handle the copy if formula text wanted.
            if (0 != (Engine.FormulaCopyFlags & GridFormulaCopyFlags.ClipBoardFormula)
                && !this.copyFormulasToClipboard)
            {
                if (this.Grid.Model.SelectedRanges.ActiveRange.Width == 1 &&
                    this.Grid.Model.SelectedRanges.ActiveRange.Height == 1 &&
                    this.Grid.Model.ActiveGridView.CurrentCell.HasCurrentCell &&
                    this.Grid.Model.ActiveGridView.CurrentCell.IsEditing && this.Grid.Model.ActiveGridView.CurrentCell.Renderer.Control is TextBoxBase
                    )
                {
                    //just copy the selected text if any...
                    if (((TextBoxBase)this.Grid.Model.ActiveGridView.CurrentCell.Renderer.Control).SelectedText.Length > 0)
                    {
                        Clipboard.SetText(((TextBoxBase)this.Grid.Model.ActiveGridView.CurrentCell.Renderer.Control).SelectedText);
                    }
                    else
                    {
                        Clipboard.Clear();
                    }
                    e.Handled = true;
                    e.Result = true;
                }
                else
                {
                    this.copyFormulasToClipboard = true;
                    this.Grid.Model.CutPaste.Copy();
                    this.copyFormulasToClipboard = false;
                    e.Handled = true;
                    e.Result = true;
                }
            }

            if (Engine.IsSheeted)
            {
                GridSheetFamilyItem family = GridFormulaEngine.GetSheetFamilyItem(this.gridModel);
                string token = family.GridModelToToken[sender as GridModel] as string;
                lastCopiedSheetName = family.TokenToSheetName[token] as string;
            }
        }

        ////used to handle reference updating during a D&D
        private void GridDragDrop(object sender, DragEventArgs e)
        {
            if (0 != (Engine.FormulaCopyFlags & GridFormulaCopyFlags.UpdateReferencesDuringDnD))
            {
                GridControlBase g = sender as GridControlBase;
                if (g != null && g.Model.DragDropData.dndSource)
                {
                    GridModel grid = g.Model;
                    int targetRow, targetCol;
                    System.Drawing.Point pt = grid.ActiveGridView.PointToClient(Control.MousePosition);
                    if (grid.ActiveGridView.PointToRowCol(pt, out targetRow, out targetCol))
                    {
                        if (grid.SelectedRanges.ActiveRange.IsCols)
                        {
                            targetRow = grid.Model.Rows.HeaderCount + 1;
                            targetCol = grid.SelectedRanges.ActiveRange.Left;
                        }
                        else if (grid.SelectedRanges.ActiveRange.IsRows)
                        {
                            targetRow = grid.SelectedRanges.ActiveRange.Top;
                            targetCol = grid.Model.Cols.HeaderCount + 1;
                        }
                        else
                        {
                            targetRow = targetRow - grid.DragDropData.dndRowOffset;
                            targetCol = targetCol - grid.DragDropData.dndColOffset;
                        }

                        foreach (GridRangeInfo range in grid.DragDropData.dndSelList)
                        {
                            int row, col;
                            if (range.GetFirstCell(out row, out col))
                            {
                                int rowOffSet = targetRow - row;
                                int colOffSet = targetCol - col;
                                AdjustFormulasForDnD(grid, row, col, rowOffSet, colOffSet);
                                while (range.GetNextCell(ref row, ref col))
                                {
                                    AdjustFormulasForDnD(grid, row, col, rowOffSet, colOffSet);
                                }
                            }
                        }

                        grid.Refresh();
                    }
                }
            }
        }

        private const string validLeadingChars = "=(*&+-/^ ,";
        private const string validTrailingChars = ")*&+-/^ ,";

        ////adjusts reference during a D&D
        private void AdjustFormulasForDnD(GridModel grid2, int row, int col, int rowOffSet, int colOffSet)
        {
            GridSheetFamilyItem family = GridFormulaEngine.GetSheetFamilyItem(grid2);

            string gridToken = family.isSheeted ? family.GridModelToToken[grid2].ToString() : string.Empty;
            string source = string.Format("{0}{1}", GridRangeInfo.GetAlphaLabel(col), row, gridToken);
            string tokenSource = gridToken.Length > 0 ? gridToken + source : source;

            if (this.engine.DependentCells.ContainsKey(tokenSource))
            {
                string target = string.Format("{0}{1}", GridRangeInfo.GetAlphaLabel(col + colOffSet), row + rowOffSet);
                Hashtable ht = this.engine.DependentCells[tokenSource] as Hashtable;
                this.engine.DependentCells.Remove(tokenSource);
                Hashtable saveCells = new Hashtable();
                foreach (string s in ht.Keys)
                {
                    if (this.engine.DependentFormulaCells.ContainsKey(s))
                    {
                        Hashtable ht1 = (Hashtable)this.engine.DependentFormulaCells[s];
                        if (ht1 != null && ht1.ContainsKey(tokenSource))
                        {
                            ht1.Remove(tokenSource);
                        }
                    }

                    string sheet = engine.SheetToken(s);
                    GridModel grd1 = (sheet.Length == 0)
                                      ? grid2 : family.TokenToGridModel[sheet] as GridModel;
                    int r = this.engine.RowIndex(s);
                    int c = this.engine.ColIndex(s);
                    string formulaString = grd1[r, c].Text;
                    if (formulaString.Length > 0 && formulaString[0] == this.formulaChar)
                    {
                        string upperString = formulaString.ToUpper();
                        int i = -1;
                        int offset = 0;
                        if (gridToken.Length > 0)
                        {
                            ////first replace all full references
                            string sheetName = family.TokenToSheetName[gridToken].ToString();
                            string sheetSource = sheetName.ToUpper() + GridFormulaEngine.sheetToken + source;
                            string sheetTarget = sheetName.ToUpper() + GridFormulaEngine.sheetToken + target;
                            i = upperString.IndexOf(sheetSource);
                            while (i > -1)
                            {
                                bool swap = true;
                                if (i > 0)
                                {
                                    swap = validLeadingChars.IndexOf(upperString[i - 1]) > -1;
                                }

                                if (swap && i + sheetSource.Length < upperString.Length)
                                {
                                    swap = validTrailingChars.IndexOf(upperString[i + sheetSource.Length]) > -1;
                                }

                                if (swap)
                                {
                                    formulaString = formulaString.Substring(0, i + sheetName.Length + 1 + offset) + target
                                        + ((i + sheetSource.Length < formulaString.Length) ?
                                        formulaString.Substring(i + sheetSource.Length + offset) : string.Empty);
                                    offset += target.Length - source.Length;
                                }

                                i = upperString.IndexOf(sheetSource, i + 1);
                            }
                        }

                        i = upperString.IndexOf(source);
                        offset = 0;
                        while (gridToken == sheet && i > -1)
                        {
                            bool swap = true;
                            if (i > 0)
                            {
                                swap = validLeadingChars.IndexOf(upperString[i - 1]) > -1;
                            }

                            if (swap && i + source.Length < upperString.Length)
                            {
                                swap = validTrailingChars.IndexOf(upperString[i + source.Length]) > -1;
                            }

                            if (swap)
                            {
                                formulaString = formulaString.Substring(0, i + offset) + target
                                    + ((i + source.Length < formulaString.Length) ?
                                    formulaString.Substring(i + source.Length + offset) : string.Empty);
                                offset += target.Length - source.Length;
                            }

                            i = upperString.IndexOf(source, i + 1);
                        }

                        grd1[r, c].Text = formulaString;
                        grd1[r, c].FormulaTag = null;
                        ////savecell for refreshing
                        if (!saveCells.ContainsKey(GridRangeInfo.Cell(r, c)))
                        {
                            saveCells.Add(GridRangeInfo.Cell(r, c), grd1);
                        }
                    }
                }

                if (this.engine.DependentFormulaCells.ContainsKey(tokenSource))
                {
                    this.engine.DependentFormulaCells.Remove(tokenSource);
                }

                ////do the refresh
                foreach (GridRangeInfo range in saveCells.Keys)
                {
                    GridModel g = saveCells[range] as GridModel;
                    this.Engine.RecalculateRange(range, g, true, true, false);
                }

                saveCells.Clear();
            }
        }

        ////Used to update references.
        private void GridColsInserted(object sender, GridRangeInsertedEventArgs e)
        {
            this.Engine.AdjustReferencesForRangeChange(sender as GridModel, true, true, e.InsertAt, e.InsertAt + e.Count - 1, false);
        }

        /////Used to update references.
        private void GridColsRemoved(object sender, GridRangeRemovedEventArgs e)
        {
            this.Engine.AdjustReferencesForRangeChange(sender as GridModel, true, false, e.From, e.To, true);
        }

        ////Used to update references.
        private void GridRowsInserted(object sender, GridRangeInsertedEventArgs e)
        {
            this.Engine.AdjustReferencesForRangeChange(sender as GridModel, false, true, e.InsertAt, e.InsertAt + e.Count - 1, false);
        }

        ////Used to update references.
        private void GridRowsRemoved(object sender, GridRangeRemovedEventArgs e)
        {
            this.Engine.AdjustReferencesForRangeChange(sender as GridModel, false, false, e.From, e.To, true);
        }

        ////Used to end editing on CurrentCell and update pasted cells after paste.
        private void GridClipboardPasted(object sender, GridCutPasteEventArgs e)
        {
            if (Grid != null && Grid.ActiveGridView != null)
            {
                GridCurrentCell gcc = Grid.ActiveGridView.CurrentCell;
                if (gcc.HasCurrentCell && gcc.IsEditing && gcc.Renderer is GridFormulaCellRenderer)
                {
                    gcc.Renderer.ResetControlText();
                    string text = gcc.Renderer.ControlText;
                    Grid.ActiveGridView.RefreshRange(gcc.RangeInfo, true);
                }

                if (gcc.HasCurrentCell)
                {
                    //raise this event so any listeners (Like GridAwareTextBox knows the cell has changed because of the paste)
                    Grid.ActiveGridView.RaiseCurrentCellChanged();

                    if (fromClipboardCut)
                    {
                        //recalculates the pasted range
                        DataObject data = (DataObject)Clipboard.GetDataObject();
                        if (data.GetDataPresent(DataFormats.Text))
                        {
                            string s = (string)data.GetData(DataFormats.Text);
                            string[] rows = s.Split(new char[] { '\n' });
                            int numRows = rows.GetLength(0);
                            if (numRows > 0 && rows[numRows - 1].Length == 0)
                                numRows--; //remove extra empty row if present
                            int numCols = 0;
                            if (numRows > 0)
                            {
                                string[] cols = rows[0].Split(new char[] { '\t' });
                                numCols = cols.GetLength(0);
                            }
                            this.Engine.RecalculateRange(GridRangeInfo.Cells(gcc.RangeInfo.Top, gcc.RangeInfo.Left, gcc.RangeInfo.Top + numRows - 1, gcc.RangeInfo.Left + numCols - 1), true, false);
                        }
                        Clipboard.Clear();
                    }
                }

                ////TODO: **** use code below to only update new cells...
                ////Grid.ActiveGridView.Refresh();
            }

            //Refresh invisible range
            IDataObject dataObject = e.DataObject;
            if(dataObject != null)
                dataObject.GetData(typeof(string));
            
            this.isPaste = false;
            Engine.inPaste = false;
        }

        ////Used to set paste flag
        private bool isPaste = false;

        private void GridClipboardPaste(object sender, GridCutPasteEventArgs e)
        {
            if (Grid.CurrentCellInfo != null)
            {
                GridCurrentCell gcc = Grid.CurrentCellInfo.CellView.CurrentCell;
                if (gcc.HasCurrentCell && e.RangeList != null && e.RangeList.ActiveRange.Width == 1 && e.RangeList.ActiveRange.Height == 1 && !gcc.IsEditing)
                {
                    //if a single cell and not  editing, just let the grid copy the style and avoid copying the current cell text
                    e.IgnoreCurrentCell = true;
                }
            }
            this.isPaste = true;
            Engine.inPaste = true;
        }

        ////Used to redraw cells changed in SaveCellInfo.
        private void GridCellsChanged(object sender, GridCellsChangedEventArgs e)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(e, this);
            }
#else
            ;
#endif
           
            if (e.Range.IsCells)
            {
                if (this.gridModel != null && this.gridModel.ActiveGridView!=null && !this.gridModel.ActiveGridView.IsVisibleCell(e.Range.Top, e.Range.Left)
                    && this.Engine.DependentCells.Count > 0
                    && (this.Engine.FormulaCopyFlags & GridFormulaCopyFlags.InsDelRangeReferencesUpdated) != GridFormulaCopyFlags.None)
                {
                    GridStyleInfo style = this.gridModel[e.Range.Top, e.Range.Right];
                    this.gridModel.ActiveGridView.ScrollCellInView(e.Range.Top, e.Range.Right);
                    this.GetFormattedOrActiveTextAt(e.Range.Top, e.Range.Right, style);
                }

                int row, col;
                if (e.Range.GetFirstCell(out row, out col))
                {
                    RefreshCells(row, col);
                    while (e.Range.GetNextCell(ref row, ref col, false))
                    {
                        RefreshCells(row, col);
                    }
                }
            }
        }

        /// <summary>
        /// Triggers the parsing and computing of formulas.
        /// </summary>
        /// <remarks>
        /// Overridden to trigger the parsing and computing of formulas. The 
        /// computed value is returned for non-editing cells. The raw formula
        /// text is returned when the cell is actively being edited.
        /// </remarks>
        /// <param name="style">The <see cref="GridStyleInfo"/> object that holds cell information.</param>
        /// <param name="value">The value to format.</param>
        /// <param name="textInfo">textInfo is a hint who is calling, default is GridCellBaseTextInfo.DisplayText.</param>
        /// <returns>Either the computed value or the raw formula text.</returns>
        public override string GetFormattedText(GridStyleInfo style, object value, int textInfo)
        {
            string s = style.Text;
            Engine.GetFormulaText(ref s);

            if (this.copyFormulasToClipboard)
            {
                return s;
            }

            bool usesFormulaChar = this.formulaChar != (char)0 && s.Length > 0 && s[0] == this.formulaChar;
            if (this.formulaChar == (char)0 || usesFormulaChar)
            {
                if (usesFormulaChar)
                {
                    s = s.Substring(1); ////strip it off
                }

                if (GridUtil.IsEmpty(s.Trim()))
                {
                    return string.Empty;
                }

                GridStyleInfoIdentity id = style.Identity as GridStyleInfoIdentity;
#if DEBUG
                if (Switches.FormulaCell.TraceVerbose)
                {
                    TraceUtil.TraceCurrentMethodInfo(style.Text, style.FormulaTag, id.RowIndex, id.ColIndex, value, Grid.CurrentCellInfo, this);
                }
#else

                ;
#endif
                bool saveTag = false;
                if (style.FormulaTag == null)
                {
                    Engine.cell = GridRangeInfo.GetAlphaLabel(id.ColIndex) + id.RowIndex.ToString();
                    style.FormulaTag = new GridFormulaTag(Engine.Parse(s), null, id.RowIndex, id.ColIndex);
#if DEBUG
                    if (Switches.FormulaCell.TraceVerbose)
                    {
                        TraceUtil.TraceCurrentMethodInfo("parsing", s, style.FormulaTag, id.RowIndex, id.ColIndex, this);
                    }
#else
                    ;
#endif
                    saveTag = true;
                }

                GridFormulaTag tag = style.FormulaTag;
                if (tag != null)
                {
                    ////Check to see if parameter references need to be updated.
                    bool needAdjustment = NeedToAdjustReferences(id, tag) & !(fromClipboardCut | Engine.fromClipboardCut);
                    if (needAdjustment && (this.isPaste || Engine.inPaste) &&
                        0 != (Engine.FormulaCopyFlags & GridFormulaCopyFlags.ClipBoardReferencesAdjusted))
                    {
                        allowAdjustReferences = true;
                        AdjustReferences(id, tag, style);
                        allowAdjustReferences = false;
                        tag.Formula = Engine.Parse(style.Text);
                        saveTag = true;
                        ////this.isPaste = false;
                    }
                    else if (needAdjustment)
                    {
                        tag.ParsedRow = id.RowIndex;
                        tag.ParsedCol = id.ColIndex;
                        saveTag = true;
                        Engine.SetDirty(tag); //// tag.Text = string.Empty;
                    }
                    ////if(GridUtil.IsEmpty(tag.Text))
                    if (Engine.IsDirty(tag))
                    {
                        Engine.cell = GridRangeInfo.GetAlphaLabel(id.ColIndex) + id.RowIndex.ToString();

                        try
                        {
                            tag.Text = Engine.ComputedValue(tag.Formula);
                            if (Engine.unParsedKey.ToUpper() == style.FormulaTag.Formula)
                            {
                                tag.Text = Engine.parsedKeyValue;
                            }
                            saveTag = true;
                        }
                        catch (Exception ex)
                        {
                            ////                            TraceUtil.TraceExceptionCatched(ex);
                            ////                            if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            ////                                throw ex;
                            return ex.Message;
                        }
                    }

                    if (saveTag)
                    {
                        if (Engine.ReturnNAN)
                        {
                            if (tag.Text == "NaN")
                            {
                                tag.Text = "#Value!";
                            }
                            else if (tag.Text == "Infinity")
                            {
                                tag.Text = "#Div/0!";
                            }
                        }

                        bool b = this.Grid.IgnoreReadOnly;
                        this.Grid.IgnoreReadOnly = true;
                        this.Grid[id.RowIndex, id.ColIndex].FormulaTag = tag;
                        this.Grid.IgnoreReadOnly = b;
                    }

                    ////FIXED() func will return text and it should not be converted to value type.
                    char UNIQUESTRINGMARKER = (char)127;
                    if (tag.Text != null && tag.Text.IndexOf(UNIQUESTRINGMARKER) == 0)
                    {
                        return tag.Text.Remove(0, 1);
                    }

                    CultureInfo ci = style.GetCulture(true);

                    double d;
                    DateTime date;
                    if (tag.Text != null && double.TryParse(tag.Text, NumberStyles.Number | NumberStyles.AllowExponent, ci.NumberFormat, out d))
                    {
                        if (!GridUtil.IsEmpty(style.Format))
                        {
                            return GridCellValueConvert.FormatValue(d, typeof(double), style.Format, ci, ci.NumberFormat);
                        }

                        // The change in effect says that if there is no style.Format property and if the tag.Text is parseable as a double,
                        // then just return the value of tag.Text instead of the raw double value returned by double.ToString() which is 
                        // what the commented base class call effectively does.
                        return tag.Text;
                        //return base.GetFormattedText(style, d, textInfo);
                    }
                    else if (tag.Text != null && DateTime.TryParse(tag.Text, ci.NumberFormat, DateTimeStyles.None, out date))
                    {
                        if (!GridUtil.IsEmpty(style.Format))
                        {
                            return GridCellValueConvert.FormatValue(date, typeof(DateTime), style.Format, ci, ci.NumberFormat);
                        }

                        // The change in effect says that if there is no style.Format property and if the tag.Text is parseable as a double,
                        // then just return the value of tag.Text instead of the raw datetime value returned by DateTime.ToString() which is 
                        // what the commented base class call effectively does.
                        return tag.Text;
                    }
                    else
                    {
                        if (tag.Text == null)
                        {
                            return string.Empty;
                        }
                        if (Engine.unParsedKey.ToUpper() == style.FormulaTag.Formula)
                        {
                            tag.Text = Engine.parsedKeyValue;
                        }

                        return tag.Text;
                    }
                }
                else
                {
                    return string.Empty;
                }
            }
            else
            {
                ////do not have a formula in the cell

                ////check for possible format and try to apply it
                if (!GridUtil.IsEmpty(style.Format))
                {
                    CultureInfo ci = style.GetCulture(true);

                    decimal dec;
                    double d;
                    DateTime date;

                    if (style.CellValueType == typeof(DateTime) && DateTime.TryParse(style.Text, ci.NumberFormat, DateTimeStyles.None, out date))
                    {
                        return GridCellValueConvert.FormatValue(date, typeof(DateTime), style.Format, ci, ci.NumberFormat);
                    }
                    if (decimal.TryParse(style.Text, NumberStyles.Number | NumberStyles.AllowExponent, ci.NumberFormat, out dec))
                    {
                        return GridCellValueConvert.FormatValue(dec, typeof(decimal), style.Format, ci, ci.NumberFormat);
                    }
                    else if (double.TryParse(style.Text, NumberStyles.Number | NumberStyles.AllowExponent, ci.NumberFormat, out d))
                    {
                        return GridCellValueConvert.FormatValue(d, typeof(double), style.Format, ci, ci.NumberFormat);
                    }
                }
                ////    TraceUtil.TraceCurrentMethodInfoIf(Switches.FormulaCell.TraceVerbose, "not a formula cell", style.Text, value, base.GetFormattedText(style, value, textInfo));
                return base.GetFormattedText(style, value, textInfo);
            }
        }

        internal bool allowAdjustReferences = false;
        private void AdjustReferences(GridStyleInfoIdentity id, GridFormulaTag tag, GridStyleInfo style)
        {
            if (allowAdjustReferences)
            {
                string sheetName = string.Empty;
                if (Engine.IsSheeted)
                {
                    GridSheetFamilyItem family = GridFormulaEngine.GetSheetFamilyItem(this.gridModel);
                    string token = family.GridModelToToken[this.gridModel] as string;
                    sheetName = family.TokenToSheetName[token] as string;
                }

                style.Text = this.Engine.AdjustReferences(style.Text, id.RowIndex - tag.ParsedRow, id.ColIndex - tag.ParsedCol, lastCopiedSheetName, sheetName);
            }

            tag.ParsedRow = id.RowIndex;
            tag.ParsedCol = id.ColIndex;

            this.Grid.SuspendChangeEvents();
            bool b = this.Grid.IgnoreReadOnly;
            this.Grid.IgnoreReadOnly = true;
            this.Grid[id.RowIndex, id.ColIndex].Text = style.Text;
            Engine.SetDirty(tag); ////tag.Text = string.Empty;
            this.Grid[id.RowIndex, id.ColIndex].FormulaTag = tag;
            this.Grid.IgnoreReadOnly = b;
            this.Grid.ResumeChangeEvents();
        }

        private bool NeedToAdjustReferences(GridStyleInfoIdentity id, GridFormulaTag tag)
        {
            return (id.RowIndex != tag.ParsedRow && tag.ParsedRow != -1)
                || (id.ColIndex != tag.ParsedCol && tag.ParsedCol != -1);
        }

        /// <summary>
        /// Overridden to return a GridFormulaCellRenderer class.
        /// </summary>
        /// <param name="control">The grid control.</param>
        /// <returns>returns a GridFormulaCellRenderer object.</returns>
        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new GridFormulaCellRenderer(control, this);
        }
    }
    #endregion

    #region CellRenderer Class
    /// <summary>
    /// Implements a GridCellRender that supports formulas.
    /// </summary>
    /// <remarks>
    /// See <see cref="GridFormulaCellModel"/> for a discussion about formula engine
    /// and customization.
    /// <para/>
    /// The following table lists some characteristics about the FormulaCell cell type:
    /// <para/>
    /// <list type="table">
    ///     <listheader>
    ///         <term>Item</term>
    ///         <description>Description</description>
    ///     </listheader>
    ///     <item>
    ///         <term>CellType</term>
    ///         <description>FormulaCell</description>
    ///     </item>
    ///     <item>
    ///         <term>Renderer</term>
    ///         <description><see cref="GridFormulaCellRenderer"/></description>
    ///     </item>
    ///     <item>
    ///         <term>Model</term>
    ///         <description><see cref="GridFormulaCellModel"/></description>
    ///     </item>
    ///     <item>
    ///         <term>XP Themes Support</term>
    ///         <description>NA</description>
    ///     </item>
    ///     <item>
    ///         <term>Interactive</term>
    ///         <description>Edit with Text and Formula Input</description>
    ///     </item>
    ///     <item>
    ///         <term>Control</term>
    ///         <description><see cref="GridTextBoxControl"/></description>
    ///     </item>
    ///     <item>
    ///         <term>Floating Support</term>
    ///         <description>Both</description>
    ///     </item>
    ///     <item>
    ///         <term>Base Type</term>
    ///         <description><see cref="GridTextBoxCellRenderer"/></description>
    ///     </item>
    /// </list>
    /// <para/>
    /// <para/>
    /// The cell's behavior and appearance can be customized with the following properties of the <see cref="GridStyleInfo"/> class:
    /// <para/>
    /// <list type="table">
    ///     <listheader>
    ///         <term>PropertyName</term>
    ///         <description>Description</description>
    ///     </listheader>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.AllowEnter"/> (<see cref="System.Boolean"/>)</term>
    ///         <description>Gets / sets if pressing the &lt;Enter&gt;-Key should insert a new line into the edited text. (Default: False)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.AutoSize"/> (<see cref="System.Boolean"/>)</term>
    ///         <description>Gets / sets if the cell height should automatically increase when the edited text does not fit into the cell and <see cref="GridStyleInfo.WrapText"/> is True. If <see cref="GridStyleInfo.WrapText"/> is False, <see cref="GridStyleInfo.AutoSize"/> will affect the column width. (Default: False)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.BaseStyle"/> (<see cref="System.String"/>)</term>
    ///         <description>The base style for this style instance with default values for properties that are not initialized for this style object. (Default: String.Empty)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.Borders"/> (<see cref="GridBordersInfo"/>)</term>
    ///         <description>Top, left, bottom, and right border settings. (Default: GridBordersInfo.Default)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.CellAppearance"/> (<see cref="GridCellAppearance"/>)</term>
    ///         <description>Specifies if cell edges shall be drawn raised, sunken, or flat (default). (Default: GridCellAppearance.Flat)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.CellTipText"/> (<see cref="System.String"/>)</term>
    ///         <description>ToolTip text to be displayed when user hovers mouse over cell. (Default: String.Empty)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.CellType"/> (<see cref="System.String"/>)</term>
    ///         <description>FormulaCell (Default: Text Box)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.CellValue"/> (<see cref="System.Object"/>)</term>
    ///         <description>This property holds the cell value. Although the cell value is typically a string, it can also be any other primitive type such as int, byte, enum, or any custom type that is derived from <see cref="System.Object"/>. (Default: String.Empty)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.CellValueType"/> (<see cref="System.Type"/>)</term>
    ///         <description>Specifies the preferred <see cref="System.Type"/> for cell values. When you assign a value to the <see cref="GridStyleInfo"/> object, the value will be converted to this type. If the value cannot be converted, <see cref="GridStyleInfo.Error"/> will contain error information. (Default: NULL)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.CultureInfo"/> (<see cref="System.Globalization.CultureInfo"/>)</term>
    ///         <description>The culture information holds rules for parsing and formatting the cell's value. (Default: null)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.Enabled"/> (<see cref="System.Boolean"/>)</term>
    ///         <description>Specifies if the cell can be activated as current cell or if the cell should be skipped when moving the current cell. (Default: True)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.Error"/> (<see cref="System.String"/>)</term>
    ///         <description>Holds error information if a value could not be converted to the <see cref="System.Type"/> specified with <see cref="GridStyleInfo.CellValueType"/>. (Default: String.Empty)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.FloatCell"/> (<see cref="System.Boolean"/>)</term>
    ///         <description>Gets / sets if text can float into the boundaries of a neighboring cell. (Default: True)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.FloodCell"/> (<see cref="System.Boolean"/>)</term>
    ///         <description>Gets / sets if this cell can be flooded by a previous cell. (Default: True)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.Font"/> (<see cref="GridFontInfo"/>)</term>
    ///         <description>The font for drawing text. (Default: GridFontInfo.Default)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.Format"/> (<see cref="System.String"/>)</term>
    ///         <description>Gets / sets the format mask for formatting the cell value. You can specify numeric format strings,
    /// date format strings or enumeration format strings as discussed in the section "Format Specifiers and Format Providers" of the .NET Framework Developers Guide (see ms-help://MS.VSCC/MS.MSDNVS/cpguide/html/cpconformatspecifiersformatproviders.htm) (Default: String.Empty)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.FormulaTag"/> (<see cref="GridFormulaTag"/>)</term>
    ///         <description>A formula tag that is associated with a cell. (Default: NULL)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.HorizontalAlignment"/> (<see cref="GridHorizontalAlignment"/>)</term>
    ///         <description>Specifies horizontal alignment of text in the cell. (Default: GridHorizontalAlignment.Left)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.HotkeyPrefix"/> (<see cref="System.Drawing.Text.HotkeyPrefix"/>)</term>
    ///         <description>Specifies how hot-key prefixes should be displayed. Hot-keys are indicated in text with an '&amp;' (ampersand). When you enable a hot-key prefix, the specific characters can be displayed underlined or regular. The '&amp;' will not be displayed. (Default: HotkeyPrefix.Show)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.ImageIndex"/> (<see cref="System.Int32"/>)</term>
    ///         <description>Specifies an index for an image in the <see cref="GridStyleInfo.ImageList"/> of a <see cref="GridStyleInfo"/>
    /// instance. (Default: -1)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.ImageList"/> (<see cref="System.Windows.Forms.ImageList"/>)</term>
    ///         <description>The <see cref="GridStyleInfo.ImageList"/> that holds a collection of images. Cells can choose images with the <see cref="GridStyleInfo.ImageIndex"/> property in a <see cref="GridStyleInfo"/>
    /// instance. (Default: NULL)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.Interior"/> (<see cref="Syncfusion.Drawing.BrushInfo"/>)</term>
    ///         <description> Lets you specify a solid backcolor, gradient, or pattern style with both back and forecolor for a cell's
    ///  background. (Default: SystemColors.Window)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.MaxLength"/> (<see cref="System.Int32"/>)</term>
    ///         <description>Limits the number of characters the user can type into the cell. Note: When selecting a text from a choice list or when pasting text, the text can be longer. Additional validation is necessary on your side. (Default: 0)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.MergeCell"/> (<see cref="GridMergeCellDirection"/>)</term>
    ///         <description>Specifies merge behavior for an individual cell when merging cells feature has been enabled in a <see cref="GridModel"/> with  <see cref="GridModelOptions.MergeCellsMode"/>. (Default: GridMergeCellDirection.None)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.ReadOnly"/> (<see cref="System.Boolean"/>)</term>
    ///         <description>Specifies if cell contents can be modified by the user. You can programmatically change Read-only cells by setting <see cref="GridModel.DiscardReadOnly"/> to True. (Default: False)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.Text"/> (<see cref="System.String"/>)</term>
    ///         <description>Gets / sets the value as a string. If a <see cref="GridStyleInfo.CellValueType"/>
    /// is specified, the text will be parsed and converted to the type specified with
    /// <see cref="GridStyleInfo.CellValueType"/> using any <see cref="GridStyleInfo.CultureInfo"/>
    /// information.
    ///  (Default: String.Empty)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.TextColor"/> (<see cref="System.Drawing.Color"/>)</term>
    ///         <description>Lets you specify the color for drawing the cell text. (Default: SystemColors.WindowText)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.TextMargins"/> (<see cref="GridMarginsInfo"/>)</term>
    ///         <description>Holds text margins in pixels. When drawing a cell, this specifies the empty area between the
    /// text rectangle and the client rectangle of the cell without borders and cell buttons. (Default: GridMarginsInfo.Default)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.Trimming"/> (<see cref="System.Drawing.StringTrimming"/>)</term>
    ///         <description>Indicates how text is trimmed when it exceeds the edges of the cell text rectangle. (Default: StringTrimming.Character)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.ValidateValue"/> (<see cref="GridCellValidateValueInfo"/>)</term>
    ///         <description>Holds validation rules for cell values that are being checked before any user changes are committed to the grid cell's style object. (Default: NULL)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.VerticalAlignment"/> (<see cref="GridVerticalAlignment"/>)</term>
    ///         <description>Specifies vertical alignment of text in the cell. (Default: GridVerticalAlignment.Top)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.VerticalScrollbar"/> (<see cref="System.Boolean"/>)</term>
    ///         <description>Specifies if text box should show a vertical scrollbar when text is being edited and does not fit in cell. WrapText must be initialized to True. (Default: False)</description>
    ///     </item>
    ///     <item>
    ///         <term><see cref="GridStyleInfo.WrapText"/> (<see cref="System.Boolean"/>)</term>
    ///         <description>Specifies if text should be wrapped when it does not fit into a single line. (Default: True)</description>
    ///     </item>
    /// </list>
    /// <para/>
    /// </remarks>
    public class GridFormulaCellRenderer : GridTextBoxCellRenderer
    {
        [ThreadStaticAttribute]
        private static bool forceEditWhenActivated = true;

        /// <summary>
        /// Gets or sets a value indicating whether GridFormulaCellRenderer should honor the <see cref="GridModelOptions.ActivateCurrentCellBehavior"/>
        /// and only start editing mode as specified in that setting or if it should ignore that setting and always
        /// switch into edit mode as soon as it is activated.
        /// </summary>
        public static bool ForceEditWhenActivated
        {
            get
            {
                return forceEditWhenActivated;
            }

            set
            {
                forceEditWhenActivated = value;
            }
        }

        /// <summary>
        /// Initializes a new GridFormulaCellRenderer object for the given GridControlBase
        /// and GridCellModelBase.
        /// </summary>
        /// <param name="grid">The <see cref="GridControlBase"/> that displays this cell renderer.</param>
        /// <param name="cellModel">The <see cref="GridCellModelBase"/> that holds data for this cell renderer that should
        /// be shared among views.</param>
        /// <remarks>References to GridControlBase, 
        /// and GridCellModelBase will be saved.</remarks>
        public GridFormulaCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(grid, cellModel);
            }
#else
            ;
#endif
        }

        /// <override/>
        protected override void OnActivated()
        {
            base.OnActivated();
            if (!CurrentCell.IsEditing && ForceEditWhenActivated)
            {
                // Set the cell editing so that formula is displayed but do not set focus to text box.
                CurrentCell.BeginEdit(false);
            }
        }

        /// <override/>
        protected override void InitializeControlText(object controlValue)
        {
            string text = controlValue != null ? controlValue.ToString() : string.Empty;
            GridCurrentCellInitializeControlTextEventArgs e = new GridCurrentCellInitializeControlTextEventArgs(this.RowIndex, this.ColIndex, this.StyleInfo, controlValue, text);
            if (Grid.RaiseCurrentCellInitializeControlText(e))
            {
                this.ControlText = e.ControlText;
            }
        }

        /// <override/>
        /// <summary>Called when user initiates ClipboardCopy operation. Used for a single cell only. Returns True if successful; False if failed.</summary>
        /// <returns>returns False.</returns>
        public override bool Copy()
        {
            ////Only used when you are copying a single formula cell.
            ////Not used when copying ranges.
            ////If this copied value is pasted to a FormulaCell target,
            ////you get the formula. If it gets pasted to a non-formula cell,
            ////you get the text.
            return false; ////let the grid copy the cell...
        }

        /// <override/>
        /// <summary>Called when user initiates ClipboardPaste operation. True if successful; False if failed.</summary>
        /// <returns>returns False.</returns>
        public override bool Paste()
        {
            ////Only used when you are pasting a single cell.
            ////Not used when pasting ranges.
            ////If the source was a formulacell, you get the formula.
            ////If the source was not a formulacell, you get text.
            return false; ////let the grid handle it
        }

        /// <summary>
        /// This method is called from GridCurrentCell.Validate after GridCurrentCell.Validating event has been
        /// fired. The default version checks if the active text fits any criteria as specified
        /// in the style object: It can be parsed into a cell value and meets GridCellValidateValueInfo criteria.
        /// </summary>
        /// <returns>
        /// True if the modified text is valid; False otherwise.
        /// </returns>
        /// <override/>
        protected override bool OnValidate()
        {
            GridFormulaEngine engine = ((GridFormulaCellModel)this.Model).Engine;

            string s = this.Control.Text;
            engine.GetFormulaText(ref s);

            GridSheetFamilyItem family = GridFormulaEngine.GetSheetFamilyItem(this.Grid.Model);
            string sheet = (family.GridModelToToken != null)
                        ? family.GridModelToToken[this.Grid.Model] as string
                        : string.Empty;
            int row = this.Grid.CurrentCell.RowIndex;
            int col = this.Grid.CurrentCell.ColIndex;
            string cell = sheet + GridRangeInfo.GetAlphaLabel(col) + row.ToString();

            if ((s.Length == 0) ||
                (engine.formulaChar != (char)0 && s.Length > 0 && s[0] != engine.formulaChar))
            {
                ////Remove any cell dependencies....
                Hashtable ht = (Hashtable)engine.DependentFormulaCells[cell];
                if (ht != null)
                {
                    foreach (string s1 in ht.Keys)
                    {
                        Hashtable ht1 = (Hashtable)engine.DependentCells[s1];
                        if (ht1 != null)
                        {
                            ht1.Remove(cell);
                            if (ht1.Count == 0)
                            {
                                engine.DependentCells.Remove(s1);
                            }
                        }
                    }

                    engine.DependentFormulaCells.Remove(cell);
                }

                if (this.Grid.Model[row, col].FormulaTag != null)
                {
                    ////also remove formulaTag
                    this.Grid.Model[row, col].FormulaTag = null;
                }

                return true;
            }
            else if (engine.formulaChar != (char)0 && s.Length > 0)
            {
                s = s.Substring(1);
            }

            bool save = engine.lockDependencies;
            try
            {
                ////Parse it to see if OK...
                engine.cell = cell;
                string parsedFormula;

                if (engine.DoCircularCheckInValidating)
                {
                    engine.lockDependencies = true;
                    parsedFormula = engine.Parse(s);
                    if (CircularCheckFailed(cell, parsedFormula))
                    {
                        this.Grid.CurrentCell.ErrorMessage = engine.FormulaErrorStrings[engine.circular_reference_] + cell;
                        engine.lockDependencies = save;
                        return false;
                    }

                    engine.lockDependencies = save;
                }

                parsedFormula = engine.Parse(s);
                ////remove old cell dependencies so new ones can be set up
                if (engine.DependentFormulaCells.ContainsKey(cell))
                {
                    Hashtable ht = (Hashtable)engine.DependentFormulaCells[engine.cell];
                    foreach (object key in ht.Keys)
                    {
                        Hashtable ht1 = (Hashtable)engine.DependentCells[(string)key];
                        ht1.Remove(engine.cell);
                        if (ht1.Count == 0)
                        {
                            engine.DependentCells.Remove(key);
                        }
                    }

                    engine.DependentFormulaCells.Remove(engine.cell);
                }

                string computeFormula = engine.ComputedValue(parsedFormula);

                ////Check for circular reference.
                if (engine.IsCircularReference(cell))
                {
                    Hashtable ht = (Hashtable)engine.DependentCells[cell];
                    //// && ht.ContainsKey(cell)) 
                    if (ht != null)
                    {
                        ht.Remove(cell);
                        engine.ClearFormulaDependentCells(engine.cell);
                        throw new ArgumentException(engine.FormulaErrorStrings[engine.circular_reference_] + cell);
                    }
                }
                ////Save the parsed formula to avoid parsing it a second time in GetFormattedText.
                if (engine.ReturnNAN)
                {
                    if (computeFormula == "NaN")
                    {
                        computeFormula = "#Value!";
                    }
                    else if (computeFormula == "Infinity")
                    {
                        computeFormula = "#Div/0!";
                    }
                }

                bool b = this.Grid.Model.IgnoreReadOnly;
                this.Grid.Model.IgnoreReadOnly = true;
                this.Grid.Model[row, col].FormulaTag = new GridFormulaTag(parsedFormula, computeFormula, row, col);
                this.Grid.Model.IgnoreReadOnly = b;
            }
            catch (ArgumentException ex)
            {
                this.Grid.CurrentCell.ErrorMessage = ex.Message;
                return false;
            }
            finally
            {
                if (save != engine.lockDependencies)
                {
                    engine.lockDependencies = save;
                }
            }

            return true;
        }

        internal bool CircularCheckFailed(string hostCell, string parsedFormula)
        {
            GridFormulaEngine engine = ((GridFormulaCellModel)this.Model).Engine;
            return engine.CircularCheckFailed(hostCell, parsedFormula);
        }
    }
    #endregion

    #region GridFormulaEngine Class

    /// <summary>
    /// Encapsulates the code required to parse and compute formulas. Hashtable
    /// properties maintain a Formula Library of functions as well as a list
    /// of dependent cells.
    /// <para/>
    /// You can add and remove library functions.
    /// </summary>
    public class GridFormulaEngine
    {
        ////the grid
        private GridModel grid;

        /// <summary>
        /// The character that indicates a formula.
        /// </summary>
        /// <remarks>
        /// If this character is not 0, the text of a cell
        /// must begin with this character if the cell contains
        /// a formula. Common usage would be to use '=' as the
        /// character to begin a formula in a cell.
        /// <para/> 
        /// This value is normally set through the cell model constructor.
        /// </remarks>
        internal char formulaChar;

        ////Holds the cell being calculated.. set in CellModel.GetFormattedText.
        internal string cell;

        // Holds the value to determine whether the range is to be executed in RecalculateRange
        private bool isCellsprocessed = false;
        
        /// <summary>
        /// Gets or sets the cell (in the column-row notation of A1 or E21) whose 
        /// formula is being parsed or computed.
        /// </summary>
        /// <remarks>
        /// This property should be set only if you are directly parsing and 
        /// computing formulas by calling the Parse or ComputedValue methods. In this 
        /// case, you should set the property before calling either method. FormulaContextCell
        /// is used to provide support for CurrentRowNotation, circular calculation checks,
        /// and reference updating.
        /// </remarks>
        public string FormulaContextCell
        {
            get { return cell; }
            set { cell = value; }
        }

        ////Used to determine if this GridFormulaEngine instance is a member of
        ////several sheets. If so, dependent cells are tracked through a static member
        ////so that they are known across instances.
        internal bool IsSheeted
        {
            get
            {
                GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                return (family == null)
                    ? false : family.isSheeted;
            }
        }

        private bool supportBlanksInSheetNames = true;

        /// <summary>
        /// Gets or sets a value indicating whether blanks are treated as significant in sheet names.
        /// </summary>
        /// <remarks>
        /// Earlier versions of GridFormulaEngine did not support the use of blanks
        /// in sheet names. (They were treated as insignificant.) The current version
        /// does treat blanks as significant. If you want to revert to the prior
        /// treatment of blanks in sheet names, then set this property to false.
        /// </remarks>
        public bool SupportBlanksInSheetNames
        {
            get { return supportBlanksInSheetNames; }
            set { supportBlanksInSheetNames = value; }
        }

        private bool fixedReferenceOnlyOnPaste = false;

        /// <summary>
        /// Gets or sets a value indicating whether fixed references should be adjusted on other than paste.
        /// </summary>
        public bool FixedReferenceOnlyOnPaste
        {
            get { return fixedReferenceOnlyOnPaste; }
            set { fixedReferenceOnlyOnPaste = value; }
        }

        [ThreadStaticAttribute]
        private static Hashtable sheetFamiliesList = null;

        [ThreadStaticAttribute]
        private static int sheetFamilyID = 0;

        [ThreadStaticAttribute]
        private static GridSheetFamilyItem defaultFamilyItem = null;

        /// <summary>
        /// Returns the GridSheetFamilyItem for the specified model. If there was no item registered for 
        /// the model, a new item is created and cached.
        /// </summary>
        /// <param name="model">The grid model.</param>
        /// <returns>The GridSheetFamilyItem for the specified model.</returns>
        public static GridSheetFamilyItem GetSheetFamilyItem(GridModel model)
        {
            if (sheetFamilyID == 0)
            {
                if (defaultFamilyItem == null)
                {
                    defaultFamilyItem = new GridSheetFamilyItem();
                }

                return defaultFamilyItem;
            }

            if (sheetFamiliesList == null)
            {
                sheetFamiliesList = new Hashtable();
            }

            int i = (int)modelToSheetID[model];

            if (sheetFamiliesList[i] == null)
            {
                sheetFamiliesList.Add(i, new GridSheetFamilyItem());
            }

            return sheetFamiliesList[i] as GridSheetFamilyItem;
        }

        private Hashtable dependentCells = null;

        /// <summary>
        /// Gets a mapping between a cell and a list of formula cells that depend on it.
        /// </summary>
        /// <remarks>
        /// The key is the given cell, and the value is a Hashtable of cells containing
        /// formulas that reference this cell.
        /// </remarks>
        /// <example> Here is code that will list formula cells affected by changing the given cell.
        /// <code lang="C#">
        ///    public void DisplayAllAffectedCells()
        ///    {
        ///        GridFormulaEngine engine = ((GridFormulaCellModel)this.gridControl1.CellModels["FormulaCell"]).Engine;
        /// <para/>    
        ///        foreach(object o in engine.DependentCells.Keys)
        ///        {
        ///            string s1 = o as string;
        ///            Console.Write(s1 + " affects ");
        ///            Hashtable ht = (Hashtable) engine.DependentCells[s1];
        ///            foreach(object o1 in ht.Keys)
        ///            {
        ///                string s2 = o1 as string;
        ///                Console.Write(s2 + " ");
        ///            }
        ///            Console.WriteLine(string.Empty);
        ///        }
        ///    }
        /// </code>
        /// <code lang="VB">
        ///        Public Sub DisplayAllAffectedCells()
        ///            Dim engine As GridFormulaEngine = CType(Me.gridControl1.CellModels("FormulaCell"), GridFormulaCellModel).Engine
        ///            Dim o As Object
        ///            For Each o In  engine.DependentCells.Keys
        ///                Dim s1 As String = CStr(o)
        ///                Console.Write((s1 + " affects "))
        ///                Dim ht As Hashtable = CType(engine.DependentCells(s1), Hashtable)
        ///                Dim o1 As Object
        ///                For Each o1 In  ht.Keys
        ///                    Dim s2 As String = CStr(o1)
        ///                    Console.Write((s2 + " "))
        ///                Next o1
        ///                Console.WriteLine(string.Empty)
        ///            Next o
        ///        End Sub 'DisplayAllAffectedCells
        /// </code>
        /// </example>
        public Hashtable DependentCells
        {
            get
            {
                if (IsSheeted)
                {
                    GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                    if (family.sheetDependentCells == null)
                    {
                        family.sheetDependentCells = new Hashtable();
                    }

                    return family.sheetDependentCells;
                }
                else
                {
                    if (dependentCells == null)
                    {
                        dependentCells = new Hashtable();
                    }

                    return dependentCells;
                }
            }
        }

        private Hashtable dependentFormulaCells = null;

        /// <summary>
        /// Gets a mapping between a formula cell and a list of cells upon which it depends.
        /// </summary>
        /// <remarks>
        /// The key is the given formula cell, and the value is a Hashtable of cells that this 
        /// formula cell references.
        /// </remarks>
        /// <example> Here is code that lists formula cells affected by changing a given cell:
        /// <code lang="C#">
        ///        public void DisplayAllFormulaDependencies()
        ///        {
        ///            GridFormulaEngine engine = ((GridFormulaCellModel)this.gridControl1.CellModels["FormulaCell"]).Engine;
        ///  <para/>           
        ///            foreach(object o in engine.DependentFormulaCells.Keys)
        ///            {
        ///                string s1 = o as string;
        ///                Console.Write(s1 + " depends upon ");
        ///                Hashtable ht = (Hashtable) engine.DependentFormulaCells[s1];
        ///                foreach(object o1 in ht.Keys)
        ///                {
        ///                    string s2 = o1 as string;
        ///                    Console.Write(s2 + " ");
        ///                }
        ///                Console.WriteLine(string.Empty);
        ///            }
        ///        }
        /// </code>
        /// <code lang="VB">
        ///        Public Sub DisplayAllFormulaDependencies()
        ///            Dim engine As GridFormulaEngine = CType(Me.gridControl1.CellModels("FormulaCell"), GridFormulaCellModel).Engine
        /// <para/>   
        ///            Dim o As Object
        ///            For Each o In  engine.DependentFormulaCells.Keys
        ///                Dim s1 As String = CStr(o)
        ///                Console.Write((s1 + " depends upon "))
        ///                Dim ht As Hashtable = CType(engine.DependentFormulaCells(s1), Hashtable)
        ///                Dim o1 As Object
        ///                For Each o1 In  ht.Keys
        ///                    Dim s2 As String = CStr(o1) 
        ///                    Console.Write((s2 + " "))
        ///                Next o1
        ///                Console.WriteLine(string.Empty)
        ///            Next o
        ///        End Sub 'DisplayAllFormulaDependencies
        /// </code>
        /// </example>
        public Hashtable DependentFormulaCells
        {
            get
            {
                if (IsSheeted)
                {
                    GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                    if (family.sheetDependentFormulaCells == null)
                    {
                        family.sheetDependentFormulaCells = new Hashtable();
                    }

                    return family.sheetDependentFormulaCells;
                }
                else
                {
                    if (dependentFormulaCells == null)
                    {
                        dependentFormulaCells = new Hashtable();
                    }

                    return dependentFormulaCells;
                }
            }
        }

        private GridFormulaCopyFlags formulaCopyFlags;

        [ThreadStaticAttribute]
        internal static GridFormulaCopyFlags sheetedFormulaCopyFlags = GridFormulaCopyFlags.ClipBoardText;

        /// <summary>
        /// Gets or sets behavior for copy, paste, and drag-and-drop operations.
        /// </summary>
        public GridFormulaCopyFlags FormulaCopyFlags
        {
            get
            {
                if (this.IsSheeted)
                {
                    return sheetedFormulaCopyFlags;
                }
                else
                {
                    return formulaCopyFlags;
                }
            }

            set
            {
                if (this.IsSheeted)
                {
                    sheetedFormulaCopyFlags = value;
                }
                else
                {
                    formulaCopyFlags = value;
                }
                ////OnOptionsChanged(EventArgs.Empty);
            }
        }

        /// <summary>
        /// Gets or sets the number of recursive checks done
        /// for circular references.
        /// </summary>
        /// <remarks>
        /// When you edit a cell, the engine parsing will attempt
        /// to flag circular references as an error. This value sets the
        /// number of dependent cells it will check before assuming there
        /// is no circular reference. Checking every reference can be 
        /// time consuming depending upon the formulas being used. This
        /// property lets you decide how many recursions the engine will 
        /// allow when looking for circular references. The default value
        /// is -1 meaning no checks are done as you type formula into the 
        /// cell. Setting this property to int.MaxValue will make the 
        /// engine check all dependent cells for a circular reference.
        /// </remarks>
        public int MaximumCircularChecks
        {
            get { return maxCircularChecks; }
            set { maxCircularChecks = value; }
        }

        private bool doCircularCheckInValidating = false;

        /// <summary>
        /// Gets or sets a value indicating whether circular references should be checked in 
        /// the CurrentCell.Validating event.
        /// </summary>
        public bool DoCircularCheckInValidating
        {
            get { return doCircularCheckInValidating; }
            set { doCircularCheckInValidating = value; }
        }

        ////Uses recursion to check for a circular dependence.
        private int maxCircularChecks = -1;
        private int circularCheckCount = 0;
        internal bool IsCircularReference(string cell)
        {
            if (maxCircularChecks == -1)
            {
                return false;
            }

            circularCheckCount = 0;
            Hashtable ht = (Hashtable)DependentCells[cell];
            return DoesHashtableOrChildContain(ht, cell);
        }

        private bool DoesHashtableOrChildContain(Hashtable ht, string cell)
        {
            if (ht == null)
            {
                return false;
            }

            if (ht.ContainsKey(cell))
            {
                return true;
            }

            if (circularCheckCount > maxCircularChecks)
            {
                return false;
            }

            circularCheckCount++;

            ////recursivecall
            foreach (string key in ht.Keys)
            {
                Hashtable ht1 = (Hashtable)DependentCells[key];
                if (ht1 != null && DoesHashtableOrChildContain(ht1, cell))
                {
                    return true;
                }
            }

            return false;
        }

        ////used internally in AdjustReferencesForRangeChange
        internal class AdjustInfo
        {
            internal AdjustInfo(GridModel grd1, int row, int col, string newFormula)
            {
                this.grd1 = grd1;
                this.row = row;
                this.col = col;
                this.newFormula = newFormula;
            }

            internal GridModel grd1;
            internal int row;
            internal int col;
            internal string newFormula;
        }

        ////Adjust references for inserting or removing rows or columns.
        internal bool AdjustReferencesForRangeChange(GridModel grd, bool isCols, bool isInsert, int start, int end, bool isDelete)
        {
            if (0 == (this.FormulaCopyFlags & GridFormulaCopyFlags.InsDelRangeReferencesUpdated)
                && 0 == (this.FormulaCopyFlags & GridFormulaCopyFlags.NamedRangeReferencesUpdated))
            {
                return false;
            }

            SortedArrayList clearedFormulaCells = new SortedArrayList();

            GridModel saveGrid = this.grid;

            this.grid = grd;

            ////Turn off undo while we possibly update references...
            bool saveUndoStatus = this.grid.Model.CommandStack.Enabled;
            this.grid.Model.CommandStack.Enabled = false;

            string gridToken = IsSheeted ? GetSheetFamilyItem(this.grid).GridModelToToken[this.grid].ToString() : string.Empty;
            string sheetName = IsSheeted ? GetSheetFamilyItem(this.grid).TokenToSheetName[gridToken].ToString().ToUpper() : string.Empty;

            Hashtable namedRangeChanges = null;
            if (0 != (this.FormulaCopyFlags & GridFormulaCopyFlags.NamedRangeReferencesUpdated))
            {
                ////The idea is to put the named range into this formula, "=sum(namedrange)"
                ////and then adjust this formula for the inserted / deleted row / col. Finally,
                ////the changed named range is put back into the collections.

                namedRangeChanges = new Hashtable();
                foreach (string s in this.NamedRanges.Keys)
                {
                    string val = this.NamedRanges[s].ToString();
                    string oldFormula = string.Format("=sum({0})", val);
                    int offSet = isInsert ? end - start + 1
                        : start - end - 1;
                    string newFormula = isCols ? this.AdjustReferences(oldFormula, 0, offSet, -1, start, sheetName, string.Empty)
                        : this.AdjustReferences(oldFormula, offSet, 0, start, -1);
                    if (oldFormula != newFormula)
                    {
                        newFormula = newFormula.Substring(5, newFormula.Length - 6);
                        namedRangeChanges.Add(s, newFormula);
                    }
                }

                foreach (string s in namedRangeChanges.Keys)
                {
                    this.NamedRanges[s] = namedRangeChanges[s];
                }
            }

            if (0 != (this.FormulaCopyFlags & GridFormulaCopyFlags.InsDelRangeReferencesUpdated))
            {
                ////Generate a list of formula cells from dependent cells.
                SortedArrayList formulaList = new SortedArrayList();
                SortedArrayList removedDependentCells = new SortedArrayList();
                this.UpdateDependencies(this.cell);

                foreach (string s in DependentCells.Keys)
                {
                    ////Check if row / col needs checking - i.e. past insertion or deletion.
                    //// Continue if no check necessary.
                    string sheet = SheetToken(s);
                    int row = this.RowIndex(s);
                    int col = this.ColIndex(s);
                    if (gridToken != sheet
                        || (isCols && col < start)
                        || (!isCols && row < start))
                    {
                        continue;
                    }
                    bool removeDependent = isCols ? (col <= end && col >= start) :(row <= end && row >= start);
                    if (removeDependent&&isDelete)
                    {
                        removedDependentCells.Add(s);
                    }
                    else
                    {
                        ////ht holds the formula cells that depend upon cell s
                        ////Check formulaList to make sure it holds each formula cell.
                        ////formulaList is dynamically created sorted arraylist.
                        Hashtable ht = (Hashtable)DependentCells[s];
                        SortedArrayList clearList = new SortedArrayList();
                        foreach (object o in ht.Keys)
                        {
                            string s1 = o as string;
                            formulaList.InsertIfNotPresent(s1);
                        }
                    }
                }
                foreach (string dependentKey in removedDependentCells)
                {
                    ClearFormulaDependentCells(dependentKey);
                }
                ////clear dependencies on any found formulas
                foreach (string s1 in formulaList)
                {
                    if (clearedFormulaCells.InsertIfNotPresent(s1))
                    {
                        ClearFormulaDependentCells(s1);
                    }
                }
                ////loop thru all formulas to see if need to remove them for formula list
                if (DependentFormulaCells.Keys.Count > 0)
                {
                    string[] cachedList = new string[DependentFormulaCells.Keys.Count];
                    DependentFormulaCells.Keys.CopyTo(cachedList, 0);
                    foreach (string o in cachedList)
                    {
                        if (formulaList.BinarySearch(o) < 0 && clearedFormulaCells.InsertIfNotPresent(o))
                        {
                            ClearFormulaDependentCells(o);
                        }
                    }
                }

                ////create a list of formulas that need adjusting...
                Hashtable adjustedFormulas = new Hashtable();
                foreach (string s in formulaList)
                {
                    GridModel grd1 = this.grid;
                    string sheet = SheetToken(s);
                    if (!isInsert && !isDelete && sheet.Length > 0)
                    {
                        grd1 = GetSheetFamilyItem(this.grid).TokenToGridModel[sheet] as GridModel;
                    }

                    string currentSheetName = IsSheeted ? GetSheetFamilyItem(this.grid).TokenToSheetName[sheet].ToString().ToUpper() : string.Empty;

                    int row = this.RowIndex(s);
                    int col = this.ColIndex(s);
                    int offSet = isInsert ? end - start + 1
                        : start - end - 1;
                    ////if adjustment is in the same grid, need to adjust for inserted rows/columns
                    if (sheet == gridToken || gridToken == string.Empty)
                    {
                        if (isCols)
                        {
                            col += (col < start || (col == start && isDelete)) ? 0 : offSet;
                        }
                        else
                        {
                            row += (row < start || (row == start && isDelete)) ? 0 : offSet;
                        }
                    }

                    string oldFormula = grd1[row, col].Text;
                    string newFormula = isCols ? this.AdjustReferences(oldFormula, 0, offSet, -1, start, sheetName, currentSheetName, isDelete)
                        : this.AdjustReferences(oldFormula, offSet, 0, start, -1, sheetName, currentSheetName, isDelete);
                    if (oldFormula != newFormula)
                    {
                        ////found a formula that needs to be adjusted
                        adjustedFormulas.Add(s, new AdjustInfo(grd1, row, col, newFormula));

                        ////make sure the old formula is not regenerated below...
                        if (clearedFormulaCells.IndexOf(s) > -1)
                        {
                            clearedFormulaCells.Remove(s);
                        }

                        bool b = grd1.Model.IgnoreReadOnly;
                        grd1.Model.IgnoreReadOnly = true;
                        grd1[row, col].Text = newFormula;
                        grd1[row, col].FormulaTag = null;
                        grd1.Model.IgnoreReadOnly = b;
                    }
                }

                ////now loop thru all adjustedFormulas formuals and force computation
                foreach (string s in adjustedFormulas.Keys)
                {
                    AdjustInfo info = adjustedFormulas[s] as AdjustInfo;

                    bool b = info.grd1.Model.IgnoreReadOnly;
                    info.grd1.Model.IgnoreReadOnly = true;

                    GridRangeInfo range = GridRangeInfo.Auto(info.row, info.col);
                    if (!info.grd1.ActiveGridView.ViewLayout.VisibleCellsRange.Contains(range) || info.grd1.Updating)
                    {
                        ////getting inside this loop should resbuilt proper dependencies
                        string save = this.cell;
                        GridModel saveModel = this.grid;
                        this.grid = info.grd1;
                        this.cell = GridRangeInfo.GetAlphaLabel(info.col) + info.row.ToString();
                        string s2 = this.Parse(info.newFormula);
                        info.grd1[info.row, info.col].FormulaTag = new GridFormulaTag(s2, this.ComputedValue(s2), info.row, info.col);
                        if (this.ForceSaveCellInfo)
                        {
                            info.grd1.RaiseSaveCellInfo(new GridSaveCellInfoEventArgs(info.row, info.col, info.grd1[info.row, info.col], StyleModifyType.Override));
                        }

                        this.grid = saveModel;
                        this.cell = save;
                    }

                    info.grd1.Model.IgnoreReadOnly = b;
                }

                adjustedFormulas.Clear();
            }

            if (0 != (this.FormulaCopyFlags & GridFormulaCopyFlags.NamedRangeReferencesUpdated))
            {
                foreach (string s in namedRangeChanges.Keys)
                {
                    ////   this.NamedRanges[s] = changes[s];
                    Hashtable ht = (Hashtable)DependentNamedRangeCells[s];
                    if (ht != null)
                    {
                        int offSet = isInsert ? end - start + 1
                        : start - end - 1;

                        string[] copy = new string[ht.Keys.Count];
                        ht.Keys.CopyTo(copy, 0);
                        foreach (string s1 in copy)
                        {
                            updateFormulasInCells(s1, clearedFormulaCells);
                        }
                    }
                }
            }

            ////make sure dependent cells are properly set up
            foreach (string s3 in clearedFormulaCells)
            {
                Hashtable ht1 = DependentFormulaCells[s3] as Hashtable;
                if (ht1 != null)
                {
                    foreach (string s4 in ht1.Keys)
                    {
                        Hashtable ht2 = DependentCells[s4] as Hashtable;
                        if (ht2 == null)
                        {
                            ht2 = new Hashtable();
                            ht2.Add(s3, s3);
                            DependentCells.Add(s4, ht2);
                        }
                        else if (!ht2.ContainsKey(s3))
                        {
                            ht2.Add(s3, s3);
                        }
                    }
                }
                else
                {
                    updateFormulasInCells(s3, null);
                }
            }

            ////reset undo
            this.grid.Model.CommandStack.Enabled = saveUndoStatus;
            if (namedRangeChanges != null)
            {
                namedRangeChanges.Clear();
            }

            clearedFormulaCells.Clear();
            clearedFormulaCells.TrimToSize();

            this.grid = saveGrid;

            return true;
        }

        private void updateFormulasInCells(string formulaCell, SortedArrayList clearedFormulaCells)
        {
            string cell1 = formulaCell;
            int i = cell1.LastIndexOf(sheetToken);
            int row, col;
            GridModel grd1 = grid;
            string sheet = string.Empty;
            if (i > -1)
            {
                GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                sheet = cell1.Substring(0, i + 1);
                this.grid = (GridModel)family.TokenToGridModel[cell1.Substring(0, i + 1)];
                row = RowIndex(cell1);
                col = ColIndex(cell1);
            }
            else
            {
                row = RowIndex(cell1);
                col = ColIndex(cell1);
            }

            ////need to clear old dependencies only once
            if (clearedFormulaCells != null && clearedFormulaCells.InsertIfNotPresent(cell1))
            {
                ClearFormulaDependentCells(cell1);
            }

            bool b = grd1.Model.IgnoreReadOnly;
            grd1.Model.IgnoreReadOnly = true;
            string save = this.cell;
            this.cell = GridRangeInfo.GetAlphaLabel(col) + row.ToString();
            string newFormula = this.grid[row, col].Text;
            this.grid[row, col].FormulaTag = null;

            ////only parse it and add teh formulatag if the cell contains a formula...
            if (newFormula.Length > 0 && newFormula[0] == formulaChar)
            {
                string s2 = this.Parse(newFormula);
                grd1[row, col].FormulaTag = new GridFormulaTag(s2, this.ComputedValue(s2), row, col);

                if (this.ForceSaveCellInfo || grid.Updating)
                {
                    GridRangeInfo range = GridRangeInfo.Auto(row, col);
                    if (!grid.ActiveGridView.ViewLayout.VisibleCellsRange.Contains(range) || grid.Updating)
                    {
                        this.grid.RaiseSaveCellInfo(new GridSaveCellInfoEventArgs(row, col, grid[row, col], StyleModifyType.Override));
                    }
                }
            }

            grd1.Model.IgnoreReadOnly = b;
            this.cell = save;
            this.grid = grd1;
        }

        /// <summary>
        /// Helper method for adjusting formulas.
        /// </summary>
        /// <param name="origText">Valid unparsed formula string.</param>
        /// <param name="rowOffset">Row offset adjusmtent to be made to the origText.</param>
        /// <param name="colOffset">Column offset adjustment to be made to the origText</param>
        /// <returns>Unparsed formula string derived from origText by adjusting the row 
        /// and column references.
        /// </returns>
        /// <remarks>
        /// For example, calling AdjustReferences("=A4+C4", 2, 1) returns the string "=B6+D6".
        /// This is a helper method that is used to adjust formula references for inserted and
        /// deleted rows and columns. Normally, you would not need this method unless you 
        /// are managing formulas outside the GridControl as in a virtual GridControl or a 
        /// GridDataBoundGrid. This AdjustReferences implementation does not support 
        /// updating references with sheet names within the formulas.</remarks>
        public string AdjustReferences(string origText, int rowOffset, int colOffset)
        {
            return AdjustReferences(origText, rowOffset, colOffset, -1, -1);
        }

        /// <summary>
        /// Helper method for adjusting formulas.
        /// </summary>
        /// <param name="origText">Valid unparsed formula string.</param>
        /// <param name="rowOffset">Row offset adjusmtent to be made to the origText.</param>
        /// <param name="colOffset">Column offset adjustment to be made to the origText</param>
        /// <param name="sheetNameWhereCopied">The name of the sheet where the copy took place.</param>
        /// <param name="currentSheetName">The name of the sheet where the cell holding origText is located.</param>
        /// <returns>Unparsed formula string derived from origText by adjusting the row 
        /// and column references.
        /// </returns>
        /// <remarks>
        /// This AdjustReferences implementation does support 
        /// updating references with sheet names within the formulas.</remarks>
        public string AdjustReferences(string origText, int rowOffset, int colOffset, string sheetNameWhereCopied, string currentSheetName)
        {
            return AdjustReferences(origText, rowOffset, colOffset, -1, -1, sheetNameWhereCopied, currentSheetName);
        }

        private string AdjustReferences(string origText, int rowOffset, int colOffset, int rowCut, int colCut)
        {
            return AdjustReferences(origText, rowOffset, colOffset, rowCut, colCut, string.Empty, string.Empty);
        }

        internal bool inPaste = false;

        /// <summary>
        /// Specifies if the clipboard buffer came from cut operation or not.
        /// </summary>
        internal bool fromClipboardCut = false;

        private bool CheckNeedToAdjust(string sheetName, string requiredSheetName, string currentSheetName)
        {
            return inPaste || requiredSheetName == sheetName || (currentSheetName == requiredSheetName && sheetName == string.Empty);
        }

        private string AdjustReferences(string origText, int rowOffset, int colOffset, int rowCut, int colCut, string requiredSheetName, string currentSheetName)
        {
            return AdjustReferences(origText, rowOffset, colOffset, rowCut, colCut, requiredSheetName, currentSheetName, false);
        }

        private string AdjustReferences(string origText, int rowOffset, int colOffset, int rowCut, int colCut, string requiredSheetName, string currentSheetName, bool isDelete)
        {
            string s = string.Empty;
            string upperText = origText.ToUpper();
            string sheetName = string.Empty;

            requiredSheetName = requiredSheetName.ToUpper();
            currentSheetName = currentSheetName.ToUpper();

            bool needToAdjust = CheckNeedToAdjust(sheetName, requiredSheetName, currentSheetName);

            int pos = 0;
            int len = origText.Length;

            while (pos < len)
            {
                if (upperText[pos] == STRING_fixedreference[0] && (inPaste || !FixedReferenceOnlyOnPaste))
                {
                    ////skip $'s on cols
                    s += origText[pos];
                    pos++;
                    while (pos < len && char.IsLetter(upperText, pos))
                    {
                        s += origText[pos];
                        pos++;
                    }

                    if (pos < len && upperText[pos] == STRING_fixedreference[0])
                    {
                        ////skip $'s on rows
                        s += origText[pos];
                        pos++;
                        while (pos < len && char.IsDigit(upperText, pos))
                        {
                            s += origText[pos];
                            pos++;
                        }
                    }
                    else if (pos < len)
                    {
                        ////no $ on row
                        int row = 0;
                        while (pos < len && char.IsDigit(upperText, pos))
                        {
                            row = (10 * row) + int.Parse(upperText[pos].ToString());
                            pos++;
                        }

                        if (row >= rowCut && needToAdjust)
                        {
                            row = Math.Max(row + rowOffset, this.grid.Model.Rows.HeaderCount + 1);
                        }

                        s += row.ToString();
                    }
                }
                else if (char.IsLetter(upperText, pos))
                {
                    int loopStart = pos;
                    pos++;
                    while (pos < len && (char.IsLetter(upperText, pos) || upperText[pos] == ' ' || upperText[pos] == '_'))
                    {
                        pos++;
                    }

                    if (pos < len && upperText[pos] == STRING_fixedreference[0])
                    {
                        ////handle no $ on col but $ on row
                        string cellRef = upperText.Substring(loopStart, pos - loopStart) + "1";
                        int col = this.ColIndex(cellRef);

                        if (col >= colCut && needToAdjust)
                        {
                            col = Math.Max(col + colOffset, this.grid.Model.Cols.HeaderCount + 1);
                        }

                        s += GridRangeInfo.GetAlphaLabel(col);

                        s += origText[pos];
                        pos++;
                        while (pos < len && char.IsDigit(upperText, pos))
                        {
                            s += origText[pos];
                            pos++;
                        }
                    }
                    else if (pos < len && char.IsDigit(upperText, pos))
                    {
                        while (pos < len && char.IsDigit(upperText, pos))
                        {
                            pos++;
                        }

                        if (pos < len-1 && (upperText[pos] == sheetToken || char.IsLetter(upperText, pos)))
                        {
                            //we have a sheet reference
                            pos++;
                            s += upperText.Substring(loopStart, pos - loopStart);
                        }
                        else
                        {

                            string cellRef = upperText.Substring(loopStart, pos - loopStart);
                            if (((loopStart - 1) < upperText.Length) && !upperText[loopStart - 1].Equals('"') && (((pos - loopStart + 1) < upperText.Length) && !upperText[pos - loopStart + 1].Equals('"'))
                                || (((pos - loopStart + 1) == upperText.Length)))//&& (rowOffset > 0 || colOffset > 0)))
                            {
                                int row = this.RowIndex(cellRef);
                                int col = this.ColIndex(cellRef);

                                if (!isDelete && ((row >= rowCut && row <= (-rowOffset + rowCut - 1)) || (col >= colCut && col <= (-colOffset + colCut - 1))))
                                {
                                    return "=#REF!";
                                }

                                if (needToAdjust && (row > rowCut || (row == rowCut && !isDelete)))
                                {
                                    row = Math.Max(row + rowOffset, this.grid.Model.Rows.HeaderCount + 1);
                                }

                                if (needToAdjust && (col > colCut || (col == colCut && !isDelete)))
                                {
                                    col = Math.Max(col + colOffset, this.grid.Model.Cols.HeaderCount + 1);
                                }

                                s += GridRangeInfo.GetAlphaLabel(col) + row.ToString();
                            }
                            else
                            {
                                s += origText.Substring(loopStart, pos - loopStart);
                            }
                        }
                    }
                    else
                    {
                        ////Not a cell reference.
                        sheetName = origText.Substring(loopStart, pos - loopStart);
                        s += sheetName;
                        sheetName = (pos < len && origText[pos] == sheetToken)
                                      ? sheetName.ToUpper(CultureInfo.InvariantCulture)
                                      : string.Empty;
                        needToAdjust = CheckNeedToAdjust(sheetName, requiredSheetName, currentSheetName);
                    }
                }
                else
                {
                    s += origText[pos];
                    pos++;
                }
            }
            ////    Console.WriteLine("out[" + s + "]");
            return s;
        }

        /// <summary>
        /// Displays information on the cell currently being calculated.
        /// </summary>
        /// <returns>String with information on the cell currently being calculated.</returns>
        public override string ToString()
        {
            return "GridFormulaEngine { Cell: " + (cell != null ? cell.ToString() : "null") + " " + (DependentCells != null ? DependentCells.Count : -1).ToString() + "}";
        }

        #region Sheet Support

        [ThreadStaticAttribute]
        internal static int TokenCount = 0;

        [ThreadStaticAttribute]
        internal static Hashtable modelToSheetID = null;

        /// <summary>
        /// Removes a sheet from a sheet family.
        /// </summary>
        /// <param name="sheetName">The name of the sheet to be removed.</param>
        /// <param name="grid">The GridModel associated with this sheet.</param>
        /// <remarks>
        /// This method only updates the GridFormulaEngine structure with respect to 
        /// removing a sheet from a family of sheets that has been created 
        /// using <see cref="RegisterGridAsSheet"/>.
        ///  <para/>
        /// This procedure first renames the sheet to be deleted as #REF using 
        /// <see cref="ChangeGridSheetName"/>. This has the effect of replacing
        /// any formula on another sheet that references this deleted sheet with a
        /// #REF to indicate that formula has a reference problem. It will then 
        /// call <see cref="UnregisterGridAsSheet"/> with the new name #REF
        /// to remove the renamed sheet. 
        ///  <para/>
        /// This method does not remove the grid from your UI structures. For example,
        /// if you have several grids in a TabControl, calling RemoveGridSheet
        /// does not remove the grid from any tabpage.
        /// </remarks>
        public static void RemoveGridSheet(string sheetName, GridModel grid)
        {
            GridSheetFamilyItem family = GetSheetFamilyItem(grid);

            ChangeGridSheetName(sheetName, "#REF", grid, true);
            UnregisterGridAsSheet("#REF", grid);

            ////redraw any visible grids to show the #REFs
            foreach (GridModel model in family.SheetNameToGridModel.Values)
            {
                if (model.ActiveGridView != null && model.ActiveGridView.Visible)
                {
                    model.ActiveGridView.Refresh();
                }
            }
        }

        /// <summary>
        /// Used by the <see cref="ChangeGridSheetName"/> method to decide
        /// what characters can precede a SheetName in a formula. The default
        /// value is "+-/*&lt;&gt;=(,:".
        /// </summary>
        [ThreadStaticAttribute]
        public static string ValidSheetChars = "+-/*<>=(,:";

        internal const char sheetToken = '!';

        /// <summary>
        /// Changes a sheetname that was previously registered using <see cref="RegisterGridAsSheet"/>. 
        /// This method iterates through all the cells in all the sheets in the sheet family, swapping all 
        /// occurrences of the oldName in any formula with the newName.
        /// </summary>
        /// <param name="oldName">The old sheet name.</param>
        /// <param name="newName">The new sheet name.</param>
        /// <param name="grid">True if the sheet was successfully renamed. If false returns,
        /// check whether the sheet family already contains the new name.</param>
        /// <returns>returns boolean value</returns>
        public static bool ChangeGridSheetName(string oldName, string newName, GridModel grid)
        {
            return ChangeGridSheetName(oldName, newName, grid, false);
        }

        private static bool ChangeGridSheetName(string oldName, string newName, GridModel grid, bool setREF)
        {
            bool returnValue = false;
            try
            {
                GridFormulaEngine engine = ((GridFormulaCellModel)grid.CellModels["FormulaCell"]).Engine;
                GridSheetFamilyItem family = GetSheetFamilyItem(grid);
                string oldNameUpper = oldName.ToUpper();
                string newNameUpper = newName.ToUpper();
                string validChars = ValidSheetChars;
                if (engine != null && family != null
                    && family.SheetNameToGridModel.ContainsKey(oldNameUpper)
                    && !family.SheetNameToGridModel.ContainsKey(newNameUpper))
                {
                    ////swap out the name mapped to the gridmodel
                    object gridModel = family.SheetNameToGridModel[oldNameUpper];
                    family.SheetNameToGridModel.Remove(oldNameUpper);
                    family.SheetNameToGridModel.Add(newNameUpper, gridModel);

                    ////swap out the name mapped to the sheet Token
                    object token = family.SheetNameToToken[oldNameUpper];
                    family.SheetNameToToken.Remove(oldNameUpper);

                    family.TokenToSheetName.Remove(token);

                    family.SheetNameToToken.Add(newNameUpper, token);
                    family.TokenToSheetName.Add(token, newName);

                    ////swap out the name in the sized sheet name list
                    family.sheetNamesSized.Remove(oldNameUpper);
                    family.sheetNamesSized.Add(newNameUpper);
                    family.sheetNamesSized.Sort(new LenComparer());

                    oldNameUpper += GridFormulaEngine.sheetToken;
                    newName += GridFormulaEngine.sheetToken;

                    ////now loop thru all the sheets and all the cells replacing
                    ////the name in the formulas. Note that formulaTags should not
                    ////be affected. Only the formula text stored in the cell's GridStyleInfo.
                    foreach (GridModel model in family.SheetNameToGridModel.Values)
                    {
                        GridData data = model.Data;
                        for (int row = 1; row <= data.RowCount; ++row)
                        {
                            for (int col = 1; col <= data.ColCount; ++col)
                            {
                                if (data[row, col] != null)
                                {
                                    GridStyleInfo style = new GridStyleInfo(data[row, col]);
                                    string text = style.Text;
                                    string textUpper = text.ToUpper();
                                    if (text.Length > 0 // && style.CellType == "FormulaCell"
                                        && text[0] == engine.formulaChar)
                                    {
                                        int i = textUpper.Length;
                                        bool changed = false;
                                        while (i > -1 && (i = textUpper.Substring(0, i).LastIndexOf(oldNameUpper)) > -1)
                                        {
                                            int startI = i;
                                            int j = i + oldNameUpper.Length;
                                            i--;
                                            int saveI = i;
                                            while (i > -1 && textUpper[i] == ' ')
                                            {
                                                i--;
                                            }

                                            if (i > -1 && (i == 0 || validChars.IndexOf(textUpper[i]) > -1))
                                            {
                                                text = text.Substring(0, saveI + 1) + newName + text.Substring(j);
                                                changed = true;
                                                //// i = j;
                                            }
                                            else if (i > -1)
                                            {
                                                i = startI - 1;
                                            }
                                        }

                                        if (changed)
                                        {
                                            style.Text = text;
                                            if (setREF && style.FormulaTag != null)
                                            {
                                                style.FormulaTag.Text = "#REF!"; ////set so it displays an error
                                            }

                                            data[row, col] = style.Store;
                                        }
                                    }
                                }
                            }
                        }
                    }

                    returnValue = true;
                }
            }
            catch
            {
                returnValue = false;
            }

            return returnValue;
        }

        /// <summary>
        /// Registers a grid so it can be referenced in a formula from another grid.
        /// </summary>
        /// <param name="refName">The reference name used to refer to this grid from formulas in other grids.
        /// </param>
        /// <param name="model">The GridModel from the grid being registered.
        /// </param>
        /// /// <param name="sheetFamilyID">An integer previously created with a call 
        /// to GridFormulaEngine.CreateSheetFamilyID. This number is used to identify
        /// the grids as belonging to a particular family of grids. You can only reference
        /// grids from within the same family.
        /// </param>
        /// <remarks>
        /// Essential Grid supports multisheet references with its formulas. For example, if you
        /// have two tabpages with a GridControl on each, you can reference cells from the first
        /// in the second grid. For this to work, both grids need to be registered using this method.
        ///  <para/>
        /// The syntax for using a sheet reference as part of a formula is to prefix a cell reference 
        /// with the sheet reference name followed by an exclamation point.
        ///  <para/>
        /// The formula "= sheet1!A1 + sheet2!C3" would add the value of cell A1 for 
        /// the grid whose reference name is sheet1 to the value from cell C3 in the grid 
        /// whose reference name is sheet2.
        ///   <para/>
        /// </remarks>
        /// <example> Use this code to use cross sheet references.
        /// <code lang="C#">
        ///        //Register 3 grids so cell can be referenced across grids.
        ///        int sheetfamilyID = GridFormulaEngine.CreateSheetFamilyID();
        ///        GridFormulaEngine.RegisterGridAsSheet("summary", this.gridControl1.Model, sheetfamilyID);
        ///        GridFormulaEngine.RegisterGridAsSheet("income", this.gridControl2.Model, sheetfamilyID);
        ///        GridFormulaEngine.RegisterGridAsSheet("expenses", this.gridControl3.Model, sheetfamilyID);
        ///        ....
        ///        //Sample formula usage for cells in gridControl1, the 'summary' grid.
        ///        //This code sums up some cells from gridControl3, the 'expenses' grid, 
        ///        //and gridControl2, the 'income' grid.
        ///  <para/>       
        ///        //Sum the range B2:B8 from the expenses grid.
        ///        this.gridControl1[3,4].Text = "= Sum(expenses!B2:expenses!B8)";
        /// <para/>
        ///        //Sum the range B2:B4 from the income grid.
        ///        this.gridControl1[4,4].Text = "= Sum(income!B2:income!B4)";
        ///  <para/>       
        /// </code>
        /// <code lang="VB">
        ///        'Register 3 grids so cells can be referenced across grids.
        ///        Dim sheetfamilyID As Integer = GridFormulaEngine.CreateSheetFamilyID();
        ///        GridFormulaEngine.RegisterGridAsSheet("summary", Me.gridControl1.Model, sheetfamilyID)
        ///        GridFormulaEngine.RegisterGridAsSheet("income", Me.gridControl2.Model, sheetfamilyID) 
        ///        GridFormulaEngine.RegisterGridAsSheet("expenses", Me.gridControl3.Model, sheetfamilyID) 
        ///        ....
        ///        'Sample formula usage for cells in gridControl1, the 'summary' grid.
        ///        'This code sums ups some cells from gridControl3, the 'expenses' grid, 
        ///        'and gridControl2, the 'income' grid.
        ///   <para/>      
        ///        'Sum the range B2:B8 from the expenses grid.
        ///        Me.gridControl1(3,4).Text = "= Sum(expenses!B2:expenses!B8)"
        /// <para/>
        ///        'Sum the range B2:B4 from the income grid.
        ///        Me.gridControl1(4,4).Text = "= Sum(income!B2:income!B4)"
        /// </code>
        /// </example>
        public static void RegisterGridAsSheet(string refName, GridModel model, int sheetFamilyID)
        {
            if (modelToSheetID == null)
            {
                modelToSheetID = new Hashtable();
            }

            if (modelToSheetID[model] == null)
            {
                modelToSheetID.Add(model, sheetFamilyID);
            }

            GridSheetFamilyItem family = GetSheetFamilyItem(model);
            family.isSheeted = true;

            string refName1 = refName.ToUpper();
            if (family.SheetNameToGridModel == null)
            {
                family.SheetNameToGridModel = new Hashtable();
            }

            if (family.TokenToGridModel == null)
            {
                family.TokenToGridModel = new Hashtable();
            }

            if (family.SheetNameToToken == null)
            {
                family.SheetNameToToken = new Hashtable();
            }

            if (family.TokenToSheetName == null)
            {
                family.TokenToSheetName = new Hashtable();
            }

            if (family.GridModelToToken == null)
            {
                family.GridModelToToken = new Hashtable();
            }

            if (family.SheetNameToGridModel.ContainsKey(refName1))
            {
                string token = (string)family.SheetNameToToken[refName1];
                family.TokenToGridModel[token] = model;
                family.GridModelToToken[model] = token;
                family.TokenToSheetName[token] = refName;
            }
            else
            {
                string token = sheetToken + TokenCount.ToString() + sheetToken;
                TokenCount++;
                family.TokenToGridModel.Add(token, model);
                family.SheetNameToToken.Add(refName1, token);
                family.SheetNameToGridModel.Add(refName1, model);
                family.GridModelToToken.Add(model, token);
                family.TokenToSheetName[token] = refName;

                model.CellsChanged += new GridCellsChangedEventHandler(GridCellsChanged);
            }
            ////need a list of names sorted by string length to simplify searches
            if (family.sheetNamesSized == null)
            {
                family.sheetNamesSized = new ArrayList();
            }

            family.sheetNamesSized.Add(refName1);
            family.sheetNamesSized.Sort(new LenComparer());
        }

        /// <summary>
        /// Returns an integer that is used to identify a family of grids.
        /// </summary>
        /// <returns>return integer</returns>
        /// <remarks>
        /// Essential Grid supports multisheet references within a family of grids. To use
        /// this functionality, you employ the method to get a unique identifier for
        /// the family. Then in the RegisterGridAsSheet method that you call to add grids
        /// to this family, you pass this unique identifier to mark the grids as belonging
        /// to this family. You can only cross reference grids within the same family.
        /// </remarks>
        public static int CreateSheetFamilyID()
        {
            if (sheetFamilyID == int.MaxValue)
            {
                sheetFamilyID = int.MinValue;
            }

            return sheetFamilyID++;
        }

        private static void GridCellsChanged(object sender, GridCellsChangedEventArgs e)
        {
            ////TraceUtil.TraceCurrentMethodInfoIf(Switches.FormulaCell.TraceVerbose, e, this);

            GridModel model = sender as GridModel;
            GridSheetFamilyItem family = GetSheetFamilyItem(model);
            if (model != null && e.Range.IsCells)
            {
                string token = family.GridModelToToken[sender] as string;
                GridFormulaEngine engine = ((GridFormulaCellModel)model.CellModels["FormulaCell"]).Engine;
                int row, col;
                if (e.Range.GetFirstCell(out row, out col))
                {
                    string s = token + GridRangeInfo.GetAlphaLabel(col) + row.ToString();
                    engine.Refresh(s);
                    while (e.Range.GetNextCell(ref row, ref col, false))
                    {
                        s = token + GridRangeInfo.GetAlphaLabel(col) + row.ToString();
                        engine.Refresh(s);
                    }
                }
            }
        }

        /// <summary>
        /// Unregisters a grid so it can no longer be referenced in a formula from another grid.
        /// </summary>
        /// <param name="refName">The reference name used to refer to this grid from formulas in other grids.
        /// </param>
        /// <param name="model">The grid model.</param>
        public static void UnregisterGridAsSheet(string refName, GridModel model)
        {
            GridSheetFamilyItem family = GetSheetFamilyItem(model);
            string refName1 = refName.ToUpper();
            if (family.SheetNameToGridModel != null && family.SheetNameToGridModel.ContainsKey(refName1))
            {
                family.SheetNameToGridModel.Remove(refName1);
                string token = (string)family.SheetNameToToken[refName1];

                if (model != null)
                {
                    model.CellsChanged -= new GridCellsChangedEventHandler(GridCellsChanged);
                }

                family.SheetNameToToken.Remove(refName1);
                family.TokenToGridModel.Remove(token);
                family.TokenToSheetName.Remove(refName);
                family.sheetNamesSized.Remove(refName1); ////added
                modelToSheetID.Remove(model);
                family.GridModelToToken.Remove(model);
            }
        }

        private void PutTokensForSheets(ref string text, char toke)
        {
            GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
            if (family.sheetNamesSized != null)
            {
                foreach (string name in family.sheetNamesSized)
                {
                    string token = (string)family.SheetNameToToken[name];
                    string s = name.ToUpper() + toke;
                    text = text.Replace(s, token);
                }
            }
        }

        private void PutTokensForSheets(ref string text)
        {
            PutTokensForSheets(ref text, sheetToken);
        }

        #endregion

        [ThreadStaticAttribute]
        private static bool yesToCloning = false;

        /// <summary>
        /// Gets or sets a value indicating whether FormulaTags are cloned when
        /// setting one FormulaTag object equal to another. 
        /// </summary>
        /// <remarks>
        /// This is by default set to False as normally the FormulaEngine
        /// expects to be working with referenced objects, and not clones 
        /// of referenced objects.
        /// </remarks>
        public static bool CloneableFormulaTags
        {
            get
            {
                return yesToCloning;
            }

            set
            {
                if (value != yesToCloning)
                {
                    yesToCloning = value;
                    GridStyleInfoStore.FormulaTagProperty.IsCloneable = yesToCloning;
                    GridStyleInfoStore.FormulaTagProperty.IsDisposable = yesToCloning;
                }
            }
        }

        /// <summary>
        /// Constructor for GridFormulaEngine.
        /// </summary>
        /// <param name="gridModel">The GridModel from the underlying grid.</param>
        public GridFormulaEngine(GridModel gridModel)
        {
            GridStyleInfoStore.FormulaTagProperty.IsCloneable = GridFormulaEngine.yesToCloning;
            GridStyleInfoStore.FormulaTagProperty.IsDisposable = GridFormulaEngine.yesToCloning;
#if DEBUG

            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(this);
            }
#else

            ;
#endif
            grid = gridModel;

            grid.CellsChanged += new GridCellsChangedEventHandler(grid_CellsChanged);
            
            this.InitLibraryFunctions();

            formulaChar = (char)0;

            tokens = new char[]
            {
                                   TOKEN_add,
                                   TOKEN_subtract,
                                   TOKEN_multiply,
                                   TOKEN_divide, 
                                   TOKEN_less,
                                   TOKEN_greater,
                                   TOKEN_equal,
                                   TOKEN_lesseq,
                                   TOKEN_greatereq,
                                   TOKEN_noequal
            };

            refreshedCells = new SortedArrayList();
            formulaCopyFlags = GridFormulaCopyFlags.ClipBoardText;
        }

        private bool useVirtualDataSource = true;

        /// <summary>
        /// Gets or sets a value indicating whether the underlying data is virtually bound to the grid.
        /// </summary>
        /// <remarks>If the formula engine knows the data is stored within a GridControl.Data
        /// object, it can optimize data access. If the grid is being populated through
        /// virtual techniques (handling QueryCellInfo), the formula engine has to get the data 
        /// through that means. The default value is to assume the data is coming from a virtual
        /// datasource. There are some other situations, like formulas in headers or frozen cells, when
        /// this property should be set to True.
        /// </remarks>
        public bool UsesVirtualDataSource
        {
            get { return useVirtualDataSource; }
            set { useVirtualDataSource = value; }
        }

        private bool calculationsSuspended = false;

        /// <summary>
        /// Gets or sets a value indicating whether formulas are immediately calculated as dependent cells are changed.
        /// </summary>
        /// <remarks>Use this property to suspend calculations while a series of changes 
        /// are made to dependent cells either by the user or programmatically. When the changes are 
        /// complete, set this property to False, and then call Engine.RecalculateRange to recalculate
        /// the affected range. See the sample in GridFormulaCellModel.CalculatingSuspended.
        /// </remarks>
        public bool CalculatingSuspended
        {
            get { return calculationsSuspended; }
            set { calculationsSuspended = value; }
        }

        private int maximumRecursiveCalls = 100;

        /// <summary>
        /// Gets or sets the maximum number of recursive calls that can be used to compute a cellvalue.
        /// </summary>
        /// <remarks>This property comes into play when you have a calculated formula cell that depends on  
        /// another calculated formula that depends on another calculated formula and so on. If the 
        /// 'depends on another formula' number exceeds MaximumRecursiveCalls, you will see a Too Complex message
        /// displayed in the cell. The default value is 100, but you can set it higher or lower depending upon 
        /// your expected needs. The purpose of the limit is to avoid a circular reference locking up your
        /// application.
        /// </remarks>
        public int MaximumRecursiveCalls
        {
            get { return maximumRecursiveCalls; }
            set { maximumRecursiveCalls = value; }
        }

        private bool currentRowNotationEnabled = true;

        /// <summary>
        /// Gets or sets a value indicating whether to enable / disable using row = 0 in formulas to represent the current row.
        /// </summary>
        /// <remarks> 
        /// When this property is set True, entering zero as a row in a formula is
        /// interpreted to be the current row. Using the current row notation allows
        /// you to sort a column in the grid and maintain the relative formula. 
        /// After sorting, you do have to call engine.RecalculateRange to allow the
        /// relative formulas to reset themselves. See the Grid\Samples\Quick Start\GridControlSort 
        /// sample to see this sorting support in action.
        /// </remarks>
        /// <example>Here is code that places a formula in column 4 that sums
        /// columns 1 and 3. Notice it just sets CellType and Text properties
        /// on the column style for the column. It is not required that you visit
        /// each cell in the column to assign such a formula. 
        /// <code lang="C#">
        /// //column 4 is formula col which used relative notation (row = 0)
        /// //to enter the formulas as a simple string as the default value
        /// //in the column
        /// this.gridControl1.ColWidths[4] = 100;
        /// this.gridControl1[0, 4].Text = "FormulaCell";
        /// this.gridControl1.ColStyles[4].CellType = "FormulaCell";
        /// this.gridControl1.ColStyles[4].Text = "= A0 + C0"; 
        /// </code>
        /// </example>
        public bool CurrentRowNotationEnabled
        {
            get { return currentRowNotationEnabled; }
            set { currentRowNotationEnabled = value; }
        }

        #region Refreshing/Recalculating cells

        ////Used to prevent infinite refreshes on circular references.
        private SortedArrayList refreshedCells = null;

        private int dependencyLevel = 0;

        //////        a debug routine to display recursively display dependent cells
        //////        sample usage at start of next routine...
        //////        private int indent = 0;
        //////        private void DisplayAffectedCells(string s)
        //////        {
        //////            string spc = new String('_', 4 * indent);
        //////            indent++;
        //////            Console.Write(spc + s + " affects ");
        //////            Hashtable ht = (Hashtable) this.DependentCells[s];
        //////            if(ht != null)
        //////            {
        //////                foreach(object o1 in ht.Keys)
        //////                {
        //////                    string s2 = o1 as string;
        //////                    Console.Write(s2 + " ");
        //////                }
        //////                Console.WriteLine(string.Empty);
        //////                foreach(object o1 in ht.Keys)
        //////                {
        //////                    string s2 = o1 as string;
        //////                    DisplayAffectedCells(s2);
        //////                }
        //////            }
        //////            else
        //////                Console.WriteLine(string.Empty);
        //////
        //////            indent--;
        //////        }

        /// <summary>
        /// Recalculates any cell that depends upon the passed in cell.
        /// </summary>
        /// <param name="s">A cell such as A21 or EE31.</param>
        public void Refresh(string s)
        {
            ////             if(s=="D2")
            ////             {
            ////                 DisplayAffectedCells(s);
            ////             }
            ////don't refresh any cells
            if (CalculatingSuspended)
            {
                return;
            }
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(s, this);
            }
#else
            ;
#endif
            if (dependencyLevel == 0)
            {
                refreshedCells.Clear();
            }

            if (dependencyLevel > MaximumRecursiveCalls)
            {
                dependencyLevel--;
                return; // Possible circular dependency or simply too complex.
            }

            if (DependentCells[s] != null)
            {
                ////Track the cells that have been refreshed since initial call to Refresh.

                refreshedCells.InsertIfNotPresent(s);

                dependencyLevel++;

                bool saveUndoStatus = true;

                try
                {
                    ////Turn off undo.
                    saveUndoStatus = this.grid.Model.CommandStack.Enabled;
                    this.grid.Model.CommandStack.Enabled = false;

                    GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);

                    Hashtable ht = (Hashtable)DependentCells[s];

                    foreach (object o in ht.Keys)
                    {
                        string s1 = o as string;
                        if (s1 != null)
                        {
                            GridModel grd = grid;

                            string sheet = SheetToken(s1);
                            if (sheet.Length > 0)
                            {
                                grid = family.TokenToGridModel[sheet] as GridModel;
                            }

                            int row = RowIndex(s1);
                            int col = ColIndex(s1);

                            GridStyleInfo style = UsesVirtualDataSource
                                ? grid.Model[row, col]
                                : new GridStyleInfo(new GridStyleInfoIdentity(grid.VolatileData, row, col, true), grid.Data[row, col]);

                            GridFormulaTag tag;

                            try
                            {
                                tag = style.FormulaTag;
                            }
                            catch
                            {
                                this.grid = grd;
                                throw new NullReferenceException(FormulaErrorStrings[virtual_mode_required]);
                            }

                            if (tag != null)
                            {
                                SetDirty(tag); //// tag.Text = string.Empty;
                                if (refreshedCells.BinarySearch(s1) < 0)
                                {
                                    Refresh(s1); ////recursive call
                                }
                            }

                            ////Option 1 to update.
                            GridRangeInfo cellRange = GridRangeInfo.Cell(row, col);
                            if (grid.ActiveGridView != null
                                && (grid.ActiveGridView.ViewLayout.VisibleCellsRange.Contains(cellRange)
                                || row <= grid.Rows.FrozenCount || col <= grid.Cols.FrozenCount))
                            {
                                grid.ActiveGridView.Invalidate(grid.ActiveGridView.RangeInfoToRectangle(cellRange));
                            }
                            else if (forceSaveCellInfo)
                            {
                                string saveCell = cell;
                                this.cell = GridRangeInfo.GetAlphaLabel(col) + row.ToString();
                                bool saveCalculatingSuspended = CalculatingSuspended;
                                CalculatingSuspended = true;
                                tag.Text = this.ComputedValue(tag.Formula);
                                CalculatingSuspended = saveCalculatingSuspended;
                                cell = saveCell;
                                this.grid.RaiseSaveCellInfo(new GridSaveCellInfoEventArgs(row, col, style, StyleModifyType.Override));
                            }

                            ////Option 2 to update.
                            //// GridRangeInfo cellRange = GridRangeInfo.Cell(row, col);
                            ////if(grid.ActiveGridView.ViewLayout.VisibleCellsRange.Contains(cellRange))
                            ////    grid.InvalidateRange(cellRange, GridRangeOptions.None);

                            this.grid = grd;
                        }
                    }
                }
                finally
                {
                    dependencyLevel--;
                    if (dependencyLevel == 0)
                    {
                        refreshedCells.Clear();
                        refreshedCells.TrimToSize();
                    }

                    this.grid.Model.CommandStack.Enabled = saveUndoStatus;
                }
            }
        }

        /// <summary>
        /// Recalculates every cell that depends upon any cell in the passed-in range.
        /// </summary>
        /// <remarks>For example, if range is GridRangeInfo(1,1,2,2), and cells (5,6) and
        /// (12,17) hold formulas that reference the cells in the range, then cells (5,6) 
        /// and (12,17) will be re-computed as the result of this call.</remarks>
        /// <param name="range">GridRangeInfo object to be refreshed.</param>
        public void RefreshRange(GridRangeInfo range)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(range, this);
            }
#else
            ;
#endif
            range = range.ExpandRange(1, 1, this.grid.RowCount, this.grid.ColCount);

            ////Loop through the range refreshing the cells...
            for (int r = range.Top; r <= range.Bottom; r++)
            {
                for (int c = range.Left; c <= range.Right; c++)
                {
                    string s = GridRangeInfo.GetAlphaLabel(c) + GridRangeInfo.GetNumericLabel(r);
                    this.Refresh(s);
                }
            }
        }

        /// <summary>
        /// Recalculates any formula cells in the specified range.
        /// </summary>
        /// <remarks>The calculations for non-visible formula cells are performed the next time 
        ///  cell are actually displayed. If you want the calculation performed immediately 
        ///  on cells (visible or not), call the two argument overload of RecalculateRange, 
        ///  passing the forceCalculations argument as True.</remarks>
        /// <param name="range">GridRangeInfo object that specifies the cells to be recalculated.</param>
        public void RecalculateRange(GridRangeInfo range)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(range, this);
            }
#else
            ;
#endif
            this.RecalculateRange(range, false);
        }

        private bool forceSaveCellInfo = false;

        /// <summary>
        /// Gets or sets a value indicating whether a SaveCellInfo event should be raised even if the affected
        /// cell is off-screen.
        /// </summary>
        /// <remarks>
        /// The default behavior is not to raise a SaveCellInfo event for off-screen
        /// cells when a value is modified that affects such cells. This is done for 
        /// performance considerations. But if you need the SaveCellInfo event raised for
        /// every change, you can set this property to ensure this event is raised.
        /// </remarks>
        public bool ForceSaveCellInfo
        {
            get { return forceSaveCellInfo; }
            set { forceSaveCellInfo = value; }
        }

        /// <summary>
        /// Recalculates any formula cells in the specified range.
        /// </summary>
        /// <param name="range">GridRangeInfo object that specifies the cells to be recalculated.</param>
        /// <param name="forceCalculations">Determines whether the calculations on non-visible
        /// cells are performed immediately or delayed until the next time the cell is drawn.
        /// For visible cells, the calculations are done immediately.</param>
        public void RecalculateRange(GridRangeInfo range, bool forceCalculations)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(range, forceCalculations, this);
            }
#else
            ;
#endif
            this.RecalculateRange(range, forceCalculations, false);
        }

        /// <summary>
        /// Recalculates any formula cells in the specified range.
        /// </summary>
        /// <param name="range">GridRangeInfo object that specifies the cells to be recalculated.</param>
        /// <param name="forceCalculations">Determines whether the calculations on non-visible
        /// cells are performed immediately or delayed until the next time the cell is drawn.
        /// For visible cells, the calculations are done immediately.</param>
        /// <param name="forceParsing">When forceParsing is False, a formula is only 
        /// re-parsed if FormulaTag is NULL, or FormulaTag.Formula is empty. Otherwise, the existing
        /// parsed formula in FormulaTag.Formula is used to perform the calculation. The value
        /// of forceParsing only affects the Engine if forceCalculations is True.</param>
        /// <remarks> This method does not do anything if <see cref="CalculatingSuspended"/> is true.
        /// </remarks>
        public void RecalculateRange(GridRangeInfo range, bool forceCalculations, bool forceParsing)
        {
            RecalculateRange(range, this.grid, forceCalculations, forceParsing);
        }

        /// <summary>
        /// Recalculates any formula cells in the specified range.
        /// </summary>
        /// <param name="range">GridRangInfo object that specifies the cells to be recalculated.</param>
        /// <param name="grd">The GridModel object where the range to be updated is located.</param>
        /// <param name="forceCalculations">Determines whether the calculations on non-visible
        /// cells are performed immediately or delayed until the next time the cell is drawn.
        /// For visible cells, the calculations are done immediately.</param>
        /// <param name="forceParsing">When forceParsing is False, a formula is only 
        /// re-parsed if FormulaTag is NULL, or FormulaTag.Formula is empty. Otherwise, the existing
        /// parsed formula in FormulaTag.Formula is used to perform the calculation. The value
        /// of forceParsing only affects the Engine if forceCalculations is True.</param>
        /// <remarks> This method does not do anything if <see cref="CalculatingSuspended"/> is True.
        /// </remarks>
        public void RecalculateRange(GridRangeInfo range, GridModel grd, bool forceCalculations, bool forceParsing)
        {
            RecalculateRange(range, grd, forceCalculations, forceParsing, true);
        }

        internal void RecalculateRange(GridRangeInfo range, GridModel grd, bool forceCalculations, bool forceParsing, bool useBeginUpdate)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(range, forceCalculations, forceParsing, this);
            }
#else
            ;
#endif
            if (CalculatingSuspended)
            {
                ////Don't refresh any cells.
                return;
            }

            GridModel saveGrid = this.grid;
            this.grid = grd;

            bool b = this.grid.IgnoreReadOnly;
            this.grid.IgnoreReadOnly = true;
            range = range.ExpandRange(1, 1, this.grid.RowCount, this.grid.ColCount);

            if (useBeginUpdate)
            {
                this.grid.BeginUpdate();
            }

            GridData data = this.grid.Model.Data;

            if (!isCellsprocessed)
            {
                for (int r = range.Top; r <= range.Bottom; r++)
                {
                    for (int c = range.Left; c <= range.Right; c++)
                    {
                        
                        if (data[r, c] == null && !UsesVirtualDataSource)
                        {
                            continue;
                        }

                        string cellAddress = GridRangeInfo.GetAlphaLabel(c) + r.ToString();

                        GridStyleInfo style = UsesVirtualDataSource
                            ? this.grid[r, c]
                            : new GridStyleInfo(new GridStyleInfoIdentity(this.grid.Model.VolatileData, r, c, true), data[r, c]);

                        if (style.Store != null && style.CellModel is GridFormulaCellModel)
                        {
                            string s = style.Text;
                            GetFormulaText(ref s);

                            bool usesFormulaChar = this.formulaChar != (char)0 && s.Length > 0 && s[0] == this.formulaChar;
                            if (this.formulaChar == (char)0 || usesFormulaChar)
                            {
                                if (usesFormulaChar)
                                {
                                    s = s.Substring(1); ////strip it off
                                }

                                ////Only force a parse if Formula not in Tag.
                                string formula = (style.FormulaTag != null && !forceParsing) ? style.FormulaTag.Formula : string.Empty;

                                style.FormulaTag = null;

                                if (forceCalculations)
                                {
                                    this.cell = GridRangeInfo.GetAlphaLabel(c) + r.ToString();

                                    if (formula.Length == 0 && s.Length > 0)
                                    {
                                        if (catchExceptionsInRecalculateRange)
                                        {
                                            try
                                            {
                                                formula = this.Parse(s);
                                            }
                                            catch (Exception)
                                            {
                                                formula = "#REF!";
                                            }
                                        }
                                        else
                                        {
                                            formula = this.Parse(s);
                                        }
                                    }

                                    string text = "";
                                    if (catchExceptionsInRecalculateRange)
                                    {
                                        if (formula == "#REF!")
                                        {
                                            text = "#REF!";
                                        }
                                        else
                                        {
                                            if (formula.Length > 0)
                                            {
                                                bool save = ExceptionManager.PassThroughExceptions;
                                                ExceptionManager.PassThroughExceptions = true;
                                                try
                                                {
                                                    text = this.ComputedValue(formula);
                                                }
                                                catch (Exception)
                                                {
                                                    text = "#REF!";
                                                }
                                                finally
                                                {
                                                    ExceptionManager.PassThroughExceptions = save;
                                                }
                                            }
                                        }
                                    }
                                    else
                                    {
                                        text = this.ComputedValue(formula);
                                    }
                                    style.FormulaTag = new GridFormulaTag(formula, text, r, c);
                                }

                                if (!UsesVirtualDataSource)
                                {
                                    data[r, c] = (GridStyleInfoStore)style.Store;
                                }
                            }
                        }
                        if (r == range.Bottom && c == range.Right)
                            isCellsprocessed = true;
                    }
                }
            }

            this.grid.IgnoreReadOnly = b;

            if (useBeginUpdate)
            {
                this.grid.EndUpdate();
            }

            ////Now draw the range so formulas are recomputed.
            this.grid.InvalidateRange(range, GridRangeOptions.None);
            this.grid = saveGrid;
        }

        /// <summary>
        /// updates the value to perform calcuation for range of cells in RecalculateRange function
        /// </summary>
        /// <param name="sender">grid model</param>
        void grid_CellsChanged(object sender, GridCellsChangedEventArgs e)
        {
            isCellsprocessed = false;
        }

        private bool catchExceptionsInRecalculateRange = false;

        /// <summary>
        /// Gets or sets whether further calculations on a formula will be attempted in the RecalculateRange method if there is an exception 
        /// raised during the parsing or calculating of the formula.
        /// </summary>
        /// <remarks>
        /// The default value is false for backward compatibilty purposes. If you set this property to true, then during calls to RecalculateRange with
        /// forceCalculations = true, if an exception is raised during parsing or computing, the cell will display "#REF!". This property only has 
        /// effect if you call RecalculateRange with the forceCalculations parameter set to true.
        /// </remarks>
        public bool CatchExceptionsInRecalculateRange
        {
            get { return catchExceptionsInRecalculateRange; }
            set { catchExceptionsInRecalculateRange = value; }
        }

        #endregion

        #region Error Messages

        /// <summary>
        /// String array that holds the strings used in error messages within the Formula Engine.
        /// </summary>
        /// <remarks>If you want to change the error messages displayed within the Formula Engine,
        /// you can set the new strings into the appropriate position in the FormulaErrorStrings 
        /// array. You should assign your new
        /// strings to the corresponding positions. 
        /// </remarks>
        /// <example>Here is the code that shows position of each string in FormulaErrorStrings:
        /// <code lang="C#">
        ///        public string[] FormulaErrorStrings = new string[]
        ///        {
        ///            "binary operators cannot start an expression",    //0
        ///            "cannot parse",                                    //1
        ///            "bad library",                                    //2
        ///            "invalid char in front of",                        //3
        ///            "number contains 2 decimal points",                //4
        ///            "expression cannot end with an operator",        //5
        ///            "invalid characters following an operator",        //6
        ///            "invalid character in number",                    //7
        ///            "mismatched parentheses",                        //8
        ///            "unknown formula name",                            //9
        ///            "requires a single argument",                    //10
        ///            "requires 3 arguments",                            //11
        ///            "invalid Math argument",                        //12
        ///            "requires 2 arguments",                            //13
        ///            "bad index",                                    //14
        ///            "too complex",                                    //15
        ///            "circular reference: ",                            //16
        ///            "missing formula",                                //17
        ///            "improper formula",                                //18
        ///            "invalid expression",                            //19
        ///            "cell empty"                                    //20
        ///            "bad formula",                                    //21
        ///            "empty expression",                                //22
        ///            "Virtual Mode required - set UsesVirtualDataSource", //23
        ///            "mismatched string quotes",                     //24
        ///            "wrong number of arguments",                    //25
        ///            "invalid arguments",                            //26
        ///            "iterations do not converge",                   //27
        ///            "Control named '{0}' is already registered",    //28
        ///         "Calculation overflow",                            //29
        ///         "missing operand"                                //30
        ///         "unknown sheet"                                //30
        ///        };
        /// </code>
        /// </example>
        public string[] FormulaErrorStrings = new string[]
            {
                "binary operators cannot start an expression",    ////0
                "cannot parse",                                    ////1
                "bad library",                                    ////2
                "invalid char in front of",                        ////3
                "number contains 2 decimal points",                ////4
                "expression cannot end with an operator",        ////5
                "invalid characters following an operator",        ////6
                "invalid character in number",                    ////7
                "mismatched parentheses",                        ////8
                "unknown formula name",                            ////9
                "requires a single argument",                    ////10
                "requires 3 arguments",                            ////11
                "invalid Math argument",                        ////12
                "requires 2 arguments",                            ////13
                "bad index",                                    ////14
                "too complex",                                    ////15
                "circular reference: ",                            ////16
                "missing formula",                                ////17
                "improper formula",                                ////18
                "invalid expression",                            ////19
                "cell empty",                                    ////20
                "bad formula",                                    ////21
                "empty expression",                                ////22
                "Virtual Mode required - set UsesVirtualDataSource", ////23
                "mismatched string quotes",                             ////24
                "wrong number of arguments",                    ////25
                "invalid arguments",                                ////26
                "iterations do not converge",                        ////27
                "Control named '{0}' is already registered",          ////28
                "Calculation overflow",                                ////29
                "missing operand",                                ////30
                "unknown sheet"                                 ////31
            };

        internal int operators_cannot_start_an_expression = 0;
        internal int cannot_parse = 1;
        internal int bad_library = 2;
        internal int invalid_char_in_front_of = 3;
        internal int number_contains_2_decimal_points = 4;
        internal int expression_cannot_end_with_an_operator = 5;
        internal int invalid_characters_following_an_operator = 6;
        internal int invalid_char_in_number = 7;
        internal int mismatched_parentheses = 8;
        internal int unknown_formula_name = 9;
        internal int requires_a_single_argument = 10;
        internal int requires_3_args = 11;
        internal int invalid_Math_argument = 12;
        internal int requires_2_args = 13;
        internal int bad_index = 14;
        internal int too_complex = 15;
        internal int circular_reference_ = 16;
        internal int missing_formula = 17;
        internal int improper_formula = 18;
        internal int invalid_expression = 19;
        internal int cell_empty = 20;
        internal int bad_formula = 21;
        internal int empty_expression = 22;
        internal int virtual_mode_required = 23;
        internal int mismatched_tics = 24;
        internal int wrong_number_arguments = 25;
        internal int invalid_arguments = 26;
        internal int iterations_dont_converge = 27;
        internal int already_registered = 28;
        internal int calculation_overflow = 29;
        internal int missing_operand = 30;
        internal int unknown_sheet = 31;

        #endregion

        #region Parsing Code

        #region String Support

        ////handle special case #N/A
        private char UNIQUESLASHMARKER = (char)129;

        private char UNIQUESTRINGMARKER = (char)127;
        private Hashtable SaveStrings(ref string text)
        {
            ////handle #N/A problem by replacing it for parsing...
            text = text.Replace("#N/A", "#N" + UNIQUESLASHMARKER + "A");

            Hashtable strings = null;
            string TICs2 = TIC + TIC;
            int id = 0;
            int i = -1;
            if ((i = text.IndexOf(TIC)) > -1)
            {
                while (i > -1 && i < text.Length)
                {
                    if (strings == null)
                    {
                        strings = new Hashtable();
                    }

                    int j = (i + 1) < text.Length ? text.IndexOf(TIC, i + 1) : -1;
                    if (j > -1)
                    {
                        string key = TIC + UNIQUESTRINGMARKER + id.ToString() + TIC;
                        if (j < text.Length - 2 && text[j + 1] == TIC[0])
                        {
                            j = text.IndexOf(TIC, j + 2);
                            if (j == -1)
                            {
                                throw new ArgumentException(FormulaErrorStrings[mismatched_tics]);
                            }
                        }

                        string s = text.Substring(i, j - i + 1);
                        strings.Add(key, s);
                        s = s.Replace(TICs2, TIC);
                        id++;
                        text = text.Substring(0, i) + key + text.Substring(j + 1);
                        i = i + key.Length;
                        if (i < text.Length)
                        {
                            i = text.IndexOf(TIC, i);
                        }
                    }
                    else
                    {
                        throw new ArgumentException(FormulaErrorStrings[mismatched_tics]);
                    }
                }
            }

            return strings;
        }

        private void SetStrings(ref string retValue, Hashtable strings)
        {
            ////handle #N/A problem by replacing it for parsing...
            retValue = retValue.Replace("#N" + UNIQUESLASHMARKER + "A", "#N/A");

            foreach (string s in strings.Keys)
            {
                retValue = retValue.Replace(s, (string)strings[s]);
            }
        }
        #endregion

        static GridFormulaEngine()
        {
            ParseDecimalSeparator = '.';
            ParseArgumentSeparator = ',';
        }

        /// <summary>
        /// Character recognized by the parsing engine as decimal separator for numbers.
        /// </summary>
        [ThreadStaticAttribute]
        public static char ParseDecimalSeparator = '.';

        /// <summary>
        /// Character recognized by the parsing code as the delimiter for arguments in a named formula's argument list.
        /// </summary>
        [ThreadStaticAttribute]
        public static char ParseArgumentSeparator = ',';

        private ArrayList errorStrings;

        /// <summary>
        /// Checks if given string holds a valid formula.
        /// </summary>
        /// <param name="text">String to be tested.</param>
        /// <param name="parsedFormula">Tokenized string holding holding valid parsed formula.</param>
        /// <param name="errorMessage">Error message from the grid if string is invalid.</param>
        /// <param name="computedValue">Computed value from parsed formula.</param>
        /// <returns>True if string holds a valid formula, false otherwise.</returns>
        public bool IsFormulaValid(string text, out string parsedFormula, out string errorMessage, out string computedValue)
        {
            string saveCell = this.cell;
            this.cell = string.Empty;

            errorMessage = string.Empty;
            parsedFormula = string.Empty;
            computedValue = string.Empty;
            bool save = this.lockDependencies;
            try
            {
                this.lockDependencies = true;
                parsedFormula = Parse(text);
                computedValue = this.ComputedValue(parsedFormula);
                if (errorStrings == null)
                {
                    errorStrings = new ArrayList();
                    this.errorStrings.AddRange(this.FormulaErrorStrings);
                }

                int index = errorStrings.IndexOf(computedValue);
                if (index > -1)
                {
                    errorMessage = errorStrings[index].ToString();
                    computedValue = string.Empty;
                }

                return index == -1;
            }
            catch (Exception ex)
            {
                errorMessage = ex.Message;
                return false;
            }
            finally
            {
                this.lockDependencies = save;
                this.cell = saveCell;
            }
        }

        /// <summary>
        /// Checks if the given string holds a valid formula.
        /// </summary>
        /// <param name="text">String to be tested.</param>
        /// <returns>True if string holds a valid formula, false otherwise.</returns>
        public bool IsFormulaValid(string text)
        {
            string s, p, c;
            return IsFormulaValid(text, out p, out s, out c);
        }

        private string unparsedText = string.Empty;
        public string unParsedKey = string.Empty;
        public string parsedKeyValue = string.Empty;
        /// <summary>
        /// Parses a formula string into a tokenized string.
        /// </summary>
        /// <param name="text">The string to be parsed.</param>
        /// <returns>The parsed string.</returns>
        /// <remarks> 
        ///  This method accepts a string that holds a formula, like =Sum(A1:B5), 
        ///  and translates this string into a tokenized expression that can
        ///  be computed using the ComputedValue method. Before 
        ///  calling the method, you should set FormulaContextCell
        ///  to properly reflect which cell owns this formula.
        ///  The return value, which is 
        ///  the tokenized string, is referred to as a parsed formula string.
        /// </remarks>
        public string Parse(string text)
        {
            ////Console.WriteLine("Parse->[{0}]", text);
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(text, this);
            }
#else
            ;
#endif
            GetFormulaText(ref text);

            bool usesFormulaChar = this.formulaChar != (char)0 && text.Length > 0 && text[0] == this.formulaChar;
            if (usesFormulaChar)
            {
                text = text.Substring(1); ////strip it off
            }

            unparsedText = text;

            if (GridUtil.IsEmpty(text))
            {
                return text;
            }

            ////Need to handle % sign as in =1%*F1
            ////if(text.IndexOf("%") > 0)
            ////    Console.WriteLine("IndexOf");
            ////text = text.Replace("%", string.Empty);

            ////Make braces strings...
            text = text.Replace(BRACELEFT, TIC);
            text = text.Replace(BRACERIGHT, TIC);

            ////Save strings...
            Hashtable formulaStrings = SaveStrings(ref text);

            if (formulaChar != (char)0 && formulaChar == text[0])
            {
                text = text.Substring(1);
            }

            int i = 0;

            text = text.ToUpper(CultureInfo.InvariantCulture);
            if (!SupportBlanksInSheetNames)
            {
                text = text.Replace(" ", string.Empty);
            }

            ////Special check for single namedrange.
            if (NamedRanges.ContainsKey(text))
            {
                SetNamedRangeDependency(text, cell);
                text = ((string)NamedRanges[text]).ToUpper();
            }
            else
            {
                System.Text.StringBuilder sb = new System.Text.StringBuilder(text);

                ////Search in order of size, match must not have a letter or digit immediately
                ////preceding or following.
                foreach (string key in this.NamedRangesSized)
                {
                    int newLoc = -1;
                    int oldLoc = 0;
                    string s = string.Empty;
                    while (oldLoc < sb.Length && (newLoc = sb.ToString().IndexOf(key, oldLoc)) > -1)
                    {
                        int len = key.Length;
                        if ((newLoc == 0 || !char.IsLetterOrDigit(sb[newLoc - 1]))
                            && (newLoc + len == sb.Length || !char.IsLetterOrDigit(sb[newLoc + len])))
                        {
                            ////only do this on first replacement
                            if (s.Length == 0)
                            {
                                SetNamedRangeDependency(key, cell);
                                ////upper and enclose in parens
                                unParsedKey = this.NamedRanges[key].ToString();
                                parsedKeyValue = this.NamedRangeValues[key].ToString();
                                s = "(" + this.NamedRanges[key].ToString().ToUpper() + ")";
                               //   PutTokensForSheets(ref s); //this will be done later
                            }

                            sb.Replace(key, s, newLoc, len);
                            len = s.Length;
                        }

                        oldLoc = newLoc + len;
                    }
                }

                text = sb.ToString();
            }

            ////Replace sheet references with tokens.
            GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
            if (family.SheetNameToGridModel != null && family.SheetNameToGridModel.Count > 0)
            {
                try
                {
                    text = text.Replace(sheetToken, SHEETTOKENCHECK);
                    PutTokensForSheets(ref text, SHEETTOKENCHECK);
                    if (text.IndexOf(SHEETTOKENCHECK) > -1)
                    {
                        if (MarkUnknownSheetRanges)
                        {
                            return "\"#REF!\"";
                        }
                        text = text.Replace(SHEETTOKENCHECK, sheetToken);
                        throw new ArgumentException(FormulaErrorStrings[unknown_sheet]);
                    }
                }
                catch (Exception ex)
                {
                    TraceUtil.TraceExceptionCatched(ex);
                    if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                    {
                        throw ex;
                    }

                    throw ex; ////rethrow so the caller can handle the bad parse on formula name
                    ////return ex.Message;
                }
            }

            if (!CheckSheetTokens(text))
            {
                throw new ArgumentException(FormulaErrorStrings[unknown_sheet]);
            }

            if (SupportBlanksInSheetNames)
            {
                text = text.Replace(" ", string.Empty);
            }

            try
            {
                MarkLibraryFormulas(ref text);
            }
            catch (Exception ex)
            {
                TraceUtil.TraceExceptionCatched(ex);
                if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                {
                    throw ex;
                }

                throw ex; ////rethrow so the caller can handle the bad parse on formula name
                ////return ex.Message;
            }

            ////Special case parsing done in function definition.

            ////Look for inner matching & parse pieces without parens with ParseSimple.
            while ((i = text.IndexOf(')')) > -1)
            {
                int k = text.Substring(0, i).LastIndexOf('(');
                if (k == -1)
                {
                    throw new ArgumentException(FormulaErrorStrings[mismatched_parentheses]);
                }

                if (k == i - 1)
                {
                    throw new ArgumentException(FormulaErrorStrings[empty_expression]);
                }

                string s = text.Substring(k + 1, i - k - 1);
                text = text.Substring(0, k) + ParseSimple(s) + text.Substring(i + 1);
            }

            ////All parens should be removed.
            if (text.IndexOf('(') > -1)
            {
                throw new ArgumentException(FormulaErrorStrings[mismatched_parentheses]);
            }

            string retValue = ParseSimple(text);

            if (formulaStrings != null && formulaStrings.Count > 0)
            {
                SetStrings(ref retValue, formulaStrings);
            }
            double cellvalue;
            if (!this.IsCellReference(retValue) && !double.TryParse(retValue,out cellvalue) && retValue.Equals(unparsedText))
            {
                return "#NAME?";
            }
            return retValue;
        }

        private bool markUnknownSheetRanges = false;

        /// <summary>
        /// Gets or sets whether formulas involving unknown sheet names are marked with #REF! text.
        /// </summary>
        /// <remarks>The default value is false. When this property is false, the engine throws an 
        /// unknown sheet name error when a formula uses a sheet name that has not been registered
        /// with the engine. If you want to avoid this exception being thrown, set this property to 
        /// true. In this case, the cell will display #REF! and be ignored by the engine.
        /// </remarks>
        public bool MarkUnknownSheetRanges
        {
            get { return markUnknownSheetRanges; }
            set { markUnknownSheetRanges = value; }
        }
        private bool CheckSheetTokens(string text)
        {
            bool b = true;
            int i = 0;
            while (b && i < text.Length &&
                ((i = text.IndexOf(sheetToken, i)) > -1))
            {
                int start = i;
                i++;
                while (i < text.Length && text[i] != sheetToken)
                {
                    i++;
                }

                if (start + 1 >= i || i == text.Length)
                {
                    b = false;
                }
                else
                {
                    GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                    if (!family.TokenToGridModel.ContainsKey(text.Substring(start, i - start + 1)))
                    {
                        b = false;
                    }
                }

                i++;
            }

            return b;
        }

        ////Operator Parsing
        ////CHAR_xxxx used in formulas; swapped for TOKEN_xxxx in parsed formula.
        ////TOKEN_xxxx is lowercase alpha char.
        ////STRING_xxxx identifies the operators that require 2 characters; get mapped to CHAR_xxxx to fit into single char algorithm.
        ////Lowercase letters used: abdefghjklmnopqrstuvwx.
        //// z is used as a temp swap character...
        private const char TOKEN_multiply = 'm';
        private const char CHAR_multiply = '*';
        private const char TOKEN_divide = 'd';
        private const char CHAR_divide = '/';
        private const char TOKEN_add = 'a';
        private const char CHAR_add = '+';
        private const char TOKEN_subtract = 's';
        private const char CHAR_subtract = '-';

        private const char TOKEN_less = 'l';
        private const char CHAR_less = '<';
        private const char TOKEN_greater = 'g';
        private const char CHAR_greater = '>';
        private const char TOKEN_equal = 'e';
        private const char CHAR_equal = '=';

        private const char TOKEN_lesseq = 'k';
        private const char CHAR_lesseq = 'f';
        private const string STRING_lesseq = "<=";
        private const char TOKEN_greatereq = 'j';
        private const char CHAR_greatereq = 'h';
        private const string STRING_greatereq = ">=";
        private const char TOKEN_noequal = 'o';
        private const char CHAR_noequal = 'p';
        private const string STRING_noequal = "<>";

        private const string STRING_fixedreference = "$";
        private const string STRING_empty = "";

        private const char TOKEN_and = 'c';
        private char CHAR_and = 'i';
        private string STRING_and = "&"; ////"AND";
        private const char TOKEN_pow = (char)126; ////'k'; //power
        private char CHAR_pow = 'w';
        private string STRING_pow = "^"; ////"OR";

        private char CHAR_EP = 'x';
        private char CHAR_EM = 'r';
        private const char TOKEN_EP = 't';
        private const char TOKEN_EM = 'v';
        private string STRING_EP = "E+";
        private string STRING_E = "E";
        private string STRING_EM = "E-";

        private string TIC = "\""; ////"'"; ////used to mark strings
        private string BRACELEFT = "{";
        private string BRACERIGHT = "}";
        private char[] BRACEDELIMETER = new char[] { ',' };

        private char LEFTBRACKET = (char)130; ////'[';
        private char RIGHTBRACKET = (char)131; ////']';

        private const char LEADMINUS = (char)132;
        private const char ZMARKER = 'z';
        private const char BMARKER = 'b';

        private const char SHEETTOKENCHECK = (char)133; ////used to make sure sheets are properly checked

        private string IFMarker = "qIF" + (char)130; ////'['; ////(char)130;

        private char[] tokens;

        private string ParseSimple(string text)
        {
            ////strip leading plus
            if (text.Length > 0 && text[0] == '+')
            {
                text = text.Substring(1);
            }

            if (text.Trim().Length == 0 || text == "#N/A")
            {
                return text;
            }

            ////TraceUtil.TraceCurrentMethodInfoIf(Switches.FormulaCell.TraceVerbose, text, this);

            text = HandleEmbeddedEs(text);

            // To check whether cell has values like "ABC+000" to support for bind operator as string
            if (text.StartsWith("\"")&&text.EndsWith("\""))
            {
                return text;
            }

            System.Text.StringBuilder sb = new System.Text.StringBuilder(text);
            bool process = true;
            while (process)
            {
                sb.Replace("--", "+");
                sb.Replace("++", "+");
                ////Mark unary minus with u-token.
                sb.Replace(",-", ",u").Replace(LEFTBRACKET + "-", LEFTBRACKET + "u").Replace("=-", "=u").Replace(">-", ">u").Replace("<-", "<u").Replace("/-", "/u").Replace("*-", "*u").Replace("+-", "+u").Replace("^-", "^u");
                ////Get rid of leading pluses.
                sb.Replace(",+", ",").Replace(LEFTBRACKET + "+", LEFTBRACKET.ToString()).Replace("=+", "=").Replace(">+", ">").Replace("<+", "<").Replace("/+", "/").Replace("*+", "*").Replace("^+", "^");
                if (sb.Length > 0 && sb[0] == '+')
                {
                    sb.Remove(0, 1);
                }

                process = text != sb.ToString();
                text = sb.ToString();
            }

            text = sb.Replace(STRING_lesseq, CHAR_lesseq.ToString())
                .Replace(STRING_greatereq, CHAR_greatereq.ToString())
                .Replace(STRING_noequal, CHAR_noequal.ToString())
                .Replace(STRING_fixedreference, STRING_empty)
                .Replace(STRING_pow, CHAR_pow.ToString())
                .Replace(STRING_and, CHAR_and.ToString())
                .ToString();

            bool needToContinue = true;
            ////Doing things sequentially imposes computation hierarchy for 6 levels.
            //// 0. E+ E- //handles exponential notation like 1.2e-1 or 1.2e+1 to 1.2e1
            //// 1. ^ exponentiation
            //// 2. * /
            //// 3. + -
            //// 4. < > = <= >= <>
            //// 5. & concatenation
            //// in each level parsing done left to right
            text = ParseSimple(text, new char[] { TOKEN_EP, TOKEN_EM }, new char[] { CHAR_EP, CHAR_EM }, ref needToContinue);
            if (needToContinue)
            {
                text = ParseSimple(text, new char[] { TOKEN_pow }, new char[] { CHAR_pow }, ref needToContinue);
            }

            if (needToContinue)
            {
                text = ParseSimple(text, new char[] { TOKEN_multiply, TOKEN_divide }, new char[] { CHAR_multiply, CHAR_divide }, ref needToContinue);
            }

            if (needToContinue)
            {
                text = ParseSimple(text, new char[] { TOKEN_add, TOKEN_subtract }, new char[] { CHAR_add, CHAR_subtract }, ref needToContinue);
            }

            if (needToContinue)
            {
                text = ParseSimple(text, new char[] { TOKEN_less, TOKEN_greater, TOKEN_equal, TOKEN_lesseq, TOKEN_greatereq, TOKEN_noequal }, new char[] { CHAR_less, CHAR_greater, CHAR_equal, CHAR_lesseq, CHAR_greatereq, CHAR_noequal }, ref needToContinue);
            }

            if (needToContinue)
            {
                text = ParseSimple(text, new char[] { TOKEN_and }, new char[] { CHAR_and }, ref needToContinue);
            }

            return text;
        }

        private bool EInsideBMARKERs(int j, string text)
        {
            int left = text.Substring(0, j).LastIndexOf(BMARKER);
            int right = text.IndexOf(BMARKER, j);
            return left > -1 && j > left && j < right;
        }

        private string HandleEmbeddedEs(string text)
        {
            int j = 0;
            while (j > -1 && ((j = text.IndexOf(STRING_EP, j)) > -1) && !EInsideBMARKERs(j, text))
            {
                int left = j;
                while (left > 0 && (char.IsDigit(text[left - 1]) || text[left - 1] == ParseDecimalSeparator))
                {
                    left--;
                }

                if (left != j && (left == 0 || !char.IsUpper(text[left - 1])))
                {
                    int right = j + STRING_EP.Length;
                    while (right < text.Length && char.IsDigit(text[right]))
                    {
                        right++;
                    }

                    if (right != j + STRING_EP.Length)
                    {
                        text = text.Substring(0, j) + CHAR_EP + text.Substring(j + STRING_EP.Length);
                    }
                }

                j += 1;
            }

            j = 0;
            while (j > -1 && ((j = text.IndexOf(STRING_EM, j)) > -1) && !EInsideBMARKERs(j, text))
            {
                int left = j;
                while (left > 0 && (char.IsDigit(text[left - 1]) || text[left - 1] == ParseDecimalSeparator))
                {
                    left--;
                }

                if (left != j && (left == 0 || !char.IsUpper(text[left - 1])))
                {
                    int right = j + STRING_EM.Length;
                    while (right < text.Length && char.IsDigit(text[right]))
                    {
                        right++;
                    }

                    if (right != j + STRING_EM.Length)
                    {
                        text = text.Substring(0, j) + CHAR_EM + text.Substring(j + STRING_EM.Length);
                    }
                }

                j += 1;
            }

            j = 0;
            ////only look for single e's if expression has not been parsed
            while (j > -1 && ((j = text.IndexOf(STRING_E, j)) > -1)
                && !EInsideBMARKERs(j, text))
            {
                ////text[0] != BMARKER
                int left = j;
                while (left > 0 && (char.IsDigit(text[left - 1]) || text[left - 1] == ParseDecimalSeparator))
                {
                    left--;
                }

                if (left != j && (left == 0 || !char.IsUpper(text[left - 1])))
                {
                    int right = j + STRING_E.Length;
                    while (right < text.Length && char.IsDigit(text[right]))
                    {
                        right++;
                    }

                    if (right != j + STRING_E.Length && (left == -1 || !char.IsUpper(text[left])))
                    {
                        text = text.Substring(0, j) + CHAR_EP + text.Substring(j + STRING_E.Length);
                    }
                }

                j += 1;
            }

            return text;
        }

        private string ZapBlocks(string text)
        {
            if (text.IndexOf(BMARKER) > -1)
            {
                int bracketLevel = 0;
                for (int i = text.Length - 1; i >= 0; --i)
                {
                    if (text[i] == RIGHTBRACKET)
                    {
                        bracketLevel--;
                    }
                    else if (text[i] == LEFTBRACKET)
                    {
                        bracketLevel++;
                    }
                    else if (text[i] == BMARKER && bracketLevel == 0)
                    {
                        text = text.Remove(i, 1);
                    }
                }
            }

            return text;
        }

        private int FindNonQB(string text)
        {
            int ret = -1;
            if (text.IndexOf(BMARKER) > -1)
            {
                int bracketLevel = 0;
                for (int i = 0; i < text.Length; ++i)
                {
                    if (text[i] == RIGHTBRACKET)
                    {
                        bracketLevel--;
                    }
                    else if (text[i] == LEFTBRACKET)
                    {
                        bracketLevel++;
                    }
                    else if (text[i] == BMARKER && bracketLevel == 0)
                    {
                        ret = i;
                        break;
                    }
                }
            }

            return ret;
        }

        private string ParseSimple(string text, char[] markers, char[] operators, ref bool needToContinue)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(text, markers, operators);
            }
#else
            ;
#endif
            int i;

            ////Op is string containing CHAR_xxxx's for each operator in this level.
            string op = string.Empty;
            foreach (char c in operators)
            {
                op = op + c;
            }

            if (text.Length > 0 && text[0] == '-')
            {
                ////Leading unary minus.

                text = text.Substring(1).Replace('-', LEADMINUS);
                text = "0-" + text;
                text = ParseSimple(text, new char[] { TOKEN_subtract }, new char[] { CHAR_subtract }, ref needToContinue);
                text = text.Replace(LEADMINUS, '-');
            }

            ////Check for simple namedrange.
            if (NamedRanges.ContainsKey(text))
            {
                SetNamedRangeDependency(text, cell);
                text = this.NamedRanges[text].ToString().ToUpper();
            }

            if (text.IndexOfAny(operators) > -1)
            {
                while ((i = text.IndexOfAny(operators)) > -1)
                {
                    string left = string.Empty;
                    string right = string.Empty;

                    int leftIndex = 0;
                    int rightIndex = 0;

                    if (i < 1 && text[i] != '-')
                    {
                        throw new ArgumentException(FormulaErrorStrings[operators_cannot_start_an_expression]);
                    }

                    ////Process left argument.
                    int j = i - 1;

                    if (i == 0 && text[i] == '-')
                    {
                        ////Unary minus - block and continue.
                        text = "bnu" + text.Substring(1) + BMARKER;
                        continue;
                    }
                    else if (text[j] == TIC[0])
                    {
                        ////string
                        int k = text.Substring(0, j - 1).LastIndexOf(TIC);
                        if (k < 0)
                        {
                            throw new ArgumentException(FormulaErrorStrings[cannot_parse]);
                        }

                        ////left = text.Substring(k+1, j-k-1);
                        ////leftIndex = k + 1;
                        left = text.Substring(k, j - k + 1); ////keep the tics
                        leftIndex = k;
                    }
                    else if (text[j] == BMARKER)
                    {
                        ////block of already parsed code

                        ////int k = text.Substring(0, j-1).LastIndexOf(BMARKER);
                        int k = FindLastNonQB(text.Substring(0, j - 1));
                        if (k < 0)
                        {
                            throw new ArgumentException(FormulaErrorStrings[cannot_parse]);
                        }

                        left = text.Substring(k + 1, j - k - 1);
                        leftIndex = k + 1;
                    }
                    else if (text[j] == RIGHTBRACKET)
                    {
                        ////library member
                        int bracketCount = 0;
                        int k = j - 1;
                        while (k > 0 && (text[k] != 'q' || bracketCount != 0))
                        {
                            if (text[k] == 'q')
                            {
                                bracketCount--;
                            }
                            else if (text[k] == RIGHTBRACKET)
                            {
                                bracketCount++;
                            }

                            k--;
                        }

                        ////int k = text.Substring(0, j-1).LastIndexOf('q');
                        if (k < 0)
                        {
                            throw new ArgumentException(FormulaErrorStrings[bad_library]);
                        }

                        left = text.Substring(k, j - k + 1);
                        left = left.Replace(BMARKER, ZMARKER); ////marked computed formulas with z instead of b
                        leftIndex = k;
                    }
                    else if (!char.IsDigit(text[j]) && text[j] != '%')
                    {
                        ////number
                        ////Check for namedrange.
                        while (j >= 0 && char.IsUpper(text[j]))
                        {
                            j--;
                        }

                        left = text.Substring(j + 1, i - j - 1); ////'n' for number
                        leftIndex = j + 1;
                        if (this.NamedRanges.ContainsKey(left))
                        {
                            left = Parse((string)this.NamedRanges[left]);
                        }
                        else if (left == TRUEVALUESTR)
                        {
                            left = 'n' + TRUEVALUESTR;
                            ////left = 'n' + TRUEVALUE.ToString();
                        }
                        else if (left == FALSEVALUESTR)
                        {
                            left = 'n' + FALSEVALUESTR;
                            ////left = 'n' + FALSEVALUE.ToString();
                        }
                        else
                        {
                            if (MarkUnknownSheetRanges)
                            {
                                return "\"#REF!\"";
                            }
                            throw new ArgumentException(FormulaErrorStrings[invalid_char_in_front_of] + " " + text[i]);
                        }
                    }
                    else
                    {
                        bool period = false;
                        bool percent = false;

                        while (j > -1 && (char.IsDigit(text[j]) || (!period && text[j] == ParseDecimalSeparator)
                            || (!percent && text[j] == '%')))
                        {
                            if (text[j] == ParseDecimalSeparator)
                            {
                                period = true;
                            }
                            else if (text[j] == '%')
                            {
                                percent = true;
                            }

                            j = j - 1;
                        }

                        if (j > -1 && period && text[j] == ParseDecimalSeparator)
                        {
                            throw new ArgumentException(FormulaErrorStrings[number_contains_2_decimal_points]);
                        }

                        if (j > -1 && text[j] == 'u')
                        {
                            j--;
                        }

                        j = j + 1;

                        if (j == 0 || (j > 0 && !char.IsUpper(text[j - 1]))
                            || (j == 1 && text[0] == 'u'))
                        {
                            if (j == 1 && text[0] == 'u')
                            {
                                j--;
                            }

                            left = 'n' + text.Substring(j, i - j); ////'n' for number
                            leftIndex = j;
                        }
                        else
                        {
                            ////We have a cell reference.
                            j = j - 1;
                            while (j > -1 && char.IsUpper(text[j]))
                            {
                                j = j - 1;
                            }

                            if (j > -1 && text[j] == sheetToken)
                            {
                                j = j - 1;
                                while (j > -1 && text[j] != sheetToken)
                                {
                                    j = j - 1;
                                }

                                if (j > -1 && text[j] == sheetToken)
                                {
                                    j = j - 1;
                                }
                            }

                            j = j + 1;
                            left = text.Substring(j, i - j);

                            ////Handle 0 as relative row reference.
                            if (currentRowNotationEnabled && left.Length > 1 && left[left.Length - 1] == '0' &&
                                !char.IsDigit(left[left.Length - 2]))
                            {
                                left = left.Substring(0, left.Length - 1) + RowIndex(cell);
                            }

                            leftIndex = j;
                        }
                    }

                    ////Process right argument.
                    if (i == text.Length - 1)
                    {
                        throw new ArgumentException(FormulaErrorStrings[expression_cannot_end_with_an_operator]);
                    }
                    else
                    {
                        j = i + 1;
                        bool isU = text[j] == 'u';
                        if (isU)
                        {
                            j++;  ////ship for now, but add back later
                        }

                        ////string
                        if (text[j] == TIC[0])
                        {
                            int k = text.Substring(j + 1).IndexOf(TIC);
                            if (k < 0)
                            {
                                throw new ArgumentException(FormulaErrorStrings[cannot_parse]);
                            }

                            right = text.Substring(j, k + 2);
                            rightIndex = k + j + 2;
                        }
                        else if (text[j] == BMARKER)
                        {
                            ////block of already parsed code

                            ////int k = text.Substring(j+1).IndexOf(BMARKER);
                            int k = FindNonQB(text.Substring(j + 1));
                            if (k < 0)
                            {
                                throw new ArgumentException(FormulaErrorStrings[cannot_parse]);
                            }

                            right = text.Substring(j + 1, k);
                            if (isU)
                            {
                                right = right + "nu1m"; ////multiply quantity by -1...
                            }

                            rightIndex = k + j + 2;
                            ////                        else if(text[j] == TIC[0]) //string
                            ////                        {
                            ////                            int k = text.Substring(j+1).IndexOf(TIC[0]);
                            ////                            if(k<0)
                            ////                                throw new ArgumentException(FormulaErrorStrings[cannot_parse]);
                            ////
                            ////                            right = text.Substring(j , k + 2);
                            ////                            rightIndex = k + j + 2;
                            ////                        }
                        }
                         else if (text[j] == 'q')
                        {
                            ////library
                            int bracketCount = 0;
                            int k = j + 1;
                            while (k < text.Length && (text[k] != RIGHTBRACKET || bracketCount != 0))
                            {
                                if (text[k] == RIGHTBRACKET)
                                {
                                    bracketCount++;
                                }
                                else if (text[k] == 'q')
                                {
                                    bracketCount--;
                                }

                                k++;
                            }

                            if (k == text.Length)
                            {
                                throw new ArgumentException(FormulaErrorStrings[cannot_parse]);
                            }

                            right = text.Substring(j, k - j + 1);
                            if (isU)
                            {
                                right = 'u' + right;
                            }

                            right = right.Replace(BMARKER, ZMARKER); ////marked computed formulas with z instead of b
                            rightIndex = k + 1;
                        }
                        else if (char.IsDigit(text[j]) || text[j] == ParseDecimalSeparator)
                        {
                            bool period = text[j] == ParseDecimalSeparator;
                            j = j + 1;
                            while (j < text.Length && (char.IsDigit(text[j]) || (!period && text[j] == ParseDecimalSeparator)))
                            {
                                if (text[j] == ParseDecimalSeparator)
                                {
                                    period = true;
                                }

                                j = j + 1;
                            }

                            if (j < text.Length && text[j] == '%')
                            {
                                j += 1;
                            }

                            if (period && j < text.Length && text[j] == ParseDecimalSeparator)
                            {
                                throw new ArgumentException(FormulaErrorStrings[number_contains_2_decimal_points]);
                            }

                            right = 'n' + text.Substring(i + 1, j - i - 1);
                            rightIndex = j;
                        }
                        else if (char.IsUpper(text[j]) || text[j] == sheetToken || text[j] == 'u')
                        {
                            if (text[j] == sheetToken)
                            {
                                j++;
                                while (j < text.Length && text[j] != sheetToken)
                                {
                                    j++;
                                }
                            }

                            j = j + 1;

                            int j0 = 0;
                            while (j < text.Length && char.IsUpper(text[j]))
                            {
                                j++;
                                j0++;
                            }

                            bool noCellReference = (j == text.Length) || !char.IsDigit(text[j]);
                            if (j0 > 4)
                            {
                                while (j < text.Length && (char.IsUpper(text[j]) || char.IsDigit(text[j])))
                                {
                                    j++;
                                }

                                noCellReference = true;
                            }

                            while (j < text.Length && char.IsDigit(text[j]))
                            {
                                j = j + 1;
                            }

                            j = j - 1;

                            right = text.Substring(i + 1, j - i);
                            if (isU)
                            {
                                right = 'u' + right;
                            }

                            if (!noCellReference)
                            {
                                ////Handle 0 as relative row reference.
                                if (currentRowNotationEnabled && right.Length > 1 && right[right.Length - 1] == '0' &&
                                    !char.IsDigit(right[right.Length - 2]))
                                {
                                    right = right.Substring(0, right.Length - 1) + RowIndex(cell);
                                }

                                UpdateDependencies(right); ////cb
                            }
                            else
                            {
                                if (NamedRanges.ContainsKey(right))
                                {
                                    SetNamedRangeDependency(right, cell);
                                    right = Parse((string)NamedRanges[right]);
                                }
                                else if (right == TRUEVALUESTR)
                                {
                                    ////right = 'n' + TRUEVALUE.ToString();
                                    right = 'n' + TRUEVALUESTR;
                                }
                                else if (right == FALSEVALUESTR)
                                {
                                    ////right = 'n' + FALSEVALUE.ToString();
                                    right = 'n' + FALSEVALUESTR;
                                }
                                else
                                {
                                    throw new ArgumentException(FormulaErrorStrings[invalid_characters_following_an_operator]);
                                }
                            }

                            rightIndex = j + 1;
                        }
                        else
                        {
                            throw new ArgumentException(FormulaErrorStrings[invalid_characters_following_an_operator]);
                        }
                    }

                    int p = op.IndexOf(text[i]);
                    ////string s = BMARKER + left.Replace("b", string.Empty) + right.Replace("b", string.Empty) + markers[p] + BMARKER;
                    string s = BMARKER + ZapBlocks(left) + ZapBlocks(right) + markers[p] + BMARKER;
                    s = s.Replace(ZMARKER, BMARKER); ////swap b markers back
                    if (leftIndex > 0)
                    {
                        s = text.Substring(0, leftIndex) + s;
                    }

                    if (rightIndex < text.Length)
                    {
                        s = s + text.Substring(rightIndex);
                    }

                    s = s.Replace("bb", "b");
                    ////s = s.Replace(TIC+TIC, TIC);
                    text = s;
                }
            }
            else
            {
                ////No operators  ..must be number, reference, or library method.

                ////Process left argument.
                int j = text.Length - 1;
                ////Block of already parsed code.
                if (text[j] == BMARKER)
                {
                    ////int k = text.Substring(0, j-1).LastIndexOf(BMARKER);
                    int k = FindLastNonQB(text.Substring(0, j - 1));
                    if (k < 0)
                    {
                        throw new ArgumentException(FormulaErrorStrings[cannot_parse]);
                    }
                }
                else if (text[j] == RIGHTBRACKET)
                {
                    ////library member
                    int bracketCount = 0;
                    int k = j - 1;
                    while (k > 0 && (text[k] != 'q' || bracketCount != 0))
                    {
                        if (text[k] == 'q')
                        {
                            bracketCount--;
                        }
                        else if (text[k] == RIGHTBRACKET)
                        {
                            bracketCount++;
                        }

                        k--;
                    }

                    ////int k = text.Substring(0, j-1).LastIndexOf('q');
                    if (k < 0)
                    {
                        throw new ArgumentException(FormulaErrorStrings[bad_library]);
                    }
                }
                else
                {
                    ////                else if(!char.IsDigit(text[j])) //number
                    ////                {
                    ////                    throw new ArgumentException(FormulaErrorStrings[invalid_char_in_number]);
                    ////                }

                    bool period = false;
                    bool percent = false;

                    while (j > -1 && (char.IsDigit(text[j]) || (!period && text[j] == ParseDecimalSeparator)
                        || (!percent && text[j] == '%')))
                    {
                        if (text[j] == ParseDecimalSeparator)
                        {
                            period = true;
                        }
                        else if (text[j] == '%')
                        {
                            percent = true;
                        }

                        j = j - 1;
                    }

                    if (j > -1 && period && text[j] == ParseDecimalSeparator)
                    {
                        throw new ArgumentException(FormulaErrorStrings[number_contains_2_decimal_points]);
                    }
                }

                if (text.Length > 0 && (char.IsUpper(text, 0) || text[0] == sheetToken))
                {
                    ////Check if cell reference.
                    bool ok = true;
                    bool checkLetter = true;
                    for (int k = 0; k < text.Length; ++k)
                    {
                        if (text[k] == sheetToken)
                        {
                            k++;
                            while (k < text.Length && char.IsDigit(text[k]))
                            {
                                k++;
                            }

                            if (k == text.Length || text[k] != sheetToken)
                            {
                                ok = false;
                                break;
                            }
                        }
                        else
                        {
                            if (!checkLetter && char.IsUpper(text, k))
                            {
                                ok = false;
                                break;
                            }
                            ////20040709 - if(char.IsLetterOrDigit(text, k) || text[k] == sheetToken)
                            if (char.IsDigit(text, k) || char.IsUpper(text, k) || text[k] == sheetToken)
                            {
                                checkLetter = char.IsUpper(text, k);
                            }
                            else
                            {
                                ok = false;
                                break;
                            }
                        }
                    }

                    if (ok)
                    {
                        ////Handle 0 as relative row reference.
                        if (currentRowNotationEnabled && text.Length > 1 && text[text.Length - 1] == '0' &&
                            !char.IsDigit(text[text.Length - 2]))
                        {
                            text = text.Substring(0, text.Length - 1) + RowIndex(cell);
                        }

                        UpdateDependencies(text); ////cb
                        needToContinue = false;
                    }
                }
            }

            return text;
        }

        private int FindLastNonQB(string text)
        {
            int ret = -1;
            if (text.IndexOf(BMARKER) > -1)
            {
                int bracketLevel = 0;
                for (int i = text.Length - 1; i >= 0; --i)
                {
                    if (text[i] == RIGHTBRACKET)
                    {
                        bracketLevel--;
                    }
                    else if (text[i] == LEFTBRACKET)
                    {
                        bracketLevel++;
                    }
                    else if (text[i] == BMARKER && bracketLevel == 0)
                    {
                        ret = i;
                        break;
                    }
                }
            }

            return ret;
        }

        /// <summary>
        /// Replaces namedranges with their values.
        /// </summary>
        /// <remarks>
        /// Accepts a string such as Sin(SumRange), and tokenizes it into
        /// bqSIN[A1A4a]b which serves as input to ComputedValue.
        /// </remarks>
        /// <param name="argList">argList containing named ranges.</param>
        private void MarkNamedRanges(ref string argList)
        {
            int rightParens = argList.IndexOf(')');
            char[] markers = new char[] { ')', ParseArgumentSeparator, '}', '+', '-', '*', '/', '<', '>', '=' };

            int leftParens = 0;

            int i = leftParens + 1; ////start of args
            int end = argList.Substring(i).IndexOfAny(markers);

            while (end > -1 && end + i < argList.Length)
            {
                string s = (string)NamedRanges[argList.Substring(i, end)];
                if (s != null)
                {
                    SetNamedRangeDependency(argList.Substring(i, end), cell);
                    s = s.ToUpper();
                    MarkLibraryFormulas(ref s);
                    PutTokensForSheets(ref s);
                }

                if (s != null)
                {
                    argList = argList.Substring(0, i) + s.ToString() + argList.Substring(i + end);
                    i += s.ToString().Length + 1;
                }
                else
                {
                    i += end + 1;
                    while (i < argList.Length && !char.IsUpper(argList[i]))
                    {
                        i++;
                    }
                }

                end = argList.Substring(i).IndexOfAny(markers);
            }
        }

        /// <summary>
        /// Tokenizes all library references.
        /// </summary>
        /// <remarks>
        /// Accepts a string such as Sin(A1+A2), and tokenizes it into
        /// bqSIN[A1A2a]b which serves as input to ComputedValue.
        /// </remarks>
        /// <param name="formula">Formula to be tokenized.</param>
        private void MarkLibraryFormulas(ref string formula)
        {
            int rightParens = formula.IndexOf(')');

            while (rightParens > -1)
            {
                int parenCount = 0;
                int leftParens = rightParens - 1;
                while (leftParens > -1 && (formula[leftParens] != '(' || parenCount != 0))
                {
                    if (formula[leftParens] == ')')
                    {
                        parenCount++;
                    }
                    else if (formula[leftParens] == ')')
                    {
                        parenCount--;
                    }

                    leftParens--;
                }

                if (leftParens == -1)
                {
                    throw new ArgumentException(FormulaErrorStrings[mismatched_parentheses]);
                }

                int i = leftParens - 1;
                while (i > -1 && char.IsLetterOrDigit(formula[i]))
                {
                    i--;
                }

                int len = leftParens - i - 1;
                if (len > 0 && this.LibraryFunctions[formula.Substring(i + 1, len)] != null)
                {
                    string s = formula.Substring(leftParens, rightParens - leftParens + 1);
                    MarkNamedRanges(ref s);
                    if (forceParsingOfLibraryFunctionArguments)
                    {
                        SwapInnerParens(ref s);
                        AddParensToArgs(ref s);
                    }

                    formula = formula.Substring(0, i + 1) + 'q' + formula.Substring(i + 1, len) + s.Replace('(', LEFTBRACKET).Replace(')', RIGHTBRACKET) + formula.Substring(rightParens + 1);
                }
                else if (len > 0)
                {
                    throw new ArgumentException(FormulaErrorStrings[unknown_formula_name]);
                }
                else
                {
                    string s = string.Empty;
                    if (leftParens > 0)
                    {
                        s = formula.Substring(0, leftParens);
                    }

                    s = s + '{' +
                        formula.Substring(leftParens + 1, rightParens - leftParens - 1) + '}';
                    if (rightParens < formula.Length)
                    {
                        s = s + formula.Substring(rightParens + 1);
                    }

                    formula = s;
                }

                rightParens = formula.IndexOf(')');
            }

            formula = formula.Replace('{', '(').Replace('}', ')');
        }

        private void SwapInnerParens(ref string s)
        {
            if (s.Length > 2)
            {
                s = s[0] + s.Substring(1, s.Length - 2).Replace("(", "{").Replace(")", "}") + s[s.Length - 1];
            }
        }

        private void AddParensToArgs(ref string formula)
        {
            if (formula.Length == 0)
            {
                return;
            }

            char[] rightSides = new char[] { ParseArgumentSeparator, RIGHTBRACKET };
            ////char[] rightSides = new char[]{ParseArgumentSeparator, RIGHTBRACKET,')'};
            int i = formula.LastIndexOf(ParseArgumentSeparator);
            if (i == -1)
            {
                if (formula.Length > 2 && formula[0] == '(' && formula[formula.Length - 1] == ')')
                {
                    if (formula[1] != '{' && formula[1] != '(')
                    {
                        formula = formula.Substring(0, formula.Length - 1) + "}" + formula.Substring(formula.Length - 1);
                        formula = formula[0] + "{" + formula.Substring(1);
                    }
                }
            }
            else
            {
                bool oneTimeOnly = true;
                while (i > -1)
                {
                    int j = formula.IndexOfAny(rightSides, i + 1);
                    if (j == -1 && formula[formula.Length - 1] == ')')
                    {
                        j = formula.Length - 1;
                    }

                    if (j > 0)
                    {
                        if (formula[i + 1] != '{' && formula[j - 1] != '}')
                        {
                            formula = formula.Substring(0, j) + "}" + formula.Substring(j);
                            formula = formula.Substring(0, i + 1) + "{" + formula.Substring(i + 1);
                        }
                    }

                    i = formula.Substring(0, i).LastIndexOf(ParseArgumentSeparator);
                    if (oneTimeOnly && i == -1 && formula[0] == '(')
                    {
                        i = 0;
                        oneTimeOnly = false;
                    }
                }
            }
        }

        private bool forceParsingOfLibraryFunctionArguments = false;

        /// <summary>
        /// Gets or sets a value indicating whether all function arguments are parsed using GridFormulaEngine.Parse.
        /// </summary>
        /// <remarks>
        /// If you are using NamedRanges or CurrentRowNotationEnabled inside function arguments, 
        /// you should set this property 
        /// to true to make sure the proper substitutions are done on the arguments.
        /// </remarks>
        public bool ForceParsingOfLibraryFunctionArguments
        {
            get { return forceParsingOfLibraryFunctionArguments; }
            set { forceParsingOfLibraryFunctionArguments = value; }
        }

        #endregion

        #region Named Ranges support

        /// <summary>
        /// Gets the named range Hashtable.
        /// </summary>
        /// <remarks>
        /// The key is the uppercase name and the value is the range for this name.
        /// </remarks>
        public Hashtable NamedRanges
        {
            get
            {
                if (IsSheeted)
                {
                    GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                    if (family.sheetNamedRanges == null)
                    {
                        family.sheetNamedRanges = new Hashtable();
                    }

                    return family.sheetNamedRanges;
                }
                else
                {
                    if (namedRanges == null)
                    {
                        namedRanges = new Hashtable();
                    }

                    return namedRanges;
                }
            }
        }

        public Hashtable NamedRangeValues
        {
            get
            {
                if (IsSheeted)
                {
                    GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                    if (family.namedRangeValues == null)
                    {
                        family.namedRangeValues = new Hashtable();
                    }

                    return family.namedRangeValues;
                }
                else
                {
                    if (rangeValues == null)
                    {
                        rangeValues = new Hashtable();
                    }

                    return rangeValues;
                }
            }
        }

        /// <summary>
        /// Gets the named range Hashtable that holds the original case name as the value.
        /// </summary>
        /// <remarks>
        /// The key is the uppercase name and the value is the original name.
        /// </remarks>
        public Hashtable NamedRangesOriginalNames
        {
            get
            {
                if (IsSheeted)
                {
                    GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                    if (family.sheetNamedRangesOriginalNames == null)
                    {
                        family.sheetNamedRangesOriginalNames = new Hashtable();
                    }

                    return family.sheetNamedRangesOriginalNames;
                }
                else
                {
                    if (namedRangesOriginalNames == null)
                    {
                        namedRangesOriginalNames = new Hashtable();
                    }

                    return namedRangesOriginalNames;
                }
            }
        }

        /// <summary>
        /// Updates all cells that depend upon the given named range.
        /// </summary>
        /// <param name="key">The named range whose cells should be updated.</param>
        public void UpdateDependentNamedRangeCell(string key)
        {
            if (DependentNamedRangeCells.ContainsKey(key))
            {
                Hashtable ht = (Hashtable)((Hashtable)DependentNamedRangeCells[key]).Clone();
                foreach (string s in ht.Keys)
                {
                    string cell1 = s;
                    int i = s.LastIndexOf(sheetToken);
                    int row, col;
                    GridModel grd = grid;
                    if (i > -1)
                    {
                        GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);

                        this.grid = (GridModel)family.TokenToGridModel[cell1.Substring(0, i + 1)];
                        row = RowIndex(cell1);
                        col = ColIndex(cell1);
                    }
                    else
                    {
                        row = RowIndex(cell1);
                        col = ColIndex(cell1);
                    }

                    this.RecalculateRange(GridRangeInfo.Cell(row, col), true, true);

                    this.grid = grd;
                }
            }
        }

        /// <summary>
        /// Removes entries in the DependentNamedRangeCells collection for the given named range.
        /// </summary>
        /// <param name="key">The named range.</param>
        public void RemoveNamedRangeDependency(string key)
        {
            if (DependentNamedRangeCells.ContainsKey(key))
            {
                Hashtable ht = (Hashtable)DependentNamedRangeCells[key];
                ht.Clear();
                DependentNamedRangeCells.Remove(key);
            }
        }

        /// <summary>
        /// Adds a cell to the DependentNamedRangeCells list.
        /// </summary>
        /// <param name="key">The named range.</param>
        /// <param name="cell1">The cell (such as C11 or AJ232).</param>
        public void SetNamedRangeDependency(string key, string cell1)
        {
            GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
            if (family.SheetNameToGridModel != null && cell1.IndexOf(sheetToken) == -1)
            {
                string token = family.GridModelToToken[this.grid] as string;
                cell1 = token + cell1;
            }

            if (DependentNamedRangeCells.ContainsKey(key))
            {
                Hashtable ht = (Hashtable)DependentNamedRangeCells[key];
                if (!ht.ContainsKey(cell1))
                {
                    ht.Add(cell1, string.Empty);
                }
            }
            else
            {
                Hashtable ht = new Hashtable();
                ht.Add(cell1, string.Empty);
                DependentNamedRangeCells.Add(key, ht);
            }
        }

        private Hashtable namedRangesOriginalNames = null;
        private Hashtable namedRanges = null;
        private Hashtable dependentNamedRangeCells = null;
        private ArrayList namedRangesSized = null;
        private Hashtable rangeValues = null;
        /// <summary>
        /// Gets namedranges in order of string length.
        /// </summary>
        protected ArrayList NamedRangesSized
        {
            get
            {
                if (namedRangesSized == null)
                {
                    namedRangesSized = new ArrayList();
                    foreach (string s in NamedRanges.Keys)
                    {
                        namedRangesSized.Add(s);
                    }

                    AdjustNameRangesForSize();
                }

                return namedRangesSized;
            }
        }

        /// <summary>
        /// Orders a namedrange collection according to string length.
        /// </summary>
        /// <remarks>The GridFormulaEngine needs an ordered list of named ranges 
        /// to be able to properly parse named ranges. If you manually add named ranges
        /// to namedranges, then you should call this method afterwards.
        /// </remarks>
        public void AdjustNameRangesForSize()
        {
            NamedRangesSized.Sort(new LenComparer());
        }

        /// <summary>
        /// Used by AdjustNameRangesForSize to create an ArrayList
        /// ordered by string length.
        /// </summary>
        [Syncfusion.Documentation.DocumentationExclude()]
        public class LenComparer : IComparer
        {
            /// <summary>
            /// Used internally.
            /// </summary>
            [Syncfusion.Documentation.DocumentationExclude()]
            public LenComparer()
                : base()
            {
            }

            /// <summary>
            /// Compares 2 strings based on their length.
            /// </summary>
            /// <param name="x">First string to compare.</param>
            /// <param name="y">Second string to compare.</param>
            /// <returns>Zero if both are equal; positive value if second string is greater; negative value if first string is greater.</returns>
            [Syncfusion.Documentation.DocumentationExclude()]
            public int Compare(object x, object y)
            {
                return y.ToString().Length - x.ToString().Length;
            }
        }

        /// <summary>
        /// Gets hashtables containing cells that depend upon namedranges.
        /// </summary>
        /// <remarks>
        /// The key properties in DependentNamedRangeCells are namedranges. The
        /// value properties are hashtables.
        /// </remarks>
        public Hashtable DependentNamedRangeCells
        {
            get
            {
                if (IsSheeted)
                {
                    GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                    if (family.sheetDependentNamedRangeCells == null)
                    {
                        family.sheetDependentNamedRangeCells = new Hashtable();
                    }

                    return family.sheetDependentNamedRangeCells;
                }
                else
                {
                    if (dependentNamedRangeCells == null)
                    {
                        dependentNamedRangeCells = new Hashtable();
                    }

                    return dependentNamedRangeCells;
                }
            }
        }

        /// <summary>
        /// Adds a named range to the namedranges collection.
        /// </summary>
        /// <param name="name">The name of the range to be added.</param>
        /// <param name="range">The range to be added.</param>
        /// <returns>True if successfully added, False otherwise.</returns>
        /// <remarks>
        /// The range should be a string such as A4:C8.
        /// </remarks>
        public bool AddNamedRange(string name, string range)
        {
            string s = name.ToUpper(CultureInfo.InvariantCulture);
            if (NamedRanges[s] == null)
            {
                if (range.StartsWith("="))
                {
                    range = range.Substring(1);
                }

                NamedRanges.Add(s, range.Replace(" ", string.Empty));
                NamedRangesOriginalNames.Add(s, name);
                if (!NamedRangeValues.ContainsKey(s))
                {
                    NamedRangeValues.Add(s, range.ToString());
                }
                namedRangesSized = null;
                return true;
            }

            return false;
        }

        /// <summary>
        /// Removes a range from the namedranges collection.
        /// </summary>
        /// <param name="name">The name of the range to be removed.</param>
        /// <returns>True is successfully remove, False otherwise.</returns>
        public bool RemoveNamedRange(string name)
        {
            name = name.ToUpper(CultureInfo.InvariantCulture);
            if (NamedRanges[name] != null)
            {
                NamedRanges.Remove(name);
                NamedRangesOriginalNames.Remove(name);
                namedRangesSized = null;
                return true;
            }

            return false;
        }
        #endregion

        #region Library Functions Maintenance

        /// <summary>
        /// Gets the current library functions.
        /// </summary>
        /// <remarks>
        /// This field gives you direct access to all Library Functions. 
        /// The function name serves as the hash key, and the function delegate
        /// serves as the hash value.
        /// </remarks>
        public Hashtable LibraryFunctions
        {
            get
            {
                return libraryFunctions;
            }
        }

        Hashtable libraryFunctions;

        /// <summary>
        /// Adds a function to the Function Library.
        /// </summary>
        /// <param name="name">The name of the function to be added.</param>
        /// <param name="func">The function to be added.</param>
        /// <returns>True if successfully removed, False otherwise.</returns>
        /// <remarks>
        /// LibraryFunction is a delegate the defines the signature of functions that
        /// you can add to the Function Library.
        /// <code>
        /// public delegate string LibraryFunction(string args);
        /// </code>
        /// </remarks>
        public bool AddFunction(string name, LibraryFunction func)
        {
            name = name.ToUpper(CultureInfo.InvariantCulture);

            if (UseCommonLibrary)
            {
                GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                if (family.GridModelToToken != null)
                {
                    foreach (GridModel g in family.GridModelToToken.Keys)
                    {
                        GridFormulaEngine e = ((GridFormulaCellModel)g.CellModels["FormulaCell"]).Engine;
                        if (e.LibraryFunctions[name] == null)
                        {
                            e.LibraryFunctions.Add(name, func);
                        }
                    }

                    return true;
                }
            }

            if (LibraryFunctions[name] == null)
            {
                LibraryFunctions.Add(name, func);
                return true;
            }

            return false;
        }

        /// <summary>
        /// Removes a function from the Function Library.
        /// </summary>
        /// <param name="name">The name of the function to be removed.</param>
        /// <returns>True if successfully removed, False otherwise.</returns>
        public bool RemoveFunction(string name)
        {
            if (UseCommonLibrary)
            {
                GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
                if (family.GridModelToToken != null)
                {
                    foreach (GridModel g in family.GridModelToToken.Keys)
                    {
                        GridFormulaEngine e = ((GridFormulaCellModel)g.CellModels["FormulaCell"]).Engine;
                        if (e.LibraryFunctions[name] == null)
                        {
                            e.LibraryFunctions.Remove(name);
                        }
                    }

                    return true;
                }
            }

            if (LibraryFunctions[name] != null)
            {
                LibraryFunctions.Remove(name);
                return true;
            }

            return false;
        }

        /// <summary>
        /// Delegate used to define functions that you add to the Function Library.
        /// </summary>
        public delegate string LibraryFunction(string args);

        ////Used to avoid recursively setting dependencies when cells are being
        ////refreshed because a dependent cell changed.
        internal bool lockDependencies = false;

        internal void UpdateDependencies(string s)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(this);
            }
#else
            ;
#endif
            if (lockDependencies)
            {
                return;
            }

            GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
            if (cell == null)
            {
                int currRow = grid.currentRow;
                cell = GridRangeInfo.GetAlphaLabel(grid.currentCol) + currRow.ToString();
            }
            string cell1 = cell;
            if (family.SheetNameToGridModel != null && cell1.IndexOf(sheetToken) == -1)
            {
                string token = family.GridModelToToken[this.grid] as string;
                cell1 = token + cell1;
            }

            if (s != null)
            {
                if (family.SheetNameToGridModel != null && s.IndexOf(sheetToken) == -1)
                {
                    string token = family.GridModelToToken[this.grid] as string;
                    s = token + s;
                }

                if (!DependentCells.ContainsKey(s))
                {
#if DEBUG
                    if (Switches.FormulaCell.TraceVerbose)
                    {
                        TraceUtil.TraceCurrentMethodInfo(this, " new trigger cell (" + s + " -> " + cell1 + ")");
                    }
#else
                ;
#endif
                    DependentCells.Add(s, new Hashtable());
                    ((Hashtable)DependentCells[s]).Add(cell1, cell1);
                    AddToFormulaDependentCells(s);
                }
                else if (!((Hashtable)DependentCells[s]).ContainsKey(cell1))
                {
                    ((Hashtable)DependentCells[s]).Add(cell1, cell1);
                    AddToFormulaDependentCells(s);
#if DEBUG
                    if (Switches.FormulaCell.TraceVerbose)
                    {
                        TraceUtil.TraceCurrentMethodInfo(this, " old trigger cell (" + s + " -> " + cell1 + ")");
                    }
#else
                ;
#endif
                }
            }
        }

        ////When a formula cell changes, call this method to clear it from its dependent cells.
        internal void ClearFormulaDependentCells(string cell)
        {
            Hashtable ht = (Hashtable)DependentFormulaCells[cell];
            if (ht != null)
            {
                foreach (object o in ht.Keys)
                {
                    string s = o as string;
                    ////DependentCells.Remove(s);
                    Hashtable ht1 = DependentCells[s] as Hashtable;
                    if (ht1.ContainsKey(cell))
                    {
                        ht1.Remove(cell);
                        if (ht1.Count == 0)
                        {
                            DependentCells.Remove(s);
                        }
                    }
#if DEBUG
                    if (Switches.FormulaCell.TraceVerbose)
                    {
                        TraceUtil.TraceCurrentMethodInfo(this, " clearing formula dependencies (" + cell + " -> " + s + ")");
                    }
#else
                    ;
#endif
                }

                DependentFormulaCells.Remove(cell);
            }

            foreach (Hashtable ht1 in DependentNamedRangeCells.Values)
            {
                if (ht1.ContainsKey(cell))
                {
                    ht1.Remove(cell);
                }
            }
        }

        ////Maintains a list of cells that a formula cell is dependent upon.
        private void AddToFormulaDependentCells(string s)
        {
            string cell1 = cell;
            GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);
            if (family.SheetNameToGridModel != null && cell1.IndexOf(sheetToken) == -1)
            {
                string token = family.GridModelToToken[this.grid] as string;
                cell1 = token + cell1;
            }

            if (!DependentFormulaCells.ContainsKey(cell1))
            {
                DependentFormulaCells.Add(cell1, new Hashtable());
                ((Hashtable)DependentFormulaCells[cell1]).Add(s, s);
            }
            else if (!((Hashtable)DependentFormulaCells[cell1]).ContainsKey(s))
            {
                ((Hashtable)DependentFormulaCells[cell1]).Add(s, s);
            }
        }

        /// <summary>
        /// Returns the simple average of all values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the simple average of all values listed in the argument.</returns>
        public string ComputeAvg(string range)
        {
            double sum = 0;
            int count = 0;
            double d;
            string s1;

            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            TraceUtil.TraceExceptionCatched(ex);
                            if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            {
                                throw ex;
                            }

                            return ex.Message;
                            ////throw new ArgumentException(ex.Message);
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                            {
                                sum = sum + d;
                                count++;
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                        {
                            sum = sum + d;
                            count++;
                        }
                    }
                }
            }

            if (count > 0)
            {
                sum = sum / (double)count;
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns the inclusive OR of all values treated as logical values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers. Each item in the list is considered True if it is nonzero,
        /// and False if it is zero.</param>
        /// <returns>A string holding the OR of all values listed in the argument.</returns>
        public string ComputeOr(string range)
        {
            bool sum = false;
            string s1;
            double d;

            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            TraceUtil.TraceExceptionCatched(ex);
                            if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            {
                                throw ex;
                            }

                            return ex.Message;
                            ////throw new ArgumentException(ex.Message);
                        }

                        if (s1.Length > 0)
                        {
                            if (s1 == this.TRUEVALUESTR)
                            {
                                sum = true;
                                break;
                            }
                            else if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                            {
                                if (Math.Abs(d) > 1e-10)
                                {
                                    sum = true;
                                    break;
                                }
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (s1 == this.TRUEVALUESTR)
                        {
                            sum = true;
                            break;
                        }
                        else if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                        {
                            if (Math.Abs(d) > 1e-10)
                            {
                                sum = true;
                                break;
                            }
                        }
                    }
                }

                if (sum)
                {
                    break;
                }
            }
            ////return sum ? "1" : "0";
            return sum ? this.TRUEVALUESTR : this.FALSEVALUESTR;
        }

        /// <summary>
        /// Returns the AND of all values treated as logical values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers. Each item in the list is considered True if it is nonzero,
        /// and False if it is zero.</param>
        /// <returns>A string holding the AND of all values listed in the argument.</returns>
        public string ComputeAnd(string range)
        {
            bool sum = true;
            string s1;
            double d;

            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            TraceUtil.TraceExceptionCatched(ex);
                            if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            {
                                throw ex;
                            }

                            return ex.Message;
                            ////throw new ArgumentException(ex.Message);
                        }

                        if (s1.Length > 0)
                        {
                            if (s1 == this.FALSEVALUESTR)
                            {
                                sum = false;
                                break;
                            }
                            else if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                            {
                                if (Math.Abs(d) < 1e-10)
                                {
                                    sum = false;
                                    break;
                                }
                            }
                        }
                        else
                        {
                            ////empty cell
                            sum = false;
                            break;
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (s1 == this.FALSEVALUESTR)
                        {
                            sum = false;
                            break;
                        }
                        else if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                        {
                            if (Math.Abs(d) < 1e-10)
                            {
                                sum = false;
                                break;
                            }
                        }
                    }
                    else
                    {
                        ////empty cell
                        sum = false;
                        break;
                    }
                }

                if (!sum)
                {
                    break;
                }
            }
            ////return sum ? "1" : "0";
            return sum ? this.TRUEVALUESTR : this.FALSEVALUESTR;
        }

        /// <summary>
        ///  Flips the logical value represented by the argument.
        /// </summary>
        /// <remarks>
        ///    The argument is
        /// treated as a logical expression with a non-zero value considered True and a zero value considered False.
        /// </remarks>
        /// <param name="args">A string holding either a single argument consisting of a 
        /// cell reference, a formula, or a number. 
        /// </param>
        /// <returns>Returns 0 if the argument evaluates to a non-zero value. Otherwise, it returns 1.</returns>
        public string ComputeNot(string args)
        {
            double d1;
            string s = args;
            ////parsed formula
            if (args.Length > 0 && !char.IsLetter(args[0]) && args.IndexOfAny(new char[] { ParseArgumentSeparator, ':' }) > -1)
            {
                return FormulaErrorStrings[requires_a_single_argument];
            }
            else
            {
                try
                {
                    s = GetValueFromArg(s);
                    if (s == this.FALSEVALUESTR)
                    {
                        s = "0";
                    }
                    else if (s == this.TRUEVALUESTR)
                    {
                        s = "1";
                    }

                    if (double.TryParse(s, NumberStyles.Number | NumberStyles.AllowExponent, null, out d1))
                    {
                        ////Flip the value.
                        if (Math.Abs(d1) > 1e-10)
                        {
                            s = this.FALSEVALUESTR;
                        }
                        else
                        {
                            s = this.TRUEVALUESTR;
                        }
                    }
                }
                catch (Exception ex)
                {
                    TraceUtil.TraceExceptionCatched(ex);
                    if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                    {
                        throw ex;
                    }

                    return ex.Message;
                }
            }

            return s;
        }

        //Used to handle unary minus replacement
        private string Checkub(string s)
        {
            return s.Replace("ub", "bu");
        }
        /// <summary>
        /// Conditionally computes one of two alternatives depending upon a logical expression.
        /// </summary>
        /// <remarks>
        ///    The first argument is
        /// treated as a logical expression with a non-zero value considered True and a zero value considered False.
        /// The value of only one of the alternatives is computed depending upon the logical expression.
        /// </remarks>
        /// <param name="args">A string holding a list of three arguments. 
        /// </param>
        /// <returns>Returns a string holding the second argument if the first argument is True (non-zero). Otherwise, it returns a string holding the third argument.</returns>
        public string ComputeIf(string args)
        {
            double d1;
            string s1 = string.Empty;
            ////parsed formula
            if (args.Length > 0 && args.IndexOfAny(new char[] { ParseArgumentSeparator, ':' }) == -1)
            {
                return FormulaErrorStrings[requires_3_args];
            }
            else
            {
                string[] s = SplitArguments(args, ParseArgumentSeparator);
                
                if (s.GetLength(0) >= 2)
                {
                    try
                    {
                        if (SplitArguments(s[0], ':').Length == 1)
                        {
                            s1 = GetValueFromArg(s[0]);
                        }
                        else
                        {
                            string[] str1 = this.GetCellsFromArgs(s[0]);
                            
                            int firstRow = this.RowIndex(str1[0]);
                            int firstCol = this.ColIndex(str1[0]);
                            int lastRow = this.RowIndex(str1[str1.Length - 1]);
                            int lastCol = this.ColIndex(str1[str1.Length - 1]);
                            string currentCell = this.cell;
                            if (this.cell == null && this.grid.CurrentCellInfo != null)
                                currentCell = GridRangeInfo.GetAlphaLabel(this.grid.CurrentCellInfo.ColIndex) + this.grid.CurrentCellInfo.RowIndex;
                            string toke = this.SheetToken(currentCell);
                            if(toke != string.Empty)
                                currentCell = currentCell.Replace(toke, string.Empty);

                            if ((firstCol != lastCol && firstRow == lastRow && (this.ColIndex(currentCell) < firstCol || this.ColIndex(currentCell) > lastCol))
                                || (firstCol == lastCol && firstRow != lastRow && (this.RowIndex(currentCell) < firstRow || this.RowIndex(currentCell) > lastRow)))
                            {
                                s1 = "#VALUE!";
                                return s1;
                            }
                            else
                            {
                                if (firstCol == lastCol)
                                {
                                    string refCell = GridRangeInfo.GetAlphaLabel(firstCol) + this.RowIndex(this.cell);
                                    s[0] = s[0].Replace(str1[0], string.Empty).Replace(":", string.Empty);
                                    s1 = GetValueFromArg(s[0]);
                                }
                                else if (firstRow == lastRow)
                                {
                                    string refCell = GridRangeInfo.GetAlphaLabel(this.ColIndex(this.cell)) + firstRow;
                                    s[0] = s[0].Replace(str1[0], string.Empty).Replace(":", string.Empty);
                                    s1 = GetValueFromArg(s[0]);
                                }
                            }
                        }

                        if (s1.ToUpper() == this.FALSEVALUESTR)
                        {
                            s1 = "0";
                        }
                        else if (s1.ToUpper() == this.TRUEVALUESTR)
                        {
                            s1 = "1";
                        }

                        if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d1))
                        {
                            if (Math.Abs(d1) > 1e-10)
                            {
                                s1 = GetValueFromArg(Checkub(s[1]));
                            }
                            else
                            {
                                s1 = s.GetLength(0) == 3 ? ((s[2].Equals(FALSEVALUESTR) || s[2].Equals(TRUEVALUESTR)) ? s[2] : GetValueFromArg(Checkub(s[2]))) : FALSEVALUESTR;
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        return ex.Message;
                    }
                }
                else
                {
                    return FormulaErrorStrings[requires_3_args];
                }
            }

            return s1;
        }

        /// <summary>
        /// Returns the sum of all values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the sum of all values listed in the argument.</returns>
        public string ComputeSum(string range)
        {
            double sum = 0;
            string s1;
            double d;

            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s).Replace(TIC, string.Empty);
                        }
                        catch (Exception ex)
                        {
                            TraceUtil.TraceExceptionCatched(ex);
                            if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            {
                                throw ex;
                            }

                            return ex.Message;
                            ////throw new ArgumentException(ex.Message);
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                            {
                                sum = sum + d;
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r).Replace(TIC, string.Empty);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent | NumberStyles.Currency, null, out d))
                        {
                            sum = sum + d;
                        }
                    }
                }
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns the maximum value of all values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the maximum value of all values listed in the argument.</returns>
        public string ComputeMax(string range)
        {
            double max = double.MinValue;
            double d;
            string s1;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////cell range
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            TraceUtil.TraceExceptionCatched(ex);
                            if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            {
                                throw ex;
                            }

                            return ex.Message;
                            ////throw new ArgumentException(ex.Message);
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent | NumberStyles.Currency, null, out d))
                            {
                                max = Math.Max(max, d);
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = (r == string.Empty) ? "0" : GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent | NumberStyles.Currency, null, out d))
                        {
                            max = Math.Max(max, d);
                        }
                    }
                }
            }

            if (max != double.MinValue)
            {
                GridStyleInfo style = this.grid[this.RowIndex(this.cell), this.ColIndex(this.cell)];
                return max.ToString(style.Format);
            }

            return "0";
        }

        /// <summary>
        /// Returns the minimum value of all values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the minimum value of all values listed in the argument.</returns>
        public string ComputeMin(string range)
        {
            double min = double.MaxValue;
            double d;
            string s1;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////cell range
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            ////TraceUtil.TraceExceptionCatched(ex);
                            ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            ////    throw ex;
                            return ex.Message;
                            ////throw new ArgumentException(ex.Message);
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent | NumberStyles.Currency, null, out d))
                            {
                                min = Math.Min(min, d);
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = (r == string.Empty) ? "0" : GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent | NumberStyles.Currency, null, out d))
                        {
                            min = Math.Min(min, d);
                        }
                    }
                }
            }

            if (min != double.MaxValue)
            {
                GridStyleInfo style = this.grid[this.RowIndex(this.cell), this.ColIndex(this.cell)];
                return min.ToString(style.Format);
            }

            return "0";
        }

        /// <summary>
        /// Returns the number pi.
        /// </summary>
        /// <param name="args">Ignored. Can be empty.</param>
        /// <returns>A string holding the number pi.</returns>
        public string ComputePI(string args)
        {
            return Math.PI.ToString();
        }

        /// <summary>
        /// Returns a number indicating the sign of the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding a number representing the sign of the argument.</returns>
        public string ComputeSign(string args)
        {
            int sign = 0;
            double d;
            string s1;
            if (args.Length > 0 && !char.IsLetter(args[0]) && args.IndexOfAny(new char[] { ParseArgumentSeparator, ':' }) == -1)
            {
                ////parsed formula
                if (double.TryParse(args, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                {
                    sign = Math.Sign(d);
                }
                else
                {
                    return FormulaErrorStrings[invalid_Math_argument];
                }
            }
            else if (args.Length > 0 && (args[0] == BMARKER || args[0] == 'u' || args[0] == 'n' || args.IndexOfAny(tokens) > -1))
            {
                ////parsed formula
                args = args.Replace('{', '(');
                args = args.Replace('}', ')');

                try
                {
                    s1 = this.ComputedValue(args);
                }
                catch (Exception ex)
                {
                    TraceUtil.TraceExceptionCatched(ex);
                    if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                    {
                        throw ex;
                    }

                    return ex.Message;
                }

                if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                {
                    sign = Math.Sign(d);
                }
                else
                {
                    return FormulaErrorStrings[invalid_Math_argument];
                }
            }
            else
            {
                foreach (string s in this.GetCellsFromArgs(args))
                {
                    try
                    {
                        s1 = GetValueFromArg(s);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                        {
                            sign = Math.Sign(d);
                            break; ////Skip out after first value.
                        }
                    }
                }
            }

            return sign.ToString();
        }

        /// <summary>
        /// Returns a specified number raised to the specified power.
        /// </summary>
        /// <param name="args">String containing two parameters separated by commas:
        /// the first being base number,
        /// the second being the exponent.</param>
        /// <returns>A string holding the value of the base number raised to the exponent.</returns>
        public string ComputePow(string args)
        {
            double pow = 0;
            double d1, d2;
            string s1, s2;
            if (args.Length > 0 && !char.IsLetter(args[0]) && args.IndexOfAny(new char[] { ParseArgumentSeparator, ':' }) == -1)
            {
                ////parsed formula
                return FormulaErrorStrings[requires_2_args];
            }
            else
            {
                string[] s = this.GetCellsFromArgs(args);

                if (s.GetLength(0) == 2)
                {
                    try
                    {
                        s1 = GetValueFromArg(s[0]);
                        s2 = GetValueFromArg(s[1]);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        return ex.Message;
                    }

                    if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d1)
                        && double.TryParse(s2, NumberStyles.Number | NumberStyles.AllowExponent, null, out d2))
                    {
                        pow = Math.Pow(d1, d2);
                    }
                }
                else
                {
                    return FormulaErrorStrings[requires_2_args];
                }
            }

            return pow.ToString();
        }

        private delegate double MathFunc(double d);

        private string ComputeMath(string args, MathFunc func)
        {
            double ans = 0;
            double d1;
            string s1;
            if (args.Length > 0 && ((!char.IsLetter(args[0]) && args[0] != sheetToken) || args[0] == 'u')
                 && args.IndexOfAny(new char[] { ParseArgumentSeparator, ':' }) == -1)
            {
                ////parsed formula
                ////Swap out unary minus.
                args = args.Replace('u', '-').Replace("n", string.Empty);

                if (double.TryParse(args, NumberStyles.Number | NumberStyles.AllowExponent, null, out d1))
                {
                    ans = func(d1);
                }
                else
                {
                    return FormulaErrorStrings[invalid_Math_argument];
                }
            }
            else if (args.Length > 0 && (args[0] == BMARKER || args[0] == 'n' || args.IndexOfAny(tokens) > -1))
            {
                ////parsed formula
                args = args.Replace('{', '(');
                args = args.Replace('}', ')');

                try
                {
                    s1 = this.ComputedValue(args);
                }
                catch (Exception ex)
                {
                    TraceUtil.TraceExceptionCatched(ex);
                    if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                    {
                        throw ex;
                    }

                    return ex.Message;
                }

                if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d1))
                {
                    ans = func(d1);
                }
                else
                {
                    return FormulaErrorStrings[invalid_Math_argument];
                }
            }
            else
            {
                foreach (string s in this.GetCellsFromArgs(args))
                {
                    try
                    {
                        s1 = GetValueFromArg(s);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d1))
                        {
                            ans = func(d1);
                        }
                    }

                    break; ////do only the first one... ignore any others
                }
            }

            return ans.ToString();
        }

        /// <summary>
        /// Computes angle whose cosine is the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding angle whose cosine is the argument.</returns>
        public string ComputeAcos(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Acos)).ToString();
        }

        /// <summary>
        /// Computes angle whose sine is the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding angle whose sine is the argument.</returns>
        public string ComputeAsin(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Asin)).ToString();
        }

        /// <summary>
        /// Computes angle whose tangent is the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the tangent of the argument.</returns>
        public string ComputeAtan(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Atan)).ToString();
        }

        /// <summary>
        /// Computes the cosine of the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the cosine of the argument.</returns>
        public string ComputeCos(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Cos)).ToString();
        }

        /// <summary>
        /// Computes the sine of the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the sine of the argument.</returns>
        public string ComputeSin(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Sin)).ToString();
        }

        /// <summary>
        /// Computes the hyperbolic cosine of the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the hyperbolic cosine of the argument.</returns>
        public string ComputeCosh(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Cosh)).ToString();
        }

        /// <summary>
        /// Computes the hyperbolic sine of the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the hyperbolic sine of the argument.</returns>
        public string ComputeSinh(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Sinh)).ToString();
        }

        /// <summary>
        /// Computes the hyperbolic tangent of the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the hyperbolic tangent of the argument.</returns>
        public string ComputeTanh(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Tanh)).ToString();
        }

        ////        /// <summary>
        ////        /// Computes the whole number nearest the argument.
        ////        /// </summary>
        ////        /// <param name="args">A cell reference, formula, or number.</param>
        ////        /// <returns>A string holding the whole number nearest the argument.</returns>
        ////        public string ComputeRound(string args)
        ////        {
        ////            return ComputeMath(args, new MathFunc(Math.Round)).ToString();
        ////        }

        /// <summary>
        /// Computes the smallest whole number greater than or equal to the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the smallest whole number greater than or equal to the argument.</returns>
        public string ComputeCeiling(string args)
        {
            ////return ComputeMath(args, new MathFunc(Math.Ceiling)).ToString();

            string[] range = args.Split(new char[] { ParseArgumentSeparator });
            int argCount = range.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double d1, d2, d3 = 0;
            if (double.TryParse(GetValueFromArg(range[0]), out d1)
                && double.TryParse(GetValueFromArg(range[1]), out d2))
            {
                if (d1 * d2 <= 0)
                {
                    return "#NUM!";
                }

                d3 = Math.Floor(d1 / d2) * d2;
                if (d2 > 0)
                {
                    while (d3 < d1)
                    {
                        d3 += d2;
                    }
                }
                else
                {
                    while (d3 > d1)
                    {
                        d3 += d2;
                    }
                }
            }

            return d3.ToString();
        }

        /// <summary>
        /// Computes the largest whole number less than or equal to the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the largest whole number less than or equal to the argument.</returns>
        public string ComputeFloor(string args)
        {
            ////return ComputeMath(args, new MathFunc(Math.Floor)).ToString();

            string[] range = args.Split(new char[] { ParseArgumentSeparator });
            int argCount = range.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double d1, d2, d3 = 0;
            if (double.TryParse(GetValueFromArg(range[0]), out d1)
                && double.TryParse(GetValueFromArg(range[1]), out d2))
            {
                if (d1 * d2 <= 0)
                {
                    return "#NUM!";
                }

                d3 = Math.Ceiling(d1 / d2) * d2;
                if (d2 > 0)
                {
                    while (d3 > d1)
                    {
                        d3 -= d2;
                    }
                }
                else
                {
                    while (d3 < d1)
                    {
                        d3 -= d2;
                    }
                }
            }

            return d3.ToString();
        }

        /// <summary>
        /// Computes the natural logarithm of the value in the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding natural logarithm of the value in the argument.</returns>
        public string ComputeLn(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Log)).ToString();
        }

        /// <summary>
        /// Computes the logarithm of the first value using the second value as the base.
        /// </summary>
        /// <param name="argList">A cell reference, formula, or number.</param>
        /// <returns>A string holding logarithm of the value in the argument using the second argument as the base.</returns>
        public string ComputeLog(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 1 && argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x = 10.0;
            double bas = 10.0;
            if (double.TryParse(GetValueFromArg(args[0]), out x)
                && x > 0 &&
                (argCount == 1 || double.TryParse(GetValueFromArg(args[1]), out bas)))
            {
                return Math.Log(x, bas).ToString();
            }
            else
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }
        }

        /// <summary>
        /// Computes the base 10 logarithm of the value in the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding base 10 logarithm of the value in the argument.</returns>
        public string ComputeLog10(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Log10)).ToString();
        }

        /// <summary>
        /// Computes e raised to the value of the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the e raised to the value of the argument.</returns>
        public string ComputeExp(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Exp)).ToString();
        }

        /// <summary>
        /// Computes the square root of the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the square root of the argument.</returns>
        public string ComputeSqrt(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Sqrt)).ToString();
        }

        /// <summary>
        /// Computes the absolute value of the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the absolute value of the argument.</returns>
        public string ComputeAbs(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Abs)).ToString();
        }

        /// <summary>
        /// Computes the tangent the argument.
        /// </summary>
        /// <param name="args">A cell reference, formula, or number.</param>
        /// <returns>A string holding the tangent of the argument.</returns>
        public string ComputeTan(string args)
        {
            return ComputeMath(args, new MathFunc(Math.Tan)).ToString();
        }

        private Random rand;

        /// <summary>
        /// Returns an evenly distributed random number greater than or equal
        /// zero and less than one.
        /// </summary>
        /// <param name="args">Ignored. Can be empty.</param>
        /// <returns>A string holding the random number.</returns>
        public string ComputeRand(string args)
        {
            if (rand == null)
            {
                rand = new Random();
            }

            return rand.NextDouble().ToString("0.0#############");
        }

        /// <summary>
        /// Computes the sum of range2
        /// if the item in the range1 satisfies the condition.
        /// </summary>
        /// <param name="range">Range1, condition, range2.</param>
        /// <returns>The conditional sum.</returns>
        public string ComputeSumIf(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            string criteriaRange = args[0];
            string criteria = args[1]; ////.Replace(TIC, string.Empty);

            if (criteria.Length > 1 && criteria[0] == TIC[0]
                && "=><".IndexOf(criteria[1]) == -1)
            {
                criteria = "=" + criteria;
            }
            else
            {
                criteria = criteria.Replace(TIC, string.Empty);
            }

            string sumRange = (argCount == 2) ? criteriaRange : args[2];

            string[] s1 = this.GetCellsFromArgs(criteriaRange);
            string[] s2 = this.GetCellsFromArgs(sumRange);

            double sum = 0;
            int count = s1.GetLength(0);
            double d;
            double[] vector = new double[count];
            string s;

            for (int index = 0; index < count; ++index)
            {
                s = s1[index] + criteria;
                ////s = this.ParseSimple(s);
                s = this.Parse(s);
                s = this.ComputedValue(s);
                if (s == this.TRUEVALUESTR)
                {
                    s = s2[index];
                    s = GetValueFromArg(s);
                    if (double.TryParse(s, out d))
                    {
                        sum += d;
                    }
                }
            }

            return sum.ToString();
            ////            string[] ss = range.Split(new char[]{ParseArgumentSeparator});
            ////            if(ss.GetLength(0) != 3) 
            ////            {
            ////                return FormulaErrorStrings[requires_3_args];
            ////            }
            ////            else 
            ////            {
            ////                string[] s1 = this.GetCellsFromArgs(ss[0]);
            ////                string[] s2 = this.GetCellsFromArgs(ss[2]);
            ////
            ////            
            ////                double sum = 0;
            ////                int count = s1.GetLength(0);
            ////                double d;
            ////                double[] vector = new double[count];
            ////                string s;
            ////                
            ////                for(int index = 0; index < count; ++index)
            ////                {
            ////                    s = s1[index] + ss[1];
            ////                    s = this.ParseSimple(s);
            ////                    s = this.ComputeAbs(s);
            ////                    if(double.TryParse(s , NumberStyles.Any, null, out d) && d > 0)
            ////                    {
            ////                        s = s2[index];
            ////                        s = GetValueFromArg(s);
            ////                        if(double.TryParse(s , NumberStyles.Any, null, out d))
            ////                        {
            ////                            sum += d;
            ////                        }
            ////                        
            ////                    }
            ////                }
            ////                
            ////
            ////                return sum.ToString();
            ////            }
        }

        ////New functions start

        /// <summary>
        /// Computes the declining balance of an asset.
        /// </summary>
        /// <param name="argList"> Delimited string containing the initial cost, 
        /// salvage value, life of asset, period of calculation, and months in initial year.</param>
        /// <returns>returns declining balance of an asset.</returns>
        public string ComputeDb(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 5 && argCount != 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double cost;
            double salvage;
            double life;
            double period;
            double month = 12;

            double deprec = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out cost)
                && double.TryParse(args[1], out salvage)
                && double.TryParse(args[2], out life)
                && double.TryParse(args[3], out period)
                && (argCount == 4 || double.TryParse(args[4], out month)))
            {
                double rate = Math.Round(1 - Math.Pow(salvage / cost, 1 / life), 3);
                double priorDeprec = 0;
                for (int i = 1; i <= period; ++i)
                {
                    if (i == 1)
                    {
                        deprec = cost * rate * month / 12;
                    }
                    else if (i > life)
                    {
                        deprec = (cost - priorDeprec) * rate * (12 - month) / 12;
                    }
                    else
                    {
                        deprec = (cost - priorDeprec) * rate;
                    }

                    priorDeprec += deprec;
                }
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return deprec.ToString();
        }

        /// <summary>
        /// Computes the double declining balance of an asset.
        /// </summary>
        /// <param name="argList"> Delimited string containing the initial cost, 
        /// salvage value, life of asset, period of calculation, factor.</param>
        /// <returns>returns double declining balance of an asset</returns>
        public string ComputeDdb(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 5 && argCount != 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double cost;
            double salvage;
            double life;
            double period;
            double factor = 2;

            double deprec = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out cost)
                && double.TryParse(args[1], out salvage)
                && double.TryParse(args[2], out life)
                && double.TryParse(args[3], out period)
                && (argCount == 4 || double.TryParse(args[4], out factor)))
            {
                double rate = factor / life;
                double priorDeprec = 0;
                for (int i = 1; i <= period; ++i)
                {
                    if (i == life)
                    {
                        deprec = cost - salvage - priorDeprec;
                    }
                    else
                    {
                        deprec = (cost - priorDeprec) * rate;
                    }

                    priorDeprec += deprec;
                }
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return deprec.ToString();
        }

        /// <summary>
        /// Computes the future value of an investment.
        /// </summary>
        /// <param name="argList"> Delimited string containing the rate as percentage per period, 
        /// number of periods, payment per period, present value, and payment type (0 = end of period, 1 = start of period).</param>
        /// <returns>returns future value of an investment</returns>
        public string ComputeFv(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 5 && argCount != 4 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double rate;
            double nper;
            double pmt = 0;
            double pv = 0;
            double type = 0;

            double fv = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out rate)
                && double.TryParse(args[1], out nper)
                && double.TryParse(args[2], out pmt))
            {
                if (argCount >= 4)
                {
                    double.TryParse(args[3], out pv);
                }

                if (argCount == 5)
                {
                    double.TryParse(args[4], out type);
                }

                if (Math.Abs(type) > 0.5)
                {
                    type = 1;
                }
                else
                {
                    type = 0;
                }
                ////FV   = (PMT*(1+rate*type)*(1-(1+ rate)^NPER)/rate)-PV*(1+rate)^NPER
                double pow = Math.Pow((1 + rate), nper);
                fv = (pmt * (1 + rate * type) * (1 - pow) / rate) - (pv * pow);
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return fv.ToString();
        }

        /// <summary>
        /// Computes the interest payment for a period.
        /// </summary>
        /// <param name="argList"> Delimited string containing the rate as percentage per period, the period,
        /// number of periods, present value, future value, and payment type (0 = end of period, 1 = start of period).</param>
        /// <returns>Interest payment.</returns>
        public string ComputeIpmt(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 6 && argCount != 5 && argCount != 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double rate;
            double per;
            double nper;
            double pv = 0;
            double fv = 0;
            double type = 0;

            double impt = 0;
            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out rate)
                && double.TryParse(args[1], out per)
                && double.TryParse(args[2], out nper)
                && double.TryParse(args[3], out pv)
                && (argCount == 4 || double.TryParse(args[4], out fv))
                && (argCount <= 5 || double.TryParse(args[5], out type)))
            {
                if (Math.Abs(type) > 0.5)
                {
                    type = 1;
                }
                else
                {
                    type = 0;
                }

                double x0 = Math.Pow((1 + rate), nper);
                double x1 = Math.Pow((1 + rate), per);
                double pmt = (rate * (fv + pv * x0)) / ((1 + rate * type) * (1 - x0));
                double fv1 = (pmt * (1 + rate * type) * (1 - x1) / rate) - pv * x1;
                double x2 = Math.Pow((1 + rate), per - 1);
                double fv2 = (pmt * (1 + rate * type) * (1 - x2) / rate) - pv * x2;

                impt = pmt - fv2 + fv1;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return impt.ToString();
        }

        /// <summary>
        /// Computes the internal rate of return of a series of cash flows.
        /// </summary>
        /// <param name="argList"> Delimited string containing a range of cells and an initial guess.</param>
        /// <returns>Internal rate of return.</returns>
        /// <remarks>
        /// This IRR calculation uses Newton's method to approximate a root of 
        ///           f(r) = Sum( values[i]/(1+r)^i) = 0
        /// where the Sum index is i = 1 to the number of values. The algorithm returns a value if 
        /// the relative difference between root approximations is less than 1e-5. It fails if this
        /// accuracy is not attained in 20 iterations.
        /// </remarks>
        public string ComputeIrr(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 1 && argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double guess = .1;
            string arg1 = args[0];
            if (argCount == 2)
            {
                double.TryParse(GetValueFromArg(args[1]), out guess);
            }

            args = GetCellsFromArgs(arg1);
            argCount = args.GetLength(0);
            double[] values = new double[argCount];
            int i = 0;
            double d;
            foreach (string s in args)
            {
                if (double.TryParse(GetValueFromArg(s), out d))
                {
                    values[i] = d;
                }

                ++i;
            }

            int iteration = 0;
            double gp1, numer, denom, powgp1;
            ////hard coded excel values
            while (iteration < 20)
            {
                numer = 0;
                denom = 0;
                gp1 = Math.Abs(guess + 1);
                powgp1 = gp1;

                for (i = 0; i < argCount; ++i)
                {
                    numer += values[i] / powgp1;
                    powgp1 *= gp1;
                    denom += (i + 1) * values[i] / powgp1;
                }

                numer = numer / denom;
                ////hard coded excel values
                if (Math.Abs(numer / guess) < 1e-5)
                {
                    guess = guess + numer;
                    break;
                }

                guess = guess + numer;
                iteration++;
            }
            ////hard coded excel values
            if (iteration >= 20)
            {
                return FormulaErrorStrings[invalid_arguments];
            }
            else
            {
                return guess.ToString();
            }
        }

        ////used in XIRR calc
        private bool GetDaySerialNumberFromDateTime(string s, out double days)
        {
            bool b = true;
            DateTime dt = DateTime.Now;
#if !(SyncfusionFramework1_0 || SyncfusionFramework1_1)
            b = DateTime.TryParse(s, out dt);
#else
            try
            {
                dt = Convert.ToDateTime(s);
                days = 1 + ((TimeSpan)(dt.Date - dateTime1900)).Days;
                if(Treat1900AsLeapYear && days > 59)
                    days += 1;
            }
            catch 
            {
                b = false;
            }
#endif
            days = 1 + ((TimeSpan)(dt.Date - dateTime1900)).Days;
            if (Treat1900AsLeapYear && days > 59)
            {
                days += 1;
            }

            return b;
        }

        private double[] guesses = new double[] { .1, -.1, .2, -.2, .3, -.3, .4, -.4, .5, -.5, .6, -.6, .7, -.7, .8, -.8, .9, -.9 };

        /// <summary>
        /// Computes the internal rate of return of a series of cash flows.
        /// </summary>
        /// <param name="argList">values, dates, guess. Values and dates are ranges of cells holding the values and dates.
        /// Guess is the initial guess. The first date is the the start date for the calculation.</param>
        /// <returns>Internal rate of return.</returns>
        /// <remarks>
        /// This XIRR calculation is similar to IRR except that the values are not equally spaced in time.
        ///  The algorithm returns a value if 
        /// the relative difference between root approximations is less than 1e-5. It fails if this
        /// accuracy is not attained in 20 iterations.
        /// </remarks>
        public string ComputeXirr(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double guess = .1;
            string arg1 = args[0];
            string arg2 = args[1];
            if (argCount == 3)
            {
                double.TryParse(GetValueFromArg(args[2]), out guess);
            }

            if (Math.Abs(guess) >= 1)
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            double initialGuess = guess;
            int numberOfGuessAdjustments = 0;
            args = GetCellsFromArgs(arg1);
            argCount = args.GetLength(0);
            double[] values = new double[argCount];
            int i = 0;
            double d;
            foreach (string s in args)
            {
                if (double.TryParse(GetValueFromArg(s), out d))
                {
                    values[i] = d;
                }

                ++i;
            }

            args = GetCellsFromArgs(arg2);
            double[] dates = new double[argCount];
            i = 0;
            foreach (string s in args)
            {
                string s1 = GetValueFromArg(s);
                if (double.TryParse(s1, out d)
                    || GetDaySerialNumberFromDateTime(s1, out d))
                {
                    dates[i] = d;
                }

                ++i;
            }

            int iteration = 0;
            double gp1, exp, pow, f, fprime;
            double diff = 0;
            double oldF = 0, oldGuess = 0;
            ////hard coded excel values
            while (iteration < 100)
            {
                f = 0;
                fprime = 0;
                gp1 = guess + 1;
                for (i = 0; i < argCount; ++i)
                {
                    exp = (dates[0] - dates[i]) / 365.0;
                    pow = Math.Pow(Math.Abs(gp1), exp);
                    f += values[i] * pow;
                    fprime += exp * values[i] * pow / gp1;
                }

                oldGuess = guess;
                oldF = f;
                f = f / fprime;
                ////hard coded excel values
                if (Math.Abs(f / guess) < 1e-6)
                {
                    guess = guess - f;
                    break;
                }
                ////truncate guess diff at one
                if (f > 1)
                {
                    f = f / 2;
                }
                else if (f < -2)
                {
                    f = -2;
                }

                if (numberOfGuessAdjustments < guesses.Length && iteration > 10 && Math.Abs(f / guess) > diff)
                {
                    guess = guesses[numberOfGuessAdjustments];
                    numberOfGuessAdjustments++;
                    iteration = 0;
                }
                else
                {
                    diff = Math.Abs(f / guess);
                    guess = guess - f;
                }

                iteration++;
            }
            ////hard coded excel values
            if (iteration >= 100)
            {
                return FormulaErrorStrings[invalid_arguments];
            }
            else
            {
                return guess.ToString();
            }
        }

        /// <summary>
        /// Computes the simple interest payment.
        /// </summary>
        /// <param name="argList"> Delimited string containing the rate as percentage per period, the period,
        /// number of periods, and present value.</param>
        /// <returns>Simple interest payment.</returns>
        public string ComputeIspmt(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double rate;
            double per;
            double nper;
            double pv = 0;

            double impt = 0;
            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out rate)
                && double.TryParse(args[1], out per)
                && double.TryParse(args[2], out nper)
                && double.TryParse(args[3], out pv))
            {
                impt = -rate * pv * (nper - per) / nper;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return impt.ToString();
        }

        /// <summary>
        /// Computes the modified internal rate of return of a series of cash flows.
        /// </summary>
        /// <param name="argList"> Delimited string containing a range of cells, 
        /// finance interest rate, and a reinvest interest rate.</param>
        /// <returns>Modified internal rate of return.</returns>
        public string ComputeMirr(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double val = 0;
            double frate = .1;
            double rrate = .1;
            double.TryParse(GetValueFromArg(args[1]), out frate);
            double.TryParse(GetValueFromArg(args[2]), out rrate);

            string arg1 = args[0];
            args = GetCellsFromArgs(arg1);
            argCount = args.GetLength(0);
            double[] values = new double[argCount];
            int i = 0;
            double d;
            foreach (string s in args)
            {
                if (double.TryParse(GetValueFromArg(s), out d))
                {
                    values[i] = d;
                }

                ++i;
            }

            double posValues = 0;
            double negValues = 0;
            double rpow = 1;
            double fpow = 1 + frate;

            for (i = 0; i < argCount; ++i)
            {
                rpow *= 1 + rrate;

                if (values[i] > 0)
                {
                    posValues += values[i] / rpow;
                }
                else
                {
                    negValues += values[i] / fpow;
                }

                fpow *= 1 + frate;
            }

            posValues = -posValues * rpow;
            negValues = negValues * (1 + frate);
            val = Math.Pow(posValues / negValues, 1d / (argCount - 1d)) - 1d;
            return val.ToString();
        }

        /// <summary>
        /// Computes the number of periods in an investment.
        /// </summary>
        /// <param name="argList">Delimited string containing the rate as percentage per period, 
        /// payment per period, present value, future value, and payment type (0 = end of period, 1 = start of period).</param>
        /// <returns>Number of periods.</returns>
        public string ComputeNper(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 5 && argCount != 4 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double rate;
            double pmt = 0;
            double pv = 0;
            double fv = 0;
            double type = 0;

            double val = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out rate)
                && double.TryParse(args[1], out pmt)
                && double.TryParse(args[2], out pv)
                && (argCount == 3 || double.TryParse(args[3], out fv))
                && (argCount <= 4 || double.TryParse(args[4], out type)))
            {
                if (Math.Abs(type) > 0.5)
                {
                    type = 1;
                }
                else
                {
                    type = 0;
                }

                ////NPER = LOG10((PMT*(1+rate*type)-FV*rate)/(PMT*(1+rate*type)+PV*rate))/ LOG10(1+rate)
                val = Math.Log10((pmt * (1 + rate * type) - fv * rate)
                    / (pmt * (1 + rate * type) + pv * rate)) / Math.Log10(1 + rate);
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return val.ToString();
        }

        /// <summary>
        /// Computes the net present value of an investment.
        /// </summary>
        /// <param name="argList">Delimited string containing the rate as percentage per period  
        /// and a list of invested values.</param>
        /// <returns>Net present value.</returns>
        public string ComputeNpv(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount < 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double rate;
            double.TryParse(GetValueFromArg(args[0]), out rate);

            double val = 0;
            double denom = 1;
            double d;

            for (int i = 1; i < argCount; ++i)
            {
                string r = args[i];
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        if (double.TryParse(GetValueFromArg(s), out d))
                        {
                            denom *= 1 + rate;
                            val += d / denom;
                        }
                    }
                }
                else
                {
                    if (double.TryParse(GetValueFromArg(r), out d))
                    {
                        denom *= 1 + rate;
                        val += d / denom;
                    }
                }
            }

            return val.ToString();
        }

        /// <summary>
        /// Computes the payment for a loan.
        /// </summary>
        /// <param name="argList">Delimited string containing the rate as percentage per period, 
        /// number of periods, present value, future value, and payment type (0 = end of period, 1 = start of period).</param>
        /// <returns>Payment amount.</returns>
        public string ComputePmt(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 5 && argCount != 4 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double rate;
            double nper = 0;
            double pv = 0;
            double fv = 0;
            double type = 0;

            double val = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out rate)
                && double.TryParse(args[1], out nper)
                && double.TryParse(args[2], out pv)
                && (argCount == 3 || double.TryParse(args[3], out fv))
                && (argCount <= 4 || double.TryParse(args[4], out type)))
            {
                if (Math.Abs(type) > 0.5)
                {
                    type = 1;
                }
                else
                {
                    type = 0;
                }

                ////PMT  = (rate*(FV+PV*(1+ rate)^NPER))/((1+rate*type)*(1-(1+ rate)^NPER))

                double pow = Math.Pow(1 + rate, nper);
                val = (rate * (fv + pv * pow)) / ((1 + rate * type) * (1 - pow));
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return val.ToString();
        }

        /// <summary>
        /// Computes the principal payment for a period.
        /// </summary>
        /// <param name="argList"> Delimited string containing the rate as percentage per period, the period,
        /// number of periods, present value, future value, and payment type (0 = end of period, 1 = start of period).</param>
        /// <returns>Principal payment.</returns>
        public string ComputePpmt(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 6 && argCount != 5 && argCount != 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double rate;
            double per;
            double nper;
            double pv = 0;
            double fv = 0;
            double type = 0;

            double ppmt = 0;
            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out rate)
                && double.TryParse(args[1], out per)
                && double.TryParse(args[2], out nper)
                && double.TryParse(args[3], out pv)
                && (argCount == 4 || double.TryParse(args[4], out fv))
                && (argCount <= 5 || double.TryParse(args[5], out type)))
            {
                if (Math.Abs(type) > 0.5)
                {
                    type = 1;
                }
                else
                {
                    type = 0;
                }

                double x0 = Math.Pow((1 + rate), nper);
                ////double x1 = Math.Pow((1 + rate), per);
                double pmt = (rate * (fv + pv * x0)) / ((1 + rate * type) * (1 - x0));
                double ipmt = 0;
                 double.TryParse(ComputeIpmt(argList), out ipmt);
                ////double fv1 = (pmt*(1 + rate * type) * (1 - x1) /rate) - pv * x1;
                ////double x2 = Math.Pow((1 + rate), per - 1);
                ////double fv2 = (pmt*(1 + rate * type) * (1 - x2) /rate) - pv * x2;

                ppmt = pmt - ipmt;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return ppmt.ToString();
        }

        /// <summary>
        /// Computes the present value of an investment.
        /// </summary>
        /// <param name="argList"> Delimited string containing the rate as percentage per period, 
        /// number of periods, payment per period, future value, and payment type (0 = end of period, 1 = start of period).</param>
        /// <returns>Present value.</returns>
        public string ComputePv(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 5 && argCount != 4 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double rate;
            double nper;
            double pmt = 0;
            double fv = 0;
            double type = 0;

            double pv = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out rate)
                && double.TryParse(args[1], out nper)
                && double.TryParse(args[2], out pmt))
            {
                if (argCount >= 4)
                {
                    double.TryParse(args[3], out pv);
                }

                if (argCount == 5)
                {
                    double.TryParse(args[4], out type);
                }

                if (Math.Abs(type) > 0.5)
                {
                    type = 1;
                }
                else
                {
                    type = 0;
                }

                if (Math.Abs(type) > 0.5)
                {
                    type = 1;
                }
                else
                {
                    type = 0;
                }

                ////PV   = (PMT*(1+rate*type)*(1-(1+rate)^NPER)-rate*FV)/(rate*(1+rate)^NPER)
                double pow = Math.Pow(1 + rate, nper);
                pv = (pmt * (1 + rate * type) * (1 - pow) - rate * fv) / (rate * pow);
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return pv.ToString();
        }

        /// <summary>
        /// Computes the internal rate of return of a series of cash flows.
        /// </summary>
        /// <param name="argList"> Delimited string containing a range of cells and an initial guess.</param>
        /// <returns>Internal rate of return.</returns>
        /// <remarks>
        /// This IRR calculation uses Newton's method to approximate a root of 
        ///           f(r) = Sum( values[i]/(1+r)^i) = 0
        /// where the Sum index is i = 1 to the number of values. The algorithm returns a value if 
        /// the relative difference between root approximations is less than 1e-7. It fails if this
        /// accuracy is not attained in 20 iterations.
        /// </remarks>
        public string ComputeRate(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 6 && argCount != 5 && argCount != 4 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double nper;
            double pmt;
            double pv;
            double fv = 0;
            double type = 0;
            double r = .1;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out nper)
                && double.TryParse(args[1], out pmt)
                && double.TryParse(args[2], out pv)
                && (argCount == 3 || double.TryParse(args[3], out fv))
                && (argCount <= 4 || double.TryParse(args[4], out type))
                && (argCount <= 5 || double.TryParse(args[5], out r)))
            {
                if (Math.Abs(type) > 0.5)
                {
                    type = 1;
                }
                else
                {
                    type = 0;
                }

                //// seeking root of f(r) = 0 where
                //// f(r) = pmt - u(r) / v(r) where
                //// u(r) = r * (fv + pv * (1+r)^nper
                //// v(r) = (1 + r * type) * (1 - (1+r)^nper);

                int iteration = 0;
                double ur,  ////u(r)
                    vr,     ////v(r)
                    upr,    ////u'(r)
                    vpr,    ////v'(r)
                    fr,     ////f(r)
                    fpr,    ////f'(r)
                    rp1,    //// r + 1
                    rn,     //// (1 + r)^nper
                    rn1,    //// (1 + r)^(nper-1)
                    h;      //// h from Taylor series expansion of f(r+h).
                ////hard coded excel values  
                while (iteration < 20)
                {
                    rp1 = Math.Abs(r + 1);
                    rn = Math.Pow(rp1, nper);
                    rn1 = Math.Pow(rp1, nper - 1);

                    ur = r * (fv + pv * rn);
                    vr = (1 + r * type) * (1 - rn);
                    upr = fv + pv * (rn + r * nper * rn1);
                    vpr = (1 + r * type) * (-nper) * rn1 + (1 - rn) * type;
                    fpr = (upr * vr - ur * vpr) / (vr * vr);
                    fr = pmt - ur / vr;
                    h = fr / fpr;
                    ////hard coded excel values
                    if (Math.Abs(h) < 1e-7)
                    {
                        r = r + h;
                        break;
                    }

                    r = r + h;
                    iteration++;
                }
                ////hard coded excel values
                if (iteration < 20)
                {
                    return r.ToString();
                }
            }

            return FormulaErrorStrings[invalid_arguments];
        }

        /// <summary>
        /// Computes the straight-line depreciation of an asset per period.
        /// </summary>
        /// <param name="argList"> Delimited string containing the cost, 
        /// salvage value, and life.</param>
        /// <returns>returns Depreciation.</returns>
        public string ComputeSln(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[this.requires_3_args];
            }

            double cost;
            double salvage;
            double life;

            double sln = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out cost)
                && double.TryParse(args[1], out salvage)
                && double.TryParse(args[2], out life))
            {
                sln = (cost - salvage) / life;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return sln.ToString();
        }

        /// <summary>
        /// Computes the sum of years digits depreciation of an asset per period.
        /// </summary>
        /// <param name="argList"> Delimited string containing the cost, 
        /// salvage value, life, and period.</param>
        /// <returns>Depreciation for the requested period.</returns>
        public string ComputeSyd(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 4)
            {
                return FormulaErrorStrings[this.wrong_number_arguments];
            }

            double cost;
            double salvage;
            double life;
            double per;

            double syd = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out cost)
                && double.TryParse(args[1], out salvage)
                && double.TryParse(args[2], out life)
                && double.TryParse(args[3], out per))
            {
                syd = (cost - salvage) * (life - per + 1) * 2 / (life * (life + 1));
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return syd.ToString();
        }

        /// <summary>
        /// Computes the variable declining balance of an asset
        /// </summary>
        /// <param name="argList"> Delimited string containing the initial cost, 
        /// salvage value, life of asset, period of calculation, factor.</param>
        /// <returns>Variable declining balance.</returns>
        public string ComputeVdb(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 5 && argCount != 6 && argCount != 7)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double cost;
            double salvage;
            double life;
            double start;
            double end;
            double factor = 2;
            double no_switch = 0;

            double deprec = 0;
            double vdb = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out cost)
                && double.TryParse(args[1], out salvage)
                && double.TryParse(args[2], out life)
                && double.TryParse(args[3], out start)
                && double.TryParse(args[4], out end)
                && (argCount == 5 || double.TryParse(args[5], out factor))
                && (argCount <= 6 || double.TryParse(args[6], out no_switch)))
            {
                double rate = factor / life;
                double priorDeprec = 0;
                double sldeprec = (cost - salvage) / life;

                bool change = Math.Abs(no_switch) > .5;

                ////for(int i = 1; i <= end; ++i)
                int i = 0;
                while (i < end)
                {
                    i += 1;
                    if (i == life)
                    {
                        deprec = cost - salvage - priorDeprec;
                    }
                    else
                    {
                        deprec = (cost - priorDeprec) * rate;
                        if (change && deprec < sldeprec)
                        {
                            deprec = sldeprec;
                        }
                    }

                    priorDeprec += deprec;
                    if (i > start && i <= end)
                    {
                        vdb += deprec;
                    }
                    else if (i - end < .99 && i - end > 0)
                    {
                        ////1                    
                        vdb += deprec * (end - i + 1);
                    }
                }
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return vdb.ToString();
        }

        /// <summary>
        /// The inverse of Cosh.
        /// </summary>
        /// <param name="args">Value >= 1.</param>
        /// <returns>returns ACosh(value).</returns>
        public string ComputeAcosh(string args)
        {
            double z;
            if (double.TryParse(GetValueFromArg(args), out z)
                && z >= 1)
            {
                z = Math.Log(z + Math.Sqrt(z * z - 1));
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return z.ToString();
        }

        /// <summary>
        /// The inverse of Sinh.
        /// </summary>
        /// <param name="args">The Value.</param>
        /// <returns>returns ASinh(value).</returns>
        public string ComputeAsinh(string args)
        {
            double z;
            if (double.TryParse(GetValueFromArg(args), out z))
            {
                z = Math.Sign(z) * Math.Log(Math.Abs(z) + Math.Sqrt(z * z + 1));
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return z.ToString();
        }

        /// <summary>
        /// The inverse of Tanh.
        /// </summary>
        /// <param name="args">|Value| &lt; 1.</param>
        /// <returns>returns ATanh(value).</returns>
        public string ComputeAtanh(string args)
        {
            double z;
            double z1;
            if (double.TryParse(GetValueFromArg(args), out z)
                && ((z1 = Math.Abs(z)) < 1))
            {
                z = .5 * Math.Sign(z) * Math.Log((1 + z1) / (1 - z1));
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return z.ToString();
        }

        /// <summary>
        /// The ArcTangent of the x and y values. 
        /// </summary>
        /// <param name="argList">x_value and y_value.</param>
        /// <returns>Angle whose tangent is y_value/x_value.</returns>
        public string ComputeAtan2(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[this.requires_2_args];
            }

            double x;
            double y;
            double atan2 = 0;
            if (double.TryParse(GetValueFromArg(args[0]), out x)
                && double.TryParse(GetValueFromArg(args[1]), out y))
            {
                atan2 = Math.Atan2(y, x);
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return atan2.ToString();
        }

        /// <summary>
        /// The number of combinations of a given number of items. 
        /// </summary>
        /// <param name="argList">number, number_items.</param>
        /// <returns>The number of combinations.</returns>
        public string ComputeCombin(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[this.requires_2_args];
            }

            ////n things taken k at the time
            double nd;
            double kd;
            double combin = 0;
            if (double.TryParse(GetValueFromArg(args[0]), out nd)
                && double.TryParse(GetValueFromArg(args[1]), out kd))
            {
                int k = (int)(kd + 0.1);
                int n = (int)(nd + 0.1);
                combin = comb(k, n);
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return combin.ToString();
        }

        /// <summary>
        /// Converts radians into degrees.
        /// </summary>
        /// <param name="args">Value in radians.</param>
        /// <returns>returns Degrees.</returns>
        public string ComputeDegrees(string args)
        {
            double radians;
            double degrees = 0;
            if (double.TryParse(GetValueFromArg(args), out radians))
            {
                degrees = 180 * radians / Math.PI;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return degrees.ToString();
        }

        /// <summary>
        /// Rounds up to larger in magnitude even number.
        /// </summary>
        /// <param name="args">Number to be rounded.</param>
        /// <returns>Rounded even value.</returns>
        public string ComputeEven(string args)
        {
            double number;
            double even = 0;
            if (double.TryParse(GetValueFromArg(args), out number))
            {
                int sgn = Math.Sign(number);
                number = Math.Abs(number);
                if ((int)number != number)
                {
                    number = (int)(number + 1);
                }

                if ((number % 2) == 1)
                {
                    even = sgn * (number + 1);
                }
                else
                {
                    even = sgn * number;
                }
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return even.ToString();
        }

        private static int[] factorialTable = new int[]
        {
                                                           1, 1, 2, 6, 24, 120, 720, 
                                                           5040, 40320, 362880, 3628800, 
                                                           39916800, 479001600
        };

        /// <summary>
        /// Factorial of a given number.
        /// </summary>
        /// <param name="args">The x args.</param>
        /// <returns>returns x!.</returns>
        public string ComputeFact(string args)
        {
            double number = 0;
            double fact = 0;
            if (double.TryParse(GetValueFromArg(args), out number)
                && number >= 0)
            {
                int x = (int)number;
                if (x > 12)
                {
                    fact = factorialTable[12];
                    for (int i = 13; i <= x; i++)
                    {
                        fact *= i;
                    }
                }
                else
                {
                    fact = factorialTable[x];
                }
            }
            else if (number < 0)
            {
                return "#NUM!";
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return fact.ToString();
        }

        /// <summary>
        /// Rounds up to larger in magnitude odd number.
        /// </summary>
        /// <param name="args">Number to be rounded.</param>
        /// <returns>Rounded odd value.</returns>
        public string ComputeOdd(string args)
        {
            double number;
            double odd = 0;
            if (double.TryParse(GetValueFromArg(args), out number))
            {
                int sgn = Math.Sign(number);
                number = Math.Abs(number);
                if ((int)number != number)
                {
                    number = (int)(number + 1);
                }

                if ((number % 2) == 0)
                {
                    odd = sgn * (number + 1);
                }
                else
                {
                    odd = sgn * number;
                }
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return odd.ToString();
        }

        /// <summary>
        /// Converts degrees into radians.
        /// </summary>
        /// <param name="args">Value in degrees.</param>
        /// <returns>returns Radians.</returns>
        public string ComputeRadians(string args)
        {
            double radians = 0;
            double degrees;
            if (double.TryParse(GetValueFromArg(args), out degrees))
            {
                radians = Math.PI * degrees / 180;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return radians.ToString();
        }

        /// <summary>
        /// Rounds a number to a specified number of digits.
        /// </summary>
        /// <param name="argList">Number and number of digits.</param>
        /// <returns>Rounded number.</returns>
        public string ComputeRound(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount == 1)
            {
                return ComputeMath(argList, new MathFunc(Math.Round)).ToString();
            }

            if (argCount != 2)
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double x = 0;
            double digits = 0;
            double round = 0;
            string numStr = GetValueFromArg(args[0]);
            string digStr = GetValueFromArg(args[1]);

            if ((numStr.Length == 0 || double.TryParse(numStr, out x))
            && (digStr.Length == 0 || double.TryParse(digStr, out digits)))
            {
                if (digits > 0)
                {
                    round = Math.Round(x, (int)digits);
                }
                else
                {
                    double mult = Math.Pow(10, -digits);
                    round = Math.Round(x / mult) * mult;
                }
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return round.ToString();
        }

        /// <summary>
        /// Rounds a number to a specified number of digits.
        /// </summary>
        /// <param name="argList">Number and number of digits.</param>
        /// <returns>Rounded number.</returns>
        public string ComputeRounddown(string argList)
        {
            double x = 0;

            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount == 1)
            {
                double.TryParse(GetValueFromArg(argList), out x);
                x = x - .5 * Math.Sign(x);
                return ComputeRound(string.Format("{0}", x));
            }
            else if (argCount != 2)
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double digits = 0;
            double.TryParse(GetValueFromArg(args[0]), out x);
            double.TryParse(GetValueFromArg(args[1]), out digits);
            x = x - .5 * Math.Pow(10, -digits) * Math.Sign(x);
            return ComputeRound(string.Format("{0}{1}{2}", x, ParseArgumentSeparator, digits));
        }

        /// <summary>
        /// Rounds a number to a specified number of digits.
        /// </summary>
        /// <param name="argList">Number and number of digits.</param>
        /// <returns>Rounded number.</returns>
        public string ComputeRoundup(string argList)
        {
            double x = 0;

            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount == 1)
            {
                 double.TryParse(GetValueFromArg(argList), out x);
                if (x > 0)
                {
                    x += .4999999999d;
                }
                else if (x < 0)
                {
                    x -= .4999999999d;
                }

                x = Math.Round(x, 0, MidpointRounding.AwayFromZero);
                return string.Format("{0}", x);
            }
            else if (argCount != 2)
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double digits = 0;
             double.TryParse(GetValueFromArg(args[0]), out x);
            double.TryParse(GetValueFromArg(args[1]), out digits);
            if (digits >= 0)
            {
                if (x > 0)
                {
                    x += .4999999999d / Math.Pow(10, digits);
                }
                else if (x < 0)
                {
                    x -= .4999999999d / Math.Pow(10, digits);
                }

                x = Math.Round(x, (int)digits, MidpointRounding.AwayFromZero);
            }
            else
            {
                if (x > 0)
                {
                    x = (x / Math.Pow(10, -digits)) + .49999;
                }
                else if (x < 0)
                {
                    x = (x / Math.Pow(10, -digits)) - .49999;
                }

                x = Math.Round(x, 0, MidpointRounding.AwayFromZero) * Math.Pow(10, -digits);
            }

            return string.Format("{0}", x);
        }

        private string TRUEVALUESTR = "TRUE";
        private string FALSEVALUESTR = "FALSE";

        ////        /// <summary>
        ////        /// Sums the cells specified by some criteria.
        ////        /// </summary>
        ////        /// <param name="range">The criteria range, the criteria, and the sum range.</param>
        ////        public string ComputeSumif(string argList)
        ////        {
        ////            string[] args = argList.Split(new char[]{ParseArgumentSeparator});
        ////            int argCount = args.GetLength(0);
        ////            if(argCount != 2 && argCount != 3)
        ////            {
        ////                return FormulaErrorStrings[wrong_number_arguments];
        ////            }
        ////            string criteriaRange = args[0];
        ////            string criteria = args[1]; ////.Replace(TIC, string.Empty);
        ////
        ////            if(criteria.Length > 1 && criteria[0] == TIC[0]
        ////                && "=><".IndexOf(criteria[1]) == -1)
        ////                criteria = "=" + criteria;
        ////            else
        ////                criteria = criteria.Replace(TIC, string.Empty);
        ////
        ////
        ////            string sumRange = (argCount == 2) ? criteriaRange : args[2];
        ////            
        ////            
        ////            
        ////            string[] s1 = this.GetCellsFromArgs(criteriaRange);
        ////            string[] s2 = this.GetCellsFromArgs(sumRange);
        ////
        ////            
        ////            double sum = 0;
        ////            int count = s1.GetLength(0);
        ////            double d;
        ////            double[] vector = new double[count];
        ////            string s;
        ////                
        ////            for(int index = 0; index < count; ++index)
        ////            {
        ////                s = s1[index] + criteria;
        ////                //s = this.ParseSimple(s);
        ////                s = this.Parse(s);
        ////                s = this.ComputedValue(s);
        ////                //if(double.TryParse(s , NumberStyles.Any, null, out d) && d > 0)
        ////                if(s == this.TRUEVALUESTR)
        ////                {
        ////                    s = s2[index];
        ////                    s = GetValueFromArg(s);
        ////                    if(double.TryParse(s , NumberStyles.Any, null, out d))
        ////                    {
        ////                        sum += d;
        ////                    }
        ////                        
        ////                }
        ////            }
        ////                
        ////
        ////            return sum.ToString();
        ////        }

        /// <summary>
        /// Returns the sum of the square of all values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the sum of the squares of all values listed in the argument.</returns>
        public string ComputeSumsq(string range)
        {
            double sum = 0;
            string s1;
            double d;

            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            ////TraceUtil.TraceExceptionCatched(ex);
                            ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            ////    throw ex;
                            return ex.Message;
                            ////throw new ArgumentException(ex.Message);
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                            {
                                sum = sum + d * d;
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                        {
                            sum = sum + (d * d);
                        }
                    }
                }
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns the sum of the differences of squares of the two ranges.
        /// </summary>
        /// <param name="range">x_range and y_range.</param>
        /// <returns>A string holding sum of the differences of squares.</returns>
        public string ComputeSumx2my2(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[this.requires_2_args];
            }

            string x_range = args[0];
            string y_range = args[1];

            string[] x = this.GetCellsFromArgs(x_range);
            string[] y = this.GetCellsFromArgs(y_range);

            int len = x.GetLength(0);
            if (len != y.GetLength(0))
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double sum = 0;
            double x1 = 0;
            double y1 = 0;
            for (int i = 0; i < len; ++i)
            {
                if (double.TryParse(GetValueFromArg(x[i]), out x1)
                    && double.TryParse(GetValueFromArg(y[i]), out y1))
                {
                    sum += (x1 * x1) - (y1 * y1);
                }
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns the sum of the sums of squares of the two ranges.
        /// </summary>
        /// <param name="range">x_range and y_range.</param>
        /// <returns>A string holding sum of the sums of squares.</returns>
        public string ComputeSumx2py2(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[this.requires_2_args];
            }

            string x_range = args[0];
            string y_range = args[1];

            string[] x = this.GetCellsFromArgs(x_range);
            string[] y = this.GetCellsFromArgs(y_range);

            int len = x.GetLength(0);
            if (len != y.GetLength(0))
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double sum = 0;
            double x1 = 0;
            double y1 = 0;
            for (int i = 0; i < len; ++i)
            {
                if (double.TryParse(GetValueFromArg(x[i]), out x1)
                    && double.TryParse(GetValueFromArg(y[i]), out y1))
                {
                    sum += (x1 * x1) + (y1 * y1);
                }
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns the sum of the squares of the differences between two ranges.
        /// </summary>
        /// <param name="range">x_range and y_range.</param>
        /// <returns>A string holding sum of the squares of the differences.</returns>
        public string ComputeSumxmy2(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[this.requires_2_args];
            }

            string x_range = args[0];
            string y_range = args[1];

            string[] x = this.GetCellsFromArgs(x_range);
            string[] y = this.GetCellsFromArgs(y_range);

            int len = x.GetLength(0);
            if (len != y.GetLength(0))
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double sum = 0;
            double x1 = 0;
            double y1 = 0;
            for (int i = 0; i < len; ++i)
            {
                if (double.TryParse(GetValueFromArg(x[i]), out x1)
                    && double.TryParse(GetValueFromArg(y[i]), out y1))
                {
                    sum += Math.Pow(x1 - y1, 2);
                }
            }

            return sum.ToString();
        }

        /// <summary>
        /// Specifies if 1900 should be treated as Leap Year (Excel Compatibility)
        /// </summary>
        public static bool Treat1900AsLeapYear = true;

        private DateTime dateTime1900 = new DateTime(1900, 1, 1, 0, 0, 0);

        /// <summary>
        /// Returns the number of days since 01 Jan 1900.
        /// </summary>
        /// <param name="argList">Year, month, and day.</param>
        /// <returns>Number of days.</returns>
        public string ComputeDate(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[this.wrong_number_arguments];
            }

            double year;
            double month;
            double day;
            int days = 0;
            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out year) && !double.IsInfinity(year)
                && double.TryParse(args[1], out month) && !double.IsInfinity(month)
                && double.TryParse(args[2], out day) && !double.IsInfinity(day))
            {
                if (year < 0)
                {
                    return "#NUM";
                }

                DateTime dt = new DateTime((int)year, 1, 1);
                dt = dt.AddMonths((int)month - 1).AddDays((int)day - 1);
                days = GetSerialDateFromDate(dt.Year, dt.Month, dt.Day);
            }
            else
            {
                return "#VALUE!";
            }

            DateTime date = DateTime.FromOADate(days);
            return date.ToString();
        }

        /// <summary>
        /// Returns the serial day number for the given date.
        /// </summary>
        /// <param name="y">The year of the date.</param>
        /// <param name="m">The month of the date.</param>
        /// <param name="d">The day of the date.</param>
        /// <returns>The serial day number.</returns>
        /// <remarks>The serial day number is the number of days since January 1, 1900 with the serial
        /// day number of January 1, 1900 being counted as 1. The serial day number is how Excel stores dates.
        /// It is useful for including dates in calculations.
        /// </remarks>
        public int GetSerialDateFromDate(int y, int m, int d)
        {
            int days = 0;
            if (y < 1900)
            {
                y += 1900;
            }

            days = 1 + ((TimeSpan)(new DateTime(y, m, d, 0, 0, 0) - dateTime1900)).Days;
            if (Treat1900AsLeapYear && days > 59)
            {
                days += 1;
            }

            return days;
        }

        /// <summary>
        /// Gets the date associated with a serial day number.
        /// </summary>
        /// <param name="serialDayNumber">The serial day number.</param>
        /// <returns>The date value.</returns>
        /// <remarks>The serial day number is the number of days since January 1, 1900 with the serial
        /// day number of January 1, 1900 being counted as 1. The serial day number is how Excel stores dates.
        /// It is useful for including dates in calculations.
        /// </remarks>
        public DateTime GetDateFromSerialDate(int serialDayNumber)
        {
            serialDayNumber -= 1;
            if (Treat1900AsLeapYear && serialDayNumber > 59)
            {
                serialDayNumber -= 1;
            }

            return dateTime1900.AddDays(serialDayNumber);
        }

        /// <summary>
        /// Returns the number of days since 01 Jan 1900.
        /// </summary>
        /// <param name="argList">Text containing a date.</param>
        /// <returns>Number of days.</returns>
        public string ComputeDatevalue(string argList)
        {
            argList = GetValueFromArg(argList);
            DateTime dt;
            try
            {
                dt = DateTime.Parse(argList.Replace(TIC, string.Empty));
            }
            catch
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            int days = 1 + ((TimeSpan)(dt - dateTime1900)).Days;
            if (Treat1900AsLeapYear && days > 59)
            {
                days += 1;
            }

            return days.ToString();
        }

        /// <summary>
        /// Returns the day of the serial number date.
        /// </summary>
        /// <param name="argList">Serial number date.</param>
        /// <returns>returns Day.</returns>
        public string ComputeDay(string argList)
        {
            double day = 1;
            double serialdate;
            string s = GetValueFromArg(argList);
            if (double.TryParse(s, out serialdate) || GetDaySerialNumberFromDateTime(s, out serialdate))
            {
                ////serialdate = serialdate - 1 - ((Treat1900AsLeapYear && serialdate > 59) ?  1 : 0);
                DateTime dt = GetDateFromSerialDate((int)serialdate); ////dateTime1900.AddDays(serialdate);
                day = dt.Day;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return day.ToString();
        }

        /// <summary>
        /// Number of days between 2 dates using 360 day year.
        /// </summary>
        /// <param name="argList">Serial number date1, serial number date1, and method.</param>
        /// <returns>Days between the dates.</returns>
        public string ComputeDays360(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double serialdate1;
            double serialdate2;
            bool method = false;
            int days = 0;

            if (double.TryParse(GetValueFromArg(args[0]), out serialdate1)
                && double.TryParse(GetValueFromArg(args[1]), out serialdate2)
                && (argCount == 2 || (method = args[2] == TRUEVALUESTR)))
            {
                DateTime dt1 = GetDateFromSerialDate((int)serialdate1);
                DateTime dt2 = GetDateFromSerialDate((int)serialdate2);
                bool flipSign = false;

                if (dt1.Day == 31)
                {
                    dt1 = dt1.AddDays(-1);
                }

                if (dt2.Day == 31 && !method && dt1.Day < 30)
                {
                    dt2 = dt2.AddDays(1);
                }
                else if (dt2.Day == 31)
                {
                    dt2 = dt2.AddDays(-1);
                }

                if (dt2 < dt1)
                {
                    flipSign = true;
                    DateTime t = dt1;
                    dt1 = dt2;
                    dt2 = t;
                }

                days = dt2.Day - dt1.Day;
                days += 30 * (dt2.Month - dt1.Month);
                days += 360 * (dt2.Year - dt1.Year);
                if (flipSign)
                {
                    days = -days;
                }
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return days.ToString();
        }

        /// <summary>
        /// Returns the hour of the given time.
        /// </summary>
        /// <param name="argList">Given time.</param>
        /// <returns>returns Hour.</returns>
        public string ComputeHour(string argList)
        {
            double time;
            DateTime dt = DateTime.Now;
            argList = GetValueFromArg(argList);
            if (argList.IndexOf(TIC) > -1)
            {
                argList = argList.Replace(TIC, string.Empty);
                dt = DateTime.Parse(argList);
            }
            else
            {
                try
                {
                    if (double.TryParse(argList, out time))
                    {
                        dt = DateTime.FromOADate(time);
                    }
                    else
                    {
                        dt = DateTime.Parse(argList);
                    }
                }
                catch
                {
                    return FormulaErrorStrings[invalid_arguments];
                }
            }

            return dt.Hour.ToString();
        }

        /// <summary>
        /// Returns the Minute of the given time.
        /// </summary>
        /// <param name="argList">Given time.</param>
        /// <returns>returns Minute.</returns>
        public string ComputeMinute(string argList)
        {
            double time;
            DateTime dt = DateTime.Now;
            argList = GetValueFromArg(argList);
            if (argList.IndexOf(TIC) > -1)
            {
                argList = argList.Replace(TIC, string.Empty);
                dt = DateTime.Parse(argList);
            }
            else
            {
                try
                {
                    if (double.TryParse(argList, out time))
                    {
                        dt = DateTime.FromOADate(time);
                    }
                    else
                    {
                        dt = DateTime.Parse(argList);
                    }
                }
                catch
                {
                    return FormulaErrorStrings[invalid_arguments];
                }
                ////                try
                ////                {
                ////                    dt = DateTime.Parse(argList);
                ////                }
                ////                catch
                ////                {
                ////                    if(double.TryParse(argList, NumberStyles.Any, null, out time))
                ////                    {
                ////                        dt = DateTime.FromOADate(time);
                ////                    } 
                ////                }
            }

            return dt.Minute.ToString();
        }

        /// <summary>
        /// Returns the second of the given time.
        /// </summary>
        /// <param name="argList">Given time.</param>
        /// <returns>returns Second.</returns>
        public string ComputeSecond(string argList)
        {
            double time;
            DateTime dt = DateTime.Now;
            argList = GetValueFromArg(argList);
            if (argList.IndexOf(TIC) > -1)
            {
                argList = argList.Replace(TIC, string.Empty);
                dt = DateTime.Parse(argList);
            }
            else
            {
                try
                {
                    if (double.TryParse(argList, out time))
                    {
                        dt = DateTime.FromOADate(time);
                    }
                    else
                    {
                        dt = DateTime.Parse(argList);
                    }
                }
                catch
                {
                    return FormulaErrorStrings[invalid_arguments];
                }
                ////                try
                ////                {
                ////                    dt = DateTime.Parse(argList);
                ////                }
                ////                catch
                ////                {
                ////                    if(double.TryParse(argList, NumberStyles.Any, null, out time))
                ////                    {
                ////                        dt = DateTime.FromOADate(time);
                ////                    } 
                ////                }
            }

            return dt.Second.ToString();
        }

        /// <summary>
        /// Returns the month of the given date.
        /// </summary>
        /// <param name="argList">Given time.</param>
        /// <returns>returns Month.</returns>
        public string ComputeMonth(string argList)
        {
            double month = 1;
            double serialdate;
            string s = GetValueFromArg(argList);
            if (double.TryParse(s, out serialdate) || GetDaySerialNumberFromDateTime(s, out serialdate))
            {
                ////serialdate = serialdate - 1 - ((Treat1900AsLeapYear && serialdate > 59) ?  1 : 0);
                DateTime dt = GetDateFromSerialDate((int)serialdate); ////dateTime1900.AddDays(serialdate);
                month = dt.Month;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return month.ToString();
        }

        /// <summary>
        /// Returns the current date and time as a date serial number.
        /// </summary>
        /// <param name="argList">value ignored.</param>
        /// <returns>rCurrent date and time as serial number.</returns>
        public string ComputeNow(string argList)
        {
            DateTime dt = DateTime.Now;
            return dt.ToString();
        }

        /// <summary>
        /// Returns the current date as a date serial number.
        /// </summary>
        /// <param name="argList">value ignored.</param>
        /// <returns>Current date as date serial number.</returns>
        public string ComputeToday(string argList)
        {
            DateTime dt = DateTime.Today;
            return dt.ToString();     
        }

        /// <summary>
        /// Returns a fraction of a day.
        /// </summary>
        /// <param name="argList">Hour, minute, and second.</param>
        /// <returns>Fraction of a day.</returns>
        public string ComputeTime(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[this.wrong_number_arguments];
            }

            double hour;
            double minute;
            double second;
            double time = 0;
            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out hour)
                && double.TryParse(args[1], out minute)
                && double.TryParse(args[2], out second))
            {
                time = (hour + (minute + second / 60d) / 60d) / 24d;
            }

            return time.ToString("0.0########");
        }

        /// <summary>
        /// Returns a fraction of a day.
        /// </summary>
        /// <param name="argList">Time as a text string.</param>
        /// <returns>Fraction of a day.</returns>
        public string ComputeTimevalue(string argList)
        {
            double time;
            DateTime dt = DateTime.Now;
            argList = GetValueFromArg(argList);
            try
            {
                if (argList.IndexOf(TIC) > -1)
                {
                    argList = argList.Replace(TIC, string.Empty);
                    dt = DateTime.Parse(argList);
                    time = (dt.Hour + (dt.Minute + dt.Second / 60d) / 60d) / 24d;
                }
                else
                {
                    dt = DateTime.Parse(argList);
                    time = (dt.Hour + (dt.Minute + dt.Second / 60d) / 60d) / 24d;
                }
            }
            catch
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            return time.ToString("0.0###########");
        }

        /// <summary>
        /// Day of the week.
        /// </summary>
        /// <param name="argList">Serial number date1 and return_type.</param>
        /// <returns>Days between the dates.</returns>
        public string ComputeWeekday(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 1 && argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double serialdate;
            double return_type = 1;

            double day = 0;
            string s = GetValueFromArg(args[0]);
            if ((double.TryParse(s, out serialdate)
                && (argCount == 1 || double.TryParse(GetValueFromArg(args[1]), out return_type))) || GetDaySerialNumberFromDateTime(s, out serialdate))
            {
                DateTime dt1 = GetDateFromSerialDate((int)serialdate);
                day = (int)dt1.DayOfWeek;
                if (return_type == 1)
                {
                    day += 1;
                }
                else
                {
                    if (day == 0)
                    {
                        day = 7;
                    }

                    if (return_type == 3)
                    {
                        day -= 1;
                    }
                }
            }

            return day.ToString();
        }

        /// <summary>
        /// Returns the year of the given date.
        /// </summary>
        /// <param name="argList">Given date.</param>
        /// <returns>returns Month.</returns>
        public string ComputeYear(string argList)
        {
            double year = 1;
            double serialdate;
            string s = GetValueFromArg(argList);
            if (double.TryParse(s, out serialdate) || GetDaySerialNumberFromDateTime(s, out serialdate))
            {
                ////serialdate = serialdate - 1 - ((Treat1900AsLeapYear && serialdate > 59) ?  1 : 0);
                DateTime dt = GetDateFromSerialDate((int)serialdate); ////dateTime1900.AddDays(serialdate);
                year = dt.Year;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return year.ToString();
        }

        /// <summary>
        /// Returns the average deviation of all values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the average deviation of all values listed in the argument.</returns>
        public string ComputeAvedev(string range)
        {
            double sum = 0;
            string s1;
            double d;
            ArrayList x = new ArrayList();
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, out d))
                            {
                                sum = sum + d;
                                x.Add(d);
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, out d))
                        {
                            sum = sum + d;
                            x.Add(d);
                        }
                    }
                }
            }

            if (x.Count > 0)
            {
                double ave = sum / x.Count;
                sum = 0;
                for (int i = 0; i < x.Count; ++i)
                {
                    sum += Math.Abs(((double)x[i]) - ave);
                }

                sum = sum / x.Count;
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns the simple average of all values (including text) listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the simple average of all values listed in the argument.</returns>
        public string ComputeAveragea(string range)
        {
            double sum = 0;
            string s1;
            double d;
            int count = 0;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s, true);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (s1.Length > 0)
                        {
                            count++;
                            if (double.TryParse(s1, out d))
                            {
                                sum = sum + d;
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r, true);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        count++;
                        if (double.TryParse(s1, out d))
                        {
                            sum = sum + d;
                        }
                    }
                }
            }

            if (count > 0)
            {
                sum = sum / count;
            }

            return sum.ToString();
        }

        // http://lib.stat.cmu.edu/griffiths-hill/acm291
        private static double[] gammaAs = new double[] { 0.918938533204673d, 0.000595238095238d, 0.000793650793651d, 0.002777777777778d, 0.083333333333333d };
        private double gammaln(double x)
        {
            double y = x;
            double f = 0d;
            if (y < 7d)
            {
                f = y;
                y++;
                while (y < 7d)
                {
                    f = f * y;
                    if (y < 7d)
                    {
                        y++;
                    }
                }

                f = -Math.Log(f);
            }

            double z = 1d / (y * y);
            return f + (y - 0.5d) * Math.Log(y) - y + gammaAs[0]
                + (((-gammaAs[1] * z + gammaAs[2]) * z - gammaAs[3]) * z + gammaAs[4]) / y;
        }

        /// <summary>
        /// Returns the natural logarithm of the gamma function.
        /// </summary>
        /// <param name="argList">The value to be evaluated.</param>
        /// <returns>Returns natural logarithm of gamma function.</returns>
        public string ComputeGammaln(string argList)
        {
            double x = 0;
            if (double.TryParse(GetValueFromArg(argList), out x) && x > 0)
            {
                x = gammaln(x);
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return x.ToString();
        }

        private double gammainv(double p, double a, double b)
        {
            double guess = (a > b) ? b * Math.Abs(Math.Log(b * p)) : a * Math.Abs(Math.Log(a * p));
            if (a == b)
            {
                guess = p;
            }

            double lastIncrement = guess / 2;
            double gi = 0;
            double eps = 1e-7;
            int k = 100;
            int tries = 3;
            while (k == 100 && tries > 0)
            {
                tries--;
                guess = guess / 2;
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k)
                {
                    gi = gammadist(a, b, guess);
                    if (Math.Abs((gi - p) / p) < eps)
                    {
                        break;
                    }

                    if (gi < p)
                    {
                        guess += lastIncrement;
                    }
                    else
                    {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0)
                        {
                            lastIncrement = guess / 2;
                        }

                        guess -= lastIncrement;
                    }
                }
            }

            if (k == 100)
            {
                guess = -1;
            }

            return guess;
        }

        private double gammadist(double a, double b, double x)
        {
            int nPanels = 4;

            double h = x / nPanels;  ////h = (b-a)/(3n)
            double mult = x / 12;
            double sum1 = gammadensity(a, b, x);
            double sum4 = 4 * (gammadensity(a, b, h) + gammadensity(a, b, 3 * h));
            double sum2 = 2 * gammadensity(a, b, 2 * h);
            double gd = mult * (sum1 + sum2 + sum4);
            double oldgd = gd;
            double eps = 1e-7;

            for (int k = 0; k < 30; ++k)
            {
                nPanels *= 2;

                sum2 += sum4 / 2;
                sum4 = 0;
                h = x / nPanels;
                for (int i = 0; i < nPanels; ++i)
                {
                    if (i % 2 == 1)
                    {
                        double d = gammadensity(a, b, h * i);
                        sum4 += d;
                    }
                }

                sum4 = 4 * sum4;
                mult /= 2;
                gd = mult * (sum1 + sum2 + sum4);
                ////Console.WriteLine(gd);
                if (Math.Abs((gd - oldgd) / oldgd) < eps)
                {
                    break;
                }

                oldgd = gd;
            }

            return gd;
        }

        private double gammadensity(double a, double b, double x)
        {
            // excel version     dist = 1 / (Math.Pow(b, a) * gammaln(a)) * Math.Pow(x, a - 1) * Math.Exp(-x / b);
            // http://www.itl.nist.gov/div898/handbook/eda/section3/eda366b.htm
            return Math.Pow(x / b, a - 1) * Math.Exp(-x / b) / (b * Math.Exp(gammaln(a)));
        }

        /// <summary>
        /// Returns the gamma distribution.
        /// </summary>
        /// <param name="argList">X, alpha, beta, cumulative.</param>
        /// <returns>Returns gamma distribution.</returns>
        /// <remarks>
        /// X, alpha and beta should be positive real numbers. Cumulative should be either 
        /// True if you want to return the value of the distribution function, or False
        /// if you want to return the value of the density function. The distribution value
        /// is computed interactively using Trapezoidal Rule to six to seven significant digits
        /// or 20 iteration maximum.
        /// </remarks>
        public string ComputeGammadist(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 4 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double a;
            double b;
            double cum = 0;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

             if (double.TryParse(args[0], out x)
                && double.TryParse(args[1], out a)
                && double.TryParse(args[2], out b))
            {
                if (argCount != 3)
                {
                    cum = (args[3] == TRUEVALUESTR || args[3] == "1") ? 1 : 0;
                }

                if (cum == 0)
                {
                    //// excel version     dist = 1 / (Math.Pow(b, a) * gammaln(a)) * Math.Pow(x, a - 1) * Math.Exp(-x / b);
                    //// http://www.itl.nist.gov/div898/handbook/eda/section3/eda366b.htm
                    dist = gammadensity(a, b, x); ////Math.Pow(x/b, a-1) * Math.Exp(-x/b) / (b * Math.Exp(gammaln(a)));
                }
                else
                {
                    ////Integrates the density function.
                    dist = gammadist(a, b, x);
                }
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the inverse of gamma distribution.
        /// </summary>
        /// <param name="argList">P, alpha, beta</param>
        /// <returns>Returns x such that gamma distribution at x is p.</returns>
        /// <remarks>
        /// P, alpha and beta should be positive real numbers, with p between 0 and 1.
        /// </remarks>
        public string ComputeGammainv(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double p;
            double a;
            double b;
            double invdist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out p)
                && (p > 0 && p < 1)
                && double.TryParse(args[1], out a)
                && double.TryParse(args[2], out b))
            {
                invdist = gammainv(p, a, b);
            }

            if (invdist <= 0)
            {
                return FormulaErrorStrings[iterations_dont_converge];
            }

            return invdist.ToString();
        }

        /// <summary>
        /// Returns the geometric mean of all values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>The geometric mean of all values listed in the argument.</returns>
        public string ComputeGeomean(string range)
        {
            double sum = 1d;
            string s1;
            double d;
            int count = 0;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, out d))
                            {
                                count++;
                                sum = sum * d;
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, out d))
                        {
                            count++;
                            sum = sum * d;
                        }
                    }
                }
            }

            if (count > 0)
            {
                sum = Math.Pow(sum, 1d / (double)count);
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns the harmonic mean of all values listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>The harmonic mean all values listed in the argument.</returns>
        public string ComputeHarmean(string range)
        {
            double sum = 0d;
            string s1;
            double d;
            int count = 0;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, out d) && d != 0)
                            {
                                count++;
                                sum = sum + 1 / d;
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, out d) && d != 0)
                        {
                            count++;
                            sum = sum + 1 / d;
                        }
                    }
                }
            }

            if (count > 0)
            {
                sum = ((double)count) / sum;
            }

            return sum.ToString();
        }

        ////N things taken k at the time.
        private double comb(int k, int n)
        {
            double top = 1;
            for (int i = k + 1; i <= n; ++i)
            {
                top = top * i;
            }

            double bottom = 1;
            for (int i = 2; i <= (n - k); ++i)
            {
                bottom = bottom * i;
            }

            return top / bottom;
        }

        /// <summary>
        /// Returns the hypergeometric distribution.
        /// </summary>
        /// <param name="argList">Number of sample successes, number of sample, number of population successes, number of population.</param>
        /// <returns>Returns the gamma distribution.</returns>
        public string ComputeHypgeomdist(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double xnss = 0;
            double xns = 0;
            double xnps = 0;
            double xnp = 0;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], NumberStyles.Integer, null, out xnss)
                && double.TryParse(args[1], NumberStyles.Integer, null, out xns)
                && double.TryParse(args[2], NumberStyles.Integer, null, out xnps)
                && double.TryParse(args[3], NumberStyles.Integer, null, out xnp))
            {
                int nss = (int)xnss;
                int ns = (int)xns;
                int nps = (int)xnps;
                int np = (int)xnp;
                if (nss < 0 || ns < nss || nps < 0 || np < nps || nss > nps || ns > np)
                {
                    return FormulaErrorStrings[invalid_Math_argument];
                }

                dist = comb(nss, nps) * comb(ns - nss, np - nps) / comb(ns, np);
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the y-intercept of the least square fit line through the given points.
        /// </summary>
        /// <param name="range">y_range, x_range.</param>
        /// <returns>returns y-intercept.</returns>
        public string ComputeIntercept(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = GetDoubleArray(args[1]);
            int n = x.GetLength(0);

            if (n <= 0 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sumx = 0;
            double sumy = 0;
            for (int i = 0; i < n; ++i)
            {
                sumx += (double)x[i];
                sumy += (double)y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            double sumxy = 0;
            double sumx2 = 0;
            double d;
            for (int i = 0; i < n; ++i)
            {
                d = (double)x[i] - sumx;
                sumxy += d * (y[i] - sumy);
                sumx2 += d * d;
            }

            return (sumy - sumxy / sumx2 * sumx).ToString();
        }

        /// <summary>
        /// Returns the binomial distribution.
        /// </summary>
        /// <param name="argList">Number of successes, number of trials, probability, cumulative.</param>
        /// <returns>Returns binomial distribution.</returns>
        public string ComputeBinomdist(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double successes;
            double trials;
            double p;
            double cum = 0;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out successes)
                && double.TryParse(args[1], out trials)
                && double.TryParse(args[2], out p))
            {
                cum = (args[3] == TRUEVALUESTR || args[3] == "1") ? 1 : 0;
                if (cum == 0)
                {
                    dist = comb((int)successes, (int)trials) * Math.Pow(p, successes) * Math.Pow(1 - p, trials - successes);
                }
                else
                {
                    dist = binomdist((int)trials, (int)successes, p);

                    ////                    for(int i = 0; i <= successes; ++ i)
                    ////                        dist += comb(i, (int)trials) * Math.Pow(p, i) * Math.Pow(1-p, trials - i);
                    ////            
                }
            }

            return dist.ToString();
        }

        private double binomdist(int trials, int successes, double p)
        {
            double pPow = 1;
            double pm1 = 1 - p;

            double p1Pow = Math.Pow(pm1, trials);

            double dist = 0;
            double cbn = pPow * Math.Pow(pm1, trials);
            if (cbn == 0)
            {
                return double.NaN;
            }

            for (int i = 0; i <= successes; ++i)
            {
                dist += cbn;
                cbn = cbn * p / pm1 * (trials - i) / (i + 1);
                if (double.IsInfinity(cbn) || double.IsNaN(cbn))
                {
                    dist = double.NaN;
                    break;
                }
            }

            return dist;
        }

        /// <summary>
        /// Returns the chi-squared distribution.
        /// </summary>
        /// <param name="argList">X degrees of freedom.</param>
        /// <returns>Returns chi-squared distribution.</returns>
        public string ComputeChidist(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double v;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out x)
                && double.TryParse(args[1], out v))
            {
                dist = 1 - chidist(x, v);
            }

            return dist.ToString();
        }

        private static int gauss_n = 16;
        private static double[] gauss_x = new double[]
        {
                                                           0.04830766569,
                                                           0.14447196158,
                                                           0.23928736225,
                                                           0.33186860228,
                                                           0.42135127613,
                                                           0.50689990893,
                                                           0.58771575724,
                                                           0.66304426693,
                                                           0.73218211874,
                                                           0.79448379597,
                                                           0.84936761373,
                                                           0.89632115577,
                                                           0.93490607594,
                                                           0.96476225559,
                                                           0.98561151155,
                                                           0.99726386185,
        };
        private static double[] gauss_w = new double[]
        {
                                                           0.09654008851,
                                                           0.09563872008,
                                                           0.09384439908,
                                                           0.09117387870,
                                                           0.08765209300,
                                                           0.08331192423,
                                                           0.07819389579,
                                                           0.07234579411,
                                                           0.06582222278,
                                                           0.05868409348,
                                                           0.05099805926,
                                                           0.04283589802,
                                                           0.03427386291,
                                                           0.02539206531,
                                                           0.01627439473,
                                                           0.00701861001,
        };

        ////uses guass quad to estimate the integral
        private double chidist(double x, double v)
        {
            double ex = v / 2;
            double mult = 1 / (Math.Pow(2, ex) * Math.Exp(this.gammaln(ex)));
            ex = ex - 1;

            double a = 0;
            double b = x;

            double midPt = (a + b) / 2;
            double radius = (b - a) / 2;

            double sum = 0;

            double offset;
            for (int i = 0; i < gauss_n; ++i)
            {
                offset = gauss_x[i] * radius;
                sum = sum + gauss_w[i] * (Math.Pow(midPt + offset, ex) * Math.Exp(-(midPt + offset) / 2)
                    + Math.Pow(midPt - offset, ex) * Math.Exp(-(midPt - offset) / 2));
            }

            sum = mult * sum * radius;
            return sum;
        }

        private double chiinv(double p, double v)
        {
            double guess = p; ////Math.Sqrt(p);

            double lastIncrement = guess / 2;
            double gi = 0;
            double eps = 1e-7;
            int k = 100;
            int tries = 3;
            while (k == 100 && tries > 0)
            {
                tries--;
                guess = guess / 2;
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k)
                {
                    gi = 1 - chidist(guess, v);
                    if (Math.Abs((gi - p) / p) < eps)
                    {
                        break;
                    }

                    if (gi > p)
                    {
                        guess += lastIncrement;
                    }
                    else
                    {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0)
                        {
                            lastIncrement = guess / 2;
                        }

                        guess -= lastIncrement;
                    }
                }
            }

            if (k == 100)
            {
                guess = -1;
            }

            return guess;
        }

        /// <summary>
        /// Returns the inverse of the chi-squared distribution.
        /// </summary>
        /// <param name="argList">X degrees of freedom.</param>
        /// <returns>Returns inverse of the chi-squared distribution.</returns>
        public string ComputeChiinv(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double p;
            double v;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out p)
                && double.TryParse(args[1], out v))
            {
                dist = chiinv(p, v);
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the Chi Test for independence.
        /// </summary>
        /// <param name="range">Actual_range, expected_range.</param>
        /// <returns>returns y-intercept.</returns>
        public string ComputeChitest(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            string r = args[0];
            string s1;
            double d = 0;
            double v = 0;
            if (r.IndexOf(':') > -1)
            {
                s1 = r.Substring(0, r.IndexOf(':'));
                int col = this.ColIndex(s1);
                int row = this.RowIndex(s1);

                s1 = r.Substring(r.IndexOf(':') + 1);
                int col1 = this.ColIndex(s1);
                int row1 = this.RowIndex(s1);
                v = Math.Abs(col - col1) * Math.Abs(row - row1);
                if (v < 1)
                {
                    return FormulaErrorStrings[invalid_Math_argument];
                }
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = GetDoubleArray(args[1]);
            int n = x.GetLength(0);

            if (n <= 0 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sumx = 0;

            for (int i = 0; i < n; ++i)
            {
                d = y[i] - x[i];
                sumx += d * d / x[i];
            }

            return (1 - this.chidist(sumx, v)).ToString();
        }

        private double normaldensity(double x, double u, double s)
        {
            return 1 / (Math.Sqrt(2 * Math.PI) * s) * Math.Exp(-(x - u) * (x - u) / (2 * s * s));
        }

        private double normaldist(double x, double u, double s)
        {
            ////                        int nTraps = 4;
            ////                        
            ////                        
            ////                        
            ////                        double h = x / 2;
            ////                        double sum = normaldensity(-x , u, s) + 2*(normaldensity(-x + h , u, s) + normaldensity(-x + 2 * h , u, s) + normaldensity(-x + 3 * h , u, s)) + normaldensity(x , u, s);
            ////                        double gd = h * sum / 2;
            ////                        double oldgd = gd;
            ////                        double eps = 1e-7;
            ////            
            ////                        for(int k = 0; k < 30; ++k)
            ////                        {
            ////                            nTraps *= 2;
            ////            
            ////                            h = 2 * x / nTraps;
            ////                            for(int i = 0; i < nTraps; ++i)
            ////                            {
            ////                                if(i % 2 == 1)
            ////                                {
            ////                                    double d = normaldensity(-x + h * i, u, s);
            ////                                    sum += 2 * d;
            ////                                }
            ////                            }
            ////                            gd = h * sum / 2;
            ////                            Console.WriteLine(gd);
            ////                            if(Math.Abs((gd - oldgd)/oldgd) < eps)
            ////                                break;
            ////                            
            ////                            oldgd = gd;
            ////                            
            ////                        }
            ////                return gd;

            int nPanels = 32;

            double leftside;
            double rightside;
            if (x > u)
            {
                leftside = u - (x - u);
                rightside = x;
            }
            else
            {
                leftside = x;
                rightside = u + (u - x);
            }

            double h = (rightside - leftside) / nPanels;
            double mult = h / 3;

            double sum1 = normaldensity(leftside, u, s) + normaldensity(rightside, u, s);
            double sum4 = 0;
            for (int i = 1; i < nPanels; i += 2)
            {
                sum4 += 4 * normaldensity(leftside + i * h, u, s);
            }

            double sum2 = 0;
            for (int i = 2; i < nPanels; i += 2)
            {
                sum2 += 2 * normaldensity(leftside + i * h, u, s);
            }

            double gd = mult * (sum1 + sum2 + sum4);
            double oldgd = gd;
            double eps = 1e-7;

            int k = 0;

            for (k = 0; k < 20; ++k)
            {
                nPanels *= 2;

                sum2 += sum4 / 2;
                sum4 = 0;
                h = h = (rightside - leftside) / nPanels;
                for (int i = 0; i < nPanels; ++i)
                {
                    if (i % 2 == 1)
                    {
                        double d = normaldensity(leftside + h * i, u, s);
                        sum4 += d;
                    }
                }

                sum4 = 4 * sum4;
                mult = h / 3;
                gd = mult * (sum1 + sum2 + sum4);

                if (Math.Abs((gd - oldgd) / oldgd) < eps)
                {
                    break;
                }

                oldgd = gd;
            }

            if (x > u)
            {
                ////Add the left tails.
                gd = gd + (1 - gd) / 2;
            }
            else
            {
                ////return only the left tial
                gd = (1 - gd) / 2;
            }
            ////Console.WriteLine(k.ToString() + "  " + gd.ToString());
            return gd;
        }

        /// <summary>
        /// Returns the normal distribution.
        /// </summary>
        /// <param name="argList">X, mean, standarddev, cumulative.</param>
        /// <returns>Returns normal distribution.</returns>
        /// <remarks>
        /// Cumulative should be either 
        /// True if you want to return the value of the distribution function or False
        /// if you want to return the value of the density function. The distribution value
        /// is computed interactively using Trapezoidal Rule to six to seven significant digits
        /// or 20 iteration maximum.
        /// </remarks>
        public string ComputeNormdist(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 4 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double u;
            double s;
            double cum = 0;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out x)
                && double.TryParse(args[1], out u)
                && double.TryParse(args[2], out s))
            {
                if (argCount != 3)
                {
                    cum = (args[3] == TRUEVALUESTR || args[3] == "1") ? 1 : 0;
                }

                if (cum == 0)
                {
                    dist = normaldensity(x, u, s);
                }
                else
                {
                    dist = normaldist(x, u, s);
                }
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the standard normal cumulative distribution function. The distribution has a mean of 0 (zero) and a standard deviation of one.
        /// <para>Syntax: NORMSDIST(z)</para>
        /// </summary>
        /// <param name="argList">Z is the value for which you want the distribution.</param>
        /// <returns>Returns string standard normal cumulative distribution function</returns>
        public string ComputeNormsDist(string argList)
        {
            string args = argList + ", 0, 1, " + TRUEVALUESTR;
            return ComputeNormdist(args);
        }

        /// <summary>
        /// Returns the inverse of normal distribution.
        /// </summary>
        /// <param name="argList">P, mean, standard deviation.</param>
        /// <returns>Returns x such that normal distribution at x is p.</returns>
        /// <remarks>
        /// P should be between 0 and 1.
        /// </remarks>
        public string ComputeNorminv(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double p;
            double u;
            double s;
            double invdist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out p)
                && (p > 0 && p < 1)
                && double.TryParse(args[1], out u)
                && double.TryParse(args[2], out s))
            {
                invdist = normalinv(p, u, s);
            }

            if (invdist <= 0)
            {
                return FormulaErrorStrings[iterations_dont_converge];
            }

            return invdist.ToString();
        }

        private double normalinv(double p, double u, double s)
        {
            double guess = u; ////(u == 0) ? 1 : u;
            if (p < .05)
            {
                guess = u - 2 * s;
            }
            else if (p < .5)
            {
                guess = u;
            }
            else if (p < .95)
            {
                guess = u + 2 * s;
            }
            else
            {
                guess = u + 5 * s;
            }

            double lastIncrement = guess / 2;
            double gi = 0;
            double eps = 1e-7;
            int k = 100;
            int tries = 3;
            int its = 0;
            while (k == 100 && tries > 0)
            {
                tries--;
                guess = guess / 2;
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k)
                {
                    its++;
                    gi = normaldist(guess, u, s);
                    if (Math.Abs((gi - p) / p) < eps)
                    {
                        break;
                    }

                    if (gi < p)
                    {
                        guess += lastIncrement;
                    }
                    else
                    {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0)
                        {
                            lastIncrement = guess / 2;
                        }

                        guess -= lastIncrement;
                    }
                }
            }

            if (k == 100)
            {
                guess = -1;
            }

            ////Console.WriteLine(its);
            return guess;
        }

        /// <summary>
        /// Returns the inverse of the standard normal cumulative distribution. The distribution has a mean of zero and a standard deviation of one.
        /// <para>Syntax: NORMSINV(p)</para>
        /// </summary>
        /// <param name="argList">p is a probability corresponding to the normal distribution.</param>
        /// <returns>Returns string for inverse of the standard normal cumulative distribution</returns>
        /// <remarks>
        /// p should be between 0 and 1.
        /// </remarks>
        public string ComputeNormsInv(string argList)
        {
            string args = argList + ", 0, 1";
            return ComputeNorminv(args);
        }

        /// <summary>
        /// Returns a confidence interval radius.
        /// </summary>
        /// <param name="argList">Alpha, standard deviation, size.</param>
        /// <returns>Returns x such that normal distribution at x is p.</returns>
        /// <remarks>
        /// P should be between 0 and 1.
        /// </remarks>
        public string ComputeConfidence(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double alpha;
            double s;
            double sz;
            double val = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out alpha)
                && double.TryParse(args[1], out s)
                && double.TryParse(args[2], out sz))
            {
                //// 1 - alpha  would be from -inf to 1-alpha. adding alpha + 2 
                //// takes away the left tail making the integral from alpha-1 to 1-alpha.
                val = normalinv(1 - alpha + alpha / 2, 0, 1);
                ////val = normalinv(1 - alpha, 0, 1, false);
                val = val * s / Math.Sqrt(sz);
            }

            return val.ToString();
        }

        /// <summary>
        /// Returns the correlation coefficient between the two sets of points.
        /// </summary>
        /// <param name="range">Range1, range2.</param>
        /// <returns>Correlation coefficient.</returns>
        public string ComputeCorrel(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = GetDoubleArray(args[1]);
            int n = x.GetLength(0);
            if (n <= 0 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sumx = 0;
            double sumy = 0;
            for (int i = 0; i < n; ++i)
            {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            double sumxy = 0;
            double sumxb2 = 0;
            double sumyb2 = 0;
            double xb = 0;
            double yb = 0;

            for (int i = 0; i < n; ++i)
            {
                xb = x[i] - sumx;
                yb = y[i] - sumy;
                sumxy += xb * yb;
                sumxb2 += xb * xb;
                sumyb2 += yb * yb;
            }

            return (sumxy / Math.Sqrt(sumxb2 * sumyb2)).ToString();
        }

        /// <summary>
        /// Returns the count of all values (including text) listed in the argument
        /// evaluate to a number.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the count of all numerical values listed in the argument.</returns>
        public string ComputeCount(string range)
        {
            int count = 0;
            string s1;
            double d;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, out d))
                            {
                                count++;
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, out d))
                        {
                            count++;
                        }
                    }
                }
            }

            return count.ToString();
        }

        /// <summary>
        /// Returns the count of all values (including text) listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the count of all values listed in the argument.</returns>
        public string ComputeCounta(string range)
        {
            int count = 0;
            string s1;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s, true);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (s1.Length > 0)
                        {
                            count++;
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r, true);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        count++;
                    }
                }
            }

            return count.ToString();
        }

        /// <summary>
        /// Returns the count of blank cells listed in the argument.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the count of blank cells listed in the argument.</returns>
        public string ComputeCountblank(string range)
        {
            int count = 0;
            string s1;
            string TICS2 = TIC + TIC;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (s1.Length == 0 || s1 == TICS2)
                        {
                            count++;
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length == 0 || s1 == TICS2)
                    {
                        count++;
                    }
                }
            }

            return count.ToString();
        }

        /// <summary>
        /// Counts the cells specified by some criteria.
        /// </summary>
        /// <param name="argList">The criteria range, the criteria.</param>
        /// <returns>returns string to the cell count</returns>
        public string ComputeCountif(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            string criteriaRange = args[0];
            ////string criteria = args[1].Replace(TIC, string.Empty);
            string criteria = args[1];
            if (criteria.Length > 1 && criteria[0] == TIC[0]
                && "=><".IndexOf(criteria[1]) == -1)
            {
                criteria = "=" + criteria;
            }
            else
            {
                criteria = criteria.Replace(TIC, string.Empty);
            }

            string[] s1 = this.GetCellsFromArgs(criteriaRange);

            int sum = 0;
            int count = s1.GetLength(0);
            double[] vector = new double[count];
            string s;

            for (int index = 0; index < count; ++index)
            {
                s = s1[index] + criteria;
                ////s = this.ParseSimple(s);
                s = this.Parse(s);
                s = this.ComputedValue(s);
                if (s == this.TRUEVALUESTR)
                {
                    sum++;
                }
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns the covariance between the two sets of points.
        /// </summary>
        /// <param name="range">Range1, range2.</param>
        /// <returns>returns Covariance.</returns>
        public string ComputeCovar(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = GetDoubleArray(args[1]);
            int n = x.GetLength(0);

            if (n <= 0 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sumx = 0;
            double sumy = 0;
            for (int i = 0; i < n; ++i)
            {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            double sumxy = 0;

            for (int i = 0; i < n; ++i)
            {
                sumxy += (x[i] - sumx) * (y[i] - sumy);
            }

            return (sumxy / n).ToString();
        }

        /// <summary>
        /// Returns the smallest value for which the cumulative binomial distribution is greater than or equal to a criterion value.
        /// </summary>
        /// <param name="argList">Number of trials, probability, alpha.</param>
        /// <returns>Returns the critcal value.</returns>
        public string ComputeCritbinom(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double trials;
            double p;
            double alpha = 0;
            int dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out trials)
                && double.TryParse(args[1], out p)
                && double.TryParse(args[2], out alpha))
            {
                if (p > 0 && p < 1 && alpha > 0 && alpha < 1)
                {
                    dist = critbinom((int)trials, p, alpha);
                    if (dist == int.MaxValue)
                    {
                        return FormulaErrorStrings[this.calculation_overflow];
                    }
                }
                else
                {
                    return FormulaErrorStrings[this.invalid_arguments];
                }
            }

            return dist.ToString();
        }

        private int critbinom(int nTrials, double p, double alpha)
        {
            int checkval = nTrials;
            int half = nTrials;
            double dist = 1;
            double dist1 = 1;
            do
            {
                half = half / 2 + 1;
                if (dist >= alpha)
                {
                    dist1 = binomdist(nTrials, checkval - 1, p);
                    if (double.IsNaN(dist1))
                    {
                        return int.MaxValue;
                    }

                    if (dist1 < alpha && dist1 > 0)
                    {
                        break;
                    }

                    checkval = checkval - half;
                }
                else
                {
                    dist1 = binomdist(nTrials, checkval + 1, p);
                    if (dist1 >= alpha)
                    {
                        checkval = checkval + 1;
                        break;
                    }

                    checkval = checkval + half;
                }

                dist = binomdist(nTrials, checkval, p);
            }
            while (checkval < nTrials && checkval > 0);

            return checkval;
        }

        /// <summary>
        /// Returns the sum of the squares of the mean deviations.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.
        /// </param>
        /// <returns>Sum of the squares of the mean deviation.</returns>
        public string ComputeDevsq(string range)
        {
            double d;

            double[] x = GetDoubleArray(range);
            int n = x.GetLength(0);
            if (n <= 0)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sumx = 0;
            for (int i = 0; i < n; ++i)
            {
                sumx += x[i];
            }

            sumx = sumx / n;
            double sumx2 = 0;

            for (int i = 0; i < n; ++i)
            {
                d = x[i] - sumx;
                sumx2 += d * d;
            }

            return sumx2.ToString();
        }

        /// <summary>
        /// Returns the exponential distribution.
        /// </summary>
        /// <param name="argList">X, lambda, cumulative.</param>
        /// <returns>Returns exponential distribution.</returns>
        /// <remarks>
        /// Cumulative should be either 
        /// True if you want to return the value of the distribution function or False
        /// if you want to return the value of the density function. 
        /// </remarks>
        public string ComputeExpondist(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double lambda;
            double cum = 0;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out x)
                && double.TryParse(args[1], out lambda))
            {
                cum = (args[2] == "1" || args[2] == TRUEVALUESTR) ? 1 : 0;
                if (cum == 0)
                {
                    dist = lambda * Math.Exp(-lambda * x);
                }
                else
                {
                    dist = 1 - Math.Exp(-lambda * x);
                }
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the F (Fisher) probability distribution.
        /// </summary>
        /// <param name="argList">X, degreesfreedom1, degreesfreedom2.</param>
        /// <returns>Returns the F probability distribution.</returns>
        public string ComputeFdist(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double df1 = 0;
            double df2 = 0;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out x)
                && double.TryParse(args[1], out df1)
                && double.TryParse(args[2], out df2))
            {
                double mult = Math.Exp(
                    gammaln((df1 + df2) / 2) -
                    gammaln(df1 / 2) -
                    gammaln(df2 / 2) +
                    (df1 / 2) * Math.Log(df1 / df2));

                dist = 1 - mult * fdist(x, (int)df1, (int)df2);
            }

            return dist.ToString();
        }

        private double fdensity(double x, int df1, int df2)
        {
            return Math.Pow(x, (df1 - 2) / 2d) / Math.Pow(1 + df1 * x / df2, (df1 + df2) / 2d);
        }

        private double fdist(double x, int df1, int df2)
        {
            int nPanels = 32;

            double leftside = 0;
            double rightside = x;

            double h = (rightside - leftside) / nPanels;
            double mult = h / 3;

            double sum1 = fdensity(leftside, df1, df2) + fdensity(rightside, df1, df2);
            double sum4 = 0;
            for (int i = 1; i < nPanels; i += 2)
            {
                sum4 += 4 * fdensity(leftside + i * h, df1, df2);
            }

            double sum2 = 0;
            for (int i = 2; i < nPanels; i += 2)
            {
                sum2 += 2 * fdensity(leftside + i * h, df1, df2);
            }

            double gd = mult * (sum1 + sum2 + sum4);
            double oldgd = gd;
            double eps = 1e-7;

            int k = 0;

            for (k = 0; k < 20; ++k)
            {
                nPanels *= 2;

                sum2 += sum4 / 2;
                sum4 = 0;
                h = (rightside - leftside) / nPanels;
                for (int i = 0; i < nPanels; ++i)
                {
                    if (i % 2 == 1)
                    {
                        double d = fdensity(leftside + h * i, df1, df2);
                        sum4 += d;
                    }
                }

                sum4 = 4 * sum4;
                mult = h / 3;
                gd = mult * (sum1 + sum2 + sum4);

                if (Math.Abs((gd - oldgd) / oldgd) < eps)
                {
                    break;
                }

                oldgd = gd;
            }

            ////Console.WriteLine(k.ToString() + "  " + gd.ToString());
            return gd;
        }

        private double finv(double p, int df1, int df2)
        {
            double mult = Math.Exp(
                gammaln((df1 + df2) / 2d) -
                gammaln(df1 / 2d) -
                gammaln(df2 / 2d) +
                (df1 / 2d) * Math.Log((double)df1 / df2));
            double guess = mult;
            ////            if( p < .05)
            ////                guess = u - 2 * s;
            ////            else if( p < .5)
            ////                guess = u;
            ////            else if( p < .95)
            ////                guess = u + 2 * s;
            ////            else
            ////                guess = u + 5 * s;

            double lastIncrement = guess / 2;
            double gi = 0;
            double eps = 1e-7;
            int k = 100;
            int tries = 3;
            int its = 0;
            while (k == 100 && tries > 0)
            {
                tries--;
                guess = guess / 2;
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k)
                {
                    its++;
                    gi = 1 - mult * fdist(guess, df1, df2);
                    ////Console.WriteLine("{0}: guess {1}  computed {2}  target {3}", its, guess, gi, p);
                    if (Math.Abs((gi - p) / p) < eps)
                    {
                        break;
                    }
                    ////greater since looking for 1 - xxxx
                    if (gi > p)
                    {
                        guess += lastIncrement;
                    }
                    else
                    {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0)
                        {
                            lastIncrement = guess / 2;
                        }

                        guess -= lastIncrement;
                    }
                }
            }

            if (k == 100)
            {
                guess = -1;
            }

            ////Console.WriteLine(its);
            return guess;
        }

        /// <summary>
        /// Returns the inverse of F distribution.
        /// </summary>
        /// <param name="argList">P, degreesfreedom1, degreesfreedom2.</param>
        /// <returns>Returns x such that F distribution at x is p.</returns>
        /// <remarks>
        /// P should be between 0 and 1.
        /// </remarks>
        public string ComputeFinv(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double p;
            double df1;
            double df2;
            double invdist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out p)
                && (p > 0 && p < 1)
                && double.TryParse(args[1], out df1)
                && double.TryParse(args[2], out df2))
            {
                invdist = finv(p, (int)df1, (int)df2);
            }

            if (invdist <= 0)
            {
                return FormulaErrorStrings[iterations_dont_converge];
            }

            return invdist.ToString();
        }

        /// <summary>
        /// Returns the Fisher transformation of the input variable.
        /// </summary>
        /// <param name="argList">Input variable x.</param>
        /// <returns>Fisher transformation of x.</returns>
        /// <remarks>
        /// X should be between -1 and 1.
        /// </remarks>
        public string ComputeFisher(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 1)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double z = 0;

            string s = GetValueFromArg(args[0]);

            if (double.TryParse(s, out x)
                && (x > -1 && x < 1))
            {
                z = 0.5 * Math.Log((1 + x) / (1 - x));
            }

            return z.ToString();
        }

        /// <summary>
        /// Returns the inverse of Fisher transformation.
        /// </summary>
        /// <param name="argList">Input variable y.</param>
        /// <returns>The value x such that the Fisher transformation y is x.</returns>
        public string ComputeFisherinv(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 1)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double y;
            double x = 0;

            string s = GetValueFromArg(args[0]);

            if (double.TryParse(s, out y))
            {
                double d = Math.Exp(2 * y);
                x = (d - 1) / (d + 1);
            }

            return x.ToString();
        }

        /// <summary>
        /// Returns a forecasted value based on two sets of points using Least Square Fit regression.
        /// </summary>
        /// <param name="range">X, rangex, rangey.</param>
        /// <returns>returns Forecasted.</returns>
        public string ComputeForecast(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x0, d;
            string s1 = GetValueFromArg(args[0]);
            if (!double.TryParse(s1, out x0))
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            double[] y = GetDoubleArray(args[1]);
            double[] x = GetDoubleArray(args[2]);
            int n = x.GetLength(0);

            if (n <= 0 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sumx = 0;
            double sumy = 0;
            for (int i = 0; i < n; ++i)
            {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            double sumxy = 0;
            double sumx2 = 0;
            for (int i = 0; i < n; ++i)
            {
                d = x[i] - sumx;
                sumxy += d * (y[i] - sumy);
                sumx2 += d * d;
            }

            double b = sumxy / sumx2;
            double a = sumy - b * sumx;
            return (a + b * x0).ToString();
        }

        ////////        /// <summary>
        ////////        /// Returns the beta distribution.
        ////////        /// </summary>
        ////////        /// <param name="args">X, alpha, beta, a, b.</param>
        ////////        /// <returns>Returns the beta distribution.</returns>
        ////////        /// <remarks>
        ////////        /// If a and b are missing, a = 0 and b = 1. 
        ////////        /// </remarks>
        ////////        public string ComputeBetadist(string argList)
        ////////        {
        ////////            string[] args = argList.Split(new char[]{ParseArgumentSeparator});
        ////////            int argCount = args.GetLength(0);
        ////////            if(argCount != 3 && argCount != 4 && argCount != 5)
        ////////            {
        ////////                return FormulaErrorStrings[wrong_number_arguments];
        ////////            }
        ////////            double x;
        ////////            double alpha = 0;
        ////////            double beta = 0;
        ////////            double a = 0;
        ////////            double b = 1;
        ////////            double dist = 0;
        ////////            
        ////////            for(int i = 0; i < argCount; ++i)
        ////////            {
        ////////                args[i] = GetValueFromArg(args[i]);
        ////////            }
        ////////
        ////////            if(double.TryParse(args[0], NumberStyles.Any, null, out x)
        ////////                && double.TryParse(args[1], NumberStyles.Any, null, out alpha)
        ////////                && double.TryParse(args[2], NumberStyles.Any, null, out beta)
        ////////                && (argCount < 4 || double.TryParse(args[3], NumberStyles.Any, null, out a))
        ////////                && (argCount < 5 || double.TryParse(args[4], NumberStyles.Any, null, out b))
        ////////                )
        ////////            {
        ////////                dist = betadist(x, alpha, beta, a ,b);
        ////////            }
        ////////
        ////////            return dist.ToString();
        ////////        }
        ////////
        ////////        private double betadist(double x, double alpha, double beta, double a, double b)
        ////////        {
        ////////            double bmult = 1 / (betac(alpha, beta) * Math.Pow(b - a, alpha + beta - 1));
        ////////
        ////////            int nPanels = 32;
        ////////    
        ////////            double leftside = a;
        ////////            double rightside = x;
        ////////                
        ////////            double h = (rightside - leftside) / nPanels;  
        ////////            double mult = bmult * h / 3;
        ////////            
        ////////            double sum1 = betadensity(leftside, alpha, beta, a, b) + betadensity(rightside, alpha, beta, a, b);
        ////////            double sum4 =  0;
        ////////            for(int i = 1; i < nPanels; i += 2)
        ////////                sum4 += 4 * betadensity(leftside + i * h, alpha, beta, a, b);
        ////////            double sum2 = 0;
        ////////            for(int i = 2; i < nPanels; i += 2)
        ////////                sum2 += 2 * betadensity(leftside + i * h, alpha, beta, a, b);
        ////////            double gd = mult * (sum1 + sum2 + sum4);
        ////////            double oldgd = gd;
        ////////            double eps = 1e-7;
        ////////
        ////////            int k = 0;
        ////////
        ////////            for(k = 0; k < 20; ++k)
        ////////            {
        ////////                nPanels *= 2;
        ////////
        ////////                sum2 += sum4 / 2;
        ////////                sum4 = 0;
        ////////                h = (rightside - leftside) / nPanels;
        ////////                for(int i = 0; i < nPanels; ++i)
        ////////                {
        ////////                    if(i % 2 == 1)
        ////////                    {
        ////////                        double d = betadensity(leftside + h * i, alpha, beta, a, b);
        ////////                        sum4 += d;
        ////////                    }
        ////////                }
        ////////                sum4 = 4 * sum4;
        ////////                mult = bmult * h / 3;
        ////////                gd = mult * (sum1 + sum2 + sum4);
        ////////                
        ////////                if(Math.Abs((gd - oldgd)/oldgd) < eps)
        ////////                    break;
        ////////                
        ////////                oldgd = gd;
        ////////                
        ////////            }
        ////////            //Console.WriteLine(k.ToString() + "  " + gd.ToString());
        ////////            return gd;
        ////////        
        ////////        }
        ////////
        ////////        private double betadensity(double x, double alpha, double beta, double a, double b)
        ////////        {
        ////////            double d = Math.Pow(x-a, alpha-1) * Math.Pow(b-x, beta-1);
        ////////            return d;
        ////////        }
        ////////
        ////////        private double betac(double p, double q)
        ////////        {
        ////////            int nPanels = 8;
        ////////    
        ////////            double leftside = 0;
        ////////            double rightside = 1;
        ////////                
        ////////            double h = (rightside - leftside) / nPanels;  
        ////////            double mult = h / 3;
        ////////            
        ////////            double sum1 = betaf(leftside , p, q) + betaf(rightside, p, q);
        ////////            double sum4 =  0;
        ////////            for(int i = 1; i < nPanels; i += 2)
        ////////                sum4 += 4 * betaf(leftside + i * h , p, q);
        ////////            double sum2 = 0;
        ////////            for(int i = 2; i < nPanels; i += 2)
        ////////                sum2 += 2 * betaf(leftside + i * h , p, q);
        ////////            double gd = mult * (sum1 + sum2 + sum4);
        ////////            double oldgd = gd;
        ////////            double eps = 1e-7;
        ////////
        ////////            int k = 0;
        ////////
        ////////            for(k = 0; k < 20; ++k)
        ////////            {
        ////////                nPanels *= 2;
        ////////
        ////////                sum2 += sum4 / 2;
        ////////                sum4 = 0;
        ////////                h = h = (rightside - leftside) / nPanels;
        ////////                for(int i = 0; i < nPanels; ++i)
        ////////                {
        ////////                    if(i % 2 == 1)
        ////////                    {
        ////////                        double d = betaf(leftside + h * i, p, q);
        ////////                        sum4 += d;
        ////////                    }
        ////////                }
        ////////                sum4 = 4 * sum4;
        ////////                mult = h / 3;
        ////////                gd = mult * (sum1 + sum2 + sum4);
        ////////                // Console.WriteLine(k.ToString() + " c " + gd.ToString());
        ////////                if(Math.Abs((gd - oldgd)/oldgd) < eps)
        ////////                    break;
        ////////                
        ////////                oldgd = gd;
        ////////                
        ////////            }
        ////////            //Console.WriteLine(k.ToString() + " c " + gd.ToString());
        ////////            return gd;
        ////////        }
        ////////
        ////////        private double betain(double x, double p, double q)
        ////////        {
        ////////            double betaln = this.gammaln(p) + gammaln(q) - gammaln(p+q);
        ////////            double retVal = x;
        ////////
        ////////            bool flip = false;
        ////////            double psq = p + q;
        ////////            double cx = 1 - x;
        ////////            double xx, qq, pp;
        ////////            if( p < psq * x)
        ////////            {
        ////////                xx = cx;
        ////////                cx = x;
        ////////                pp = q;
        ////////                qq = p;
        ////////                flip = true;
        ////////            }
        ////////            else
        ////////            {
        ////////                xx = x;
        ////////                pp = p;
        ////////                qq = q;
        ////////                flip = false;
        ////////            }
        ////////
        ////////            double term = 1;
        ////////            double ai = 1;
        ////////                retVal = 1;
        ////////            double ns = qq + cx * psq;
        ////////            double rx = xx / cx;
        ////////    three:        double temp = qq - ai;
        ////////            if(ns == 0)
        ////////                rx = xx;
        ////////    four:        term = term * temp * rx / (pp+ai);
        ////////            retVal += term;
        ////////            temp = Math.Abs(term);
        ////////            if(temp <= 1e-7 && temp <= 1e-7 * retVal) goto five;
        ////////            ai += 1;
        ////////            ns -= 1;
        ////////            if(ns >= 0) goto three;
        ////////            temp = psq;
        ////////            psq += 1;
        ////////            goto four;
        ////////            five:
        ////////                retVal = retVal * Math.Exp(pp * Math.Log(xx) + qq-1) * Math.Log(cx) - betaln /pp;
        ////////
        ////////            if(flip)
        ////////                retVal = 1 - retVal;
        ////////            
        ////////            return retVal;
        ////////
        ////////        }
        ////////
        ////////        private double betaf(double x, double p, double q)
        ////////        {
        ////////            return Math.Pow(x, p-1) * Math.Pow((1 - x), q-1);
        ////////        }
        ////////
        ////////        /// <summary>
        ////////        /// Returns the inverse of beta distribution.
        ////////        /// </summary>
        ////////        /// <param name="argList">P, alpha, beta, a, b.</param>
        ////////        /// <returns>Returns x such that normal distribution at x is p.</returns>
        ////////        /// <remarks>
        ////////        /// P should be between 0 and 1.
        ////////        /// </remarks>
        ////////        public string ComputeBetainv(string argList)
        ////////        {
        ////////            string[] args = argList.Split(new char[]{ParseArgumentSeparator});
        ////////            int argCount = args.GetLength(0);
        ////////            if(argCount != 3 && argCount != 4 && argCount != 5)
        ////////            {
        ////////                return FormulaErrorStrings[wrong_number_arguments];
        ////////            }
        ////////            double p;
        ////////            double alpha = 0;
        ////////            double beta = 0;
        ////////            double a = 0;
        ////////            double b = 1;
        ////////            double invdist = 0;
        ////////            
        ////////            for(int i = 0; i < argCount; ++i)
        ////////            {
        ////////                args[i] = GetValueFromArg(args[i]);
        ////////            }
        ////////
        ////////            if(double.TryParse(args[0], NumberStyles.Any, null, out p)
        ////////                && double.TryParse(args[1], NumberStyles.Any, null, out alpha)
        ////////                && double.TryParse(args[2], NumberStyles.Any, null, out beta)
        ////////                && (argCount < 4 || double.TryParse(args[3], NumberStyles.Any, null, out a))
        ////////                && (argCount < 5 || double.TryParse(args[4], NumberStyles.Any, null, out b))
        ////////                )
        ////////            {
        ////////                invdist = betainv(p, alpha, beta, a, b);
        ////////            }
        ////////            
        ////////            if(invdist <= 0)
        ////////                return FormulaErrorStrings[iterations_dont_converge];
        ////////
        ////////            return invdist.ToString();
        ////////        }
        ////////
        ////////        private double betainv(double p, double alpha, double beta, double a, double b)
        ////////        {
        ////////            double guess = (a + b) / 2; ////(u == 0) ? 1 : u;
        ////////    
        ////////            double lastIncrement = (b - a) / 4;
        ////////            double gi = 0;
        ////////            double eps = 1e-7;
        ////////            int k = 100;
        ////////            int tries = 3;
        ////////            int its = 0;
        ////////            while(k == 100 && tries > 0)
        ////////            {
        ////////                tries--;
        ////////                lastIncrement = (guess - a)/2;
        ////////                guess = guess - (guess - a)/2;
        ////////                
        ////////                 
        ////////                for(k = 0; k < 100; ++k)
        ////////                {
        ////////                    its++;
        ////////                    gi = betadist(guess, alpha, beta, a, b);
        ////////                    if(Math.Abs((gi - p)/p) < eps)
        ////////                    {
        ////////                        break;
        ////////                    }
        ////////                    if(gi < p)
        ////////                    {
        ////////                        if(guess + lastIncrement > b)
        ////////                            lastIncrement = (b - guess) /2;
        ////////                        guess += lastIncrement;
        ////////                    }
        ////////                    else
        ////////                    {
        ////////                        lastIncrement = lastIncrement / 2;
        ////////                        if(guess - lastIncrement < a)
        ////////                            lastIncrement = (guess - a) / 2;
        ////////                        guess -= lastIncrement;
        ////////                    }
        ////////                }
        ////////            }
        ////////            if(k == 100)
        ////////                guess = -1;
        ////////
        ////////            //Console.WriteLine(its);
        ////////            return guess;
        ////////        }
        ////////
        ////////        /// <summary>
        ////////        /// Returns an F-test statistic.
        ////////        /// </summary>
        ////////        /// <param name="range">Range1, range2.</param>
        ////////        /// <returns>F-test statistic.</returns>
        ////////        public string ComputeFtest(string range)
        ////////        {
        ////////
        ////////            string[] args = range.Split(new char[]{ParseArgumentSeparator});
        ////////            int argCount = args.GetLength(0);
        ////////            if(argCount != 2 )
        ////////            {
        ////////                return FormulaErrorStrings[wrong_number_arguments];
        ////////            }
        ////////
        ////////            double[] yy = GetDoubleArray(args[0]);
        ////////            double[] xx = GetDoubleArray(args[1]);
        ////////             
        ////////            if(xx.GetLength(0) < 2 ||  yy.GetLength(0) < 2)
        ////////                return FormulaErrorStrings[wrong_number_arguments];
        ////////
        ////////            double varx = var(xx);
        ////////            double vary = var(yy);
        ////////
        ////////            double vratio;
        ////////            double dft;
        ////////            double dfb;
        ////////            if(varx < vary)
        ////////            {
        ////////                vratio = vary / varx;
        ////////                dft = yy.GetLength(0) - 1;
        ////////                dfb = xx.GetLength(0) - 1;
        ////////            }
        ////////            else
        ////////            {
        ////////                vratio = varx / vary;
        ////////                dft = xx.GetLength(0) - 1;
        ////////                dfb = yy.GetLength(0) - 1;
        ////////            }
        ////////            
        ////////            double d = dfb / (dfb + dft * vratio);
        ////////            d = 2 * betadist(d, dfb / 2, dft / 2, 0, 1); //both halves
        ////////            if(d > 1)
        ////////            {
        ////////                d = 2 - d;
        ////////            }
        ////////            return d.ToString();
        ////////        }

        private double var(double[] x)
        {
            double sumx = 0;
            int n = x.GetLength(0);
            for (int i = 0; i < n; ++i)
            {
                sumx += x[i];
            }

            sumx = sumx / n;

            double sumx2 = 0;
            double d;
            for (int i = 0; i < n; ++i)
            {
                d = x[i] - sumx;
                sumx2 += d * d;
            }

            return sumx2 / (n - 1);
        }

        /// <summary>
        /// Returns the kurtosis of the passed-in values.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.
        /// </param>
        /// <returns>The kurtosis of the data.</returns>
        public string ComputeKurt(string range)
        {
            double[] xx = GetDoubleArray(range);
            int n = xx.GetLength(0);
            if (n <= 3)
            {
                return FormulaErrorStrings[this.wrong_number_arguments];
            }

            double xbar = 0;
            double sdev = sd(xx, out xbar);

            double sum = 0;
            for (int i = 0; i < n; ++i)
            {
                sum += Math.Pow((xx[i] - xbar) / sdev, 4);
            }

            xbar = (double)n; // make it a double to avoid integer arithmetic
            sum = xbar * (xbar + 1) / ((xbar - 1) * (xbar - 2) * (xbar - 3)) * sum - 3 * (xbar - 1) * (xbar - 1) / ((xbar - 2) * (xbar - 3));
            return sum.ToString();
        }

        /// <summary>
        /// Returns the kth largest value in the range.
        /// </summary>
        /// <param name="range">The Range, k.</param>
        /// <returns>Kth largest value.</returns>
        public string ComputeLarge(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x0;
            string s1 = GetValueFromArg(args[1]);
            if (!double.TryParse(s1, NumberStyles.Integer, null, out x0))
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            int k = (int)x0;
            double[] dd = GetDoubleArray(args[0]);
            int n = dd.GetLength(0);
            if (k < 1 || k > n)
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            Array.Sort(dd);
            return dd[n - k].ToString();
        }

        /// <summary>
        /// Returns the lognormal distribution.
        /// </summary>
        /// <param name="argList">X, mean, standarddev.</param>
        /// <returns>Returns lognormal distribution.</returns>
        public string ComputeLognormdist(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double u;
            double s;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out x)
                && double.TryParse(args[1], out u)
                && double.TryParse(args[2], out s))
            {
                dist = normaldist(Math.Log(x), u, s);
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the inverse of the lognormal distribution.
        /// </summary>
        /// <param name="argList">P, mean, standarddev.</param>
        /// <returns>Returns the value x where the lognormal distribution of x is p.</returns>
        public string ComputeLoginv(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double p;
            double u;
            double s;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out p)
                && double.TryParse(args[1], out u)
                && double.TryParse(args[2], out s))
            {
                dist = Math.Exp(normalinv(p, u, s));
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the maximum value of all values listed in the argument including logical values.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the maximum value of all values listed in the argument.</returns>
        /// <remarks> True is treated as 1 and False is treated as 0.
        /// </remarks>
        public string ComputeMaxa(string range)
        {
            double max = double.MinValue;
            double d;
            string s1;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                if (r.IndexOf(':') > -1)
                {
                    ////cell range
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            ////TraceUtil.TraceExceptionCatched(ex);
                            ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            ////    throw ex;
                            return ex.Message;
                            ////throw new ArgumentException(ex.Message);
                        }
                        ////ignore if empty
                        if (s1.Length > 0)
                        {
                            d = 0;
                            if (s1.ToUpper() == TRUEVALUESTR)
                            {
                                d = 1;
                            }
                            else
                            {
                                double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d);
                            }

                            max = Math.Max(max, d);
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        d = 0;
                        if (s1.ToUpper() == TRUEVALUESTR)
                        {
                            d = 1;
                        }
                        else
                        {
                            double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d);
                        }

                        max = Math.Max(max, d);
                    }
                }
            }

            if (max != double.MinValue)
            {
                return max.ToString();
            }

            return string.Empty;
        }

        /// <summary>
        /// Returns the median value in the range.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.
        /// </param>
        /// <returns>Median value.</returns>
        public string ComputeMedian(string range)
        {
            double[] dd = GetDoubleArray(range);
            Array.Sort(dd);
            int n = dd.GetLength(0) / 2;
            string s1 = string.Empty;
            if (dd.GetLength(0) % 2 == 1)
            {
                s1 = dd[n].ToString();
            }
            else
            {
                s1 = ((dd[n] + dd[n - 1]) / 2).ToString();
            }

            return s1;
        }

        /// <summary>
        /// Returns the minimum value of all values listed in the argument including logical values.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>A string holding the minimum value of all values listed in the argument.</returns>
        /// <remarks> True is treated as 1 and False is treated as 0.
        /// </remarks>
        public string ComputeMina(string range)
        {
            double min = double.MaxValue;
            double d;
            string s1;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////cell range
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            ////TraceUtil.TraceExceptionCatched(ex);
                            ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            ////    throw ex;
                            return ex.Message;
                            ////throw new ArgumentException(ex.Message);
                        }

                        if (s1.Length > 0)
                        {
                            d = 0;
                            if (s1.ToUpper() == TRUEVALUESTR)
                            {
                                d = 1;
                            }
                            else
                            {
                                double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d);
                            }

                            min = Math.Min(min, d);
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        ////TraceUtil.TraceExceptionCatched(ex);
                        ////if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        ////    throw ex;
                        return ex.Message;
                    }

                    if (s1.Length > 0)
                    {
                        d = 0;
                        if (s1.ToUpper() == TRUEVALUESTR)
                        {
                            d = 1;
                        }
                        else
                        {
                            double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d);
                        }

                        min = Math.Min(min, d);
                    }
                }
            }

            if (min != double.MaxValue)
            {
                return min.ToString();
            }

            return string.Empty;
        }

        /// <summary>
        /// Returns the most frequent value in the range.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.
        /// </param>
        /// <returns>The most frequent value.</returns>
        public string ComputeMode(string range)
        {
            double[] dd = GetDoubleArray(range);
            double[] ddSave = dd.Clone() as Double[];
            int n = dd.GetLength(0);
            if (n <= 1)
            {
                return "#N/A";
            }

            Array.Sort(dd);
            ArrayList ties = new ArrayList();
            double mode = double.NaN;
            int count = 0;
            int maxCount = 0;

            for (int i = 1; i < n; ++i)
            {
                if (dd[i] == dd[i - 1])
                {
                    count++;
                }
                else
                {
                    if (count > maxCount)
                    {
                        maxCount = count;
                        mode = dd[i - 1];
                        ties.Clear();
                        ties.Add(mode);
                    }
                    else if (count == maxCount)
                    {
                        ties.Add(dd[i - 1]);
                    }

                    count = 0;
                }
            }

            if (count > maxCount)
            {
                maxCount = count;
                mode = dd[n - 1];
                ties.Clear();
                ties.Add(mode);
            }

            if (maxCount > 0)
            {
                if (count == maxCount)
                {
                    ties.Add(dd[n - 1]);
                }

                if (ties.Count > 1)
                {
                    foreach (double d in ddSave)
                    {
                        if (ties.IndexOf(d) > -1)
                        {
                            mode = d;
                            break;
                        }
                    }
                }

                return mode.ToString();
            }

            return "#N/A";
        }

        /// <summary>
        /// Returns the negative binomial distribution.
        /// </summary>
        /// <param name="argList">Number of failures, success threshold, probability, cumulative.</param>
        /// <returns>Returns negative binomial distribution.</returns>
        public string ComputeNegbinomdist(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double successes;
            double failures;
            double p;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out failures)
                && double.TryParse(args[1], out successes)
                && double.TryParse(args[2], out p))
            {
                dist = negbinomdensity((int)failures, (int)successes, p);
            }

            return dist.ToString();
        }

        private double negbinomdensity(int failures, int successes, double p)
        {
            return this.comb(successes - 1, failures + successes - 1) * Math.Pow(p, successes) * Math.Pow(1 - p, failures);
        }

        /// <summary>
        /// Returns the Pearson product moment correlation coefficient.
        /// </summary>
        /// <param name="range">range1, range2</param>
        /// <returns>Pearson product</returns>
        public string ComputePearson(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = GetDoubleArray(args[1]);

            int n = x.GetLength(0);

            if (n < 2 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sumx = 0;
            double sumy = 0;
            for (int i = 0; i < n; ++i)
            {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            double sumxy = 0;
            double sumx2 = 0;
            double sumy2 = 0;
            double d1, d;
            for (int i = 0; i < n; ++i)
            {
                d = x[i] - sumx;
                d1 = y[i] - sumy;
                sumxy += d * d1;
                sumx2 += d * d;
                sumy2 += d1 * d1;
            }

            return (sumxy / Math.Sqrt(sumx2 * sumy2)).ToString();
        }

        /// <summary>
        /// Returns the percentile position in the range.
        /// </summary>
        /// <param name="range">The Range, k.</param>
        /// <returns>Percentile position.</returns>
        /// <remarks>K is a value between 0 and 1.</remarks>
        public string ComputePercentile(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double k;
            string s1 = GetValueFromArg(args[1]);
            if (!double.TryParse(s1, out k) && (k < 0 || k > 1))
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            double[] dd = GetDoubleArray(args[0]);
            int n = dd.GetLength(0);

            Array.Sort(dd);

            double h = 1d / (n - 1);
            double d = dd[n - 1];
            for (int i = 0; i < n - 1; ++i)
            {
                if ((i + 1) * h > k)
                {
                    k = (k - i * h) / h;
                    d = dd[i] + k * (dd[i + 1] - dd[i]);
                    break;
                }
            }

            return d.ToString();
        }

        /// <summary>
        /// Returns the percentage rank in the range.
        /// </summary>
        /// <param name="range">Range, x, significant digits.</param>
        /// <returns>Percentile position.</returns>
        /// <remarks>Significant digits are optional, defaulting to 3.</remarks>
        public string ComputePercentrank(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double signif = 3;
            string s1 = GetValueFromArg(args[1]);
            if (!double.TryParse(s1, out x))
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            if (argCount == 3)
            {
                s1 = GetValueFromArg(args[2]);
                if (!double.TryParse(s1, NumberStyles.Integer, null, out signif) && signif < 1)
                {
                    return FormulaErrorStrings[invalid_arguments];
                }
            }

            double[] dd = GetDoubleArray(args[0]);
            int n = dd.GetLength(0);

            Array.Sort(dd);

            double d = 1;
            for (int i = 0; i < n; ++i)
            {
                if (dd[i] > x)
                {
                    int k = 0;
                    while (k + i < n && dd[k + i] == x)
                    {
                        k++;
                    }

                    d = ((double)(i - 1)) / (i + n - i - k - 1);

                    if (i > 0 && dd[i - 1] < x)
                    {
                        double di = ((double)i) / (n - 1);
                        d = di + (d - di) * (1 - (x - dd[i - 1]) / (dd[i] - dd[i - 1]));
                    }

                    break;
                }
            }

            string fmt = "0." + new string('#', (int)(signif + 1));
            fmt = d.ToString(fmt);
            return fmt.Substring(0, fmt.Length - 1);
        }

        /// <summary>
        /// The number of permutations of n items taken k at the time. 
        /// </summary>
        /// <param name="argList">The arglist, N, k</param>
        /// <returns>The number of combinations.</returns>
        public string ComputePermut(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[this.requires_2_args];
            }
            ////N things taken k at the time.
            double nd;
            double kd;
            double combin = 0;
            if (double.TryParse(GetValueFromArg(args[0]), out nd)
                && double.TryParse(GetValueFromArg(args[1]), out kd))
            {
                int k = (int)(kd + 0.1);
                int n = (int)(nd + 0.1);

                double top = 1;
                for (int i = (n - k) + 1; i <= n; ++i)
                {
                    top = top * i;
                }

                combin = top;
            }
            else
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            return combin.ToString();
        }

        /// <summary>
        /// Returns the Poisson distribution.
        /// </summary>
        /// <param name="argList">X, mean, cumulative.</param>
        /// <returns>Returns the exponential distribution.</returns>
        /// <remarks>
        /// Cumulative should be either 
        /// True if you want to return the value of the distribution function or False
        /// if you want to return the value of the density function. 
        /// </remarks>
        public string ComputePoisson(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double u;
            double cum = 0;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out x)
                && double.TryParse(args[1], out u))
            {
                cum = (args[2] == TRUEVALUESTR || args[2] == "1") ? 1 : 0;
                int n = (int)x;
                if (cum == 0)
                {
                    double prod = 1;
                    for (int i = 2; i <= n; ++i)
                    {
                        prod *= i;
                    }

                    dist = Math.Exp(-u) * Math.Pow(u, n) / prod;
                }
                else
                {
                    double prod = 1;
                    dist = 0;
                    double ui = 1;
                    for (int i = 0; i <= n; ++i)
                    {
                        dist += ui / prod;
                        prod *= i + 1;
                        ui *= u;
                    }

                    dist = Math.Exp(-u) * dist;
                }
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the probability that a value in the given range occurs.
        /// </summary>
        /// <param name="range">Xrange1, prange2, lowerbound, upperbound.</param>
        /// <returns>The probability.</returns>
        public string ComputeProb(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3 && argCount != 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            string s1 = GetValueFromArg(args[2]);
            double lower = 0;
            if (!double.TryParse(s1, out lower))
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double upper = lower;
            if (argCount == 4)
            {
                s1 = GetValueFromArg(args[3]);
                double.TryParse(s1, out upper);
            }

            double[] y = GetDoubleArray(args[1]);
            double[] x = GetDoubleArray(args[0]);
            int n = x.GetLength(0);

            if (n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sum = 0;
            double d;
            for (int i = 0; i < n; ++i)
            {
                d = x[i];
                if (d >= lower && d <= upper)
                {
                    sum += y[i];
                }
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns the quartile position in the range.
        /// </summary>
        /// <param name="range">The Range, q.</param>
        /// <returns>Percentile position.</returns>
        /// <remarks>Q is 0, 1, 2, 3, 4.</remarks>
        public string ComputeQuartile(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double q;
            string s1 = GetValueFromArg(args[1]);
            if (double.TryParse(s1, NumberStyles.Integer, null, out q) && q >= 0 && q <= 4)
            {
                return this.ComputePercentile(args[0] + ParseArgumentSeparator + (q * .25).ToString());
            }

            return FormulaErrorStrings[this.invalid_arguments];
        }

        /// <summary>
        /// Returns the rank of x in the range.
        /// </summary>
        /// <param name="range">X, range, order.</param>
        /// <returns>returns Rank.</returns>
        public string ComputeRank(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            int rank = 0;
            double x;
            string s1 = GetValueFromArg(args[0]);
            if (double.TryParse(s1, out x))
            {
                double order = 0;
                if (argCount == 3)
                {
                    s1 = GetValueFromArg(args[2]);
                    double.TryParse(s1, NumberStyles.Integer, null, out order);
                }

                string r = args[1];

                double d = 0;
                bool eq = false;
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (double.TryParse(s1, out d))
                        {
                            if (order == 1)
                            {
                                if (d < x)
                                {
                                    rank += 1;
                                }
                                else if (d == x)
                                {
                                    eq = true;
                                }
                            }
                            else
                            {
                                if (d > x)
                                {
                                    rank += 1;
                                }
                                else if (d == x)
                                {
                                    eq = true;
                                }
                            }
                        }
                    }

                    if (eq)
                    {
                        rank += 1;
                    }
                }
            }

            return rank.ToString();
        }

        /// <summary>
        /// Returns the square of the Pearson product moment correlation coefficient.
        /// </summary>
        /// <param name="range">Range1, range2.</param>
        /// <returns>Square of the Pearson product.</returns>
        public string ComputeRsq(string range)
        {
            string s = this.ComputePearson(range);
            double d = 0;
            if (double.TryParse(s, out d))
            {
                d = d * d;
            }

            return d.ToString();
        }

        /// <summary>
        /// Returns the skewness of a distribution.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.
        /// </param>
        /// <returns>Skewness of a distribution.</returns>
        public string ComputeSkew(string range)
        {
            double[] x = GetDoubleArray(range);
            int n = x.GetLength(0);

            if (n < 3)
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double xbar = 0;
            double sdev = sd(x, out xbar);
            double skew = 0;
            for (int i = 0; i < n; ++i)
            {
                skew += Math.Pow((x[i] - xbar) / sdev, 3);
            }

            return (n * skew / (n - 1) / (n - 2)).ToString();
        }

        private double sd(double[] x, out double xbar)
        {
            int n = x.GetLength(0);
            xbar = 0;
            for (int i = 0; i < n; ++i)
            {
                xbar += x[i];
            }

            xbar = xbar / n;
            double sumx2 = 0;
            double d = 0;

            for (int i = 0; i < n; ++i)
            {
                d = x[i] - xbar;
                sumx2 += d * d;
            }

            return Math.Sqrt(sumx2 / (n - 1));
        }

        /// <summary>
        /// Returns the m parameter of the exponential curve y = b * m^x that best fits the given points. Only the first two Excel parameters are used.
        /// </summary>
        /// <param name="range">Y_range, x_range.</param>
        /// <returns>returns m parameter.</returns>
        public string ComputeLogest(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            bool padXValues = argCount == 1;
            if (argCount == 0 || argCount > 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = null;
            if (argCount > 1 && args[1] == "")
            {
                x = y;
            }
            else
            {
                x = padXValues ? new double[y.GetLength(0)] : GetDoubleArray(args[1]);
            }
            int n = x.GetLength(0);

            if (n <= 2 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }
            if (padXValues)
            {
                for (int i = 0; i < n; ++i)
                {
                    x[i] = (double)(i + 1);
                }
            }

            for (int i = 0; i < n; ++i)
            {
                y[i] = Math.Log10(y[i]);
            }

            double m = 0;
            double b = 1;

            ComputeLogestMB(y, x, out m, out b);
            return m.ToString();
        }

        /// <summary>
        /// Returns the growth estimate using the exponential curve y = b * m^x that best fits the given points. Only the first two Excel parameters are used.
        /// </summary>
        /// <param name="range">Y_range, x_range.</param>
        /// <returns>returns estimated value.</returns>
        public string ComputeGrowth(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            bool padXValues = argCount == 1;
            if (argCount == 0 || argCount > 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = null;
            if (argCount > 1 && args[1] == "")
            {
                x = y;
            }
            else
            {
                x = padXValues ? new double[y.GetLength(0)] : GetDoubleArray(args[1]);
            }
            double[] newx = argCount < 3 ? new double[x.GetLength(0)] : GetDoubleArray(args[2]);

            int n = x.GetLength(0);

            if (n <= 2 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }
            if (padXValues)
            {
                for (int i = 0; i < n; ++i)
                {
                    x[i] = (double)(i + 1);
                }
            }

            for (int i = 0; i < n; ++i)
            {
                y[i] = Math.Log10(y[i]);
            }

            double m = 0;
            double b = 1;

            ComputeLogestMB(y, x, out m, out b);

            int n1 = newx.GetLength(0);
            if (argCount < 3)
            {
                for (int i = 0; i < n1; ++i)
                {
                    newx[i] = x[i];
                }
            }

            for (int i = 0; i < n1; ++i)
            {
                newx[i] = b * Math.Pow(m, newx[i]);
            }
            if (n1 > 1)
            {
                //populate cells under or to the left of the current cell with the other values...
                string[] cells = GetCellsFromArgs(args[2]);
                bool isColumn = RowIndex(cells[0]) == RowIndex(cells[1]);
                for (int i = 1; i < n1; ++i)
                {
                    SetValueToGridColumnOrRow(cells[i], newx[i], isColumn, isColumn ? RowIndex(cell) : ColIndex(cell));
                }
            }
            return newx[0].ToString();
        }

        /// <summary>
        /// A method that sets the data value to the specified cell.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <param name="fixedIndex">Index value</param>
        internal void SetValueToGridColumnOrRow(string cell1, object value, bool isColumn, int fixedIndex)
        {
            int i = cell1.LastIndexOf(sheetToken);
            int row, col;
            GridModel grd = grid;
            if (i > -1)
            {
                GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);

                grd = (GridModel)family.TokenToGridModel[cell1.Substring(0, i + 1)];
                row = RowIndex(cell1);
                col = ColIndex(cell1);
            }
            else
            {
                row = RowIndex(cell1);
                col = ColIndex(cell1);
            }
            if (isColumn)
            {
                grd[fixedIndex, col].CellValue = value;
            }
            else
            {
                grd[row, fixedIndex].CellValue = value;
            }

        }

        /// <summary>
        /// Returns the m and b value used in exponential curve y = b * m^x.
        /// </summary>
        /// <param name="y">The set of y-values required in y = b*m^x</param>
        /// <param name="x">The set of x-values in y = b*m^x</param>
        /// <returns>returns calculated b,m value.</returns>
        internal void ComputeLogestMB(double[] y, double[] x, out double m, out double b)
        {
            m = 0;
            b = 1;


            int n = x.GetLength(0);
            double sumx = 0;
            double sumy = 0;
            for (int i = 0; i < n; ++i)
            {
                sumx += (double)x[i];
                sumy += (double)y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            double sumxy = 0;
            double sumx2 = 0;
            double d;
            for (int i = 0; i < n; ++i)
            {
                d = (double)x[i] - sumx;
                sumxy += d * (y[i] - sumy);
                sumx2 += d * d;
            }

            b = Math.Pow(10, sumy - sumxy / sumx2 * sumx);
            m = Math.Pow(10, sumxy / sumx2);
        }

        /// <summary>
        /// Returns the b parameter of the exponential curve y = b * m^x that best fits the given points.
        /// </summary>
        /// <param name="range">Y_range, x_range.</param>
        /// <returns>returns b parameter.</returns>
        public string ComputeLogestb(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            bool padXValues = argCount == 1;
            if (argCount == 0 || argCount > 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = padXValues ? new double[y.GetLength(0)] : GetDoubleArray(args[1]);
            int n = x.GetLength(0);

            if (n <= 2 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }
            if (padXValues)
            {
                for (int i = 0; i < n; ++i)
                {
                    x[i] = (double)(i + 1);
                }
            }

            for (int i = 0; i < n; ++i)
            {
                y[i] = Math.Log10(y[i]);
            }

            double m = 0;
            double b = 1;

            ComputeLogestMB(y, x, out m, out b);

            return b.ToString();
        }
        /// <summary>
        /// Returns the slope of the least square fit line through the given points.
        /// </summary>
        /// <param name="range">Y_range, x_range.</param>
        /// <returns>returns y-intercept.</returns>
        public string ComputeSlope(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = GetDoubleArray(args[1]);
            int n = x.GetLength(0);

            if (n <= 2 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sumx = 0;
            double sumy = 0;
            for (int i = 0; i < n; ++i)
            {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            double sumxy = 0;
            double sumx2 = 0;
            double d;
            for (int i = 0; i < n; ++i)
            {
                d = x[i] - sumx;
                sumxy += d * (y[i] - sumy);
                sumx2 += d * d;
            }

            return (sumxy / sumx2).ToString();
        }

        /// <summary>
        /// Returns the kth smallest value in the range.
        /// </summary>
        /// <param name="range">The Range, k.</param>
        /// <returns>Kth smallest value.</returns>
        public string ComputeSmall(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x0;
            string s1 = GetValueFromArg(args[1]);
            if (!double.TryParse(s1, NumberStyles.Integer, null, out x0))
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            int k = (int)x0;
            double[] dd = GetDoubleArray(args[0]);
            if (k < 1 || k > dd.GetLength(0))
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            Array.Sort(dd);
            return dd[k - 1].ToString();
        }

        /// <summary>
        /// Returns a normalized value.
        /// </summary>
        /// <param name="argList">X, mean, stddev.</param>
        /// <returns>Normalized value.</returns>
        public string ComputeStandardize(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double u;
            double sd = 0;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out x)
                && double.TryParse(args[1], out u)
                && double.TryParse(args[2], out sd))
            {
                dist = (x - u) / sd;
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the sample standard deviation.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.
        /// </param>
        /// <returns>The sample standard deviation.</returns>
        public string ComputeStdev(string range)
        {
            double[] dd = GetDoubleArray(range);
            int n = dd.GetLength(0);

            if (n < 2)
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double xbar = 0;
            double sdev = sd(dd, out xbar);
            return sdev.ToString();
        }

        /// <summary>
        /// Returns the sample standard deviation.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.
        /// </param>
        /// <returns>The sample standard deviation.</returns>
        /// <remarks>Treats True as 1 and False as 0.
        /// </remarks>
        public string ComputeStdeva(string range)
        {
            double[] dd = GetDoubleArrayA(range);
            int n = dd.GetLength(0);

            if (n < 2)
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double xbar = 0;
            double sdev = sd(dd, out xbar);
            return sdev.ToString();
        }

        /// <summary>
        /// Returns the population standard deviation.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.
        /// </param>
        /// <returns>The population standard deviation.</returns>
        public string ComputeStdevp(string range)
        {
            double[] dd = GetDoubleArray(range);
            int n = dd.GetLength(0);

            if (n < 2)
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double xbar = 0;
            double sdev = sd(dd, out xbar);
            return (sdev * Math.Sqrt(n - 1) / Math.Sqrt(n)).ToString();
        }

        /// <summary>
        /// Returns the population standard deviation.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.
        /// </param>
        /// <returns>The population standard deviation.</returns>
        /// <remarks>Treats True as 1 and False as 0.
        /// </remarks>
        public string ComputeStdevpa(string range)
        {
            double[] dd = GetDoubleArrayA(range);
            int n = dd.GetLength(0);

            if (n < 2)
            {
                return FormulaErrorStrings[this.invalid_arguments];
            }

            double xbar = 0;
            double sdev = sd(dd, out xbar);
            return (sdev * Math.Sqrt(n - 1) / Math.Sqrt(n)).ToString();
        }

        /// <summary>
        /// Returns the standard error of the least square fit line through the given points.
        /// </summary>
        /// <param name="range">Y_range, x_range.</param>
        /// <returns>Standard error.</returns>
        public string ComputeSteyx(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double[] y = GetDoubleArray(args[0]);
            double[] x = GetDoubleArray(args[1]);
            int n = x.GetLength(0);

            if (n <= 2 || n != y.GetLength(0))
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double sumx = 0;
            double sumy = 0;
            for (int i = 0; i < n; ++i)
            {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            double sumxy = 0;
            double sumx2 = 0;
            double sumy2 = 0;
            double dy = 0;
            double d = 0;
            for (int i = 0; i < n; ++i)
            {
                d = x[i] - sumx;
                dy = y[i] - sumy;
                sumxy += d * dy;
                sumx2 += d * d;
                sumy2 += dy * dy;
            }

            return Math.Sqrt((sumy2 - sumxy * sumxy / sumx2) / (n - 2)).ToString();
        }

        /// <summary>
        /// Returns the mean of the range after removing points on either extreme.
        /// </summary>
        /// <param name="range">Range, percent.</param>
        /// <returns>Kth smallest value.</returns>
        public string ComputeTrimmean(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double percent;
            string s1 = GetValueFromArg(args[1]);
            if (!double.TryParse(s1, out percent))
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            double[] dd = GetDoubleArray(args[0]);
            int n = dd.GetLength(0);
            int k = (int)(percent * n);
            k = k / 2;
            if (k < 1 || 2 * k >= n)
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            Array.Sort(dd);

            double sum = 0;
            n = n - k;
            for (int i = k; i < n; ++i)
            {
                sum += dd[i];
            }

            return (sum / (n - k)).ToString();
        }

        /// <summary>
        /// Returns sample variance of the listed values.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>The sample variance.</returns>
        public string ComputeVar(string range)
        {
            double[] dd = GetDoubleArray(range);
            return var(dd).ToString();
        }

        private double[] GetDoubleArray(string range)
        {
            double d;
            string s1 = string.Empty;
            ArrayList x = new ArrayList();
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////cell range
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch
                        {
                        }
                        ////ignore if empty
                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                            {
                                x.Add(d);
                            }
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch
                    {
                    }

                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                        {
                            x.Add(d);
                        }
                    }
                }
            }

            return (double[])x.ToArray(typeof(double));
        }

        private double[] GetDoubleArrayA(string range)
        {
            double d;
            string s1 = string.Empty;
            ArrayList x = new ArrayList();
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                ////cell range
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch
                        {
                        }

                        if (s1.Length > 0)
                        {
                            ////ignore if empty
                            d = 0;
                            if (s1.ToUpper() == TRUEVALUESTR)
                            {
                                d = 1;
                            }
                            else
                            {
                                double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d);
                            }

                            x.Add(d);
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = GetValueFromArg(r);
                    }
                    catch
                    {
                    }

                    if (s1.Length > 0)
                    {
                        ////ignore if empty
                        d = 0;
                        if (s1.ToUpper() == TRUEVALUESTR)
                        {
                            d = 1;
                        }
                        else
                        {
                            double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d);
                        }

                        x.Add(d);
                    }
                }
            }

            return (double[])x.ToArray(typeof(double));
        }

        /// <summary>
        /// Returns sample variance of the listed values.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>The sample variance.</returns>
        /// <remarks> True is treated as 1 and False is treated as 0.
        /// </remarks>
        public string ComputeVara(string range)
        {
            double[] dd = GetDoubleArrayA(range);
            return var(dd).ToString();
        }

        /// <summary>
        /// Returns population variance of the listed values.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>The population variance.</returns>
        public string ComputeVarp(string range)
        {
            double[] dd = GetDoubleArray(range); ////(double[]) x.ToArray(typeof(double));
            int n = dd.GetLength(0);
            return ((n - 1) * var(dd) / n).ToString();
        }

        /// <summary>
        /// Returns population variance of the listed values.
        /// </summary>
        /// <param name="range">A string holding a list (separated by commas) of:
        /// cell references,
        /// formulas, or numbers.</param>
        /// <returns>The population variance.</returns>
        /// <remarks> True is treated as 1 and False is treated as 0.
        /// </remarks>
        public string ComputeVarpa(string range)
        {
            double[] dd = GetDoubleArrayA(range);
            int n = dd.GetLength(0);
            return ((n - 1) * var(dd) / n).ToString();
        }

        /// <summary>
        /// Returns the Weibull distribution.
        /// </summary>
        /// <param name="argList">X, alpha, beta, cumulative.</param>
        /// <returns>Returns Weibull distribution.</returns>
        public string ComputeWeibull(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 4 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double x;
            double a;
            double b;
            double cum = 0;
            double dist = 0;

            for (int i = 0; i < argCount; ++i)
            {
                args[i] = GetValueFromArg(args[i]);
            }

            if (double.TryParse(args[0], out x)
                && double.TryParse(args[1], out a)
                && double.TryParse(args[2], out b))
            {
                if (argCount != 3)
                {
                    cum = (args[3] == TRUEVALUESTR || args[3] == "1") ? 1 : 0;
                }

                if (cum == 0)
                {
                    dist = a / Math.Pow(b, a) * Math.Pow(x, a - 1) * Math.Exp(-Math.Pow(x / b, a));
                }
                else
                {
                    dist = 1 - Math.Exp(-Math.Pow(x / b, a));
                }
            }

            return dist.ToString();
        }

        /// <summary>
        /// Returns the one-tailed probability value of a Z test.
        /// </summary>
        /// <param name="range">Range, mu, sigma.</param>
        /// <returns>Kth smallest value.</returns>
        public string ComputeZtest(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2 && argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            double mu;
            double sigma;
            string s1 = GetValueFromArg(args[1]);
            if (!double.TryParse(s1, out mu))
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            double[] dd = GetDoubleArray(args[0]);
            double xbar;
            sigma = sd(dd, out xbar);
            if (argCount == 3)
            {
                double.TryParse(s1, out sigma);
            }

            return (1 - this.normaldist((xbar - mu) / (sigma / Math.Sqrt(dd.GetLength(0))), 0, 1)).ToString();
        }

        /// <summary>
        /// Returns a horizontal table look up value.
        /// </summary>
        /// <param name="range">Contains search value, table, return index and match properties.</param>
        /// <returns>Matching value found in the table.</returns>
        /// <remarks> For example, =HLOOKUP("Axles",A1:C4,2,TRUE) looks for the exact 
        /// match for Axles in A1:C1 and returns the corresponding value in A2:C2. 
        /// </remarks>
        public string ComputeHLookUp(string range)
        {
            string[] s = range.Split(new char[] { ParseArgumentSeparator });
            string lookUp = GetValueFromArg(s[0]);
            lookUp = lookUp.Replace(TIC, string.Empty).ToUpper();
            string r = s[1];
            string o1 = GetValueFromArg(s[2]).ToString().ToUpper();
            double d = 0;
            if (!double.TryParse(o1, out d))
            {
                return "#N/A";
            }

            ////int row = int.Parse(o1);    
            int row = (int)d;

            bool match = true;
            if (s.GetLength(0) == 4)
            {
                string matchString = GetValueFromArg(s[3]);
                match = matchString == TRUEVALUESTR || matchString == "1";
            }

            bool typeIsNumber = match ? double.TryParse(lookUp, out d) : false;

            int i = r.IndexOf(":");
            if (i == -1)
            {
                ////single cell
                r = r + ":" + r;
                i = r.IndexOf(":");
            }

            int k = r.Substring(0, i).LastIndexOf(sheetToken);
            GridModel grd = this.grid;
            GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);

            if (k > -1)
            {
                this.grid = (GridModel)family.TokenToGridModel[r.Substring(0, k + 1)];
            }

            int row1 = RowIndex(r.Substring(0, i));
            int col1 = ColIndex(r.Substring(0, i));

            int row2 = RowIndex(r.Substring(i + 1));
            int col2 = ColIndex(r.Substring(i + 1));

            string val = string.Empty;
            int lastCol = col1;
            string s1 = string.Empty;
            double d1 = 0;
            for (int col = col1; col <= col2; ++col)
            {
                s1 = this.grid[row1, col].FormattedText.ToUpper().Replace("\"", string.Empty);
                ////if(s1 == lookUp ||(match && s1.CompareTo(lookUp) > 0))
                if (s1 == lookUp ||
                    (match && (typeIsNumber
                     ? (double.TryParse(s1, out d1) && (d1.CompareTo(d) > 0))
                    : (s1.CompareTo(lookUp) > 0))))
                {
                    if (s1 == lookUp)
                    {
                        lastCol = col;
                    }

                    break;
                }

                lastCol = col;
            }

            if (match || s1 == lookUp)
            {
                val = this.grid[row + row1 - 1, lastCol].FormattedText;
                if (val.Length > 0 && val[0] == this.formulaChar)
                {
                    val = this.Parse(val);
                }

                d = 0; ////zzz
                if (val.Length > 0 && val[0] != TIC[0] && !double.TryParse(val, out d))
                {
                    val = TIC + val + TIC;
                }
                ////Console.WriteLine("found {0} returned {1}", lookUp, val);
            }
            else
            {
                val = "#N/A";
            }
            ////val = "\"#N/A\"";

            ////Console.WriteLine(" ");
            this.grid = grd;

            return val;
        }

        /// <summary>
        /// Returns a vertical table look up value.
        /// </summary>
        /// <param name="range">Contains search value, table, return index and match properties.</param>
        /// <returns>Matching value found in the table.</returns>
        /// <remarks> For example, =VLOOKUP("Axles",A1:C4,2,TRUE) looks for the exact 
        /// match for Axles in A1:A4 and returns the corresponding value in B1:B4. 
        /// </remarks>
        public string ComputeVLookUp(string range)
        {
            string[] s = range.Split(new char[] { ParseArgumentSeparator });
            string lookUp = GetValueFromArg(s[0]);
            lookUp = lookUp.Replace(TIC, string.Empty).ToUpper();
            string r = s[1];
            string o1 = GetValueFromArg(s[2]).ToString().Replace("\"", string.Empty);

            double d = 0;
            if (!double.TryParse(o1, out d) || o1 == "NaN")
            {
                return "#N/A";
            }

            int col = (int)d;
            bool match = true;
            if (s.GetLength(0) == 4)
            {
                string matchString = GetValueFromArg(s[3]);
                match = matchString == TRUEVALUESTR || matchString == "1";
            }

            bool typeIsNumber = match ? double.TryParse(lookUp, out d) : false;

            int i = r.IndexOf(":");
            if (i == -1)
            {
                ////single cell
                r = r + ":" + r;
                i = r.IndexOf(":");
            }

            int k = r.Substring(0, i).LastIndexOf(sheetToken);
            GridModel grd = this.grid;
            GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);

            if (k > -1)
            {
                this.grid = (GridModel)family.TokenToGridModel[r.Substring(0, k + 1)];
            }

            ////Console.WriteLine(string.Format("{0},{1},{2},{3}", lookUp, r, col, match));

            int row1 = RowIndex(r.Substring(0, i));
            int col1 = ColIndex(r.Substring(0, i));

            int row2 = RowIndex(r.Substring(i + 1));
            int col2 = ColIndex(r.Substring(i + 1));

            string val = string.Empty;
            int lastRow = row1;
            string s1 = string.Empty;
            double d1 = 0;
            for (int row = row1; row <= row2; ++row)
            {
                s1 = this.grid[row, col1].FormattedText.ToUpper().Replace("\"", string.Empty);
                ////    if(s1 == lookUp || (match && s1.CompareTo(lookUp) > 0))
                if (s1 == lookUp ||
                    (match && (typeIsNumber
                    ? (double.TryParse(s1, out d1) && (d1.CompareTo(d) > 0))
                    : (s1.CompareTo(lookUp) > 0))))
                {
                    if (s1.ToUpper() == lookUp)
                    {
                        lastRow = row;
                    }

                    break;
                }

                lastRow = row;
            }

            if ((match && lastRow != row1) || s1 == lookUp)
            {
                val = this.grid[lastRow, col + col1 - 1].FormattedText;
                if (val.Length > 0 && val[0] == this.formulaChar)
                {
                    val = this.Parse(val);
                }

                d = 0;
                if (val.Length > 0 && val[0] != TIC[0] && !double.TryParse(val, out d))
                {
                    val = TIC + val + TIC;
                }
            }
            else
            {
                val = "#N/A";
            }
            ////val = "\"#N/A\"";

            ////Console.WriteLine("found {0} returned {1}", lookUp, val);

            ////Console.WriteLine(" ");
            this.grid = grd;

            return val;
        }

        /// <summary>
        /// Returns the left so many characters in the given string.
        /// </summary>
        /// <param name="range">Contains the string and the number of characters.</param>
        /// <returns>returns a string.</returns>
        public string ComputeLeft(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 1 && argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            string s1 = args[0];
            s1 = GetValueFromArg(s1);
            bool hasTics = s1.StartsWith(TIC) && s1.EndsWith(TIC);
            string s2 = (argCount == 2) ? args[1] : "1";
            s2 = this.ComputedValue(s2);
            int len = int.Parse(s2) + (hasTics ? 1 : 0);
            len = (s1.Length >= len) ? len : s1.Length;
            s1 = s1.Substring(0, len);
            if (hasTics && !s1.EndsWith(TIC))
            {
                s1 = s1 + TIC;
            }

            return s1;
        }

        /// <summary>
        /// Returns the length of the given string.
        /// </summary>
        /// <param name="range">Contains the string.</param>
        /// <returns>An integer length.</returns>
        public string ComputeLen(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 1)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            string s1 = args[0];
            s1 = GetValueFromArg(s1);
            bool hasTics = s1.StartsWith(TIC) && s1.EndsWith(TIC);
            return (hasTics ? s1.Length - 2 : s1.Length).ToString();
        }

        /// <summary>
        /// Returns a substring of the given string.
        /// </summary>
        /// <param name="range">Contains the original string, start position of the substring, 
        /// and the number of characters in the substring.</param>
        /// <returns>returns a string.</returns>
        public string ComputeMid(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 3)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            string s1 = args[0];
            s1 = GetValueFromArg(s1);
            bool hasTics = s1.StartsWith(TIC) && s1.EndsWith(TIC);
            string s2 = args[2];
            int len = int.Parse(s2);
            s2 = args[1];
            int start = int.Parse(s2) + (hasTics ? 1 : 0);

            if (start + len > s1.Length)
            {
                return FormulaErrorStrings[invalid_arguments];
            }

            s1 = s1.Substring(start, len);
            if (hasTics && !s1.StartsWith(TIC))
            {
                s1 = TIC + s1;
            }

            if (hasTics && !s1.EndsWith(TIC))
            {
                s1 = s1 + TIC;
            }

            return s1;
        }

        /// <summary>
        /// Returns the right so many characters in the given string.
        /// </summary>
        /// <param name="range">Contains the string and the number of characters.</param>
        /// <returns>returns a string.</returns>
        public string ComputeRight(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 1 && argCount != 2)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            string s1 = args[0];
            s1 = GetValueFromArg(s1);
            bool hasTics = s1.StartsWith(TIC) && s1.EndsWith(TIC);
            string s2 = (argCount == 2) ? args[1] : "1";
            s2 = this.ComputedValue(s2);
            int len = int.Parse(s2) + (hasTics ? 1 : 0);
            int start = (s1.Length >= len) ? s1.Length - len : 0;
            s1 = s1.Substring(start);
            if (hasTics && !s1.StartsWith(TIC))
            {
                s1 = TIC + s1;
            }

            return s1;
        }

        /// <summary>
        /// Returns True if the ParseArgumentSeparator character is included in a string.
        /// </summary>
        /// <param name="s">The string to be searched.</param>
        /// <returns>True or False.</returns>
        public bool IsSeparatorInTIC(string s)
        {
            int i = s.IndexOf(TIC) + 1;
            bool inTic = true;
            while (i > 0 && i < s.Length)
            {
                if (s[i] == ParseArgumentSeparator && inTic)
                {
                    return true;
                }

                if (s[i] == TIC[0])
                {
                    inTic = !inTic;
                }

                i++;
            }

            return false;
        }

        /// <summary>
        /// Returns an array of strings from an argument list.
        /// </summary>
        /// <param name="s">A delimited argument list.</param>
        /// <returns>Array of strings from an argument list.</returns>
        public string[] GetStringArray(string s)
        {
#if !(SyncfusionFramework1_0 || SyncfusionFramework1_1)
            System.Collections.Generic.List<string> argList = new System.Collections.Generic.List<string>();
#else
            ArrayList argList = new ArrayList();
#endif
            int argStart = 0;
            bool inQuote = false;
            for (int argEnd = 0; argEnd < s.Length; argEnd++)
            {
                char ch = s[argEnd];
                if (ch == TIC[0])
                {
                    inQuote = !inQuote;
                }
                else if (!inQuote && ch == ParseArgumentSeparator)
                {
                    argList.Add(s.Substring(argStart, argEnd - argStart));
                    argStart = argEnd + 1;
                }
            }

            argList.Add(s.Substring(argStart));
            string[] ar = new string[argList.Count];
            argList.CopyTo(ar);
            return ar;
        }

        ////public string[] GetStringArray(string s)
        ////{
        ////    int i = s.IndexOf(TIC);
        ////    int count = 0;
        ////    if (i > 0)
        ////    {
        ////        count = s.Substring(0, i + 1).Split(new char[] { ParseArgumentSeparator }).GetLength(0) - 1;
        ////    }
        ////    int j = s.LastIndexOf(TIC);
        ////    if (j > -1 && j < s.Length - 1)
        ////        count += s.Substring(j).Split(new char[] { ParseArgumentSeparator }).GetLength(0) - 1;
        ////    count += s.Substring(i, j - i).Split(new char[] { TIC[0] }).GetLength(0) / 2;

        ////    string[] ar = new string[count];

        ////    int k = 0;
        ////    if (i > 0)
        ////    {
        ////        foreach (string s1 in s.Substring(0, i - 1).Split(new char[] { ParseArgumentSeparator }))
        ////        {
        ////            ar[k] = s1;
        ////            k++;
        ////        }
        ////    }

        ////    foreach (string s1 in s.Substring(i, j - i + 1).Split(new char[] { TIC[0] }))
        ////    {
        ////        if (s1.Length != 0 && s1[0] != ParseArgumentSeparator)
        ////        {
        ////            ar[k] = TIC + s1 + TIC;
        ////            k++;
        ////        }
        ////    }

        ////    if (j > -1 && j < s.Length - 2)
        ////    {
        ////        foreach (string s1 in s.Substring(j + 2).Split(new char[] { ParseArgumentSeparator }))
        ////        {
        ////            ar[k] = s1;
        ////            k++;
        ////        }
        ////    }
        ////    return ar;
        ////}

        /// <summary>
        /// Returns a single character string.
        /// </summary>
        /// <param name="range">List of strings to be concatenated.</param>
        /// <returns>A single string.</returns>
        public string ComputeConcatenate(string range)
        {
            string text = string.Empty;

            string[] ar = IsSeparatorInTIC(range) ////range.IndexOf(TIC) > 0 
                ? GetStringArray(range) : range.Split(new char[] { ParseArgumentSeparator });
            foreach (string r in ar)
            {
                if (r.Length > 0 && r[0] == TIC[0])
                {
                    string val = string.Empty;
                    val = r.Replace(TIC, string.Empty);
                    text += val;
                }
                else
                {
                    text += GetValueFromArg(r); ////TIC + GetValueFromArg(r) + TIC;
                }
            }

            if (text.IndexOf("#N/A") > -1)
            {
                text = "#N/A";
            }
            else
            {
                ////text = text.Replace(TIC + TIC, string.Empty);
                ////if (text.Length > 0 && text[0] != TIC[0])
                text = TIC + text + TIC;
            }

            return text;

            ////            string text = string.Empty;
            ////            
            ////            foreach(string r in range.Split(new char[]{ParseArgumentSeparator}))
            ////            {
            ////                if(r.IndexOf(':') > -1)
            ////                {
            ////                    foreach(string s in GetCellsFromArgs(r))
            ////                    {
            ////                        text += TIC + GetValueFromArg(s) + TIC;
            ////                    }
            ////                }
            ////                else
            ////                {
            ////                    text += TIC + GetValueFromArg(r) + TIC;
            ////                }
            ////            }
            ////            text = text.Replace(TIC + TIC, string.Empty);
            ////            return text;
        }

        /// <summary>
        /// Returns the value at a specified row and column from within a given range.
        /// </summary>
        /// <param name="arg">look_range, row, col</param>
        /// <returns>The value.</returns>
        /// <remarks>
        /// Both array and reference form of this function are supported.
        /// </remarks>
        public string ComputeIndex(string arg)
        {
            string r = string.Empty;
            string sheet = string.Empty;
            string s = string.Empty;
            bool sum = false;
            int row = -1, col = -1, top = -1, bot = -1, left = -1, right = -1;
            if (arg[arg.Length - 1] == BMARKER)
            {
                arg = GetValueFromArg(arg);
            }

            string[] args = arg.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);

            s = this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].Text;
            s = s.ToUpper();
            if (s.Contains(this.ActiveFunctionName))
            {
                if (s.IndexOf(this.ActiveFunctionName) - 2 != -1)
                {
                    string func = string.Empty;
                    for (int i = s.IndexOf(this.ActiveFunctionName) - 2; i >= 0; i--)
                    {
                        func = s[i] + func;
                        if (func.Equals("SUM"))
                        {
                            sum = true;
                            break;
                        }
                    }
                }
                else
                {
                    s = s.Replace("=" + this.ActiveFunctionName, string.Empty);
                    this.SwapInnerParens(ref s);
                }
            }


            if (s.Contains("{") && s.Contains("}"))
            {
                string innerArg = string.Empty;
                for (int j = s.IndexOf('{') + 1; j < s.IndexOf('}'); j++)
                {
                    innerArg += s[j];
                }
                string[] innerArgs = innerArg.Split(new char[] { ParseArgumentSeparator });

                int innerArgCount = innerArgs.GetLength(0);
                int cnt = 0;

                if (argCount < 2 || argCount > innerArgCount + 3 || argCount == innerArgCount)
                {
                    return this.FormulaErrorStrings[wrong_number_arguments];
                }

                if (innerArgCount >= 1)
                {
                    cnt = argCount - innerArgCount;
                    if (cnt == 2)
                    {
                        string rcCell = innerArgs[0].ToUpper().Replace(TIC, string.Empty);
                        string[] cells = rcCell.Split(new char[] { ':' }, StringSplitOptions.RemoveEmptyEntries);

                        if (cells.Length == 2)
                        {
                            if (this.ColIndex(cells[1]) != this.ColIndex(cells[0]))
                            {
                                return this.FormulaErrorStrings[wrong_number_arguments];
                            }
                        }
                        r = innerArgs[0];
                    }
                    else
                    {
                        if (int.Parse(args[argCount - 1]) <= innerArgCount)
                            if (int.Parse(args[argCount - 1]) > 0)
                                r = innerArgs[int.Parse(args[argCount - 1]) - 1];
                            else
                                return "#VALUE!";
                        else
                            return "#REF!";
                    }
                    r = r.Replace(TIC, "");
                    int i = r.IndexOf(":");
                    ////single cell
                    if (i == -1)
                    {
                        return this.FormulaErrorStrings[invalid_arguments];
                    }

                    if (arg.IndexOf("#N/A") > -1 || arg.IndexOf("#N~A") > -1)
                    {
                        return "#N/A";
                    }

                    sheet = SheetToken(args[innerArgCount - 1].Replace(TIC, string.Empty)); ;

                    double d;
                    row = (cnt == 1) ? 1 : double.TryParse(this.GetValueFromArg(args[innerArgCount]), NumberStyles.Integer, null, out d) ? (int)d : -1;
                    col = (cnt <= 2) ? 1 : double.TryParse(this.GetValueFromArg(args[innerArgCount + 1]), NumberStyles.Integer, null, out d) ? (int)d : -1;
                    if (row <= 0 || col <= 0)
                    {
                        return "#VALUE!";
                    }

                    top = this.RowIndex(r.Substring(0, i));
                    bot = this.RowIndex(r.Substring(i + 1));
                    left = this.ColIndex(r.Substring(0, i));
                    right = this.ColIndex(r.Substring(i + 1));
                    if (row > bot - top + 1 || col > right - left + 1)
                    {
                        return "#REF!";
                    }

                    row = this.RowIndex(r.Substring(0, i)) + row - 1;
                    col = this.ColIndex(r.Substring(0, i)) + col - 1;
                }

            }
            else
            {
                if (argCount < 2 || argCount > 4)
                {
                    return this.FormulaErrorStrings[wrong_number_arguments];
                }

                if (argCount == 2)
                {
                    string rcCell = args[0].ToUpper().Replace(TIC, string.Empty);
                    string[] cells = rcCell.Split(new char[] { ':' }, StringSplitOptions.RemoveEmptyEntries);

                    if (cells.Length == 2)
                    {
                        if (!((this.ColIndex(cells[1]) == this.ColIndex(cells[0])) || (this.RowIndex(cells[1]) == this.RowIndex(cells[0]))))
                        {
                            return this.FormulaErrorStrings[invalid_expression];
                        }
                    }
                    int val;
                    if (!int.TryParse(args[1], out val))
                        return this.FormulaErrorStrings[invalid_expression];

                }
                else if (argCount == 4)
                {
                    int val;
                    if (int.TryParse(args[argCount - 1], out val))
                    {
                        if (val <= 0)
                            return "#VALUE!";
                        else if (val > 1)
                            return "#REF!";
                        else if (sum)
                        {
                            sum = false;
                            string rcCell = args[0].ToUpper().Replace(TIC, string.Empty);
                            string[] cells = rcCell.Split(new char[] { ':' }, StringSplitOptions.RemoveEmptyEntries);
                            if (cells.Length == 2)
                            {
                                int topRow = this.RowIndex(cells[0]);
                                int result = -1;
                                string st = string.Empty;
                                string leftCol = string.Empty;
                                if (int.TryParse(args[argCount - 2], out result))
                                {
                                    leftCol = GridRangeInfo.GetAlphaLabel(result);
                                    st = leftCol + topRow + ":" + cells[1];
                                    return st;
                                }
                            }
                            sum = false;
                        }
                    }

                }
                r = args[0];
                r = r.Replace(TIC, "");
                int i = r.IndexOf(":");
                ////single cell
                if (i == -1)
                {
                    return this.FormulaErrorStrings[invalid_arguments];
                }

                if (arg.IndexOf("#N/A") > -1 || arg.IndexOf("#N~A") > -1)
                {
                    return "#N/A";
                }

                sheet = SheetToken(args[0].Replace(TIC, string.Empty)); ;

                double d;
                row = (argCount == 1) ? 1 : double.TryParse(this.GetValueFromArg(args[1]), NumberStyles.Integer, null, out d) ? (int)d : -1;
                col = (argCount <= 2) ? 1 : double.TryParse(this.GetValueFromArg(args[2]), NumberStyles.Integer, null, out d) ? (int)d : -1;
                if (row == -1 || col == -1)
                {
                    return "#REF!";
                }

                top = this.RowIndex(r.Substring(0, i));
                bot = this.RowIndex(r.Substring(i + 1));
                left = this.ColIndex(r.Substring(0, i));
                right = this.ColIndex(r.Substring(i + 1));
                if (row > bot - top + 1 || col > right - left + 1)
                {
                    return "#REF!";
                }

                row = this.RowIndex(r.Substring(0, i)) + row - 1;
                col = this.ColIndex(r.Substring(0, i)) + col - 1;
            }
            return this.GetValueFromArg(string.Format("{2}{0}{1}", GridRangeInfo.GetAlphaLabel(col), row, sheet));
        }

        /// <summary>
        /// Returns the reference specified by a text string. References are immediately evaluated to display their contents.
        /// <para>Syntax: INDIRECT(CellRefString, [IsA1Style])</para>
        /// </summary>
        /// <param name="args">Cell reference string.</param>
        /// <returns>Reference specified the argument.</returns>
        public string ComputeIndirect(string args)
        {
            if (args[args.Length - 1] == BMARKER)
            {
                args = GetValueFromArg(args);
            }

            string[] arg = args.Split(new char[] { ParseArgumentSeparator });
            if (arg.Length == 0 || arg.Length > 2)
            {
                throw new ArgumentException("No. of argument cant be less than 1 or more than 2.");
            }

            arg[0] = arg[0].ToUpper();
            PutTokensForSheets(ref arg[0]);
            string sheetToken = SheetToken(arg[0].Replace(TIC, string.Empty));

            ////Remove SheetToken
            if (!String.IsNullOrEmpty(sheetToken))
            {
                arg[0] = arg[0].Replace(sheetToken, string.Empty);
            }

            if (arg.Length == 2 && arg[1] == FALSEVALUESTR)
            {
                bool hasTIC = arg[0].StartsWith(TIC) && arg[0].EndsWith(TIC);
                string rcCell = arg[0].ToUpper().Replace(TIC, string.Empty);
                string[] cells = rcCell.Split(new char[] { ':' }, StringSplitOptions.RemoveEmptyEntries);

                string[] rc = cells[0].Split(new char[] { 'R', 'C' }, StringSplitOptions.RemoveEmptyEntries);
                if (cells.Length > 2)
                {
                    return "#REF!";
                }

                arg[0] = GridRangeInfo.GetAlphaLabel(Convert.ToInt32(rc[1])) + rc[0];

                if (cells.Length == 2)
                {
                    string st = SheetToken(cells[1]);
                    if (!String.IsNullOrEmpty(st))
                    {
                        cells[1] = cells[1].Replace(st, string.Empty);
                    }

                    rc = cells[1].Split(new char[] { 'R', 'C' }, StringSplitOptions.RemoveEmptyEntries);
                    arg[0] += ":" + GridRangeInfo.GetAlphaLabel(Convert.ToInt32(rc[1])) + rc[0];
                }

                if (hasTIC)
                {
                    arg[0] = TIC + arg[0] + TIC;
                }
            }

            string cellReference = string.Empty;
            if (arg[0].StartsWith(TIC))
            {
                ////String
                cellReference = sheetToken + arg[0].Replace(TIC, string.Empty);
            }
            else
            {
                if (IsCellReference(arg[0]))
                {
                    cellReference = GetValueFromArg(sheetToken + arg[0]);
                }
            }

            if (!IsCellReference(cellReference))
            {
                if (NamedRanges.Contains(cellReference.ToUpper()))
                {
                    cellReference = Convert.ToString(NamedRanges[cellReference.ToUpper()]);
                }
                else
                {
                    return "#REF!";
                }
            }

            if (cellReference.Contains(":"))
            {
                return cellReference;
            }

            return GetValueFromArg(cellReference);
        }

        /// <summary>
        /// In a given string, this method substitutes an occurrence of one string with another string.
        /// </summary>
        /// <param name="range">A list of 3 or 4 arguments: the original string, the search string, the 
        /// replacement string, and optionally, an integer representing the occurrence to be replaced.
        ///    </param>
        /// <returns>The modified string.</returns>
        public string ComputeSubstitute(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            if (args.Length != 3 && args.Length != 4)
            {
                return FormulaErrorStrings[wrong_number_arguments];
            }

            ////strip off outside tics (if any) on the evaluated arguments
            string s1 = StripTics0(GetValueFromArg(args[0]));
            string s2 = StripTics0(GetValueFromArg(args[1]));
            string s3 = StripTics0(GetValueFromArg(args[2]));
            if (args.Length == 3)
            {
                s1 = s1.Replace(s2, s3);
            }
            else
            {
                string s4 = GetValueFromArg(args[3]);
                double d = 0;
                if (double.TryParse(s4, NumberStyles.Integer, null, out d))
                {
                    int count = (int)d;
                    int loc = -1;
                    while (count > 0 && (loc = s1.IndexOf(s2, loc + 1)) > -1)
                    {
                        count--;
                    }

                    if (count == 0)
                    {
                        s1 = s1.Substring(0, loc) + s3 + s1.Substring(loc + s2.Length);
                    }
                }
            }

            return TIC + s1 + TIC;
        }

        private string StripTics0(string s)
        {
            if (s.Length > 1 && s[0] == TIC[0] && s[s.Length - 1] == TIC[0])
            {
                s = s.Substring(1, s.Length - 2);
            }

            return s;
        }

        /// <summary>
        /// Returns the product of the arguments in the list.
        /// </summary>
        /// <param name="range">List of arguments.</param>
        /// <returns>Product of the arguments.</returns>
        public string ComputeProduct(string range)
        {
            double prod = 1;
            double d;
            string s1;
            bool nohits = true;
            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s, true);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (s1.Length > 0)
                        {
                             if (double.TryParse(s1, out d))
                            {
                                prod = prod * d;
                                nohits = false;
                            }
                        }
                    }
                }
                else
                {
                    if (double.TryParse(GetValueFromArg(r), out d))
                    {
                        prod = prod * d;
                        nohits = false;
                    }
                }
            }

            return nohits ? "0" : prod.ToString();
        }

        /// <summary>
        /// Returns a quoted string from a date or number.
        /// </summary>
        /// <param name="range">Value to be converted to a string.</param>
        /// <returns>Quoted string.</returns>
        public string ComputeText(string range)
        {
            string s1 = range.Split(new char[] { ParseArgumentSeparator })[0];
            string s2 = range.Split(new char[] { ParseArgumentSeparator })[1].Replace(TIC, string.Empty);
            s1 = GetValueFromArg(s1);
            double d;
            if (double.TryParse(s1, out d))
            {
                s1 = d.ToString(s2);
                if (s1 == s2)
                {
                    s2 = s2.Replace('m', 'M').Replace('Y', 'y').Replace('D', 'd');
                    DateTime dt = DateTime.FromOADate(d);
                    s1 = dt.ToString(s2);
                }
            }

            return TIC + s1 + TIC;
        }

        /// <summary>
        /// Returns a number.
        /// </summary>
        /// <param name="range">A date or number string.</param>
        /// <returns>returns a number.</returns>
        public string ComputeValue(string range)
        {
            return ComputedValue(range);
            //double d;
            ////if (!double.TryParse(range.Replace(TIC, string.Empty), out d))
            //if (!double.TryParse(range.Replace(TIC, string.Empty), NumberStyles.Any, null, out d))
            //{
            //    try
            //    {
            //        DateTime dt = DateTime.Parse(range.Replace(TIC, string.Empty));
            //        d = dt.ToOADate();
            //    }
            //    catch
            //    {
            //    }
            //}

            //return d.ToString();
        }

        /// <summary>
        /// Returns the remainder after dividing one number by another.
        /// </summary>
        /// <param name="range">Two numbers in a list.</param>
        /// <returns>The remainder.</returns>
        public string ComputeMod(string range)
        {
            string s1 = range.Split(new char[] { ParseArgumentSeparator })[0];
            string s2 = range.Split(new char[] { ParseArgumentSeparator })[1];
            s1 = GetValueFromArg(s1);
            s2 = GetValueFromArg(s2);
            int i = int.Parse(s2);
            return (Math.Sign(i) * Math.Abs((int.Parse(s1) % i))).ToString();
        }

        /// <summary>
        /// Returns the integer value.
        /// </summary>
        /// <param name="range">Number to be truncated.</param>
        /// <returns>An integer.</returns>
        public string ComputeInt(string range)
        {
            range = GetValueFromArg(range);
            double d;
            if (double.TryParse(range, out d))
            {
                double d1 = (d < 0) ? -1 : 1;
                double d2 = (d < 0) ? 1 : 0;
                return (d1 * Math.Floor(d2 + Math.Abs(d))).ToString("F0");
            }

            return "0";
        }

        /// <summary>
        /// Truncates a number to an integer.
        /// </summary>
        /// <param name="range">Value and number of digits.</param>
        /// <returns>Truncated value.</returns>
        public string ComputeTrunc(string range)
        {
            string[] args = range.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            double digits = 0;

            if (argCount == 2)
            {
                ////ignore return value...
                double.TryParse(GetValueFromArg(args[1]), NumberStyles.Integer, null, out digits);
            }

            range = GetValueFromArg(args[0]);

            double d;
            if (double.TryParse(range, out d))
            {
                string format = digits == 0 ? "F0" : ("0." + new string('0', (int)digits));
                double normalizer = Math.Pow(10, digits);
                double d1 = (d < 0) ? -1 : 1;
                return (d1 * Math.Floor(normalizer * Math.Abs(d)) / normalizer).ToString(format);
            }

            return "0";
        }

        /// <summary>
        /// Returns the sum of the products of corresponding values.
        /// </summary>
        /// <param name="range">Two cell ranges.</param>
        /// <returns>Sum of the products.</returns>
        public string ComputeSumProduct(string range)
        {
            double sum = 0;
            int count = 0;
            double d;
            double[] vector = null;

            foreach (string r in range.Split(new char[] { ParseArgumentSeparator }))
            {
                if (r.IndexOf(':') > -1)
                {
                    int i = r.IndexOf(":");
                    int row1 = RowIndex(r.Substring(0, i));
                    int col1 = ColIndex(r.Substring(0, i));
                    int row2 = RowIndex(r.Substring(i + 1));
                    int col2 = ColIndex(r.Substring(i + 1));

                    if (vector == null)
                    {
                        count = (row2 - row1 + 1) * (col2 - col1 + 1);
                        vector = new double[count];
                        for (i = 0; i < count; ++i)
                        {
                            vector[i] = 1d;
                        }
                    }

                    int k = 0;
                    string s1;
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            return ex.Message;
                        }

                        if (s1.Length > 0)
                        {
                            if (double.TryParse(s1, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                            {
                                vector[k] = vector[k] * d;
                            }
                        }
                        //Fix for issue SD3204
                        else if (s1.Equals(string.Empty))
                        {
                            vector[k] = 0d; // empty needs to be considered as zero and hence zero multiplied by anything will result in zero
                        }

                        k++;
                    }
                }
                else
                {
                    break;
                }
            }

            for (int i = 0; i < count; ++i)
            {
                sum += vector[i];
            }

            return sum.ToString();
        }

        /// <summary>
        /// Returns True is the string denotes an error.
        /// </summary>
        /// <param name="range">String to be tested.</param>
        /// <returns>True if the value is an error.</returns>
        public string ComputeIsError(string range)
        {
            if (range.StartsWith(TIC.ToString()))
            {
                return FALSEVALUESTR;
            }

            try
            {
                range = GetValueFromArg(range).ToUpper().Replace(TIC, string.Empty);
            }
            catch
            {
                range = range.ToUpper();
            }

            if (range.StartsWith("NAN")
                || range.StartsWith("-NAN")
                || range.StartsWith("INFINITY")
                || range.StartsWith("-INFINITY")
                || range.StartsWith("#")
                || range.StartsWith("n#"))
            {
                return TRUEVALUESTR;
            }
            else
            {
                return FALSEVALUESTR;
            }
        }

        /// <summary>
        /// Determines whether the string contains a number or not.
        /// </summary>
        /// <param name="range">String to be tested.</param>
        /// <returns>True if the string is a number.</returns>
        public string ComputeIsNumber(string range)
        {
            double d;
            try
            {
                range = GetValueFromArg(range);
            }
            catch
            {
            }

            if (double.TryParse(range, out d))
            {
                return TRUEVALUESTR;
            }
            else
            {
                return FALSEVALUESTR;
            }
        }

        /// <summary>
        /// Returns the logical value False. 
        /// </summary>
        /// <returns>Logical False value string.</returns>
        public string ComputeFalse(string empty)
        {
            return FALSEVALUESTR;
        }

        /// <summary>
        /// Returns the logical value True.
        /// </summary>
        /// <returns>Logical True value string.</returns>
        public string ComputeTrue(string empty)
        {
            return TRUEVALUESTR;
        }

        /// <summary>
        /// Converts a number to text using currency format.
        /// </summary>
        /// <param name="args">Number and the number of digits.</param>
        /// <returns>Currency format string.</returns>
        public string ComputeDollar(string args)
        {
            string[] argsArray = args.Split(new char[] { ParseArgumentSeparator });
            string s1 = argsArray[0];
            string s2 = "2";
            if (argsArray.GetLength(0) == 2)
            {
                s2 = argsArray[1];
            }

            s1 = GetValueFromArg(s1);
            s2 = GetValueFromArg(s2);

            double number, decimals;

            double.TryParse(s1, out number);
            if (!double.TryParse(s2, out decimals))
            {
                decimals = 2;
            }

            NumberFormatInfo nfi = (NumberFormatInfo)CultureInfo.CurrentCulture.NumberFormat.Clone();
            nfi.CurrencyDecimalDigits = (int)decimals;

            return string.Format(nfi, "{0:C}", number);
        }

        /// <summary>
        /// Rounds a number to the specified number of decimals, formats the number
        /// in decimal format using a period and commas, and return the result as text.
        /// </summary>
        /// <param name="args">
        /// Number, number of digits, a flag that prevents from include
        /// commas in the returned text.
        /// </param>
        /// <returns>Formatted number as string.</returns>
        public string ComputeFixed(string args)
        {
            string[] argsArray = args.Split(new char[] { ParseArgumentSeparator });
            string s1 = argsArray[0];
            string s2 = "2";
            string s3 = "FALSE";

            int argCount = argsArray.GetLength(0);
            if (argCount > 1)
            {
                s2 = argsArray[1];
            }

            if (argCount > 2)
            {
                s3 = argsArray[2];
            }

            s1 = GetValueFromArg(s1);
            s2 = GetValueFromArg(s2);
            s3 = GetValueFromArg(s3);

            double number, decimals;
            double.TryParse(s1, out number);
            if (!double.TryParse(s2, out decimals))
            {
                decimals = 2;
            }

            double no_commas_flag;
            bool no_commas;
            if (double.TryParse(s3, out no_commas_flag))
            {
                if (no_commas_flag == 0)
                {
                    no_commas = false;
                }
                else
                {
                    no_commas = true;
                }
            }
            else
            {
                if (s3.ToUpper() == FALSEVALUESTR)
                {
                    no_commas = false;
                }
                else
                {
                    if (s3.ToUpper() == TRUEVALUESTR)
                    {
                        no_commas = true;
                    }
                    else
                    {
                        return "#NAME?";
                    }
                }
            }

            NumberFormatInfo nfi = (NumberFormatInfo)CultureInfo.CurrentCulture.NumberFormat.Clone();
            nfi.NumberDecimalDigits = (int)decimals;
            if (no_commas)
            {
                nfi.NumberGroupSeparator = string.Empty;
            }

            ////Added UNIQUESTRINGMARKER to avoid parsing in GetFormattedText().
            return string.Format(nfi, "{0}{1:N}", UNIQUESTRINGMARKER, number);
            ////return string.Format(nfi, "{0:N}", number); 
        }

        /// <summary>
        /// Converts text to lowercase.
        /// </summary>
        /// <param name="args">Value to convert.</param>
        /// <returns>Converted string.</returns>
        public string ComputeLower(string args)
        {
            return GetValueFromArg(args).ToLower();
        }

        /// <summary>
        /// Converts text to uppercase.
        /// </summary>
        /// <param name="args">Value to convert.</param>
        /// <returns>Converted string.</returns>
        public string ComputeUpper(string args)
        {
            return GetValueFromArg(args).ToUpper();
        }

        /// <summary>
        /// Removes all leading and trailing white-space characters.
        /// </summary>
        /// <param name="args">Value to trim.</param>
        /// <returns>
        /// The string that remains after all leading and trailing white-space characters
        /// were removed.
        /// </returns>
        public string ComputeTrim(string args)
        {
            string s = GetValueFromArg(args).Trim(new char[] { '\"', ' ' });
            int len = 0;
            ////strip out interior double, triple, etc spaces...
            while (s.Length != len)
            {
                len = s.Length;
                s = s.Replace("  ", " ");
            }

            return s;
        }

        /// <summary>
        /// Determines whether the value is a logical value.
        /// </summary>
        /// <param name="args">Value to be tested.</param>
        /// <returns>True if the value is a logical value, False otherwise.</returns>
        public string ComputeIsLogical(string args)
        {
            string value = args.ToUpper();
            try
            {
                value = GetValueFromArg(args).ToUpper();
            }
            catch
            {
                return FALSEVALUESTR;
            }

            if (value == FALSEVALUESTR || value == TRUEVALUESTR
                || args.ToUpper() == FALSEVALUESTR || args.ToUpper() == TRUEVALUESTR)
            {
                return TRUEVALUESTR;
            }

            return FALSEVALUESTR;
        }

        /// <summary>
        /// Determines whether the value is the #NA error value.
        /// </summary>
        /// <param name="args">Value to be tested.</param>
        /// <returns>True if the value is the #NA error value, False otherwise.</returns>
        public string ComputeIsNA(string args)
        {
            if (args.StartsWith(TIC.ToString()))
            {
                return FALSEVALUESTR;
            }

            try
            {
                args = GetValueFromArg(args).ToUpper();
            }
            catch
            {
                args = args.ToUpper();
            }

            if (args.StartsWith("#N/A"))
            {
                return TRUEVALUESTR;
            }

            return FALSEVALUESTR;
        }

        /// <summary>
        /// Returns True is the string denotes an error except #N/A.
        /// </summary>
        /// <param name="range">Value to be tested.</param>
        /// <returns>True if the value is an error except #N/A, false otherwise.</returns>
        public string ComputeIsErr(string range)
        {
            if (range.StartsWith(TIC.ToString()))
            {
                return FALSEVALUESTR;
            }

            try
            {
                range = GetValueFromArg(range).ToUpper().Replace(TIC, string.Empty);
            }
            catch
            {
                range = range.ToUpper();
            }

            if ((range.StartsWith("NAN")
                || range.StartsWith("-NAN")
                || range.StartsWith("INFINITY")
                || range.StartsWith("-INFINITY")
                || range.StartsWith("#")
                || range.StartsWith("n#")) && !range.StartsWith("#N/A"))
            {
                return TRUEVALUESTR;
            }
            else
            {
                return FALSEVALUESTR;
            }
        }

        /// <summary>
        /// Determines whether the value is empty string.
        /// </summary>
        /// <param name="args">Value to be tested.</param>
        /// <returns>True if the value is empty, False otherwise.</returns>
        public string ComputeIsBlank(string args)
        {
            if (args.Length == 0)
            {
                throw new ArgumentException("Argument cant be empty.");
            }

            try
            {
                if (GetValueFromArg(args).Equals(string.Empty))
                {
                    return TRUEVALUESTR;
                }
            }
            catch
            {
                return FALSEVALUESTR;
            }

            return FALSEVALUESTR;
        }

        /// <summary>
        /// Determines whether the value is string or not.
        /// </summary>
        /// <param name="args">Value to be tested.</param>
        /// <returns>True if the value is a string, false otherwise.</returns>
        public string ComputeIsText(string args)
        {
            string s;
            double d;
            try
            {
                s = GetValueFromArg(args);
            }
            catch
            {
                return FALSEVALUESTR;
            }

            if (s.Length > 0 && !double.TryParse(s, out d) &&
                (s.ToUpper() != FALSEVALUESTR && s.ToUpper() != TRUEVALUESTR) &&
                !(s.StartsWith("NAN")
                || s.StartsWith("-NAN")
                || s.StartsWith("INFINITY")
                || s.StartsWith("-INFINITY")
                || s.StartsWith("#")
                || s.StartsWith("n#")))
            {
                return TRUEVALUESTR;
            }
            else
            {
                return FALSEVALUESTR;
            }
            ////UpdateDependencies
            ////if(IsCellReference(args))
            ////    GetValueFromArg(args);
            ////if (IsFormulaResultString(args, cell))
            ////    return TRUEVALUESTR;
            ////else
            ////    return FALSEVALUESTR;
        }

        /// <summary>
        /// Determines whether the value is not a string.
        /// </summary>
        /// <param name="args">Value to be tested.</param>
        /// <returns>True if the value is not a string, false otherwise.</returns>
        public string ComputeIsNonText(string args)
        {
            if (ComputeIsText(args) == TRUEVALUESTR)
            {
                return FALSEVALUESTR;
            }
            else
            {
                return TRUEVALUESTR;
            }
        }
        /// <summary>
        /// Returns the Multinominal value of given range of numbers.
        /// </summary>
        /// <param name="range">Given numbers</param>
        /// <returns>Multinominal value of given range of numbers</returns>
        public string ComputeMultinomial(string range)
        {
            double d;
            string s1;
            string str = string.Empty;
            int arrayList = 0, elements = 0;
            string total = ComputeSum(range);
            string totalFact = ComputeFact(total);
            string[] ranges = range.Split(new char[] { ParseArgumentSeparator });
            if (ranges.Length > 0)
            {
                foreach (string r in ranges)
                {
                    elements++;
                    if (r.IndexOf(':') > -1)
                    {
                        --elements;
                        foreach (string s in this.GetCellsFromArgs(r))
                            elements++;
                    }
                }
            }
            string[] ranges2 = new string[elements];
            foreach (string r in ranges)
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in this.GetCellsFromArgs(r))
                    {
                        try
                        {
                            s1 = this.GetValueFromArg(s);
                        }
                        catch (Exception ex)
                        {
                            TraceUtil.TraceExceptionCatched(ex);
                            if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                            {
                                throw ex;
                            }
                            return ex.Message;
                        }
                        if (s1.Length > 0 && double.TryParse(s1, NumberStyles.Any, null, out d))
                        {
                            string fact = ComputeFact(d.ToString());
                            ranges2[arrayList] = fact;
                            arrayList++;
                        }
                    }
                }
                else
                {
                    try
                    {
                        s1 = this.GetValueFromArg(r);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }
                        return ex.Message;
                    }
                    if (s1.Length > 0)
                    {
                        if (double.TryParse(s1, NumberStyles.Any, null, out d))
                        {
                            string fact = ComputeFact(d.ToString());
                            ranges2[arrayList] = fact;
                            arrayList++;
                        }
                    }
                }
            }
            str = ranges2[0];
            for (int i = 1; i < ranges2.Length; i++)
                str = str + "," + ranges2[i];
            string multi = ComputeProduct(str);
            double totalFactVal = double.Parse(totalFact);
            double multiValue = double.Parse(multi);
            double multinominalVal = totalFactVal / multiValue;
            return multinominalVal.ToString();
        }

        /// <summary>
        /// Rounds the given number to a specified multiple number
        /// </summary>
        /// <param name="argList">Given number, multiple value</param>
        /// <returns>Mround value of given number</returns>
        public string ComputeMround(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
                return this.FormulaErrorStrings[this.invalid_arguments];
            double x = 0, digits = 0;
            double numberstr, mround;
            string round;
            string numStr = this.GetValueFromArg(args[0]);
            string digStr = this.GetValueFromArg(args[1]);
            if (digStr == "0")
                return digStr;
            if (ComputeSign(numStr) != ComputeSign(digStr))
                return "#NUM!";
            else
            {
                if ((numStr.Length == 0 || double.TryParse(numStr, NumberStyles.Any, null, out x))
                && (digStr.Length == 0 || double.TryParse(digStr, NumberStyles.Any, null, out digits)))
                {
                    numberstr = x / digits;
                    round = ComputeRoundup(numberstr.ToString());
                    double round1 = double.Parse(round);
                    mround = round1 * digits;
                }
                else
                    return this.FormulaErrorStrings[invalid_arguments];
            }
            return mround.ToString();
        }

        /// <summary>
        /// Returns the random integer number between the given two numbers.
        /// </summary>
        /// <param name="argList">StartNumber, EndNumber</param>
        /// <returns>Random number between two values</returns>
        public string ComputeRandbetween(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            int argCount = args.GetLength(0);
            if (argCount != 2)
                return this.FormulaErrorStrings[this.invalid_arguments];
            string arg1 = this.GetValueFromArg(args[0]);
            string arg2 = this.GetValueFromArg(args[1]);
            double bottom = double.Parse(arg1);
            double top = double.Parse(arg2);
            string number1 = ComputeRound(arg1);
            string number2 = ComputeRound(arg2);
            if (bottom > top)
                return "#NUM!";
            if (this.rand == null)
                this.rand = new Random();
            return this.rand.Next(int.Parse(number1), int.Parse(number2)).ToString();
        }
        /// <summary>
        /// Return the square root of product of given number with PI.
        /// </summary>
        /// <param name="args">number to find Sqrtpi.</param>
        /// <returns>Return Sqrtpi value of giveen number</returns>
        public string ComputeSqrtpi(string args)
        {
            double pi = 3.14159;
            double sqrtValue;
            string s1;
            string sqrtpi = string.Empty;
            string[] numList = args.Split(new char[] { ParseArgumentSeparator });
            if (numList.GetLength(0) > 1)
                return this.FormulaErrorStrings[this.invalid_arguments];
            if (args.Length > 0 && !char.IsLetter(args[0]) && args.IndexOfAny(new char[] { ParseArgumentSeparator, ':' }) == -1)
            {
                if (double.Parse(args) >= 0)
                {
                    sqrtValue = double.Parse(args) * pi;
                    sqrtpi = ComputeSqrt(sqrtValue.ToString());
                }
                else
                    return "#NUM!";
            }
            else if (args.Length > 0 && (args[0] == BMARKER || args[0] == 'u' || args[0] == 'n' || args.IndexOfAny(this.tokens) > -1))
            {
                ////parsed formula
                args = args.Replace('{', '(');
                args = args.Replace('}', ')');
                try
                {
                    s1 = this.ComputedValue(args);
                }
                catch (Exception ex)
                {
                    TraceUtil.TraceExceptionCatched(ex);
                    if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                    {
                        throw ex;
                    }
                    return ex.Message;
                }
                if (double.Parse(s1) >= 0)
                {
                    sqrtValue = double.Parse(args) * pi;
                    sqrtpi = ComputeSqrt(sqrtValue.ToString());
                }
                else
                    return "#NUM!";
            }
            else
            {
                foreach (string s in this.GetCellsFromArgs(args))
                {
                    try
                    {
                        s1 = this.GetValueFromArg(s);
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }
                        return ex.Message;
                    }
                    if (s1.Length > 0)
                    {
                        if (double.Parse(s1) >= 0)
                        {
                            sqrtValue = double.Parse(s1) * pi;
                            sqrtpi = ComputeSqrt(sqrtValue.ToString());
                            break; ////Skip out after first value.
                        }
                        else
                            return "#NUM!";
                    }
                }
            }
            return sqrtpi;
        }

        /// <summary>
        /// Returns the integer portion of division function.
        /// </summary>
        /// <param name="range">Numerator, Denominator to find the quotient</param>
        /// <returns>Returns integer value.</returns>
        public string ComputeQuotient(string range)
        {
            string[] ranges = range.Split(new char[] { ParseArgumentSeparator });
            string s1 = ranges[0];
            string s2 = ranges[1];
            string quotient = string.Empty;
            s1 = this.GetValueFromArg(s1);
            s2 = this.GetValueFromArg(s2);
            double div = (double.Parse(s1) / double.Parse(s2));
            quotient = ComputeInt(div.ToString());
            return quotient;
        }

        /// <summary>
        /// Returns the Double Factorial value for given number
        /// </summary>
        /// <param name="args">number to find FactDouble</param>
        /// <returns>FactDouble of given number </returns>
        public string ComputeFactdouble(string args)
        {
            double number;
            double fact = 1;
            string[] numList = args.Split(new char[] { ParseArgumentSeparator });
            if (numList.GetLength(0) > 1)
                return this.FormulaErrorStrings[this.invalid_arguments];
            if (double.TryParse(this.GetValueFromArg(args), NumberStyles.Any, null, out number)
                && number >= 0)
            {
                int x = (int)number;
                int n = x;
                if (x > 3)
                {
                    while (n > 0)
                    {
                        fact = fact * n;
                        n = n - 2;
                    }
                }
                else
                    fact = ((x == 0) ? 1 : x);
            }
            else
            {
                return this.FormulaErrorStrings[invalid_arguments];
            }
            return fact.ToString();
        }

        /// <summary>
        /// Returns the largest integer divides the numbers without any reminders.
        /// </summary>
        /// <param name="range">number1,number2,...</param>
        /// <returns>GCD value of given numbers.</returns>
        public string ComputeGcd(string range)
        {
            int arrayList = 0;
            int reminder, elements = 0;
            double number;
            string[] ranges = range.Split(new char[] { ParseArgumentSeparator });
            if (ranges.Length > 0)
            {
                foreach (string r in ranges)
                {
                    elements++;
                    if (r.IndexOf(':') > -1)
                    {
                        --elements;
                        foreach (string s in this.GetCellsFromArgs(r))
                        {
                            elements++;
                        }
                    }
                }
            }
            int[] ranges2 = new int[elements];
            foreach (string r in ranges)
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in this.GetCellsFromArgs(r))
                    {
                        if (double.TryParse(this.GetValueFromArg(s), NumberStyles.Any, null, out number) || s.Length > 0 && (int)number > 0)
                        {
                            ranges2[arrayList] = (int)number;
                            arrayList++;
                        }
                        else
                            return "#VALUE!";
                    }
                }
                else
                {
                    if (double.TryParse(this.GetValueFromArg(r), NumberStyles.Any, null, out number) || r.Length > 0 && (int)number > 0)
                    {

                        ranges2[arrayList] = (int)number;
                        arrayList++;
                    }
                    else
                        return "#VALUE!";
                }
            }
            if (ranges2.Length == 1)
                return ranges2[0].ToString();
            for (int i = 0; i < ranges2.Length - 1; i++)
            {
                while (ranges2[1] != 0)
                {
                    reminder = ranges2[0] % ranges2[1];
                    ranges2[0] = ranges2[1];
                    ranges2[1] = reminder;
                }
                if (ranges2.Length > 2 && i < ranges2.Length - 2)
                    ranges2[1] = ranges2[i + 2];
            }
            return ranges2[0].ToString();
        }

        /// <summary>
        /// Returns the smallest positive integer that is a multiple of all given values.
        /// </summary>
        /// <param name="range">Number1,Number2,...</param>
        /// <returns>LCM value of given numbers</returns>
        public string ComputeLcm(string range)
        {
            int arrayList = 0, elements = 0;
            double number;
            string gcd, str = string.Empty;
            string[] ranges = range.Split(new char[] { ParseArgumentSeparator });
            if (ranges.Length > 0)
            {
                foreach (string r in ranges)
                {
                    elements++;
                    if (r.IndexOf(':') > -1)
                    {
                        --elements;
                        foreach (string s in this.GetCellsFromArgs(r))
                        {
                            elements++;
                        }
                    }
                }
            }
            int[] ranges2 = new int[elements];
            foreach (string r in ranges)
            {
                ////is a cellrange
                if (r.IndexOf(':') > -1)
                {
                    foreach (string s in this.GetCellsFromArgs(r))
                    {
                        if (double.TryParse(this.GetValueFromArg(s), NumberStyles.Any, null, out number) || s.Length > 0 && (int)number > 0)
                        {
                            ranges2[arrayList] = (int)number;
                            arrayList++;
                        }
                        else
                            return "#VALUE!";
                    }
                }
                else
                {
                    if (double.TryParse(this.GetValueFromArg(r), NumberStyles.Any, null, out number) || r.Length > 0 && (int)number > 0)
                    {
                        ranges2[arrayList] = (int)number;
                        arrayList++;
                    }
                    else
                        return "#VALUE!";
                }
            }
            if (ranges2.Length == 1)
                return ranges2[0].ToString();
            int multi = ranges2[0] * ranges2[1];
            str = ranges2[0].ToString() + "," + ranges2[1].ToString();
            gcd = ComputeGcd(str);
            int lcm = multi / int.Parse(gcd);
            for (int i = 2; i < ranges2.Length; i++)
            {
                multi = lcm * ranges2[i];
                str = lcm.ToString() + "," + ranges2[i].ToString();
                gcd = ComputeGcd(str);
                lcm = multi / int.Parse(gcd);
            }
            return lcm.ToString();
        }

        /// <summary>
        /// Returns the arabic numeral to roman in TEXT format
        /// </summary>
        /// <param name="argList">Number ,Form for style of roman text 
        /// 0 or omitted Classic. 
        /// 1 More concise.  
        /// 2 More concise.
        /// 3 More concise.
        /// 4 Simplified. 
        /// TRUE Classic. 
        /// FALSE Simplified. 
        /// </param>
        /// <returns>Retuns the Roman string of given numaric value based on the style form</returns>
        public string ComputeRoman(string argList)
        {
            string[] args = argList.Split(new char[] { ParseArgumentSeparator });
            string ans0 = string.Empty, ans1 = string.Empty, ans2 = string.Empty, ans3 = string.Empty, ans4 = string.Empty;
            int no = 0;
            double doubleNumber;
            if (args.Length > 2)
                return this.FormulaErrorStrings[this.invalid_arguments];
            int[] ranges2 = new int[2];
            foreach (string r in args)
            {
                if (IsCellReference(r) && GetCellsFromArgs(r).Length > 2)
                {
                    return "#VALUE!";
                }
                else
                {
                    if ((double.TryParse(this.GetValueFromArg(r), NumberStyles.Any, null, out doubleNumber) || r.Length > 0) && (int)doubleNumber > 0)
                    {
                        ranges2[no] = (int)doubleNumber;
                        no++;
                    }
                    else
                        return "#NAME?";
                }
            }
            if (ranges2[0] < 0 || ranges2[0] > 3999)
                return "#VALUE!";
            if (ranges2[1] < 0 || ranges2[1] > 4)
                return this.FormulaErrorStrings[this.invalid_arguments];
            int[] values = new int[] { 1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1 };
            string[] numerals = new string[] { "M", "CM", "D", "CD", "C", "XC", "L", "XL", "X", "IX", "V", "IV", "I" };
            StringBuilder result = new StringBuilder();
            int number = ranges2[0];
            string num = number.ToString();
            bool flag = true, containForty = false, containNinety = false, endsForty = false, endsNinety = false;
            for (int i = 0; i < 13; i++)
            {
                while (number >= values[i])
                {
                    number -= values[i];
                    result.Append(numerals[i]);
                }
            }
            ans0 = result.ToString();
            if ((num.Contains("45")) || (num.Contains("46")) || (num.Contains("47")) || (num.Contains("48")) || (num.Contains("49")))
                containForty = true;
            if ((num.Contains("95")) || (num.Contains("96")) || (num.Contains("97")) || (num.Contains("98")) || (num.Contains("99")))
                containNinety = true;
            if ((num.EndsWith("45")) || (num.EndsWith("46")) || (num.EndsWith("47")) || (num.EndsWith("48")) || (num.EndsWith("49")))
                endsForty = true;
            if ((num.EndsWith("95")) || (num.EndsWith("96")) || (num.EndsWith("97")) || (num.EndsWith("98")) || (num.EndsWith("99")))
                endsNinety = true;
            if (!(containForty|| containNinety))
            {
                return result.ToString();
            }
            if (num.EndsWith("499") || num.EndsWith("999"))
            {
                string temp = ans0;
                ans1 = temp.Replace('C', 'L');
                ans1 = ans1.Replace('X', 'V');
                ans2 = ans1.Replace('L', 'X');
                string[] sp = ans2.Split('V');
                char[] arr = sp[1].ToCharArray();
                char[] arr1 = sp[0].ToCharArray();
                Array.Reverse(arr);
                temp = new string(arr);
                ans2 = new string(arr1);
                ans2 = ans2 + temp;
                ans3 = ans2.Replace('X', 'V');
                string[] sp1 = ans3.Split('V');
                char[] arr2 = sp1[1].ToCharArray();
                Array.Reverse(arr2);
                temp = new string(arr2);
                for (int i = 0; i < sp.Length - 1; i++)
                {
                    ans4= (i != 1)?  ans4 + sp1[i] :ans4 + temp; 
                }
                flag = false;
            }
            if (containNinety)
            {
                string str = "4";
                str = str.Replace('4', num[num.IndexOf('9')]);
                int digit = int.Parse(str);
                if (digit == 4 || digit == 9)
                {
                    ans1 = ans0.Replace('C', 'L');
                    int x = ans1.IndexOf('X');
                    string[] sp = ans1.Split('X');
                    int z = 0;
                    int i = 0;
                    for (i = 0; z < x + 1; i++)
                    {
                        z += sp[i].Length;
                    }
                    char[] arr = sp[1].ToCharArray();
                    Array.Reverse(arr);
                    ans1 = new string(arr);
                    int ans1Length = ans1.Length;
                    int ans1Length1 = ans1Length;
                    int k = 0;
                    for (k = 0; k < ans1Length1 - 2; k++)
                    {
                        ans1 = ans1 + ans1[ans1Length - 3];
                        ans1Length--;
                    }
                    ans1 = ans1.Substring(k);
                    ans1 = sp[0] + ans1;
                    string[] sp2 = ans1.Split('L');
                    ans2 = sp2[0] + "X";
                    for (int d = 1; d < sp2.Length; d++)
                    {
                        ans2 = ans2 + sp2[d];
                    }
                    string[] sp3 = ans2.Split('X');
                    char[] arr1 = sp3[1].ToCharArray();
                    Array.Reverse(arr1);
                    ans3 = new string(arr1);
                    int ans3Length = ans3.Length;
                    int ans3Length1 = ans3Length;
                    int k1 = 0;
                    for (k1 = 0; k1 < ans3Length1 - 2; k1++)
                    {
                        ans3 = ans3 + ans3[ans3Length - 3];
                        ans3Length--;
                    }
                    ans3 = ans3.Substring(k);
                    ans4 = ans3 = sp3[0] + ans3;
                    flag = false;
                }
            }
            if (flag && (endsForty  || endsNinety ))
            {
                int x = ans0.IndexOf('X');
                string[] sp = ans0.Split('X');
                if (sp.Length == 2)
                {
                    int z = 0;
                    int i = 0;
                    for (i = 0; z < x + 1; i++)
                    {
                        z += sp[i].Length;
                    }
                    char[] arr = sp[i - 1].ToCharArray();
                    Array.Reverse(arr);
                    ans1 = new string(arr);
                    int ans1Length = ans1.Length;
                    int ans1Length1 = ans1Length;
                    int k = 0;
                    for (k = 0; k < ans1Length1 - 2; k++)
                    {
                        ans1 = ans1 + ans1[ans1Length - 3];
                        ans1Length--;
                    }
                    ans1 = ans1.Substring(k);
                    ans1 = ans2 =ans3 =ans4 = sp[0] + ans1;
                }

                if (sp.Length == 3)
                {
                    string temp = ans0;
                    int i;
                    ans1 = temp.Replace("X", "V");
                    string[] arrV = ans1.Split('V');
                    ans2 = "";
                    char[] arr = arrV[1].ToCharArray();
                    Array.Reverse(arr);
                    string revStr = new string(arr);
                    for (i = 0; i < arrV.Length; i++)
                    {
                       ans2 = (i == 1) ? ans2 + revStr : ans2 + arrV[i];
                    }
                    ans3 = ans4 = ans2;
                }
            }
            else if (flag && (!((num.EndsWith("45")) || (num.EndsWith("46")) || (num.EndsWith("47")) || (num.EndsWith("48")) || (num.EndsWith("95")) || (num.EndsWith("96")) || (num.EndsWith("97")) || (num.EndsWith("98")))))
            {
                int x = ans0.IndexOf('C');
                string[] sp = ans0.Split('C');
                if (sp.Length == 2)
                {
                    int z = 0;
                    int i = 0;
                    for (i = 0; z < x + 1; i++)
                    {
                        z += sp[i].Length;
                    }
                    string temp = ans0;
                    char[] arr = sp[i - 1].ToCharArray();
                    Array.Reverse(arr);
                    ans1 = new string(arr);
                    int ans1Length = ans1.Length;
                    int ans1Length1 = ans1Length;
                    int k = 0;
                    for (k = 0; k < ans1Length1 - 2; k++)
                    {
                        ans1 = ans1 + ans1[ans1Length - 3];
                        ans1Length--;
                    }
                    ans1 = ans1.Substring(k);
                    ans1 = ans2 = ans3 = ans4 = sp[0] + ans1;
                }
                if (sp.Length == 3)
                {
                    string temp = ans0;
                    int i;
                    ans1 = temp.Replace("C", "L");
                    string[] arrV = ans1.Split('L');
                    ans2 = string.Empty;
                    char[] arr = arrV[1].ToCharArray();
                    Array.Reverse(arr);
                    string revStr = new string(arr);
                    for (i = 0; i < arrV.Length; i++)
                    {
                        ans2 = (i == 1) ? ans2 + revStr : ans2 + arrV[i];
                    }
                    ans3 = ans4 = ans2;
                }
            }
            else if (flag)
            {
                return ans0;
            }
            switch (ranges2[0])
            {
                case 1:
                    ans0 = ans1;
                    break;
                case 2:
                    ans0 = ans2;
                    break;
                case 3:
                    ans0 = ans3;
                    break;
                case 4:
                    ans0 = ans4;
                    break;
            }
            return ans0;
        }
        /// <summary>
        /// Returns a value you specify if a formula evaluates to an error
        /// otherwise, returns the result of the formula.
        /// </summary>
        /// <param name="args">String to be tested.</param>
        public string ComputeIfError(string args)
        {
            string[] argsArray = args.Split(new char[] { ParseArgumentSeparator });
            if (argsArray.Length != 2)
                return FormulaErrorStrings[wrong_number_arguments];
            string range = argsArray[0];
            if (range.StartsWith(TIC.ToString()))
                return GetValueFromArg(argsArray[0]);
            try
            {
                if (range[0] == BMARKER)
                {
                    range = range.Replace(BMARKER, ' ');
                    range = range.Trim();
                    if (range.StartsWith("NAN")
                || range.StartsWith("-NAN")
                || range.StartsWith("INFINITY")
                || range.StartsWith("-INFINITY")
                || range.StartsWith("#")
                || range.StartsWith("n#")
                )
                        return GetValueFromArg(argsArray[1]);
                }

                range = argsArray[0];

                range = GetValueFromArg(range).ToUpper().Replace(TIC, "");
            }
            catch
            {
                range = range.ToUpper();
            }
            if (range.StartsWith("NAN")
                || range.StartsWith("-NAN")
                || range.StartsWith("INFINITY")
                || range.StartsWith("-INFINITY")
                || range.StartsWith("#")
                || range.StartsWith("n#")
                )
                return GetValueFromArg(argsArray[1]);
            else
                return GetValueFromArg(argsArray[0]);
        }

        /// <summary>
        /// Returns the string or text referred by the given value.
        /// </summary>
        /// <param name="args">value to find the refrred text - Required </param>
        /// <returns>Referred Text</returns>
        public string ComputeT(string args)
        {
            int elements = 0;
            string[] arg = SplitArguments(args, ParseArgumentSeparator);
            if (arg.Length > 0)
            {
                foreach (string r in arg)
                {
                    elements++;
                    if (r.IndexOf(':') > -1)
                    {
                        --elements;
                        foreach (string s1 in this.GetCellsFromArgs(r))
                        {
                            elements++;
                        }
                    }
                }
            }
            int argCount = arg.GetLength(0);
            if (elements > 1)
                return this.FormulaErrorStrings[wrong_number_arguments];
            string s = GetValueFromArg(args);
            string pattern = @"[a-zA-Z0-9!#$%&'()*+,/:;<=>?@\^_`{|}~-]";
            if (ComputeIsNumber(args) == "TRUE" || ComputeIsLogical(args) == "TRUE")
                return string.Empty;
            else if (Regex.IsMatch(s, pattern))
            {
                s = s.Replace(this.TIC, "");
                return s;
            }
            else
                return string.Empty;
        }
        /// <summary>
        /// Determines whether the value is even or not.
        /// </summary>
        /// <param name="args">Value to be tested.</param>
        /// <returns>True, if the value is even, false otherwise.</returns>
        public string ComputeIsEven(string args)
        {
            string cellReference = string.Empty;
            double val = 0.0;
            string[] arg = SplitArguments(args, ParseArgumentSeparator);
            int argCount = arg.GetLength(0);
            if (argCount != 1)
                return this.FormulaErrorStrings[requires_a_single_argument];
            if (!args.Equals(string.Empty))
            {
                if (args.IndexOfAny(new char[] { ParseArgumentSeparator, ':' }) != -1)
                {
                    return "#VALUE!";
                }

                this.PutTokensForSheets(ref args);
                cellReference = this.GetValueFromArg(args);
                if (double.TryParse(cellReference, NumberStyles.Any, null, out val))
                {
                    int result = (int)val;
                    if (result != 0 && result % 2 == 0)
                    {
                        return this.TRUEVALUESTR;
                    }
                    return this.FALSEVALUESTR;
                }
            }
            return this.TRUEVALUESTR;
        }

        /// <summary>
        /// Determines whether the value is odd or not.
        /// </summary>
        /// <param name="args">Value to be tested.</param>
        /// <returns>True, if the value is odd, false otherwise.</returns>
        public string ComputeIsOdd(string args)
        {
            string cellReference = string.Empty;
            double val = 0.0;
            string[] arg = SplitArguments(args, ParseArgumentSeparator);
            int argCount = arg.GetLength(0);
            if (argCount != 1)
                return this.FormulaErrorStrings[requires_a_single_argument];
            if (!args.Equals(string.Empty))
            {
                if (args.IndexOfAny(new char[] { ParseArgumentSeparator, ':' }) != -1)
                {
                    return "#VALUE!";
                }

                cellReference = this.GetValueFromArg(args);
                if (double.TryParse(cellReference, NumberStyles.Any, null, out val))
                {
                    int result = (int)val;
                    if (result % 2 != 0)
                    {
                        return this.TRUEVALUESTR;
                    }
                    return this.FALSEVALUESTR;
                }
            }
            return this.TRUEVALUESTR;
        }

        /// <summary>
        /// Returns a number converted from the provided value.
        /// </summary>
        /// <param name="args">Value to be converted.</param>
        /// <returns>A number in string format or an error string.</returns>
        public string ComputeN(string args)
        {
            string cellReference = string.Empty;
            double val = 0.0;
            DateTime date;
            string[] arg = SplitArguments(args, ParseArgumentSeparator);
            int argCount = arg.GetLength(0);
            if (argCount != 1)
                return this.FormulaErrorStrings[requires_a_single_argument];
            cellReference = this.GetValueFromArg(args);
            if (double.TryParse(cellReference, NumberStyles.Any, null, out val))
            {
                return val.ToString();
            }
            else if (DateTime.TryParse(cellReference, out date))
            {
                val = double.Parse(ComputeDatevalue(args));
            }
            else if (cellReference.Equals(this.TRUEVALUESTR))
            {
                val = 1;
            }
            else if (cellReference.Equals(this.FALSEVALUESTR))
            {
                val = 0;
            }
            return val.ToString();
        }

        

        /// <summary>
        /// Returns the error value(#N/A). Used internally.
        /// </summary>
        /// <param name="args">empty string.</param>
        /// <returns>error value.</returns>
        private string ComputeNA(string args)
        {
            return ComputeNA();
        }

        /// <summary>
        /// Returns the error value (#N/A - value not available).
        /// </summary>
        /// <returns>error value.</returns>
        public string ComputeNA()
        {
            return "#N/A";
        }

        ///<summary>
        /// Returns a number corresponding to the predefined error values(#NULL!, #VALUE!, #REF!, #NAME?, #NUM!, #N/A, "#GETTING_DATA). Returns #N/A if not or any value enclosed within double quotes.
        ///</summary>
        public string ComputeErrorType(string args)
        {
            string cellReference = string.Empty;
            string[] arg = SplitArguments(args, ParseArgumentSeparator);
            int argCount = arg.GetLength(0);
            if (argCount != 1)
                return this.FormulaErrorStrings[requires_a_single_argument];
            if (this.IsCellReference(args))
            {
                cellReference = this.GetValueFromArg(args);
            }
            else
            {
                cellReference = args;
            }
            if (cellReference.Length > 1
                           && cellReference[0] == this.TIC[0]
                           && cellReference[cellReference.Length - 1] == this.TIC[0])
            {
                return "#N/A";
            }
            if (cellReference.Equals("#NULL!"))
                return "1";
            else if (cellReference.Equals("#DIV/0!"))
                return "2";
            else if (cellReference.Equals("#VALUE!"))
                return "3";
            else if (cellReference.Equals("#REF!"))
                return "4";
            else if (cellReference.Equals("#NAME?"))
                return "5";
            else if (cellReference.Equals("#NUM!"))
                return "6";
            else if (cellReference.Equals("#N/A"))
                return "7";
            else if (cellReference.Equals("#GETTING_DATA"))
                return "8";
            return "#N/A";
        }

        ////New functions end

        #region Methods for string function(ISTEXT/ISNONTEXT)
        /// <summary>
        /// Determines whether the arg is a valid cell name.
        /// </summary>
        /// <param name="args">Cell name.</param>
        /// <returns>True is the arg is a valid cell name, false otherwise.</returns>
        bool IsCellReference(string args)
        {
            PutTokensForSheets(ref args);
            string sheetToken = SheetToken(args);
            if (!String.IsNullOrEmpty(sheetToken))
            {
                args = args.Replace(sheetToken, string.Empty);
            }

            bool isAlpha = false, isNum = false;
            if (args.IndexOf(':') != args.LastIndexOf(':'))
            {
                return false;
            }

            foreach (char c in args.ToCharArray())
            {
                if (char.IsLetter(c))
                {
                    if (isNum)
                    {
                        return false;
                    }

                    isAlpha = true;
                }

                if (char.IsDigit(c))
                {
                    if (!isAlpha)
                    {
                        return false;
                    }

                    isNum = true;
                }

                if (char.Equals(c, ':'))
                {
                    isAlpha = false;
                    isNum = false;
                }
            }

            if (isAlpha && isNum)
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// Determines whether the computed value is a valid cell name.
        /// </summary>
        /// <param name="formula">Formula to </param>
        /// <param name="cell">The cell name</param>
        /// <returns>True if the computed value is string type, false otherwise.</returns>
        bool IsFormulaResultString(string formula, string cell)
        {
            int row = RowIndex(cell), col = ColIndex(cell);
            GridStyleInfo style = grid[row, col];
            if (style.CellValueType == typeof(string))
            {
                return true;
            }

            if (style.CellType == GridCellTypeName.FormulaCell)
            {
                string innerFormula = cell.Equals(this.cell) ? "=ISTEXT(" + formula + ")" : grid[row, col].Text;
                if (!(innerFormula.IndexOf(formulaChar) < 0 || innerFormula.Equals(formula))
                    && IsCellReference(formula))
                {
                    string pf, err, compVal;
                    if (IsFormulaValid(formula, out pf, out err, out compVal))
                    {
                        if (compVal.EndsWith("\""))
                        {
                            return true;
                        }

                        if (formula.Equals(pf) && !formula.Equals(compVal))
                        {
                            row = RowIndex(pf);
                            col = ColIndex(pf);
                            innerFormula = grid[row, col].Text;
                            if (innerFormula != string.Empty && !innerFormula.Equals(compVal))
                            {
                                if (!RecurciveFormulaCheck(innerFormula, cell))
                                {
                                    return false;
                                }
                                else
                                {
                                    return true;
                                }
                            }
                        }

                        cell = GridRangeInfo.GetAlphaLabel(col) + row.ToString();
                        style = grid[row, col];
                    }
                }
            }

            if (((style.CellType == GridCellTypeName.TextBox || style.CellType == GridCellTypeName.RichText
                || style.CellType == GridCellTypeName.OriginalTextBox) &&
                style.CellValueType == null) || style.CellValueType == typeof(string))
            {
                return true;
            }
            else if (style.CellValueType == null && style.CellType == GridCellTypeName.FormulaCell)
            {
                string val = formula;
                if (cell.Equals(this.cell))
                {
                    if (!formula.EndsWith("\""))
                    {
                        return false;
                    }
                }
                else
                {
                    val = GetValueFromArg(cell);
                }

                if (val == string.Empty)
                {
                    return false;
                }

                if (val.EndsWith("\""))
                {
                    return true;
                }

                double d;
                if (!double.TryParse(val, out d))
                {
                    if (!(val.StartsWith("NAN")
                || val.StartsWith("-NAN")
                || val.StartsWith("INFINITY")
                || val.StartsWith("-INFINITY")
                || val.StartsWith("#")
                || val.StartsWith("n#")))
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Determines whether the computed value is a valid cell name
        /// by checking all it dependent cells.
        /// </summary>
        /// <returns>return boolean value to indicate whether the computed value is a valid cell name</returns>
        bool RecurciveFormulaCheck(string formula, string cell)
        {
            int row = RowIndex(cell), col = ColIndex(cell);
            GridStyleInfo style = grid[row, col];
            ////string formula = style.Text;
            if (formula.Length == 0)
            {
                if (style.CellValueType == null || style.CellValueType == typeof(string))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }

            ////Check if the last function returns string type value.
            int rightParens = formula.IndexOf(')');
            if (rightParens == formula.Length - 1)
            {
                int firstLeftParens = formula.IndexOf('(');
                if (formula.StartsWith("="))
                {
                    formula = formula.Remove(0, 1);
                }

                string funcName = formula.Substring(0, firstLeftParens - 1).ToUpper().Trim();
                ////check for all functions that returns Text.
                foreach (string strFunction in textFunctions)
                {
                    if (funcName.Equals(strFunction))
                    {
                        return true;
                    }
                }

                return false;
            }

            if (formula.IndexOf('(') > 0)
            {
                return false;
            }

            Hashtable depFormulaCells = (Hashtable)DependentFormulaCells[cell];
            if (depFormulaCells == null)
            {
                if (IsFormulaResultString(formula, cell))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }

            if (depFormulaCells.Count > 0)
            {
                foreach (DictionaryEntry entry in depFormulaCells)
                {
                    cell = (string)entry.Value;
                }

                formula = grid[RowIndex(cell), ColIndex(cell)].Text;
                if (formula.IndexOf(formulaChar) == 0)
                {
                    formula = formula.Remove(0, 1);
                }

                if (IsFormulaResultString(formula, cell))
                {
                    return true;
                }
            }

            return false;
        }
        #endregion

        private static bool useCommonLibraryFunction = false;

        /// <summary>
        /// Gets or sets a value indicating whether you want all grids to share the same collection of library functions.
        /// </summary>
        public static bool UseCommonLibrary
        {
            get { return useCommonLibraryFunction; }
            set { useCommonLibraryFunction = value; }
        }

        System.Collections.Specialized.StringCollection textFunctions;

        /// <summary>
        /// Creates and initially loads the Function Library with the supported functions.
        /// </summary>
        public virtual void InitLibraryFunctions()
        {
            libraryFunctions = new Hashtable();

            bool save = UseCommonLibrary;
            UseCommonLibrary = false;

            ////hand coded forumlas
            AddFunction("Sum", new LibraryFunction(ComputeSum));
            AddFunction("Avg", new LibraryFunction(ComputeAvg));
            AddFunction("Max", new LibraryFunction(ComputeMax));
            AddFunction("Min", new LibraryFunction(ComputeMin));
            AddFunction("Pi", new LibraryFunction(ComputePI));
            AddFunction("Sign", new LibraryFunction(ComputeSign));
            AddFunction("Pow", new LibraryFunction(ComputePow)); ////for compatibility
            AddFunction("Power", new LibraryFunction(ComputePow));

            ////logical functions
            AddFunction("And", new LibraryFunction(ComputeAnd));
            AddFunction("Or", new LibraryFunction(ComputeOr));
            AddFunction("If", new LibraryFunction(ComputeIf));
            AddFunction("Not", new LibraryFunction(ComputeNot));
            AddFunction("False", new LibraryFunction(ComputeFalse));
            AddFunction("True", new LibraryFunction(ComputeTrue));

            ////uses ComputeMath delegate (System.Math)
            AddFunction("Acos", new LibraryFunction(ComputeAcos));
            AddFunction("Asin", new LibraryFunction(ComputeAsin));
            AddFunction("Atan", new LibraryFunction(ComputeAtan));
            AddFunction("Cos", new LibraryFunction(ComputeCos));
            AddFunction("Sin", new LibraryFunction(ComputeSin));
            AddFunction("Cosh", new LibraryFunction(ComputeCosh));
            AddFunction("Sinh", new LibraryFunction(ComputeSinh));
            AddFunction("Tanh", new LibraryFunction(ComputeTanh));
            AddFunction("Round", new LibraryFunction(ComputeRound));
            AddFunction("Ceiling", new LibraryFunction(ComputeCeiling));
            AddFunction("Floor", new LibraryFunction(ComputeFloor));
            AddFunction("Log", new LibraryFunction(ComputeLog));
            AddFunction("Log10", new LibraryFunction(ComputeLog10));
            AddFunction("Exp", new LibraryFunction(ComputeExp));
            AddFunction("Sqrt", new LibraryFunction(ComputeSqrt));
            AddFunction("Abs", new LibraryFunction(ComputeAbs));
            AddFunction("Tan", new LibraryFunction(ComputeTan));

            ////added...
            AddFunction("Rand", new LibraryFunction(ComputeRand));
            AddFunction("SumIf", new LibraryFunction(ComputeSumIf));

            ////missing formulas
            AddFunction("HLookUp", new LibraryFunction(ComputeHLookUp));
            AddFunction("VLookUp", new LibraryFunction(ComputeVLookUp));
            AddFunction("Left", new LibraryFunction(ComputeLeft));
            AddFunction("Len", new LibraryFunction(ComputeLen));
            AddFunction("Mid", new LibraryFunction(ComputeMid));
            AddFunction("Right", new LibraryFunction(ComputeRight));
            AddFunction("Product", new LibraryFunction(ComputeProduct));
            AddFunction("Value", new LibraryFunction(ComputeValue));
            AddFunction("Mod", new LibraryFunction(ComputeMod));
            AddFunction("Trunc", new LibraryFunction(ComputeTrunc));
            AddFunction("SumProduct", new LibraryFunction(ComputeSumProduct));
            AddFunction("Average", new LibraryFunction(ComputeAvg));
            AddFunction("Int", new LibraryFunction(ComputeInt));

            ////information functions
            AddFunction("IsError", new LibraryFunction(ComputeIsError));
            AddFunction("IsNumber", new LibraryFunction(ComputeIsNumber));
            AddFunction("IsLogical", new LibraryFunction(ComputeIsLogical));
            AddFunction("IsNA", new LibraryFunction(ComputeIsNA));
            AddFunction("IsErr", new LibraryFunction(ComputeIsErr));
            AddFunction("IsBlank", new LibraryFunction(ComputeIsBlank));
            AddFunction("IsText", new LibraryFunction(ComputeIsText));
            AddFunction("IsNonText", new LibraryFunction(ComputeIsNonText));

            ////string and data functions
            AddFunction("Dollar", new LibraryFunction(ComputeDollar));
            AddFunction("Fixed", new LibraryFunction(ComputeFixed));
            AddFunction("Lower", new LibraryFunction(ComputeLower));
            AddFunction("Upper", new LibraryFunction(ComputeUpper));
            AddFunction("Trim", new LibraryFunction(ComputeTrim));
            AddFunction("Text", new LibraryFunction(ComputeText));
            AddFunction("Concatenate", new LibraryFunction(ComputeConcatenate));
            AddFunction("Substitute", new LibraryFunction(ComputeSubstitute));

            ////storing string functions in array for ISTEXT/ISNONTEXT function to check
            textFunctions = new System.Collections.Specialized.StringCollection();
            textFunctions.AddRange(new string[] { "DOLLAR", "FIXED", "LOWER", "UPPER", "TRIM", "TEXT", "CONCATENATE", "SUBSTITUTE" });

            AddFunction("Index", new LibraryFunction(ComputeIndex));
            AddFunction("Indirect", new LibraryFunction(ComputeIndirect));

            ////financial
            //// http://support.microsoft.com/default.aspx?scid=kb;en-us;123757
            AddFunction("Db", new LibraryFunction(ComputeDb));
            AddFunction("Ddb", new LibraryFunction(ComputeDdb));
            AddFunction("Fv", new LibraryFunction(ComputeFv));
            AddFunction("Ipmt", new LibraryFunction(ComputeIpmt));
            AddFunction("Irr", new LibraryFunction(ComputeIrr));
            AddFunction("Xirr", new LibraryFunction(ComputeXirr));
            AddFunction("Ispmt", new LibraryFunction(ComputeIspmt));
            AddFunction("Mirr", new LibraryFunction(ComputeMirr));
            AddFunction("Nper", new LibraryFunction(ComputeNper));
            AddFunction("Npv", new LibraryFunction(ComputeNpv));
            AddFunction("Pmt", new LibraryFunction(ComputePmt));
            AddFunction("Ppmt", new LibraryFunction(ComputePpmt));
            AddFunction("Pv", new LibraryFunction(ComputePv));
            AddFunction("Rate", new LibraryFunction(ComputeRate));
            AddFunction("Sln", new LibraryFunction(ComputeSln));
            AddFunction("Syd", new LibraryFunction(ComputeSyd));
            AddFunction("Vdb", new LibraryFunction(ComputeVdb));

            ////new functions
            ////> asinh(x) = sgn(x) ln(|x| + sqrt(x^2 + 1))     all x
            ////> acosh(x) = ln(x + sqrt(x^2-1))                x >= 1
            ////> atanh(x) = 1/2 sgn(x) ln((1+|x|)/(1-|x|))     |x| < 1

            AddFunction("Rand", new LibraryFunction(ComputeRand));
            AddFunction("Acosh", new LibraryFunction(ComputeAcosh));
            AddFunction("Asinh", new LibraryFunction(ComputeAsinh));
            AddFunction("Atanh", new LibraryFunction(ComputeAtanh));
            AddFunction("Atan2", new LibraryFunction(ComputeAtan2));
            AddFunction("Combin", new LibraryFunction(ComputeCombin));
            AddFunction("Degrees", new LibraryFunction(ComputeDegrees));
            AddFunction("Even", new LibraryFunction(ComputeEven));
            AddFunction("Fact", new LibraryFunction(ComputeFact));
            AddFunction("Ln", new LibraryFunction(ComputeLn));
            AddFunction("Odd", new LibraryFunction(ComputeOdd));
            AddFunction("Radians", new LibraryFunction(ComputeRadians));
            AddFunction("Round", new LibraryFunction(ComputeRound));
            AddFunction("Rounddown", new LibraryFunction(ComputeRounddown));
            AddFunction("Roundup", new LibraryFunction(ComputeRoundup));

            ////needs strings AddFunction("Sumif", new LibraryFunction(ComputeSumif));
            AddFunction("Sumsq", new LibraryFunction(ComputeSumsq));
            AddFunction("Sumx2my2", new LibraryFunction(ComputeSumx2my2));
            AddFunction("Sumx2py2", new LibraryFunction(ComputeSumx2py2));
            AddFunction("Sumxmy2", new LibraryFunction(ComputeSumxmy2));

            AddFunction("Date", new LibraryFunction(ComputeDate));
            AddFunction("Datevalue", new LibraryFunction(ComputeDatevalue));
            AddFunction("Day", new LibraryFunction(ComputeDay));
            AddFunction("Days360", new LibraryFunction(ComputeDays360));
            AddFunction("Hour", new LibraryFunction(ComputeHour));
            AddFunction("Minute", new LibraryFunction(ComputeMinute));
            AddFunction("Second", new LibraryFunction(ComputeSecond));
            AddFunction("Month", new LibraryFunction(ComputeMonth));
            AddFunction("Now", new LibraryFunction(ComputeNow));
            AddFunction("Time", new LibraryFunction(ComputeTime));
            AddFunction("Timevalue", new LibraryFunction(ComputeTimevalue));
            AddFunction("Today", new LibraryFunction(ComputeToday));
            AddFunction("Weekday", new LibraryFunction(ComputeWeekday));
            AddFunction("Year", new LibraryFunction(ComputeYear));

            ////stats
            AddFunction("Avedev", new LibraryFunction(ComputeAvedev));
            AddFunction("Averagea", new LibraryFunction(ComputeAveragea));
            AddFunction("Gammaln", new LibraryFunction(ComputeGammaln));
            AddFunction("Gammadist", new LibraryFunction(ComputeGammadist));
            AddFunction("Gammainv", new LibraryFunction(ComputeGammainv));
            AddFunction("Geomean", new LibraryFunction(ComputeGeomean));
            AddFunction("Harmean", new LibraryFunction(ComputeHarmean));
            AddFunction("Hypgeomdist", new LibraryFunction(ComputeHypgeomdist));
            AddFunction("Intercept", new LibraryFunction(ComputeIntercept));
            AddFunction("Binomdist", new LibraryFunction(ComputeBinomdist));
            AddFunction("Chidist", new LibraryFunction(ComputeChidist));
            AddFunction("Chiinv", new LibraryFunction(ComputeChiinv));
            AddFunction("Chitest", new LibraryFunction(ComputeChitest));
            AddFunction("Normdist", new LibraryFunction(ComputeNormdist));
            AddFunction("Norminv", new LibraryFunction(ComputeNorminv));
            AddFunction("NormsInv", new LibraryFunction(ComputeNormsInv));
            AddFunction("NormsDist", new LibraryFunction(ComputeNormsDist));
            AddFunction("Confidence", new LibraryFunction(ComputeConfidence));
            AddFunction("Correl", new LibraryFunction(ComputeCorrel));
            AddFunction("Count", new LibraryFunction(ComputeCount));
            AddFunction("Counta", new LibraryFunction(ComputeCounta));
            AddFunction("Countblank", new LibraryFunction(ComputeCountblank));
            AddFunction("Countif", new LibraryFunction(ComputeCountif));
            AddFunction("Covar", new LibraryFunction(ComputeCovar));
            AddFunction("CritBinom", new LibraryFunction(ComputeCritbinom));
            AddFunction("Devsq", new LibraryFunction(ComputeDevsq));
            AddFunction("Expondist", new LibraryFunction(ComputeExpondist));
            AddFunction("Fdist", new LibraryFunction(ComputeFdist));
            AddFunction("Finv", new LibraryFunction(ComputeFinv));
            AddFunction("Fisher", new LibraryFunction(ComputeFisher));
            AddFunction("Fisherinv", new LibraryFunction(ComputeFisherinv));
            AddFunction("Forecast", new LibraryFunction(ComputeForecast));
            AddFunction("Growth", new LibraryFunction(ComputeGrowth));
            AddFunction("Kurt", new LibraryFunction(ComputeKurt));
            AddFunction("Large", new LibraryFunction(ComputeLarge));
            AddFunction("Logest", new LibraryFunction(ComputeLogest));
            AddFunction("Logestb", new LibraryFunction(ComputeLogestb));
            AddFunction("Lognormdist", new LibraryFunction(ComputeLognormdist));
            AddFunction("Loginv", new LibraryFunction(ComputeLoginv));
            AddFunction("Maxa", new LibraryFunction(ComputeMaxa));
            AddFunction("Median", new LibraryFunction(ComputeMedian));
            AddFunction("Mina", new LibraryFunction(ComputeMina));
            AddFunction("Mode", new LibraryFunction(ComputeMode));
            AddFunction("Negbinomdist", new LibraryFunction(ComputeNegbinomdist));
            AddFunction("Pearson", new LibraryFunction(ComputePearson));
            AddFunction("Percentile", new LibraryFunction(ComputePercentile));
            AddFunction("Percentrank", new LibraryFunction(ComputePercentrank));
            AddFunction("Permut", new LibraryFunction(ComputePermut));
            AddFunction("Poisson", new LibraryFunction(ComputePoisson));
            AddFunction("Prob", new LibraryFunction(ComputeProb));
            AddFunction("Quartile", new LibraryFunction(ComputeQuartile));
            AddFunction("Rank", new LibraryFunction(ComputeRank));
            AddFunction("Rsq", new LibraryFunction(ComputeRsq));
            AddFunction("Skew", new LibraryFunction(ComputeSkew));
            AddFunction("Slope", new LibraryFunction(ComputeSlope));
            AddFunction("Small", new LibraryFunction(ComputeSmall));
            AddFunction("Standardize", new LibraryFunction(ComputeStandardize));
            AddFunction("Stdev", new LibraryFunction(ComputeStdev));
            AddFunction("Stdeva", new LibraryFunction(ComputeStdeva));
            AddFunction("Stdevp", new LibraryFunction(ComputeStdevp));
            AddFunction("Stdevpa", new LibraryFunction(ComputeStdevpa));
            AddFunction("Steyx", new LibraryFunction(ComputeSteyx));
            AddFunction("Trimmean", new LibraryFunction(ComputeTrimmean));
            AddFunction("Var", new LibraryFunction(ComputeVar));
            AddFunction("Vara", new LibraryFunction(ComputeVara));
            AddFunction("Varp", new LibraryFunction(ComputeVarp));
            AddFunction("Varpa", new LibraryFunction(ComputeVarpa));
            AddFunction("Weibull", new LibraryFunction(ComputeWeibull));
            AddFunction("Ztest", new LibraryFunction(ComputeZtest));

            AddFunction("Multinomial", new LibraryFunction(ComputeMultinomial));
            AddFunction("IsEven", new LibraryFunction(ComputeIsEven));
            AddFunction("IsOdd", new LibraryFunction(ComputeIsOdd));
            AddFunction("N", new LibraryFunction(ComputeN));
            AddFunction("NA", new LibraryFunction(ComputeNA));
            AddFunction("ERROR.TYPE", new LibraryFunction(ComputeErrorType));
            AddFunction("IfError", new LibraryFunction(ComputeIfError));
            AddFunction("T", new LibraryFunction(ComputeT));
            AddFunction("Multinomial", new LibraryFunction(ComputeMultinomial));
            AddFunction("Mround", new LibraryFunction(ComputeMround));
            AddFunction("Randbetween", new LibraryFunction(ComputeRandbetween));
            AddFunction("Sqrtpi", new LibraryFunction(ComputeSqrtpi));
            AddFunction("Quotient", new LibraryFunction(ComputeQuotient));
            AddFunction("Factdouble", new LibraryFunction(ComputeFactdouble));
            AddFunction("Gcd", new LibraryFunction(ComputeGcd));
            AddFunction("Lcm", new LibraryFunction(ComputeLcm));
            AddFunction("Roman", new LibraryFunction(ComputeRoman));

            UseCommonLibrary = save;
        }
        #endregion

        #region Processing Arguments

        private string ZapQuotedColons(string s)
        {
            int start = s.IndexOf(TIC);
            while (start > -1)
            {
                int end = s.Substring(start + 1).IndexOf(TIC);
                if (end > -1)
                {
                    int i = s.Substring(start + 1).IndexOf(':');
                    while (i > -1 && i < end)
                    {
                        s = s.Substring(i + start) + " " + s.Substring(i + start + 1);
                        i = s.Substring(start + 1).IndexOf(':');
                    }
                }

                start = end;
            }

            return s;
        }

        private char MarkerChar = '`';

        private void MarkColonsInQuotes(ref string args)
        {
            bool inQuotes = false;
            for (int i = 0; i < args.Length; ++i)
            {
                if (args[i] == TIC[0])
                {
                    inQuotes = !inQuotes;
                }
                else if (args[i] == ':' && inQuotes)
                {
                    args = args.Replace(':', MarkerChar);
                }
            }
        }

        /// <summary>
        /// Accepts an argument string and returns a string array of cells.
        /// </summary>
        /// <remarks>
        ///    Converts arguments in these forms to a string array of individual cells.<para/>
        ///        A1,A2,B4,C1,...,D8<para/>
        ///        A1:A5<para/>
        ///        A1:C5<para/>
        /// </remarks>
        /// <param name="args">String containing a cell range.</param>
        /// <returns>String array of cells.</returns>
        public string[] GetCellsFromArgs(string args)
        {
            RemoveQuotesOnSingleRange(ref args);
            MarkColonsInQuotes(ref args);

            int row1;
            int col1;

            int i = args.IndexOf(':');
            if (i == -1)
            {
                args = args.Replace(MarkerChar, ':');
                i = args.IndexOf(ParseArgumentSeparator);
                if (i == -1)
                {
                    int l, initial = 0;
                    for (l = 0; l < args.Length; l++)
                    {
                        if (args[l].Equals(TIC))
                        {
                            l++;
                            initial = 0;
                            while (l < args.Length && args[l] != TIC[0])
                            {
                                l++;
                            }
                        }
                        if (l < args.Length)
                        {
                            if (!Char.IsUpper(args, l))
                                continue;
                            else
                                initial = l;
                        }
                    }
                    string subarg = string.Empty;
                    for (int final = args.Length; final >= 0; final--)
                    {
                        subarg = args.Substring(initial, final - initial);
                        if (!this.IsCellReference(subarg))
                            continue;
                        break;
                    }
                    //row1 = RowIndex(args); ////maybe throw exception
                    //col1 = ColIndex(args); ////maybe throw exception
                    row1 = RowIndex(subarg);
                    col1 = ColIndex(subarg);
                    return new string[] { args };
                }
                else
                {
                    return SplitArguments(args, ParseArgumentSeparator);
                }
            }

            string sheet = string.Empty;
            int j = args.IndexOf(sheetToken);
            if (j > -1)
            {
                int j1 = args.IndexOf(sheetToken, j + 1);
                if (j1 > -1)
                {
                    sheet = args.Substring(j, j1 + 1);
                    args = args.Replace(sheet, string.Empty);
                    i = args.IndexOf(':');
                }
            }

            row1 = RowIndex(args.Substring(0, i));
            col1 = ColIndex(args.Substring(0, i));

            string subargs = args.Substring(i + 1, args.Length - i - 1);
            string sub = subargs;
            int first = 0;
            for (first = 0; first < subargs.Length; first++)
            {
                if (!Char.IsUpper(subargs, first))
                    continue;
                break;
            }
            for (int last = subargs.Length; last >= 0; last--)
            {
                sub = subargs.Substring(first, last - first);
                if (!this.IsCellReference(sub))
                    continue;
                break;
            }
            int row2 = RowIndex(sub);
            int col2 = ColIndex(sub);
            //int row2 = RowIndex(args.Substring(i + 1, args.Length - i - 1));
            //int col2 = ColIndex(args.Substring(i + 1, args.Length - i - 1));

            if (row1 > row2)
            {
                i = row2;
                row2 = row1;
                row1 = i;
            }

            if (col1 > col2)
            {
                i = col2;
                col2 = col1;
                col1 = i;
            }

            int numCells = (row2 - row1 + 1) * (col2 - col1 + 1);
            string[] cells = new String[numCells];
            int k = 0;
            for (i = row1; i <= row2; ++i)
            {
                for (j = col1; j <= col2; ++j)
                {
                    try
                    {
                        cells[k++] = sheet + GridRangeInfo.GetAlphaLabel(j) + i.ToString();
                    }
                    catch (Exception ex)
                    {
                        TraceUtil.TraceExceptionCatched(ex);
                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                        {
                            throw ex;
                        }

                        continue;
                    }
                }
            }

            return cells;
        }

        /// <summary>
        /// A method to split the arguments using argument seperator.
        /// </summary>
        /// <param name="args">arguments to split.</param>
        /// <param name="argSeperator">argument seperator used to split.</param>
        /// <returns>A string array.</returns>
        private string[] SplitArguments(string args, char argSeperator)
        {
            ArrayList splitArg = new ArrayList();
            int start = 0;
            int ticCount = 0;
            for (int i = 0; i < args.Length; i++)
            {
                if (args[i].Equals('"'))
                {
                    if (ticCount == 0)
                        ticCount++;
                    else
                        ticCount = 0;
                }
                if ((args[i].Equals(argSeperator) && !ticCount.Equals(1)))
                {
                    splitArg.Add(args.Substring(start, i - start));
                    start = i + 1;
                }
                if (i.Equals(args.Length - 1))
                {
                    splitArg.Add(args.Substring(start, i - start + 1));
                }
            }
            return (string[])splitArg.ToArray(typeof(string));
        }

        private void RemoveQuotesOnSingleRange(ref string args)
        {
            if (args.Length > 2
                && args[0] == TIC[0]
                && args[args.Length - 1] == TIC[0]
                && args.IndexOf(":") > 0
                && args.IndexOf(ParseArgumentSeparator) == -1)
            {
                args = args.Substring(1, args.Length - 2);
            }
        }

        /// <summary>
        /// Conditionally gets either the formula value or the cell 
        /// value depending upon whether the requested cell is a FormulaCell.
        /// </summary>
        /// <param name="row">Row index of the requested cell.</param>
        /// <param name="col">Column index of the requested cell.</param>
        /// <returns>String holding either the cell value or the computed formula value.</returns>
        public string GetValueFromGrid(int row, int col)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(row, col);
            }
#else
            ;
#endif
            GridStyleInfo style = grid[row, col];
            return GetValueFromGrid(style, row, col);
        }

        /// <summary>
        /// Conditionally gets either the formula value or the cell 
        /// value depending upon whether the requested cell is a FormulaCell.
        /// </summary>
        /// <param name="cell1">The alphanumeric cell label, like A1, or EE14.</param>
        /// <returns>String holding either the cell value or the computed formula value.</returns>
        public string GetValueFromGrid(string cell1)
        {
            if (cell1 == TRUEVALUESTR || cell1 == FALSEVALUESTR)
            {
                return cell1;
            }

            int i = cell1.LastIndexOf(sheetToken);
            int row, col;
            GridModel grd = grid;
            bool isDigit = CanGetIndex(cell1);
            if (i > -1)
            {
                GridSheetFamilyItem family = GetSheetFamilyItem(this.grid);

                grid = (GridModel)family.TokenToGridModel[cell1.Substring(0, i + 1)];
                row = RowIndex(cell1);
                col = ColIndex(cell1);
            }
            else
            {
                if (!isDigit)
                {
                    return cell1;
                }

                row = RowIndex(cell1);
                col = ColIndex(cell1);
            }

            string saveCell = this.cell;
            this.cell = cell1;
            GridStyleInfo style = grid[row, col];
            string s = GetValueFromGrid(style, row, col);
            this.grid = grd;
            this.cell = saveCell;
            return s;
        }
        private bool CanGetIndex(string s)
        {
            int i = 0;
            if (i < s.Length && s[i] == sheetToken)
            {
                i++;
                while (i < s.Length && s[i] != sheetToken)
                {
                    i++;
                }

                i++;
            }

            while (i < s.Length && char.IsLetter(s[i]))
            {
                i++;
            }

            if (i < s.Length)
            {
                i = int.Parse(s.Substring(i));
                if (i <= this.grid.RowCount)
                    return true;
                else
                    return false;
            }
            return false;
        }
        private string GetValueFromGrid(GridStyleInfo style, int row, int col)
        {
            string styleText = style.Text;
            GetFormulaText(ref styleText);

            if (style.CellModel is GridFormulaCellModel
                && styleText.Length > 0 && (this.formulaChar == (char)0 || (this.formulaChar != (char)0 &&
                this.formulaChar == styleText[0])))
            {
                if (style.FormulaTag == null)
                {
                    style.FormulaTag = new GridFormulaTag(Parse(styleText), null, row, col);
                }

                GridFormulaTag tag = style.FormulaTag;
                if (tag != null)
                {
                    if ((tag.Text == null || tag.Text.Length == 0) && tag.Formula.Length > 0)
                    {
                        string save = this.cell;
                        this.cell = GridRangeInfo.GetAlphaLabel(col) + row.ToString();

                        tag.Text = ComputedValue(tag.Formula);

                        if (forceSaveCellInfo)
                        {
                            this.grid.RaiseSaveCellInfo(new GridSaveCellInfoEventArgs(row, col, style, StyleModifyType.Override));
                        }

                        this.cell = save;
                    }

                    return tag.Text;
                }
                else
                {
                    return string.Empty;
                }
            }
            else
            {
                if (styleText == this.TRUEVALUESTR || styleText == this.FALSEVALUESTR)
                {
                    return styleText;
                }

                double d;
                if (styleText.Length > 0)
                {
                    if (stringOK || double.TryParse(styleText, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                    {
                        return styleText;
                    }
                }

                return "0";
            }
        }

        /// <summary>
        /// Computes the value contained in the argument.
        /// </summary>
        /// <remarks>
        /// This method takes the argument and checks whether it is a 
        /// parsed formula, raw number, or cell reference like A21.
        /// The return value is a string that holds the computed value of 
        /// the passed-in argument.
        /// </remarks>
        /// <param name="arg">A parsed formula, raw number, or cell reference.</param>
        /// <returns>A string with the computed number in it.</returns>
        public string GetValueFromArg(string arg)
        {
            return GetValueFromArg(arg, false);
        }
        private bool getValueFromArgPreserveLeadingZeros = false;

        /// <summary>
        /// Gets or sets whether leading zeros are preserved in a call to <see cref="GetValueFromArg"/>.
        /// </summary>
        /// <remarks>
        /// If the value of arg is "0123" or a cell reference like A1 where A1 holds 0123, then the default behavior
        /// is for GetValueFromArg(arg) to return 123, stripping away any leading zeros. If you want calls to
        /// GetValueFromArg to preserve the leading zeros, then set GetValueFromArgPreserveLeadingZeros to true.
        /// </remarks>
        public bool GetValueFromArgPreserveLeadingZeros
        {
            get { return this.getValueFromArgPreserveLeadingZeros; }
            set { this.getValueFromArgPreserveLeadingZeros = value; }
        }

        private bool stringOK = false;

        /// <summary>
        /// Computes the value contained in the argument.
        /// </summary>
        /// <remarks>
        /// This method takes the argument and checks whether it is a 
        /// parsed formula, raw number, or cell reference like A21.
        /// The return value is a string that holds the computed value of 
        /// the passed-in argument.
        /// </remarks>
        /// <param name="arg">A parsed formula, raw number, or cell reference.</param>
        /// <param name="returnString">If False, this method returns the empty 
        /// string if the arg is a string. If True, this method will return the string value.</param>
        /// <returns>A string with the arg value in it.</returns>
        public string GetValueFromArg(string arg, bool returnString)
        {
            double d;
            if (GridUtil.IsEmpty(arg))
            {
                return string.Empty;
            }
            else if (arg[0] == TIC[0])
            {
                ////string
                return arg;
            }
            else if (arg[0] == BMARKER)
            {
                ////parsed formula
                arg = arg.Replace('{', '(');
                arg = arg.Replace('}', ')');
                return this.ComputedValue(arg);
            }
            else if (arg == this.TRUEVALUESTR || arg == 'n' + this.TRUEVALUESTR)
            {
                return "1";
            }
            else if (arg == this.FALSEVALUESTR || arg == 'n' + this.FALSEVALUESTR)
            {
                return "0";
            }
            else if (arg[0] == this.TIC[0])
            {
                return arg;
            }
            else
            {
                arg = arg.Replace('u', '-');
                if (!char.IsUpper(arg[0])
                    && (char.IsDigit(arg[0]) || arg[0] == ParseDecimalSeparator || arg[0] == '-' || arg[0] == 'n'))
                {
                    if (arg[0] == 'n')
                    {
                        arg = arg.Substring(1);
                    }

                    if (double.TryParse(arg, NumberStyles.Number | NumberStyles.AllowExponent, null, out d))
                    {
                        return this.GetValueFromArgPreserveLeadingZeros ? arg : d.ToString();
                    }
                }
            }

            ////not a number
            if ((arg.IndexOfAny(new char[] { '+', '-', '/', '*', ')', ')', '{' }) == -1
                && char.IsUpper(arg[0])) || arg[0] == sheetToken)
            {
                ////check for circular reference
                if (arg == cell) 
                {
                    Hashtable ht = (Hashtable)DependentCells[arg];
                    if (ht != null && ht.ContainsKey(arg))
                    {
                        ht.Remove(arg);
                    }

                    ClearFormulaDependentCells(cell);
                    throw new ArgumentException(FormulaErrorStrings[circular_reference_] + arg);
                }

                this.stringOK = true;
                string s1 = this.GetValueFromGrid(arg);
                this.stringOK = false;

                if (!GetValueFromArgPreserveLeadingZeros && s1.Length > 0
                        && double.TryParse(s1.Replace(this.TIC, string.Empty), NumberStyles.Any, null, out d))
                {
                    s1 = d.ToString();
                }

                UpdateDependencies(arg);
                return s1;
            }

            ////Must be a formula, so try to parse it and compute.

            arg = arg.Replace('{', '(');
            arg = arg.Replace('}', ')');
            arg = this.Parse(arg);

            if (arg.EndsWith("%") && double.TryParse(arg.Substring(0, arg.Length - 1), out d))
            {
                arg = (d / 100).ToString();
            }

            return this.ComputedValue(arg);
        }

        internal string SheetToken(string s)
        {
            int i = 0;
            string s1 = string.Empty;
            if (i < s.Length && s[i] == sheetToken)
            {
                i++;
                while (i < s.Length && s[i] != sheetToken)
                {
                    i++;
                }

                s1 = s.Substring(0, i + 1);
            }

            if (i < s.Length)
            {
                return s1;
            }

            throw new ArgumentException(FormulaErrorStrings[bad_index]);
        }

        /// <summary>
        /// Returns the row index from a cell reference.
        /// </summary>
        /// <param name="s">String holding a cell reference such as C21 or AB11.</param>
        /// <returns>An integer with the corresponding row number.</returns>
        public int RowIndex(string s)
        {
            if (this.CurrentRowNotationEnabled && s.Length == 0)
            {
                return 0;
            }
            else
            {
                int i = 0;
                if (i < s.Length && s[i] == sheetToken)
                {
                    i++;
                    while (i < s.Length && s[i] != sheetToken)
                    {
                        i++;
                    }

                    i++;
                }

                while (i < s.Length && char.IsLetter(s[i]))
                {
                    i++;
                }
                if (i < s.Length && s != TRUEVALUESTR && s != FALSEVALUESTR)
                {
                    i = int.Parse(s.Substring(i));
                    if (i == 0 && this.CurrentRowNotationEnabled && this.cell.Length > 1)
                    {
                        i = RowIndex(this.cell);
                    }

                }
                    return i;
            }

            throw new ArgumentException(FormulaErrorStrings[bad_index]);
        }

        /// <summary>
        /// Returns a column index from a cell reference.
        /// </summary>
        /// <param name="s">String holding a cell reference such as C21 or AB11.</param>
        /// <returns>An integer with the corresponding column number.</returns>
        public int ColIndex(string s)
        {
            int i = 0;
            int k = 0;
            s = s.ToUpper(CultureInfo.InvariantCulture);

            if (i < s.Length && s[i] == sheetToken)
            {
                i++;
                while (i < s.Length && s[i] != sheetToken)
                {
                    i++;
                }

                i++;
            }

            while (i < s.Length && char.IsLetter(s[i]))
            {
                k = ((int)k * 26) + (int)s[i] - (int)'A' + 1;
                i++;
            }

            if (k == 0)
            {
                k = -1;
            }

            return k;
        }

        #endregion

        #region Computations

        int FindLastqNotInBrackets(string s)
        {
            int found = -1;
            bool lastBracket = false;
            int i = s.Length - 1;
            while (i > -1)
            {
                if (s[i] == 'q' && lastBracket)
                {
                    found = i;
                    break;
                }

                if (s[i] == LEFTBRACKET)
                {
                    lastBracket = true;
                }
                else if (s[i] == RIGHTBRACKET)
                {
                    lastBracket = false;
                }

                i--;
            }

            return found;
        }

        int computeFunctionLevel = 0;
        internal string ComputeInteriorFunctions(string formula, ArrayList circCheckList)
        {
            try
            {
                if (GridUtil.IsEmpty(formula))
                {
                    return formula;
                }

                computeFunctionLevel++;

                ////int q = formula.LastIndexOf('q');
                int q = FindLastqNotInBrackets(formula);

                while (q > 0)
                {
                    int last = formula.Substring(q).IndexOf(RIGHTBRACKET);
                    if (last == -1)
                    {
                        return FormulaErrorStrings[bad_formula];
                    }

                    string s = formula.Substring(q, last + 1);
                    s = ComputedValue(s, circCheckList);
                    if (!GridUtil.IsEmpty(s) && s[0] == '-')
                    {
                        s = "nu" + s.Substring(1);
                    }
                    else if (s.Length > 0 &&
                        (s[0] == TIC[0] || s[0] == BMARKER || s[0] == '#'))
                    {
                        ////pass on the string...
                    }
                    else if (s.StartsWith(TRUEVALUESTR) || s.StartsWith(FALSEVALUESTR))
                    {
                        ////pass on the bool...
                    }
                    else
                    {
                        double d = 0;
                        if (double.TryParse(s, out d))
                        {
                            ////need to strip out any group separators - possible with FIXED function
                            s = d.ToString();
                            s = 'n' + s;
                        }
                        else
                        {
                            s = TIC + s + TIC;
                        }
                    }

                    formula = formula.Substring(0, q) + s + formula.Substring(q + last + 1);
                    ////q = formula.LastIndexOf('q');
                    q = FindLastqNotInBrackets(formula);
                }
            }
            catch (Exception ex)
            {
                TraceUtil.TraceExceptionCatched(ex);
                if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                {
                    throw ex;
                }

                return ex.Message;
            }
            finally
            {
                computeFunctionLevel--;
            }

            return formula;
        }

        private bool useNoAmpersandQuotes = false;

        /// <summary>
        /// Gets or sets a value indicating whether strings concatenated using the ampersand operator should 
        /// be returned inside double quote marks.
        /// </summary>
        public bool UseNoAmpersandQuotes
        {
            get { return useNoAmpersandQuotes; }
            set { useNoAmpersandQuotes = value; }
        }

        ////max calculation stack depth
        private int maxStackDepth = 100;

        /// <summary>
        /// Gets or sets depth of the engine's calculation stack.
        /// </summary>
        /// <remarks>
        /// It is unlikely that you will need to adjust this value, as
        /// its default value of 100 is quite large.
        /// Any time a formula is to be computed, a Stack object is created
        /// with this number of elements to hold temporary calculations
        /// as the formula is being computed. 
        /// For example, this formula: 1+1+1+...+1 requires a stack 
        /// depth of 2 as there are only 2 temporary values needed as the 
        /// formula is evaluated. However, this formula: 1+(1+(1+(1+1)))
        /// requires a depth of 5, as the five 1's are pushed onto the 
        /// stack before the first addition (the right-most one) is
        /// performed.
        /// </remarks>
        public int MaximumStackDepth
        {
            get { return maxStackDepth; }
            set { maxStackDepth = value; }
        }

        private int computedValueLevel = 0;

        [ThreadStaticAttribute]
        private static GridFormulaEngine functionEngineContext = null;

        /// <summary>
        /// Gets the formula engine that holds the proper state information for LibraryFunction call.
        /// </summary>
        /// <remarks>
        /// If you are adding your own custom library functions and need to access GridFormulaEngine members like
        /// GetCellsFromArgs or GetValueFromArg from your code, then you should use this static property to retrieve the proper
        /// GridFormulaEngine object. This only matters if you are using multiple grids that you have registered using
        /// RegisterGridAsSheet, and are adding your own custom formulas. Note that GridFormulaEngine.FunctionEngineContext
        /// is only defined within the scope of a library function call, and will be null at all other times.
        /// </remarks>
        /// <example> Use GridFormulaEngine.FunctionEngineContext to retrieve the engine 
        /// when writing custom functions.
        /// <code lang="C#">
        /// public string ComputeSumPosNums(string args)
        /// {
        ///     GridFormulaEngine engine = GridFormulaEngine.FunctionEngineContext;
        ///     string sum = string.Empty;
        ///     foreach(string r in args.Split(new char[]{','}))
        ///     {
        ///         if(r.IndexOf(':') > -1) //is a cellrange
        ///         {
        ///             foreach(string s in engine.GetCellsFromArgs(r))
        ///             {
        ///                 s1 = engine.GetValueFromArg(s).Replace("'",string.Empty);
        ///                 //... do some calculations to compute sum
        ///             }
        ///         }
        ///     }    
        ///     return sum.ToString();
        /// }
        /// </code>
        /// <code lang="VB">
        /// Public Function ComputeSumPosNums(args As String) As String
        ///    Dim engine As GridFormulaEngine = GridFormulaEngine.FunctionEngineContext
        ///    Dim sum As String = string.Empty
        ///    Dim r As String
        ///    For Each r In  args.Split(New Char() {","c})
        ///            If r.IndexOf(":"c) > - 1 Then 'is a cellrange
        ///                Dim s As String
        ///                For Each s In  engine.GetCellsFromArgs(r)
        ///                    s1 = engine.GetValueFromArg(s).Replace("'", string.Empty)
        ///                    '... do some calculations to compute sum
        ///                Next s 
        ///            End If
        ///    Next r
        ///    Return sum.ToString()
        /// End Function 'ComputeSumPosNums
        /// </code>
        /// </example>
        public static GridFormulaEngine FunctionEngineContext
        {
            get { return functionEngineContext; }
        }

        private string activeFunctionName;

        /// <summary>
        /// Gets the function name of the active function call. Empty if not in a call.
        /// </summary>
        public string ActiveFunctionName
        {
            get { return activeFunctionName; }
        }

        /// <summary>
        /// Computes the value of a parsed formula.
        /// </summary>
        /// <param name="formula">The formula to be computed.</param>
        /// <returns>A string holding the computed value.</returns>
        /// <remarks> 
        ///  The string passed into ComputedValue must have been parsed using the Parse
        ///  method. Before calling the method, you should set FormulaContextCell
        ///  to properly reflect which cell owns this formula.
        /// </remarks>
        public string ComputedValue(string formula)
        {
            return ComputedValue(formula, null);
        }

        internal void GetCellsInArgs(ArrayList circCheckList, string args)
        {
            foreach (string r in args.Split(new char[] { ParseArgumentSeparator }))
            {
                if (r.IndexOf(':') > -1)
                {
                    ////is a cellrange
                    foreach (string s in GetCellsFromArgs(r))
                    {
                        circCheckList.Add(s);
                    }
                }
                else
                {
                    if (r.Length > 2 && r[0] == BMARKER && r[r.Length - 1] == BMARKER)
                    {
                        this.ComputedValue(r, circCheckList);
                    }
                    else if (r.Length > 2 && char.IsUpper(r[0]) && char.IsDigit(r[r.Length - 1]))
                    {
                        circCheckList.Add(r);
                    }
                }
            }
        }

        internal string ComputedValue(string formula, ArrayList circCheckList)
        {
#if DEBUG
            if (Switches.FormulaCell.TraceVerbose)
            {
                TraceUtil.TraceCurrentMethodInfo(formula, computedValueLevel, this);
            }
#else
            ;
#endif
            if (GridUtil.IsEmpty(formula))
            {
                return formula;
            }

            try
            {
                computedValueLevel++;

                if (computedValueLevel > maximumRecursiveCalls)
                {
                    computedValueLevel = 0;
                    if (circCheckList != null)
                    {
                        circCheckList.Add(this.cell);
                        return "0";
                    }

                    throw new ArgumentException(FormulaErrorStrings[too_complex]);
                }

                Stack _stack = new Stack(maxStackDepth);

                int i = 0;
                _stack.Clear();
                string sheet = string.Empty;

                if (AllowShortCircuitIFs)
                {
                    ////code replaces all IF functions with just the proper alternative 
                    while (i < formula.Length && (i = formula.IndexOf(IFMarker, i)) > -1)
                    {
                        int loc = MatchingRightBracket(formula.Substring(i));
                        if (loc > -1)
                        {
                            ////Console.WriteLine("  " + formula);
                            formula = formula.Substring(0, i) + HandleEmbeddedIf(formula.Substring(i)) + formula.Substring(i + loc + 1);
                            //// Console.WriteLine("      " + formula);
                        }

                        i++;
                    }
                }

                i = 0;
                if (formula.Equals("#NAME?"))
                {
                    _stack.Push("#NAME?");
                    i = formula.Length;
                }

                while (i < formula.Length)
                {
                    if (formula[i] == BMARKER)
                    {
                        i = i + 1;
                        continue;
                    }

                    bool uFound = formula[i] == 'u';
                    if (uFound)
                    {
                        i++;
                        if (i >= formula.Length)
                        {
                            continue; ////error case....
                        }
                    }

                    if (formula[i] == '%' && _stack.Count > 0)
                    {
                        object o = _stack.Peek();
                        double d;
                        if (double.TryParse(o.ToString(), out d))
                        {
                            _stack.Pop(); ////remove it
                            _stack.Push(d / 100); ////push it back
                        }

                        i = i + 1;
                        continue;
                    }

                    if (formula[i] == sheetToken)
                    {
                        sheet = formula[i].ToString();
                        i++;
                        while (i < formula.Length && formula[i] != sheetToken)
                        {
                            sheet += formula[i];
                            i++;
                        }

                        if (i < formula.Length)
                        {
                            sheet += formula[i];
                            i++;
                        }
                        else
                        {
                            continue; ////skip out as something is wrong
                        }
                    }

                    if (formula.Substring(i).StartsWith(TRUEVALUESTR))
                    {
                        _stack.Push(TRUEVALUESTR);
                        i += TRUEVALUESTR.Length;
                    }
                    else if (formula.Substring(i).StartsWith(FALSEVALUESTR))
                    {
                        _stack.Push(FALSEVALUESTR);
                        i += FALSEVALUESTR.Length;
                    }
                    else if (formula[i] == TIC[0])
                    {
                        string s = formula[i].ToString();
                        i++;
                        while (i < formula.Length && formula[i] != TIC[0])
                        {
                            s = s + formula[i];
                            i = i + 1;
                        }

                        _stack.Push(s + TIC);
                        i += 1;
                    }
                    else if (char.IsUpper(formula[i]))
                    {
                        ////cell loc
                        string s = string.Empty;
                        while (i < formula.Length && char.IsUpper(formula[i]))
                        {
                            s = s + formula[i];
                            i = i + 1;
                        }

                        while (i < formula.Length && char.IsDigit(formula[i]))
                        {
                            s = s + formula[i];
                            i = i + 1;
                        }

                        s = sheet + s;
                        sheet = string.Empty;
                        if (s == cell)
                        {
                            computedValueLevel = 0;
                            Hashtable ht = (Hashtable)DependentCells[s];
                            if (ht != null && ht.ContainsKey(s))
                            {
                                ht.Remove(s);
                            }

                            ClearFormulaDependentCells(cell);

                            if (circCheckList != null)
                            {
                                circCheckList.Add(this.cell);
                                return "0";
                            }

                            _stack.Push("#REF!");
                            break;
                            //throw new ArgumentException(FormulaErrorStrings[circular_reference_] + s);
                        }

                        if (circCheckList != null)
                        {
                            circCheckList.Add(s);
                        }

                        UpdateDependencies(s);

                        stringOK = true;
                        string o = GetValueFromGrid(s);
                        stringOK = false;
                        double d;
                        if (o != null && o.EndsWith("%") && double.TryParse(o.Substring(0, o.Length - 1), out d))
                        {
                            o = (d / 100).ToString();
                        }

                        if (uFound && o != null && double.TryParse(o, out d))
                        {
                            o = (-d).ToString();
                        }

                        uFound = false;
                        _stack.Push(o);
                    }
                    else if (formula[i] == 'q')
                    {
                        ////library
                        formula = ComputeInteriorFunctions(formula, circCheckList);

                        int ii = formula.Substring(i + 1).IndexOf(LEFTBRACKET);
                        if (ii > 0)
                        {
                            int bracketCount = 0;
                            bool bracketFound = false;
                            int start = ii + i + 2;
                            int k = start;
                            while (k < formula.Length && (formula[k] != RIGHTBRACKET || bracketCount > 0))
                            {
                                if (formula[k] == LEFTBRACKET)
                                {
                                    bracketCount++;
                                    bracketFound = true;
                                }
                                else if (formula[k] == LEFTBRACKET)
                                {
                                    bracketCount--;
                                }

                                k++;
                            }

                            if (bracketFound)
                            {
                                string s = formula.Substring(start, k - start - 2);
                                string s1 = string.Empty;
                                foreach (string t in s.Split(new Char[] { ParseArgumentSeparator }))
                                {
                                    if (s1.Length > 0)
                                    {
                                        s1 += ",";
                                    }

                                    ////int j = t.LastIndexOf('q');
                                    int j = FindLastqNotInBrackets(t);
                                    if (j > 0)
                                    {
                                        s1 += t.Substring(0, j) + ComputedValue(t.Substring(j), circCheckList);
                                    }
                                    else
                                    {
                                        s1 += ComputedValue(t, circCheckList);
                                    }
                                }

                                formula = formula.Substring(0, start) + s1 + formula.Substring(k - 2);
                            }

                            string name = formula.Substring(i + 1, ii);
                            if (this.LibraryFunctions[name] != null)
                            {
                                int j = formula.Substring(i + ii + 1).IndexOf(RIGHTBRACKET);
                                string args = formula.Substring(i + ii + 2, j - 1);
                                if (circCheckList != null)
                                {
                                    GetCellsInArgs(circCheckList, args);
                                    _stack.Push("0");
                                }
                                else
                                {
                                    try
                                    {
                                        functionEngineContext = this;
                                        LibraryFunction func = (LibraryFunction)this.LibraryFunctions[name];
                                        this.activeFunctionName = name;
                                        String result = func(args);
                                        if (UseNoAmpersandQuotes && result.Length > 1 && result.StartsWith(TIC) && result.EndsWith(TIC))
                                        {
                                            result = result.Substring(1, result.Length - 2);
                                        }

                                        double d = 0;
                                        if (uFound && result != null && double.TryParse(result, out d))
                                        {
                                            result = (-d).ToString();
                                        }

                                        uFound = false;
                                        _stack.Push(result);
                                        this.activeFunctionName = string.Empty;
                                        functionEngineContext = null;
                                    }
                                    catch (Exception ex)
                                    {
                                        TraceUtil.TraceExceptionCatched(ex);
                                        if (circCheckList != null)
                                        {
                                            circCheckList.Add(this.cell);
                                            return "0";
                                        }

                                        if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                                        {
                                            throw ex;
                                        }

                                        return ex.Message;
                                    }
                                }

                                i += j + ii + 2;
                            }
                            else
                            {
                                if (circCheckList != null)
                                {
                                    circCheckList.Add(this.cell);
                                    return "0";
                                }

                                return FormulaErrorStrings[missing_formula];
                            }
                        }
                        else if (formula[0] == BMARKER)
                        {
                            ////Restart the processing with the formula without library finctions.
                            i = 0;
                            continue;
                        }
                        else
                        {
                            if (circCheckList != null)
                            {
                                circCheckList.Add(this.cell);
                                return "0";
                            }

                            return FormulaErrorStrings[improper_formula];
                        }
                    }
                    else if (char.IsDigit(formula[i]) || formula[i] == 'u')
                    {
                        string s = string.Empty;
                        if (formula[i] == 'u')
                        {
                            s = "-";
                            i++;
                        }
                        else if (uFound)
                        {
                            s = "-";
                        }

                        uFound = false;
                        while (i < formula.Length && (char.IsDigit(formula[i]) || formula[i] == ParseDecimalSeparator))
                        {
                            s = s + formula[i];
                            i = i + 1;
                        }

                        _stack.Push(s);
                    }
                    else
                    {
                        switch (formula[i])
                        {
                            case '#':
                                {
                                    i = i + 4;
                                    _stack.Push("#N/A");

                                    break;
                                }

                            case 'n':
                                {
                                    i = i + 1;

                                    string s = string.Empty;
                                    if (formula.Substring(i).StartsWith("Infinity"))
                                    {
                                        s = "Infinity";
                                        i += s.Length;
                                    }
                                    else if (formula.Substring(i).StartsWith(TRUEVALUESTR))
                                    {
                                        s = TRUEVALUESTR;
                                        i += s.Length;
                                    }
                                    else if (formula.Substring(i).StartsWith(FALSEVALUESTR))
                                    {
                                        s = FALSEVALUESTR;
                                        i += s.Length;
                                    }
                                    else if (i < formula.Length - 3 && formula.Substring(i, 3) == "NaN")
                                    {
                                        i += 3;
                                        s = "0";
                                    }
                                    else
                                    {
                                        if (formula[i] == 'u' || uFound)
                                        {
                                            s = "-";
                                            if (!uFound)
                                            {
                                                i = i + 1;
                                            }
                                            else
                                            {
                                                uFound = false;
                                            }
                                        }

                                        while (i < formula.Length && (char.IsDigit(formula[i]) || formula[i] == ParseDecimalSeparator))
                                        {
                                            s = s + formula[i];
                                            i = i + 1;
                                        }

                                        ////Check for exp values.
                                        if (i < formula.Length - 3 && formula[i] == 'E' && (formula[i + 1] == '+' || formula[i + 1] == '-'))
                                        {
                                            s = s + formula[i]; ////E
                                            i = i + 1;
                                            s = s + formula[i]; ////sign
                                            i = i + 1;
                                            s = s + formula[i]; ////first digit
                                            i = i + 1;
                                            while (i < formula.Length && (char.IsDigit(formula[i]) || formula[i] == ParseDecimalSeparator))
                                            {
                                                s = s + formula[i];
                                                i = i + 1;
                                            }
                                        }

                                        if (i < formula.Length && formula[i] == '%')
                                        {
                                            i = i + 1;
                                            if (s.Length == 0)
                                            {
                                                if (_stack.Count > 0)
                                                {
                                                    object o = _stack.Peek();
                                                    double d;
                                                    if (double.TryParse(o.ToString(), out d))
                                                    {
                                                        _stack.Pop(); ////remove it
                                                        s = (d / 100).ToString(); ////push it back
                                                    }
                                                }
                                            }
                                            else
                                            {
                                                s = (double.Parse(s) / 100).ToString();
                                            }
                                        }
                                    }

                                    _stack.Push(s);
                                }

                                break;
                            case TOKEN_add:
                                {
                                    double d, d1;
                                    DateTime result;
                                    if (this.cell != null && double.TryParse(_stack.ToArray()[1].ToString(), NumberStyles.Number | NumberStyles.Currency, this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].CultureInfo, out d1) && double.TryParse(_stack.Peek().ToString(), NumberStyles.Number | NumberStyles.Currency, this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].CultureInfo, out d))
                                    {
                                        _stack.Pop();
                                        _stack.Pop();
                                        _stack.Push((d1 + d).ToString());
                                    }
                                    else if (this.cell != null && DateTime.TryParse(_stack.Peek().ToString(), out result) && DateTime.TryParse(_stack.ToArray()[1].ToString(), out result))
                                    {
                                        DateTime date = DateTime.Parse(_stack.Pop().ToString());
                                        DateTime date1 = DateTime.Parse(_stack.Pop().ToString());
                                        Double TotalDays = date1.ToOADate() + date.ToOADate();
                                        if (this.grid.CurrentCellRenderer != null)
                                        {
                                            GridStyleInfo style = this.grid.CurrentCellRenderer.CurrentStyle;
                                            if (!style.Format.Equals(string.Empty))
                                            {
                                                if (TotalDays > 0)
                                                {
                                                    DateTime dt = DateTime.FromOADate(TotalDays + 1);
                                                    string DateString = string.Format("{0:" + style.Format + "}", dt);
                                                    _stack.Push(DateString);
                                                }
                                                else
                                                {
                                                    _stack.Push("#N/A");
                                                }
                                            }
                                            else
                                            {
                                                _stack.Push(TotalDays.ToString());
                                            }
                                        }
                                        else
                                        {
                                            _stack.Push(TotalDays.ToString());
                                        }
                                    }
                                    else
                                    {
                                        d = Pop(_stack);
                                        d1 = Pop(_stack);
                                        _stack.Push((d1 + d).ToString());
                                    }
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_subtract:
                                {
                                    double d, d1;
                                    DateTime result;
                                    if (this.cell != null && double.TryParse(_stack.ToArray()[1].ToString(), NumberStyles.Number | NumberStyles.Currency, this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].CultureInfo, out d1) && double.TryParse(_stack.Peek().ToString(), NumberStyles.Number | NumberStyles.Currency, this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].CultureInfo, out d))
                                    {
                                        _stack.Pop();
                                        _stack.Pop();
                                        _stack.Push((d1 - d).ToString());
                                    }
                                    else if (this.cell != null && DateTime.TryParse(_stack.Peek().ToString(), out result)
                                            && DateTime.TryParse(_stack.ToArray()[1].ToString(), out result))
                                    {
                                        DateTime date = DateTime.Parse(_stack.Pop().ToString());
                                        DateTime date1 = DateTime.Parse(_stack.Pop().ToString());
                                        Double TotalDays = (date1 - date).TotalDays;
                                        if (this.grid.CurrentCellRenderer != null)
                                        {
                                            GridStyleInfo style = this.grid.CurrentCellRenderer.CurrentStyle;
                                            if (!style.Format.Equals(string.Empty))
                                            {
                                                if (TotalDays > 0)
                                                {
                                                    DateTime dt = DateTime.FromOADate(TotalDays + 1);
                                                    string DateString = string.Format("{0:" + style.Format + "}", dt);
                                                    _stack.Push(DateString);
                                                }
                                                else
                                                {
                                                    _stack.Push("#N/A");
                                                }
                                            }
                                            else
                                            {
                                                _stack.Push(TotalDays.ToString());
                                            }
                                        }
                                        else
                                        {
                                            _stack.Push(TotalDays.ToString());
                                        }
                                    }
                                    else
                                    {
                                        d = Pop(_stack);
                                        d1 = Pop(_stack);
                                        _stack.Push((d1 - d).ToString());
                                    }
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_multiply:
                                {
                                    double d, d1;
                                    if (this.cell != null && double.TryParse(_stack.ToArray()[1].ToString(), NumberStyles.Number | NumberStyles.Currency, this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].CultureInfo, out d1) && double.TryParse(_stack.Peek().ToString(), NumberStyles.Number | NumberStyles.Currency, this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].CultureInfo, out d))
                                    {
                                        _stack.Pop();
                                        _stack.Pop();
                                    }
                                    else
                                    {
                                        d = Pop(_stack);
                                        d1 = Pop(_stack);
                                    }

                                    _stack.Push((d1 * d).ToString());
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_divide:
                                {
                                    double d, d1;
                                    if (this.cell != null && double.TryParse(_stack.ToArray()[1].ToString(), NumberStyles.Number | NumberStyles.Currency, this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].CultureInfo, out d1) && double.TryParse(_stack.Peek().ToString(), NumberStyles.Number | NumberStyles.Currency, this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].CultureInfo, out d))
                                    {
                                        _stack.Pop();
                                        _stack.Pop();
                                    }
                                    else
                                    {
                                        d = Pop(_stack);
                                        d1 = Pop(_stack);
                                    }

                                    _stack.Push((d1 / d).ToString());
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_EP:
                                {
                                    double d = Pop(_stack); ////double.Parse(g);
                                    double d1 = Pop(_stack); ////double.Parse(g1);

                                    _stack.Push((d1 * Math.Pow(10, d)).ToString());
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_EM:
                                {
                                    double d = Pop(_stack);
                                    double d1 = Pop(_stack);

                                    _stack.Push((d1 * Math.Pow(10, -d)).ToString());
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_less:
                                {
                                    string s1 = PopString(_stack);
                                    string s2 = PopString(_stack);
                                    double d, d1;
                                    string val;
                                    if (ParsePossibleEmptyStrings(s1, s2, out d, out d1))
                                    {
                                        val = (d1 < d) ? TRUEVALUESTR : FALSEVALUESTR;
                                    }
                                    else
                                    {
                                        val = (s2.ToUpper().Replace(TIC, string.Empty).CompareTo(s1.ToUpper().Replace(TIC, string.Empty)) < 0) ? TRUEVALUESTR : FALSEVALUESTR;
                                    }

                                    _stack.Push(val);
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_greater:
                                {
                                    string s1 = PopString(_stack);
                                    string s2 = PopString(_stack);
                                    double d, d1, d2;
                                    string val;
                                    if (ParsePossibleEmptyStrings(s1, s2, out d, out d1))
                                    {
                                            val = (d1 > d) ? TRUEVALUESTR : FALSEVALUESTR;
                                    }
                                    else
                                    {
                                        if (double.TryParse(s2, NumberStyles.Currency, this.grid.Model[this.RowIndex(this.cell), this.ColIndex(this.cell)].CultureInfo, out d2))
                                        {
                                            val = (d2.ToString().ToUpper().Replace(TIC, string.Empty).CompareTo(s1.ToUpper().Replace(TIC, string.Empty)) > 0) ? TRUEVALUESTR : FALSEVALUESTR;
                                        }
                                        else
                                        {
                                            val = (s2.ToUpper().Replace(TIC, string.Empty).CompareTo(s1.ToUpper().Replace(TIC, string.Empty)) > 0) ? TRUEVALUESTR : FALSEVALUESTR;
                                        }
                                    }
                                    if ((s2.Equals(string.Empty) || s2.Equals(" ")))
                                        val = TRUEVALUESTR;

                                    _stack.Push(val);
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_equal:
                                {
                                    string s1 = PopString(_stack);
                                    string s2 = PopString(_stack);
                                    double d, d1;
                                    string val;
                                    if (ParsePossibleEmptyStrings(s1, s2, out d, out d1))
                                    {
                                        val = (d1 == d) ? TRUEVALUESTR : FALSEVALUESTR;
                                    }
                                    else
                                    {
                                        if ((s1.Length > 0 && s1[0] == '#')
                                            || (s2.Length > 0 && s2[0] == '#'))
                                        {
                                            val = "#N/A";
                                        }
                                        else
                                        {
                                            val = (s1.ToUpper().Replace(TIC, string.Empty) == s2.ToUpper().Replace(TIC, string.Empty)) ? TRUEVALUESTR : FALSEVALUESTR;
                                        }
                                    }

                                    _stack.Push(val);
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_lesseq:
                                {
                                    string s1 = PopString(_stack);
                                    string s2 = PopString(_stack);
                                    double d, d1;
                                    DateTime date, date1;
                                    string val;
                                    if (ParsePossibleEmptyStrings(s1, s2, out d, out d1))
                                    {
                                        val = (d1 <= d) ? TRUEVALUESTR : FALSEVALUESTR;
                                    }
                                    else
                                    {
                                        if (DateTime.TryParse(s1.ToUpper().Replace(TIC, string.Empty), out date) && DateTime.TryParse(s2.ToUpper().Replace(TIC, string.Empty), out date1))
                                        {
                                            val = (DateTime.Compare(date1, date) <= 0) ? TRUEVALUESTR : FALSEVALUESTR;
                                        }
                                        else
                                        {
                                            val = (s2.ToUpper().Replace(TIC, string.Empty).CompareTo(s1.ToUpper().Replace(TIC, string.Empty)) <= 0) ? TRUEVALUESTR : FALSEVALUESTR;
                                        }
                                    }

                                    _stack.Push(val);
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_greatereq:
                                {
                                    string s1 = PopString(_stack);
                                    string s2 = PopString(_stack);
                                    double d, d1;
                                    DateTime date, date1;
                                    string val;
                                    if (ParsePossibleEmptyStrings(s1, s2, out d, out d1))
                                    {
                                        val = (d1 >= d) ? TRUEVALUESTR : FALSEVALUESTR;
                                    }
                                    else
                                    {
                                        if (DateTime.TryParse(s1.ToUpper().Replace(TIC, string.Empty), out date) && DateTime.TryParse(s2.ToUpper().Replace(TIC, string.Empty), out date1))
                                        {
                                            val = (DateTime.Compare(date1, date) >= 0) ? TRUEVALUESTR : FALSEVALUESTR;
                                        }
                                        else
                                        {
                                            val = (s2.ToUpper().Replace(TIC, string.Empty).CompareTo(s1.ToUpper().Replace(TIC, string.Empty)) >= 0) ? TRUEVALUESTR : FALSEVALUESTR;
                                        }
                                    }

                                    _stack.Push(val);
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_noequal:
                                {
                                    string s1 = PopString(_stack);
                                    string s2 = PopString(_stack);
                                    double d, d1;
                                    string val;
                                    if (ParsePossibleEmptyStrings(s1, s2, out d, out d1))
                                    {
                                        val = (d1 != d) ? TRUEVALUESTR : FALSEVALUESTR;
                                    }
                                    else
                                    {
                                        val = (s1.ToUpper().Replace(TIC, string.Empty) != s2.ToUpper().Replace(TIC, string.Empty)) ? TRUEVALUESTR : FALSEVALUESTR;
                                    }

                                    _stack.Push(val);
                                    i = i + 1;
                                }
                                ////                                    {
                                ////                                        double d = Pop(_stack);
                                ////                                        double d1 = Pop(_stack); 
                                ////                                        int val = (d1 != d) ? 1 : 0;
                                ////                                        _stack.Push(val.ToString());
                                ////                                        i = i + 1;
                                ////                                    }
                                break;
                            ////                            case TOKEN_like://like
                            ////                            {
                            ////                                string s1 = PopString(_stack);
                            ////                                string s = PopString(_stack); 
                            ////                                bool b = true;
                            ////                                if (s.Length == 0 && s1.Length == 0)
                            ////                                    b = true;
                            ////                                else if (s.Length == 0 || s1.Length == 0)
                            ////                                    b = false;
                            ////                                else
                            ////                                    b = Microsoft.VisualBasic.CompilerServices.StringType.StrLike(
                            ////                                        s, s1, Microsoft.VisualBasic.CompareMethod.Text);    
                            ////
                            ////                                int val = (b) ? TRUEVALUE : FALSEVALUE;
                            ////                                _stack.Push(val.ToString());
                            ////                                i = i + 1;
                            ////                            }
                            ////                                break;
                            ////                            case TOKEN_match://match
                            ////                            {
                            ////                                string s1 = PopString(_stack);
                            ////                                string s = PopString(_stack); 
                            ////                                bool b = true;
                            ////                                if (s.Length == 0 && s1.Length == 0)
                            ////                                    b = true;
                            ////                                else if (s.Length == 0 || s1.Length == 0)
                            ////                                    b = false;
                            ////                                else
                            ////                                {
                            ////                                    if (regexValue == null || oldregexValueCompare != s1)
                            ////                                    {
                            ////                                        regexValue = new System.Text.RegularExpressions.Regex(s1);
                            ////                                        oldregexValueCompare = s1;
                            ////                                    }
                            ////                                    b = regexValue.IsMatch(s);
                            ////                                }
                            ////                                int val = (b) ? TRUEVALUE : FALSEVALUE;
                            ////                                _stack.Push(val.ToString());
                            ////                                i = i + 1;
                            ////                            } 
                            ////                                break;
                            ////                            case TOKEN_in://in uses Like
                            ////                            {
                            ////                                string s1 = PopString(_stack);
                            ////                                string s = PopString(_stack); 
                            ////                                bool b = false;
                            ////                                if (s.Length == 0 && s1.Length == 0)
                            ////                                    b = true;
                            ////                                else if (s.Length == 0 || s1.Length == 0)
                            ////                                    b = false;
                            ////                                else
                            ////                                {
                            ////                                    foreach(string s2 in s1.Split(BRACEDELIMETER))
                            ////                                    {
                            ////                                        b |= Microsoft.VisualBasic.CompilerServices.StringType.StrLike(
                            ////                                            s, s2, Microsoft.VisualBasic.CompareMethod.Text);    
                            ////
                            ////                                        //                                if (regexValue == null || oldregexValueCompare != s2)
                            ////                                        //                                {
                            ////                                        //                                    regexValue = new System.Text.RegularExpressions.Regex(s2);
                            ////                                        //                                    oldregexValueCompare = s2;
                            ////                                        //                                }
                            ////                                        //                                b |= regexValue.IsMatch(s);
                            ////                                    }
                            ////                                }
                            ////                                int val = (b) ? TRUEVALUE : FALSEVALUE;
                            ////                                _stack.Push(val.ToString());
                            ////                                i = i + 1;
                            ////                            } 
                            ////                                break;
                            ////                            case TOKEN_between: //handles dates - use TODAY for todays's date
                            ////                            {
                            ////                                string s1 = PopString(_stack);
                            ////                                string s = PopString(_stack); 
                            ////                                bool b = false;
                            ////                                if (s.Length == 0 && s1.Length == 0)
                            ////                                    b = true;
                            ////                                else if (s.Length != 0 && s1.Length != 0)
                            ////                                {
                            ////                                    //Uses only dates, no time...
                            ////                                    DateTime leftDate = (s.Length == 0) ? DateTime.MinValue.Date : DateTime.Parse(s).Date;
                            ////                                    DateTime startDate = DateTime.MinValue.Date;
                            ////                                    DateTime endDate = DateTime.MaxValue.Date;
                            ////                                    string[] dates = s1.Split(BRACEDELIMETER);
                            ////                                    if(dates.GetLength(0) == 2)
                            ////                                    {
                            ////                                        if(dates[0] == "today")
                            ////                                            startDate = DateTime.Now.Date;
                            ////                                        else if(dates[0].Length > 0)
                            ////                                            startDate = DateTime.Parse(dates[0]).Date;
                            ////                                        if(dates[1] == "today")
                            ////                                            endDate = DateTime.Now.Date;
                            ////                                        else if(dates[1].Length > 0)
                            ////                                            endDate = DateTime.Parse(dates[1]).Date;
                            ////                                    }
                            ////                                    b = (startDate <= leftDate && endDate > leftDate)
                            ////                                        || (endDate == startDate && endDate == leftDate);
                            ////                            
                            ////                                }
                            ////                                int val = (b) ? TRUEVALUE : FALSEVALUE;
                            ////                                _stack.Push(val.ToString());
                            ////                                i = i + 1;
                            ////                            } 
                            ////                                break;
                            case TOKEN_and: ////and strings....
                                {
                                    string s1 = PopString(_stack);
                                    if (s1.Length > 0 && s1[0] == TIC[0])
                                    {
                                        if (s1.Length > 1 && s1[s1.Length - 1] == TIC[0])
                                        {
                                            s1 = s1.Substring(1, s1.Length - 2);
                                        }
                                    }

                                    string s2 = PopString(_stack);
                                    if (s2.Length > 0 && s2[0] == TIC[0])
                                    {
                                        if (s2.Length > 1 && s2[s2.Length - 1] == TIC[0])
                                        {
                                            s2 = s2.Substring(1, s2.Length - 2);
                                        }
                                    }

                                    if (useNoAmpersandQuotes)
                                    {
                                        _stack.Push(s2 + s1);
                                    }
                                    else
                                    {
                                        _stack.Push(TIC + s2 + s1 + TIC);
                                    }
                                    ////                                double d = Pop(_stack);
                                    ////                                double d1 = Pop(_stack);
                                    ////                    
                                    ////                                bool b = IsEqual(d1, TRUEVALUE) && IsEqual(d, TRUEVALUE);
                                    ////                                int val = (b) ? TRUEVALUE : FALSEVALUE;
                                    ////                                _stack.Push(val.ToString());
                                    i = i + 1;
                                }

                                break;
                            case TOKEN_pow: ////^ power
                                {
                                    double d = Pop(_stack);
                                    double d1 = Pop(_stack);
                                    double val = Math.Pow(d1, d);
                                    _stack.Push(val.ToString());
                                    i = i + 1;
                                }

                                break;
                            default:
                                {
                                    computedValueLevel = 0;
                                    if (circCheckList != null)
                                    {
                                        circCheckList.Add(this.cell);
                                        return "0";
                                    }

                                    throw new ArgumentException(FormulaErrorStrings[invalid_expression]);
                                }
                        }
                    }
                }

                if (_stack.Count == 0)
                {
                    return string.Empty;
                }
                else
                {
                    ////                    if(_stack.Count > 1 && computedValueLevel == 1)
                    ////                    {
                    ////                        return FormulaErrorStrings[missing_operand];
                    ////                    }
                    string s = _stack.Pop().ToString();
                    if (s.Equals("\"#REF!\"") || s.Equals("\"\"") || s.Equals("\"#N/A\""))
                    {
                           s = s.Substring(1, s.Length - 2);
                    }

                    if (GridUtil.IsEmpty(s) && !UseNoAmpersandQuotes)
                    {
                        s = "0"; ////empty is zero in calculation
                    }

                    return s;
                }
            }
            catch (Exception ex)
            {
                computedValueLevel = 0;
                if (circCheckList != null)
                {
                    circCheckList.Add(this.cell);
                    return "0";
                }

                if (ex.Message.IndexOf(FormulaErrorStrings[circular_reference_]) > -1)
                {
                    return "#REF!";
                }

                if (!ExceptionManager.RaiseExceptionCatched(this, ex))
                {
                    throw ex;
                }

                if (ex.Message.IndexOf(FormulaErrorStrings[cell_empty]) > -1)
                {
                    return string.Empty;
                }
                else
                {
                    return ex.Message; // + ":" + formula;
                }
            }
            finally
            {
                computedValueLevel--;
            }
        }

        #region code to handle support for short circuiting IF calculations

        private bool allowShortCircuitIFs = false;

        /// <summary>
        /// Gets or sets a value indicating whether IF function calculations should specifically avoid
        /// computing the non-used alternative.
        /// </summary>
        /// <remarks>
        /// The default value is false for code legacy consistency. When AllowShortCircuitIFs  
        /// is set true, only the necessary alternative of an IF function is computed. To support
        /// this behavior, a change in how nested IF function calculations are done is necessary.
        /// The default way of calculating nested functions is inside-out, with the inner most 
        /// functions being computed to a value before the next outer function is evaluated. To 
        /// support short circuiting IF functions, nested IF functions need to be computed from 
        /// the outside-in to know what alternative needs to be evaluated. This outside-in calculation
        /// pattern only applies to IF functions, and only when AllowShortCircuitIFs is true.
        /// </remarks>
        public bool AllowShortCircuitIFs
        {
            get { return allowShortCircuitIFs; }
            set { allowShortCircuitIFs = value; }
        }

        private int MatchingRightBracket(string formula)
        {
            int ret = -1;
            int loc = 1;
            int bracketLevel = 0;
            while (ret == -1 && loc < formula.Length)
            {
                if (formula[loc] == RIGHTBRACKET)
                {
                    if (bracketLevel == 0)
                    {
                        ret = loc;
                    }
                    else
                    {
                        bracketLevel--;
                    }
                }
                else if (formula[loc] == 'q')
                {
                    bracketLevel++;
                }

                loc++;
            }

            return ret;
        }

        private string HandleEmbeddedIf(string formula)
        {
            int sep1 = IFMarker.Length;
            if (FindNextSeparator(formula, ref sep1))
            {
                int sep2 = sep1 + 1;
                if (!FindNextSeparator(formula, ref sep2))
                {
                    sep2 = formula.IndexOf(RIGHTBRACKET, sep1);
                    if (sep2 > -1)
                    {
                        formula = formula.Substring(0, sep2) + ParseArgumentSeparator + FALSEVALUESTR + formula.Substring(sep2);
                    }
                }

                string ifValue = ComputedValue(formula.Substring(IFMarker.Length, sep1 - IFMarker.Length));
                double d = 0;
                if (ifValue == TRUEVALUESTR ||
                    (double.TryParse(ifValue, out d) && d != 0))
                {
                    formula = formula.Substring(sep1 + 1, sep2 - sep1 - 1);
                }
                else
                {
                    int rightBracket = sep2 + 1;
                    if (FindRightBracket(formula, ref rightBracket))
                    {
                        formula = formula.Substring(sep2 + 1, rightBracket - sep2 - 1);
                    }
                }
            }

            return formula;
        }

        private bool FindNextSeparator(string formula, ref int location)
        {
            int qCount = 0;
            bool found = false;
            while (!found && location < formula.Length)
            {
                if (formula[location] == 'q')
                {
                    qCount++;
                }
                else if (formula[location] == RIGHTBRACKET)
                {
                    qCount--;
                }
                else if (qCount == 0 && formula[location] == ParseArgumentSeparator)
                {
                    found = true;
                    location--;
                }

                location++;
            }

            return found;
        }

        private bool FindRightBracket(string formula, ref int location)
        {
            int qCount = 0;
            bool found = false;
            while (!found && location < formula.Length)
            {
                if (formula[location] == 'q')
                {
                    qCount++;
                }
                else if (qCount == 0 && formula[location] == RIGHTBRACKET)
                {
                    found = true;
                    location--;
                }
                else if (formula[location] == RIGHTBRACKET)
                {
                    qCount--;
                }

                location++;
            }

            return found;
        }

        #endregion

        ////parses so an empty string is treated as 0 instead of being an invalid double.
        ////used in logical expression calculations
        private bool ParsePossibleEmptyStrings(string s1, string s2, out double d1, out double d2)
        {
            bool b = false;
            d1 = 0;
            d2 = 0;
            if ((s1.Length == 0 || double.TryParse(s1, out d1))
                && (s2.Length == 0 || double.TryParse(s2, out d2)))
            {
                b = true;
            }

            return b;
        }

        private double Pop(Stack _stack)
        {
            object o = _stack.Pop();
            string s = "0";
            if (o != null)
            {
                s = o.ToString().Replace(TIC, string.Empty);
                if (s.Length > 0 && s[0] == UNIQUESTRINGMARKER)
                {
                    s = s.Substring(1);
                }
                ////change 070704
                if (s == TRUEVALUESTR)
                {
                    return 1;
                }
                else if (s == FALSEVALUESTR)
                {
                    return 0;
                }
                /////////////////////////
            }

            double d;
            //Fix for defect #13212.
            //if (double.TryParse(s, NumberStyles.Any, null, out d))
            if (double.TryParse(s, out d))
            {
                return d;
            }
            else
            {
                return returnNAN && s.Length > 0 ? double.NaN : 0;
            }
            ////return 0;
        }

        private bool returnNAN = false;

        /// <summary>
        /// Gets or sets a value indicating whether numerical formulas involving cells that hold strings return an error.
        /// </summary>
        /// <remarks>
        /// The default behavior is for the values of cells that hold strings to be treated as a zero if those 
        /// cells are used in numerical calculations. When this property is true, the calculation value is returned
        /// as double.NAN and an #Value! is displayed in the computed formula.
        /// </remarks>
        public bool ReturnNAN
        {
            get { return returnNAN; }
            set { returnNAN = value; }
        }

        ////Logical and string variable used in parsing / computing.
        ////System.Text.RegularExpressions.Regex regexValue = null; //used in string matches
        ////string oldregexValueCompare = string.Empty;
        private double ABSOLUTEZERO = 1e-20;
        ////private int TRUEVALUE = 1;
        ////private int FALSEVALUE = 0;

        private string PopString(Stack _stack)
        {
            object o = _stack.Pop();
            if (o == null)
            {
                ////throw new ArgumentException("Cell Empty");
                o = string.Empty;
            }

            return o.ToString();
        }

        private bool IsEqual(double d1, double d2)
        {
            return Math.Abs(d1 - d2) < ABSOLUTEZERO;
        }

        /// <summary>
        /// Returns whether or not the calculation for this GridFormulaTag is current or not.
        /// </summary>
        /// <param name="tag">The GridFormulaTag to be tested.</param>
        /// <returns>Returns true if the tag has been computed with the most recent values, false otherwise.
        /// </returns>
        public bool IsDirty(GridFormulaTag tag)
        {
            return tag == null || tag.Text == null;
        }

        /// <summary>
        /// Marks the underlying formula as dirty, indicating it needs to be recomputed. It does this by setting the Text property to null.
        /// </summary>
        /// <param name="tag">The GridFormulaTag to be marked as not computed.</param>
        public void SetDirty(GridFormulaTag tag)
        {
            if (tag != null)
            {
                tag.Text = null;
            }
        }

        #endregion

        #region GridFormulaParsing event

        /// <summary>
        /// Occurs whenever a string needs to be tested to determine whether it should be treated as a formula string and parsed,
        /// or be treated as a non-formula string. This event allows for preprocessing the unparsed formula.
        /// </summary>
        /// <remarks>This event may be raised more than once in the processing of a string into a formula.
        /// </remarks>
        public event GridFormulaParsingEventHandler FormulaParsing;

        /// <summary>
        /// Given a string, it returns a string that is passed through the FormulaParsing event
        /// to allow any listener to modify it.
        /// </summary>
        /// <param name="s">A string that is to be potentially parsed.</param>
        //// <returns>The string returned by GridFormulaParsingEventArgs.Text.</returns>
        internal void GetFormulaText(ref string s)
        {
            if (FormulaParsing != null)
            {
                GridFormulaParsingEventArgs e = new GridFormulaParsingEventArgs(s);
                FormulaParsing(this, e);
                s = e.Text;
            }
        }
        #endregion

        #region Circular Check
        /// <summary>
        /// Determines whether the given formula at the given cell will cause a circular reference.
        /// </summary>
        /// <param name="cell">The alphanumeric cell label, like A1, or EE14.</param>
        /// <param name="formula">The formula to be tested.</param>
        /// <returns>True if the given formula causes a circular reference, false otherwise.</returns>
        public bool IsCircularReference(string cell, string formula)
        {
            string parsedFormula = this.Parse(formula);
            bool save = this.lockDependencies;
            this.lockDependencies = true;
            try
            {
                if (CircularCheckFailed(cell, parsedFormula))
                {
                    return true;
                }
            }
            finally
            {
                this.lockDependencies = save;
            }

            return false;
        }

        internal bool CircularCheckFailed(string hostCell, string parsedFormula)
        {
            GridFormulaEngine engine = this;
            GridSheetFamilyItem family = GridFormulaEngine.GetSheetFamilyItem(this.grid);
            ArrayList dependCells = GetCellsList(parsedFormula);
            if (dependCells.IndexOf(hostCell) > -1)
            {
                return true;
            }

            foreach (string s in dependCells)
            {
                string sheet = engine.SheetToken(s);

                GridModel grd1 = (sheet.Length == 0)
                    ? this.grid : family.TokenToGridModel[sheet] as GridModel;
                int row = engine.RowIndex(s);
                int col = engine.ColIndex(s);
                string formula = grd1[row, col].Text;
                engine.GetFormulaText(ref formula);
                bool usesFormulaChar = engine.formulaChar != (char)0 && formula.Length > 0 && formula[0] == engine.formulaChar;
                if (engine.formulaChar == (char)0 || usesFormulaChar)
                {
                    if (grd1[row, col].FormulaTag != null && grd1[row, col].FormulaTag.Formula.Length > 0)
                    {
                        if (CircularCheckFailed(s, grd1[row, col].FormulaTag.Formula))
                        {
                            return true;
                        }
                    }
                }
            }

            return false;
        }

        internal ArrayList GetCellsList(string parsedFormula)
        {
            GridFormulaEngine engine = this;
            ArrayList checkList = new ArrayList();
            engine.ComputedValue(parsedFormula, checkList);

            ////fix for defect 786 - make sure sheet reference is part of cell
            string sheet = engine.SheetToken(engine.cell);
            if (sheet.Length > 0)
            {
                for (int i = 0; i < checkList.Count; ++i)
                {
                    string s = checkList[i].ToString();
                    if (engine.SheetToken(s).Length == 0)
                    {
                        checkList[i] = sheet + s;
                    }
                }
            }

            return checkList;
        }

        #endregion

        #region SortedArrayList
        /// <summary>
        /// ArrayList derived class that exposes a method to insert the string in a sorted position if not present.
        /// </summary>
        /// <remarks>This class can be used to maintain a sorted list of strings so ArrayList.BinarySearch can be used 
        /// to quickly locate a member string.</remarks>
        internal class SortedArrayList : ArrayList
        {
            /// <summary>
            /// Inserts a string into its sorted position if not present in the list.
            /// </summary>
            /// <param name="s">The string to be tested.</param>
            /// <returns>True if the string was inserted.</returns>
            internal bool InsertIfNotPresent(string s)
            {
                bool b = false;
                int i = BinarySearch(s);
                if (i < 0)
                {
                    this.Insert(-i - 1, s);
                    b = true;
                }

                return b;
            }
        }

        #endregion
    }
    #endregion

    #region GridSheetFamilyItem Class
    /// <summary>
    /// Encapsulates the properties that are needed to support multiple families of crossed-referenced grids.
    /// This class is for internal use only.
    /// </summary>
    public class GridSheetFamilyItem
    {
        internal Hashtable sheetDependentCells = null;
        internal Hashtable sheetDependentFormulaCells = null;
        internal Hashtable sheetDependentNamedRangeCells = null;
        internal Hashtable sheetNamedRanges = null;
        internal Hashtable namedRangeValues = null;
        internal Hashtable sheetNamedRangesOriginalNames = null;
        internal Hashtable SheetNameToGridModel = null;
        internal Hashtable TokenToGridModel = null;
        internal Hashtable GridModelToToken = null;
        internal Hashtable SheetNameToToken = null;
        internal Hashtable TokenToSheetName = null;
        internal bool isSheeted = false;
        internal ArrayList sheetNamesSized = null;

        /// <summary>
        /// Default Constructor.
        /// </summary>
        public GridSheetFamilyItem()
            : base()
        {
        }
    }
    #endregion

    #region GridFormulaCopyFlags enums

    /// <summary>
    /// Specifies behavior for copy, paste, and drag-and-drop operations.
    /// </summary>
    /// <remarks>
    /// Settings determine whether the formula or the calculated result is copied
    /// and whether formula references are updated when cells containing formulas are pasted or dropped.
    /// </remarks>
    [Flags]
    public enum GridFormulaCopyFlags
    {
        /// <summary>
        /// No copy flags set.
        /// </summary>
        None = 0x00,

        /// <summary>
        /// The computed value text is placed on the Clipboard.
        /// </summary>
        ClipBoardText = 0x01,

        /// <summary>
        /// The formula text is placed on the Clipboard.
        /// </summary>
        ClipBoardFormula = 0x02,

        /// <summary>
        /// Cell references in pasted formulas are adjusted for their new location.
        /// </summary>
        ClipBoardReferencesAdjusted = 0x04,

        ////        /// <summary>
        ////        /// The computed value text is placed in the DataObject.
        ////        /// </summary>
        ////        DnDText        = 0x08,
        ////        /// <summary>
        ////        /// The formula text is placed in the DataObject.
        ////        /// </summary>
        ////        DnDFormula    = 0x10,
        ////        /// <summary>
        ////        /// Allow extend existing selection when user holds Shift Key and clicks on a cell.
        ////        /// </summary>
        ////        DndReferencesAdjusted      = 0x20,
        ////        /// <summary>
        ////        /// Cell references are updated when rows or columns are moved.
        ////        /// </summary>
        ////        MovedRangeReferencesUpdated      = 0x40,

        /// <summary>
        /// Cell references are updated when rows or columns are removed or inserted.
        /// </summary>
        InsDelRangeReferencesUpdated = 0x80,

        /// <summary>
        /// Cell references are updated when rows or columns are removed or inserted.
        /// </summary>
        NamedRangeReferencesUpdated = 0x100,

        /// <summary>
        /// Cell references are updated if the cell is moved using an Ole Drag and Drop operation within the same GridControl.
        /// </summary>
        UpdateReferencesDuringDnD = 0x200
    }
    #endregion

    #region GridFormulaParsingEventArgs class

    /// <summary>
    /// Event delegate for the GridFormulaParsing event 
    /// </summary>
    public delegate void GridFormulaParsingEventHandler(object sender, GridFormulaParsingEventArgs e);

    /// <summary>
    /// Used by the <see cref="GridFormulaParsing"/> event, GridFormulaParsingEventArgs holds a reference 
    /// to the string that is to be parsed. The GridFormulaParsing event allows the listener to preprocess
    /// the string that is being parsed.
    /// </summary>
    /// <remarks>
    /// Please note that this event may be raised more than once as a string is parsed. 
    /// </remarks>
    /// <example> Here is a code snippet that shows how to tell a grid to also treat any text in a formula cell 
    /// that begins with a minus(-) or a plus(+) as formulas. The default behavior is to treat only text beginning
    /// with equal(=) as formulas.
    /// <code lang="C#">
    ///        //subscribe to the event before any formulas are loaded into the grid...
    ///        this.engine = ((GridFormulaCellModel)gridControl1.CellModels["FormulaCell"]).Engine;
    ///     this.engine.FormulaParsing += new GridFormulaParsingEventHandler(engine_FormulaParsing);
    ///  <para/>    
    ///        //Here is the handler code that adds an = if necessary so any string beginning with +, - or =
    ///     //is treated as a formula.
    ///       void engine_FormulaParsing(object sender, GridFormulaParsingEventArgs e)
    ///       {
    ///           //allow cells starting with + and - to be treated as formula cells.
    ///           if (e.Text.StartsWith("-"))
    ///                e.Text = "=" + e.Text;
    ///           else if (e.Text.StartsWith("+"))
    ///                e.Text = "=" + e.Text.Substring(1);
    ///       }         
    /// </code>
    /// </example>
    public class GridFormulaParsingEventArgs : EventArgs
    {
        /// <summary>
        /// Default constructor.
        /// </summary>
        public GridFormulaParsingEventArgs()
        {
        }

        /// <summary>
        /// Holds a reference to the string that is to be parsed..
        /// </summary>
        /// <param name="text">The formula that is to be parsed.</param>
        public GridFormulaParsingEventArgs(string text)
        {
            this.text = text;
        }

        string text;

        /// <summary>
        /// Gets or sets the formula about to be parsed.
        /// </summary>
        public string Text
        {
            get { return text; }
            set { text = value; }
        }
    }
    #endregion
}
